@@ -87,25 +87,25 @@ action_boperror( Reduce *rc, Compile *compile, const char *str,
 	const char *top_str = str ? str : _( "Bad arguments." );
 	const char *op_name = op >= 0 ? decode_BinOp( op ) : name;
 
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[MAX_ERROR_FRAG];
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt2[MAX_ERROR_FRAG];
-	BufInfo buf3;
+	VipsBuf buf3;
 	char txt3[MAX_ERROR_FRAG];
 
-	buf_init_static( &buf, txt, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf, txt, MAX_ERROR_FRAG );
 	itext_value_ev( rc, &buf, a );
 
-	buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
 	itext_value_ev( rc, &buf2, b );
 
-	buf_init_static( &buf3, txt3, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf3, txt3, MAX_ERROR_FRAG );
 	if( compile ) {
 		/* Expands to eg. 'bad args to "+", called from "fred"'
 		 */
-		buf_appends( &buf3, _( "Called from" ) );
-		buf_appends( &buf3, " " );
+		vips_buf_appends( &buf3, _( "Called from" ) );
+		vips_buf_appends( &buf3, " " );
 		compile_name( compile, &buf3 );
 	}
 
@@ -113,7 +113,7 @@ action_boperror( Reduce *rc, Compile *compile, const char *str,
 	error_sub( _( "Error in binary \"%s\".\n"
 		"left = %s\n"
 		"right = %s\n%s" ),
-		op_name, buf_all( &buf ), buf_all( &buf2 ), buf_all( &buf3 ) );
+		op_name, vips_buf_all( &buf ), vips_buf_all( &buf2 ), vips_buf_all( &buf3 ) );
 	reduce_throw( rc );
 }
 
@@ -122,16 +122,16 @@ action_boperror( Reduce *rc, Compile *compile, const char *str,
 static void
 action_nomerror( Reduce *rc, Compile *compile, PElement *a, PElement *b )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[500];
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt2[MAX_ERROR_FRAG];
-	BufInfo buf3;
+	VipsBuf buf3;
 	char txt3[MAX_ERROR_FRAG];
 
-	buf_init_static( &buf, txt, 500 );
-	buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
-	buf_init_static( &buf3, txt3, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf, txt, 500 );
+	vips_buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf3, txt3, MAX_ERROR_FRAG );
 
 	if( PEISCLASS( a ) ) 
 		symbol_qualified_name( PEGETCLASSCOMPILE( a )->sym, &buf3 );
@@ -142,30 +142,30 @@ action_nomerror( Reduce *rc, Compile *compile, PElement *a, PElement *b )
 	else if( PEISCOMPILEREF( a ) ) 
 		symbol_qualified_name( PEGETCOMPILE( a )->sym, &buf3 );
 	else
-		buf_appends( &buf3, "<thing>" );
+		vips_buf_appends( &buf3, "<thing>" );
 	itext_value_ev( rc, &buf2, b );
-	buf_appendf( &buf, _( "Member \"%s\" not found in class \"%s\"." ),
-		buf_all( &buf2 ), buf_all( &buf3 ) );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, _( "Member \"%s\" not found in class \"%s\"." ),
+		vips_buf_all( &buf2 ), vips_buf_all( &buf3 ) );
+	vips_buf_appendf( &buf, "\n" );
 
-	buf_rewind( &buf3 );
+	vips_buf_rewind( &buf3 );
 	itext_value_ev( rc, &buf3, a );
-	buf_appendf( &buf, "  " );
-	buf_appendf( &buf, _( "object = %s" ), buf_all( &buf3 ) );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "  " );
+	vips_buf_appendf( &buf, _( "object = %s" ), vips_buf_all( &buf3 ) );
+	vips_buf_appendf( &buf, "\n" );
 
-	buf_appendf( &buf, "  " );
-	buf_appendf( &buf, _( "tag = %s" ), buf_all( &buf2 ) );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "  " );
+	vips_buf_appendf( &buf, _( "tag = %s" ), vips_buf_all( &buf2 ) );
+	vips_buf_appendf( &buf, "\n" );
 
-	buf_rewind( &buf3 );
+	vips_buf_rewind( &buf3 );
 	symbol_qualified_name( compile->sym, &buf3 );
-	buf_appendf( &buf, _( "Reference attempted in \"%s\"." ),
-		buf_all( &buf3 ) );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, _( "Reference attempted in \"%s\"." ),
+		vips_buf_all( &buf3 ) );
+	vips_buf_appendf( &buf, "\n" );
 
 	error_top( _( "Member not found." ) );
-	error_sub( "%s", buf_all( &buf ) );
+	error_sub( "%s", vips_buf_all( &buf ) );
 	reduce_throw( rc );
 }
 
@@ -178,27 +178,27 @@ action_uoperror( Reduce *rc, Compile *compile,
 	const char *top_str = str ? str : _( "Bad argument." );
 	const char *op_name = op >= 0 ? decode_UnOp( op ) : name;
 
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[MAX_ERROR_FRAG];
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt2[MAX_ERROR_FRAG];
 
-	buf_init_static( &buf, txt, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf, txt, MAX_ERROR_FRAG );
 	itext_value_ev( rc, &buf, a );
 
-	buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf2, txt2, MAX_ERROR_FRAG );
 	if( compile ) {
 		/* Expands to eg. 'bad args to "+", called from "fred"'
 		 */
-		buf_appends( &buf2, _( "Called from" ) );
-		buf_appends( &buf2, " " );
+		vips_buf_appends( &buf2, _( "Called from" ) );
+		vips_buf_appends( &buf2, " " );
 		compile_name( compile, &buf2 );
 	}
 
 	error_top( "%s", top_str );
 	error_sub( _( "Error in unary \"%s\".\n"
 		"argument = %s\n%s" ),
-		op_name, buf_all( &buf ), buf_all( &buf2 ) );
+		op_name, vips_buf_all( &buf ), vips_buf_all( &buf2 ) );
 	reduce_throw( rc );
 }
 
@@ -325,6 +325,17 @@ action_proc_dot_add_link( Expr *expr, Symbol *child )
 	return( link_add( child, expr, TRUE ) );
 }
 
+static char *
+action_proc_dot_tag( PElement *b, char *tag, int n )
+{
+	if( PEISTAG( b ) ) 
+		return( PEGETTAG( b ) );
+	else if( heap_get_string( b, tag, n ) )
+		return( tag );
+
+	return( NULL );
+}
+
 /* Extract field from object. Be careful, a can be equal to out. 
  */
 static void
@@ -332,12 +343,13 @@ action_proc_dot( Reduce *rc, Compile *compile,
 	int op, const char *name, PElement *a, PElement *b, PElement *out )
 {
 	char tag[256];
+	char *p;
 
 	if( PEISCLASS( a ) ) {
 		PElement c;
 
-		if( PEISTAG( b ) ) {
-			if( !class_get_member( a, PEGETTAG( b ), NULL, &c ) )
+		if( (p = action_proc_dot_tag( b, tag, 256 )) ) {
+			if( !class_get_member( a, p, NULL, &c ) )
 				action_nomerror( rc, compile, a, b );
 
 			PEPUTPE( out, &c );
@@ -348,12 +360,6 @@ action_proc_dot( Reduce *rc, Compile *compile,
 
 			PEPUTPE( out, &c );
 		}
-		else if( heap_get_string( b, tag, 256 ) ) {
-			if( !class_get_member( a, tag, NULL, &c ) )
-				action_nomerror( rc, compile, a, b );
-
-			PEPUTPE( out, &c );
-		}
 		else
 			action_boperror( rc, compile, 
 				_( "Bad right hand side of '.'." ),
@@ -370,19 +376,13 @@ action_proc_dot( Reduce *rc, Compile *compile,
 				"is not scope" ), 
 				op, name, a, b );
 
-		assert( sym->expr );
+		g_assert( sym->expr );
 
-		if( PEISTAG( b ) ) 
-			child = compile_lookup( sym->expr->compile, 
-				PEGETTAG( b ) );
-		else if( heap_get_string( b, tag, 256 ) ) 
-			child = compile_lookup( sym->expr->compile, tag );
-		else
+		if( !(p = action_proc_dot_tag( b, tag, 256 )) ) 
 			action_boperror( rc, compile, 
 				_( "Bad right hand side of '.'." ),
 				op, name, a, b );
-
-		if( !child )
+		if( !(child = compile_lookup( sym->expr->compile, p )) )
 			action_nomerror( rc, compile, a, b );
 
 		/* Add all exprs which use compile to dynamic link graph.
@@ -402,6 +402,31 @@ action_proc_dot( Reduce *rc, Compile *compile,
 			PEPUTP( out, ELEMENT_SYMREF, child );
 		}
 	}
+	else if( PEISMANAGEDGOBJECT( a ) ) {
+		GObject *gobject = PEGETMANAGEDGOBJECT( a );
+		GObjectClass *gclass = G_OBJECT_GET_CLASS( gobject );
+		GValue value = { 0 };
+		GParamSpec *pspec;
+
+		if( !(p = action_proc_dot_tag( b, tag, 256 )) )
+			action_boperror( rc, compile, 
+				_( "Bad right hand side of '.'." ),
+				op, name, a, b );
+		if( !(pspec = g_object_class_find_property( gclass, p )) )
+			action_boperror( rc, compile, 
+				_( "Property not found." ),
+				op, name, a, b );
+
+		g_value_init( &value, G_PARAM_SPEC_VALUE_TYPE( pspec ) );
+		g_object_get_property( gobject, p, &value);
+
+		if( !heap_gvalue_to_ip( &value, out ) ) {
+			g_value_unset( &value );
+			reduce_throw( rc );
+		}
+
+		g_value_unset( &value );
+	}
 	else
 		action_boperror( rc, compile, _( "Bad left hand side of '.'." ),
 			op, name, a, b );
@@ -419,7 +444,8 @@ action_proc_lesseq( Reduce *rc, Compile *compile,
 	else if( PEISCHAR( a ) && PEISCHAR( b ) ) {
 		PEPUTP( out, ELEMENT_BOOL, PEGETCHAR( a ) <= PEGETCHAR( b ) );
 	}
-	else if( PEISLIST( a ) && PEISLIST( b ) ) {
+	else if( PEISLIST( a ) && PEISLIST( b ) &&
+		reduce_is_string( rc, a ) && reduce_is_string( rc, b ) ) {
 		char a_string[MAX_STRSIZE];
 		char b_string[MAX_STRSIZE];
 
@@ -452,7 +478,8 @@ action_proc_less( Reduce *rc, Compile *compile,
 	else if( PEISCHAR( a ) && PEISCHAR( b ) ) {
 		PEPUTP( out, ELEMENT_BOOL, PEGETCHAR( a ) < PEGETCHAR( b ) );
 	}
-	else if( PEISLIST( a ) && PEISLIST( b ) ) {
+	else if( PEISLIST( a ) && PEISLIST( b ) &&
+		reduce_is_string( rc, a ) && reduce_is_string( rc, b ) ) {
 		char a_string[MAX_STRSIZE];
 		char b_string[MAX_STRSIZE];
 
@@ -520,7 +547,7 @@ action_node_equal( Reduce *rc, HeapNode *a, HeapNode *b )
 	case TAG_SHARED:
 	case TAG_REFERENCE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -541,7 +568,7 @@ action_image_equal( Reduce *rc, Imageinfo *a, Imageinfo *b )
 	 */
 	ii[0] = a;
 	ii[1] = b;
-	assert( !ii[0] && !ii[1] );
+	g_assert( !ii[0] && !ii[1] );
 	if( ii[0] == ii[1] )
 		/* Trivial!
 		 */
@@ -585,6 +612,24 @@ action_image_equal( Reduce *rc, Imageinfo *a, Imageinfo *b )
 	return( mn == 255 );
 }
 
+/* One of p1/p2 is a managedstring. 
+ *
+ * This is pretty dumb. We could have a special loop down the list side which
+ * compared to the managedstring directly, but I doubt if this will ever be a
+ * performance issue.
+ */
+static gboolean
+action_string_equal( Reduce *rc, PElement *p1, PElement *p2 )
+{
+	char a_string[MAX_STRSIZE];
+	char b_string[MAX_STRSIZE];
+
+	reduce_get_string( rc, p1, a_string, MAX_STRSIZE );
+	reduce_get_string( rc, p2, b_string, MAX_STRSIZE );
+
+	return( strcmp( a_string, b_string ) == 0 );
+}
+
 /* Test two elements for equality. Force computation as required.
  */
 static gboolean
@@ -595,7 +640,31 @@ action_element_equal( Reduce *rc, PElement *p1, PElement *p2 )
 	reduce_spine( rc, p1 );
 	reduce_spine( rc, p2 );
 
-	/* Easy!
+	/* We can often test for eg. "fred" == "fred" by just checking
+	 * pointers.
+	 */
+	if( PEGETTYPE( p1 ) == PEGETTYPE( p2 ) ) {
+		switch( PEGETTYPE( p1 ) ) {
+		case ELEMENT_CHAR:
+		case ELEMENT_NODE:
+		case ELEMENT_BOOL:
+		case ELEMENT_MANAGED:
+			if( PEGETVAL( p1 ) == PEGETVAL( p2 ) )
+				return( TRUE );
+			break;
+
+		case ELEMENT_ELIST:
+			return( TRUE );
+		}
+	}
+
+	/* Special case if either is a managedstring.
+	 */
+	if( PEISMANAGEDSTRING( p1 ) ||
+		PEISMANAGEDSTRING( p2 ) ) 
+		return( action_string_equal( rc, p1, p2 ) );
+
+	/* No other implicit conversions, so types must match.
 	 */
 	if( PEGETTYPE( p1 ) != PEGETTYPE( p2 ) )
 		return( FALSE );
@@ -636,9 +705,8 @@ action_element_equal( Reduce *rc, PElement *p1, PElement *p2 )
 		return( TRUE );
 
 	case ELEMENT_NOVAL:
-	case ELEMENT_STATIC:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -1552,9 +1620,9 @@ action_proc_construct( Reduce *rc,
 	Compile *compile, HeapNode **arg, PElement *out )
 {
 	if( trace_flags & TRACE_CLASS_NEW ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "constructor \"%s\" ", 
+		vips_buf_appendf( buf, "constructor \"%s\" ", 
 			IOBJECT( compile->sym )->name );
 		trace_args( arg, compile->nparam + compile->nsecret );
 	}
@@ -1589,7 +1657,7 @@ action_proc_class_binary_sub( Reduce *rc, PElement *pe,
 	base = *pe;
 	heap_appl_init( &base, fn );
 	if( !heap_appl_add( rc->heap, &base, &rhs ) ||
-		!heap_string_new( rc->heap, name, &rhs ) ||
+		!heap_managedstring_new( rc->heap, name, &rhs ) ||
 		!heap_appl_add( rc->heap, &base, &rhs ) ) 
 		return( out );
 
@@ -1609,16 +1677,16 @@ action_proc_class_binary( Reduce *rc, Compile *compile,
 	PElement fn;
 
 	if( trace_flags & flags ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "%s\n", _( "invoking method:" ) );
-		buf_appends( buf, "     " );
+		vips_buf_appendf( buf, "%s\n", _( "invoking method:" ) );
+		vips_buf_appends( buf, "     " );
 		trace_pelement( a );
-		buf_appendf( buf, ".%s \"%s\" ", MEMBER_OO_BINARY, name );
+		vips_buf_appendf( buf, ".%s \"%s\" ", MEMBER_OO_BINARY, name );
 		trace_pelement( b );
-		buf_appends( buf, "\n" );
+		vips_buf_appends( buf, "\n" );
 
-		trace_text( flags, buf_all( buf ) );
+		trace_text( flags, "%s", vips_buf_all( buf ) );
 
 		trace_pop();
 	}
@@ -1645,16 +1713,16 @@ action_proc_class_binary2( Reduce *rc, Compile *compile,
 	PElement fn;
 
 	if( trace_flags & flags ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "%s\n", _( "invoking method:" ) );
-		buf_appends( buf, "     " );
+		vips_buf_appendf( buf, "%s\n", _( "invoking method:" ) );
+		vips_buf_appends( buf, "     " );
 		trace_pelement( b );
-		buf_appendf( buf, ".%s \"%s\" ", MEMBER_OO_BINARY2, name );
+		vips_buf_appendf( buf, ".%s \"%s\" ", MEMBER_OO_BINARY2, name );
 		trace_pelement( a );
-		buf_appends( buf, "\n" );
+		vips_buf_appends( buf, "\n" );
 
-		trace_text( flags, buf_all( buf ) );
+		trace_text( flags, "%s", vips_buf_all( buf ) );
 
 		trace_pop();
 	}
@@ -1772,15 +1840,15 @@ action_if( Reduce *rc, Compile *compile,
 		 */
 		if( PEISBOOL( a ) ) {
 			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_push();
+				VipsBuf *buf = trace_push();
 
-				buf_appendf( buf, "if " );
+				vips_buf_appendf( buf, "if " );
 				trace_pelement( a );
-				buf_appendf( buf, " then " );
+				vips_buf_appendf( buf, " then " );
 				trace_pelement( &t );
-				buf_appendf( buf, " else " );
+				vips_buf_appendf( buf, " else " );
 				trace_pelement( &e );
-				buf_appendf( buf, " ->\n" );
+				vips_buf_appendf( buf, " ->\n" );
 			}
 
 			if( PEGETBOOL( a ) ) {
@@ -1887,7 +1955,7 @@ action_proc_class_unary_sub( Reduce *rc, PElement *pe,
 	base = *pe;
 	heap_appl_init( &base, fn );
 	if( !heap_appl_add( rc->heap, &base, &rhs ) ||
-		!heap_string_new( rc->heap, name, &rhs ) )
+		!heap_managedstring_new( rc->heap, name, &rhs ) )
 		return( out );
 
 	PEPUTPE( out, pe );
@@ -1903,20 +1971,20 @@ action_proc_class_unary( Reduce *rc, Compile *compile,
 	PElement fn;
 
 	if( trace_flags & flags ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "%s\n", _( "invoking method:" ) );
-		buf_appends( buf, "     " );
+		vips_buf_appendf( buf, "%s\n", _( "invoking method:" ) );
+		vips_buf_appends( buf, "     " );
 		trace_pelement( a );
-		buf_appendf( buf, ".%s \"%s\"\n", MEMBER_OO_UNARY, name );
+		vips_buf_appendf( buf, ".%s \"%s\"\n", MEMBER_OO_UNARY, name );
 
-		trace_text( flags, buf_all( buf ) );
+		trace_text( flags, "%s", vips_buf_all( buf ) );
 
 		trace_pop();
 	}
 
 	/* Look up a.dispatch_unary and build 
-	 * (a.dispatch_unary "minus") application.
+	 * (a.oo_unary "minus") application.
 	 */
 	if( !class_get_member( a, MEMBER_OO_UNARY, NULL, &fn ) )
 		action_uoperror( rc, compile, error_get_sub(), op, name, a );
@@ -1943,7 +2011,11 @@ action_dispatch( Reduce *rc, Compile *compile, ReduceFunction rfn,
 	/* Don't allow nargs == 0. We rely on having a bit of graph we can
 	 * replace with (I result) for caching.
 	 */
-	assert( nargs > 0 );
+	g_assert( nargs > 0 );
+
+	/* We need to have the 
+	 */
+	g_assert( noperator_table == UN_LAST );
 
 	/* Reduce all the args.
 	 */
@@ -1964,9 +2036,9 @@ action_dispatch( Reduce *rc, Compile *compile, ReduceFunction rfn,
 	PEPOINTRIGHT( arg[1], &a );
 
 	if( trace_flags & flags ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "\"%s\" ", name );
+		vips_buf_appendf( buf, "\"%s\" ", name );
 		trace_args( arg, nargs );
 	}
 
