@@ -207,7 +207,7 @@ box_yesno( GtkWidget *par,
 		(iWindowBuildFn) box_build, 
 			buf, GTK_STOCK_DIALOG_QUESTION, NULL );
 	idialog_set_callbacks( IDIALOG( idlg ), cancelcb, NULL, NULL, client );
-	idialog_add_ok( IDIALOG( idlg ), okcb, yes_label );
+	idialog_add_ok( IDIALOG( idlg ), okcb, "%s", yes_label );
 	idialog_set_notify( IDIALOG( idlg ), nfn, sys );
 	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
@@ -265,26 +265,26 @@ about_build( iDialog *idlg, GtkWidget *work )
 	GtkWidget *lab;
 	char txt[MAX_DIALOG_TEXT];
 	char txt2[MAX_DIALOG_TEXT];
-	BufInfo buf;
+	VipsBuf buf;
 	GtkWidget *image;
 
-	buf_init_static( &buf, txt, MAX_DIALOG_TEXT );
+	vips_buf_init_static( &buf, txt, MAX_DIALOG_TEXT );
 
 	im_snprintf( txt2, MAX_DIALOG_TEXT, _( "About %s." ), PACKAGE );
-	buf_appendf( &buf, "<b><big>%s</big></b>\n\n", txt2 );
+	vips_buf_appendf( &buf, "<b><big>%s</big></b>\n\n", txt2 );
 	im_snprintf( txt2, MAX_DIALOG_TEXT, 
 		_( "%s is an image processing package." ), PACKAGE );
-	buf_appendf( &buf, "%s\n\n", txt2 );
+	vips_buf_appendf( &buf, "%s\n\n", txt2 );
 
 	im_snprintf( txt2, MAX_DIALOG_TEXT, 
 		_( "%s comes with ABSOLUTELY NO WARRANTY. This is "
 		"free software and you are welcome to redistribute "
 		"it under certain conditions, see http://www.gnu.org." ), 
 		PACKAGE );
-	buf_appendf( &buf, "%s\n\n", txt2 );
+	vips_buf_appendf( &buf, "%s\n\n", txt2 );
 
 	im_snprintf( txt2, MAX_DIALOG_TEXT, _( NIP_COPYRIGHT ), PACKAGE );
-	buf_appendf( &buf, "%s\n", txt2 );
+	vips_buf_appendf( &buf, "%s\n\n", txt2 );
 
 {
 	char buf1[FILENAME_MAX];
@@ -294,32 +294,35 @@ about_build( iDialog *idlg, GtkWidget *work )
 		get_savedir() );
 	expand_variables( buf1, buf2 );
         nativeize_path( buf2 );
-	buf_appendf( &buf, "%s %s\n", _( "Personal start folder:" ), buf2 );
+	escape_markup( buf2, buf1, FILENAME_MAX );
+	vips_buf_appendf( &buf, "<b>%s:</b> %s\n", 
+		_( "Personal start folder" ), buf1 );
 }
 
-	buf_appendf( &buf, "%s %s\n", _( "Homepage:" ), VIPS_HOMEPAGE );
+	vips_buf_appendf( &buf, "<b>%s:</b> %s\n", 
+		_( "Homepage" ), VIPS_HOMEPAGE );
 	escape_markup( im_version_string(), txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "%s %s\n", 
-		_( "Linked to VIPS:" ), txt2 );
+	vips_buf_appendf( &buf, "<b>%s:</b> %s\n", 
+		_( "Linked to VIPS" ), txt2 );
 	escape_markup( IM_VERSION_STRING, txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "%s %s\n", 
-		_( "Built against VIPS:" ), txt2 );
+	vips_buf_appendf( &buf, "<b>%s:</b> %s\n", 
+		_( "Built against VIPS" ), txt2 );
 	escape_markup( PACKAGE, txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "$PACKAGE: %s\n", txt2 );
+	vips_buf_appendf( &buf, "<b>$PACKAGE:</b> %s\n", txt2 );
 	escape_markup( VERSION, txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "$VERSION: %s\n", txt2 );
+	vips_buf_appendf( &buf, "<b>$VERSION:</b> %s\n", txt2 );
 	escape_markup( NN( g_getenv( "VIPSHOME" ) ), txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "$VIPSHOME: %s\n", txt2 );
+	vips_buf_appendf( &buf, "<b>$VIPSHOME:</b> %s\n", txt2 );
 	escape_markup( NN( g_getenv( "HOME" ) ), txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "$HOME: %s\n", txt2 );
+	vips_buf_appendf( &buf, "<b>$HOME:</b> %s\n", txt2 );
 	escape_markup( NN( g_getenv( "SAVEDIR" ) ), txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "$SAVEDIR: %s\n", txt2 );
+	vips_buf_appendf( &buf, "<b>$SAVEDIR:</b> %s\n", txt2 );
 	escape_markup( PATH_TMP, txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "%s %s", 
-		_( "Temp files in:" ), txt2 );
+	vips_buf_appendf( &buf, "<b>%s:</b> %s", 
+		_( "Temp files in" ), txt2 );
 	if( strcmp( translator_credits, "translator_credits" ) != 0 ) {
-		buf_appendf( &buf, "\n" ); 
-		buf_appends( &buf, translator_credits );
+		vips_buf_appendf( &buf, "\n" ); 
+		vips_buf_appends( &buf, translator_credits );
 	}
 
 	hb = gtk_hbox_new( FALSE, 0 );
@@ -333,7 +336,7 @@ about_build( iDialog *idlg, GtkWidget *work )
 	gtk_widget_show( image );
 
 	lab = gtk_label_new( "" );
-	gtk_label_set_markup( GTK_LABEL( lab ), buf_all( &buf ) );
+	gtk_label_set_markup( GTK_LABEL( lab ), vips_buf_all( &buf ) );
         gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
         gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
 	gtk_label_set_line_wrap( GTK_LABEL( lab ), TRUE );
@@ -777,6 +780,8 @@ imageheader_build( GtkWidget *widget )
 
 	GtkWidget *label;
 	GtkWidget *swin;
+	GtkWidget *pane;
+	GtkWidget *vbox;
 	PangoFontDescription *font_desc;
 
 #ifdef DEBUG
@@ -788,29 +793,36 @@ imageheader_build( GtkWidget *widget )
 	if( IWINDOW_CLASS( imageheader_parent_class )->build )
 		(*IWINDOW_CLASS( imageheader_parent_class )->build)( widget );
 
+	pane = gtk_vpaned_new();
+        gtk_box_pack_start( GTK_BOX( idlg->work ), pane, TRUE, TRUE, 2 );
+
 	/* Scrolled area to hold fields.
 	 */
+	vbox = gtk_vbox_new( FALSE, 2 );
+	gtk_paned_pack1( GTK_PANED( pane ), vbox, TRUE, FALSE );
 	label = gtk_label_new( _( "Image header fields" ) );
 	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), label, FALSE, FALSE, 2 );
+        gtk_box_pack_start( GTK_BOX( vbox ), label, FALSE, FALSE, 2 );
 	imageheader->swin_fields = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( 
 		GTK_SCROLLED_WINDOW( imageheader->swin_fields ),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), 
+        gtk_box_pack_start( GTK_BOX( vbox ), 
 		imageheader->swin_fields, TRUE, TRUE, 2 );
 
 	/* Created in _refresh()
 	 */
 	imageheader->fields = NULL;
 
+	vbox = gtk_vbox_new( FALSE, 2 );
+	gtk_paned_pack2( GTK_PANED( pane ), vbox, TRUE, FALSE );
 	label = gtk_label_new( _( "Image history" ) );
 	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), label, FALSE, FALSE, 2 );
+        gtk_box_pack_start( GTK_BOX( vbox ), label, FALSE, FALSE, 2 );
 	swin = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( swin ),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-        gtk_box_pack_end( GTK_BOX( idlg->work ), swin, TRUE, TRUE, 2 );
+        gtk_box_pack_end( GTK_BOX( vbox ), swin, TRUE, TRUE, 2 );
 	imageheader->history = gtk_text_view_new();
 	gtk_text_view_set_editable( GTK_TEXT_VIEW( imageheader->history ), 
 		FALSE );
@@ -881,8 +893,8 @@ imageheader_get_type( void )
 static void
 imageheader_conv_changed( Model *model, Imageheader *imageheader )
 {
-	assert( IS_MODEL( model ) );
-	assert( IS_IMAGEHEADER( imageheader ) );
+	g_assert( IS_MODEL( model ) );
+	g_assert( IS_IMAGEHEADER( imageheader ) );
 
 	imageheader_refresh( imageheader );
 }
@@ -945,27 +957,27 @@ box_url( GtkWidget *par, const char *url )
 #else /*default unix-y*/
 	static gboolean shown = FALSE;
 
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[512];
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt2[512];
 
 	char url2[FILENAME_MAX];
 
 	expand_variables( url, url2 );
 
-	buf_init_static( &buf, txt, 512 );
-	buf_appendf( &buf, "%s %s", BOX_BROWSER, BOX_BROWSER_REMOTE );
-	buf_init_static( &buf2, txt2, 512 );
-	buf_appendf( &buf2, buf_all( &buf ), url2 );
+	vips_buf_init_static( &buf, txt, 512 );
+	vips_buf_appendf( &buf, "%s %s", BOX_BROWSER, BOX_BROWSER_REMOTE );
+	vips_buf_init_static( &buf2, txt2, 512 );
+	vips_buf_appendf( &buf2, vips_buf_all( &buf ), url2 );
 
-	if( systemf( "%s", buf_all( &buf2 ) ) ) {
+	if( systemf( "%s", vips_buf_all( &buf2 ) ) ) {
 		error_top( _( "Unable to view help file." ) );
 		error_sub( _( 
 			"Attempted to launch browser with command:\n"
 			"  %s\n"
 			"You can change this command in Preferences." ),
-			buf_all( &buf2 ) );
+			vips_buf_all( &buf2 ) );
 		box_alert( par );
 	}
 	else if( !shown ) {
@@ -978,200 +990,51 @@ box_url( GtkWidget *par, const char *url )
 #endif /*lots*/
 }
 
-/* Progress dialog.
- */
-
-static iDialogClass *progress_parent_class = NULL;
-
-static void
-progress_destroy( GtkObject *object )
-{
-	Progress *progress;
-
-	g_return_if_fail( object != NULL );
-	g_return_if_fail( IS_PROGRESS( object ) );
-
-	progress = PROGRESS( object );
-
-	/* My instance destroy stuff.
-	 */
-	if( progress->imageinfo && 
-		progress->imageinfo->eval_progress == progress )
-		progress->imageinfo->eval_progress = NULL;
-
-	GTK_OBJECT_CLASS( progress_parent_class )->destroy( object );
-}
-
-static void 
-progress_set_status( Progress *progress, const char *text )
-{
-	char buf1[MAX_DIALOG_TEXT];
-	char buf2[MAX_DIALOG_TEXT];
-
-	escape_markup( text, buf1, MAX_DIALOG_TEXT );
-	im_snprintf( buf2, MAX_DIALOG_TEXT, "<i>%s</i>", buf1 );
-	gtk_label_set_markup( GTK_LABEL( progress->status ), buf2 );
-}
-
-static void 
-progress_cancel_cb( iWindow *iwnd, void *client, 
-	iWindowNotifyFn nfn, void *sys ) 
-{ 
-	Progress *progress = PROGRESS( client );
-
-	progress->cancelled = TRUE;
-	progress_set_status( progress, _( "Cancelling" ) );
-
-	nfn( sys, IWINDOW_FALSE );
-}
-
-static void
-progress_build( GtkWidget *widget )
-{
-	Progress *progress = PROGRESS( widget );
-	iDialog *idlg = IDIALOG( widget );
-	GtkWidget *lab;
-	char buf[MAX_DIALOG_TEXT];
-
-#ifdef DEBUG
-	printf( "progress_build: %s\n", IWINDOW( progress )->title );
-#endif /*DEBUG*/
-
-	idialog_set_callbacks( IDIALOG( progress ), 
-		progress_cancel_cb, NULL, NULL, progress );
-
-	/* Call all builds in superclasses.
-	 */
-	if( IWINDOW_CLASS( progress_parent_class )->build )
-		(*IWINDOW_CLASS( progress_parent_class )->build)( widget );
-
-	box_markup( buf,
-		_( "Computing image." ),
-		_( "Calculating all the pixels in an image. Press the "
-		"Cancel button to stop computation early." ) );
-	lab = gtk_label_new( NULL );
-	gtk_label_set_markup( GTK_LABEL( lab ), buf );
-        gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
-        gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
-	gtk_label_set_line_wrap( GTK_LABEL( lab ), TRUE );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), lab, TRUE, TRUE, 0 );
-        gtk_misc_set_alignment( GTK_MISC( lab ), 0.0, 0.5 );
-	gtk_widget_show( lab );
-
-	progress->bar = gtk_progress_bar_new();
-        gtk_box_pack_start( GTK_BOX( idlg->work ), progress->bar, 
-		TRUE, TRUE, 0 );
-
-	progress->status = lab = gtk_label_new( NULL );
-	progress_set_status( progress, _( "Calculating" ) );
-        gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
-        gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), lab, TRUE, TRUE, 0 );
-        gtk_misc_set_alignment( GTK_MISC( lab ), 0.0, 0.5 );
-	gtk_widget_show( lab );
-
-	gtk_widget_show_all( idlg->work );
-}
-
-static void
-progress_class_init( ProgressClass *class )
-{
-	GtkObjectClass *object_class;
-	iWindowClass *iwindow_class;
-
-	object_class = (GtkObjectClass *) class;
-	iwindow_class = (iWindowClass *) class;
-
-	object_class->destroy = progress_destroy;
-
-	iwindow_class->build = progress_build;
-
-	progress_parent_class = g_type_class_peek_parent( class );
-}
-
-static void
-progress_init( Progress *progress )
-{
-#ifdef DEBUG
-	printf( "progress_init: %s\n", IWINDOW( progress )->title );
-#endif /*DEBUG*/
-
-	progress->cancelled = FALSE;
-}
-
-GtkType
-progress_get_type( void )
-{
-	static GtkType progress_type = 0;
-
-	if( !progress_type ) {
-		static const GtkTypeInfo info = {
-			"Progress",
-			sizeof( Progress ),
-			sizeof( ProgressClass ),
-			(GtkClassInitFunc) progress_class_init,
-			(GtkObjectInitFunc) progress_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		progress_type = gtk_type_unique( TYPE_IDIALOG, &info );
-	}
-
-	return( progress_type );
-}
-
-static void
-progress_link( Progress *progress, Imageinfo *imageinfo )
-{
-	progress->imageinfo = imageinfo;
-}
-
-Progress *
-progress_new( Imageinfo *imageinfo )
-{
-	Progress *progress = gtk_type_new( TYPE_PROGRESS );
-
-	progress_link( progress, imageinfo );
-
-	return( progress );
-}
-
-void 
-progress_update( Progress *progress, int percent, int eta )
-{
-	GtkProgressBar *bar = GTK_PROGRESS_BAR( progress->bar );
-	char buf[1000];
-
-	if( eta > 30 ) {
-		int minutes = (eta + 30) / 60;
-
-		im_snprintf( buf, 1000, ngettext( 
-			"About %d minute left", 
-			"About %d minutes left", 
-			minutes ), minutes );
-	}
-	else
-		im_snprintf( buf, 1000, _( "About %d seconds left" ), eta );
-	gtk_progress_bar_set_text( bar, buf );
-	gtk_progress_bar_set_fraction( bar, 
-		IM_CLIP( 0.0, (double) percent / 100.0, 1.0 ) );
-	animate_hourglass();
-
-}
-
 /* Splash screen.
  */
 
 static GtkWindowClass *splash_parent_class = NULL;
 
+static void
+splash_destroy( GtkObject *object )
+{
+	Splash *splash;
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_SPLASH( object ) );
+
+	splash = SPLASH( object );
+
+#ifdef DEBUG
+	printf( "splash_destroy\n" );
+#endif /*DEBUG*/
+
+	/* My instance destroy stuff.
+	 */
+	FREESID( splash->update_sid, progress_get() );
+
+	GTK_OBJECT_CLASS( splash_parent_class )->destroy( object );
+}
+
 static void
 splash_class_init( SplashClass *class )
 {
+	GtkObjectClass *object_class = (GtkObjectClass *) class;
+
 	splash_parent_class = g_type_class_peek_parent( class );
+
+	object_class->destroy = splash_destroy;
 }
 
+static void
+splash_progress_update( Progress *progress, gboolean *cancel, Splash *splash )
+{
+	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( splash->progress ), 
+		vips_buf_all( &progress->feedback ) );
+	gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( splash->progress ), 
+		IM_CLIP( 0.0, (double) progress->percent / 100.0, 1.0 ) );
+} 
+
 static void
 splash_toggle_change_cb( GtkWidget *widget )
 {
@@ -1221,9 +1084,14 @@ splash_init( Splash *splash )
 		"$VIPSHOME/share/$PACKAGE/data/vips-128.png" );
         gtk_box_pack_start( GTK_BOX( vbox ), image, FALSE, FALSE, 0 );
 
-	splash->label = gtk_label_new( NULL );
-	gtk_misc_set_alignment( GTK_MISC( splash->label ), 0.0, 0.5 );
-        gtk_box_pack_start( GTK_BOX( vbox ), splash->label, FALSE, FALSE, 0 );
+	splash->progress = gtk_progress_bar_new();
+	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( splash->progress ), 
+		_( "Starting ..." ) );
+        gtk_box_pack_start( GTK_BOX( vbox ), 
+		splash->progress, FALSE, FALSE, 0 );
+	splash->update_sid = g_signal_connect( progress_get(), "update", 
+		G_CALLBACK( splash_progress_update ), splash );
+        gtk_widget_show( splash->progress );
 
         toggle = build_gtoggle( GTK_WIDGET( vbox ), 
 		_( "Display this splash screen during startup" ) );
@@ -1263,40 +1131,11 @@ splash_get_type( void )
 	return( splash_type );
 }
 
-void 
-splash_updatev( Splash *splash, const char *fmt, va_list ap )
-{
-	char buf1[100];
-	char buf2[150];
-	char buf3[150];
-
-	(void) im_vsnprintf( buf1, 100, fmt, ap );
-	escape_markup( buf1, buf3, 150 );
-	(void) im_snprintf( buf2, 150, 
-		"<span style=\"italic\" size=\"smaller\">%s</span>" , buf3 );
-	gtk_label_set_markup( GTK_LABEL( splash->label ), buf2 );
-
-	while( g_main_context_iteration( NULL, FALSE ) )
-		;
-}
-
-void 
-splash_update( Splash *splash, const char *fmt, ... )
-{
-	va_list ap;
-
-	va_start( ap, fmt );
-	splash_updatev( splash, fmt, ap );
-	va_end( ap );
-}
-
 Splash *
 splash_new( void )
 {
 	Splash *splash = gtk_type_new( TYPE_SPLASH );
 
-	splash_update( splash, "%s", _( "Initialising toolkit" ) );
-
 	return( splash );
 }
 
@@ -1439,7 +1278,7 @@ fontbutton_ok_cb( iWindow *iwnd, void *client,
 	fontbutton_set_font_name( fontbutton, font_name );
 	g_free( font_name );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 static void 
@@ -1450,7 +1289,7 @@ fontbutton_popdown_cb( iWindow *iwnd, void *client,
 
 	fontbutton->fontchooser = NULL;
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 static void
