@@ -116,7 +116,7 @@ apply_underscore_call( Reduce *rc,
 
 	/* Pump though gettext.
 	 */
-	if( !heap_string_new( rc->heap, _( text ), out ) )
+	if( !heap_managedstring_new( rc->heap, _( text ), out ) )
 		reduce_throw( rc );
 }
 
@@ -208,7 +208,7 @@ static BuiltinTypeSpot *flist_args[] = {
         &flist_spot
 };
 
-/* Do a complex op. Args already spotted.
+/* Do a list op. Args already spotted.
  */
 static void
 apply_list_call( Reduce *rc, 
@@ -218,7 +218,9 @@ apply_list_call( Reduce *rc,
         PElement a;
 
 	PEPOINTRIGHT( arg[0], &rhs );
-	assert( PEISFLIST( &rhs ) );
+	g_assert( PEISFLIST( &rhs ) );
+
+	reduce_get_list( rc, &rhs );
 
 	if( strcmp( name, "hd" ) == 0 ) {
 		PEGETHD( &a, &rhs );
@@ -307,97 +309,6 @@ apply_image_call( Reduce *rc,
 	MANAGED_UNREF( ii );
 }
 
-/* Args for "vips_image_new". 
- */
-static BuiltinTypeSpot *image_new_args[] = { 
-	&string_spot,
-	&list_spot 
-};
-
-/* Do a image_new call.
- */
-static void
-apply_image_new_call( Reduce *rc, 
-	const char *name, HeapNode **arg, PElement *out )
-{
-	PElement rhs;
-	char buf[FILENAME_MAX];
-	Managed *managed;
-
-	/* Get string. 
-	 */
-	PEPOINTRIGHT( arg[1], &rhs );
-	(void) reduce_get_string( rc, &rhs, buf, FILENAME_MAX );
-
-	/* List of options.
-	 */
-	PEPOINTRIGHT( arg[0], &rhs );
-
-	managed = call8_image_new( rc, buf, &rhs );
-
-	PEPUTP( out, ELEMENT_MANAGED, managed );
-	managed_destroy_nonheap( managed );
-}
-
-/* Args for "vips_call". 
- */
-static BuiltinTypeSpot *call_args[] = { 
-	&string_spot,
-	&list_spot,
-	&list_spot 
-};
-
-/* Do a vips_call call.
- */
-static void
-apply_call_call( Reduce *rc, 
-	const char *name, HeapNode **arg, PElement *out )
-{
-	PElement rhs;
-	char buf[256];
-	PElement required;
-	PElement optional;
-
-	PEPOINTRIGHT( arg[2], &rhs );
-	reduce_get_string( rc, &rhs, buf, 256 );
-	PEPOINTRIGHT( arg[1], &required );
-	PEPOINTRIGHT( arg[0], &optional );
-
-	call8_vips( rc, buf, &required, &optional, out );
-}
-
-/* Args for "vips_header_get". 
- */
-static BuiltinTypeSpot *header_get_args[] = { 
-	&string_spot,
-	&gobject_spot
-};
-
-/* Do a vips_header_get call.
- */
-static void
-apply_header_get_call( Reduce *rc, 
-	const char *name, HeapNode **arg, PElement *out )
-{
-	PElement rhs;
-	char buf[256];
-	GObject *object;
-
-	PEPOINTRIGHT( arg[1], &rhs );
-	reduce_get_string( rc, &rhs, buf, 256 );
-	PEPOINTRIGHT( arg[0], &rhs );
-	assert( PEISMANAGEDGOBJECT( &rhs ) );
-	object = PEGETMANAGEDGOBJECT( &rhs );
-
-	if( !VIPS_IS_OBJECT( object ) ) {
-		error_top( _( "Bad argument." ) );
-		error_sub( _( "GObject is not a VipsObject." ) );
-		reduce_throw( rc );
-	}
-
-	call8_header_get( rc, buf, VIPS_OBJECT( object ), out );
-}
-
 /* Args for "read". 
  */
 static BuiltinTypeSpot *read_args[] = { 
@@ -600,7 +511,7 @@ apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
 		reduce_throw( rc );
 
-	if( !heap_string_new( rc->heap, fn, out ) ) {
+	if( !heap_managedstring_new( rc->heap, fn, out ) ) {
 		IM_FREE( fn );
 		reduce_throw( rc );
 	}
@@ -619,14 +530,14 @@ static void
 apply_print_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
 	PElement rhs;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[MAX_STRSIZE];
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 	PEPOINTRIGHT( arg[0], &rhs );
 	itext_value_ev( rc, &buf, &rhs );
 
-	if( !heap_string_new( rc->heap, buf_all( &buf ), out ) )
+	if( !heap_managedstring_new( rc->heap, vips_buf_all( &buf ), out ) )
 		reduce_throw( rc );
 }
 
@@ -643,7 +554,7 @@ dir_object_member( Symbol *sym, PElement *value,
 	PElement t;
 
 	if( !heap_list_add( rc->heap, list, &t ) ||
-		!heap_string_new( rc->heap, IOBJECT( sym )->name, &t ) )
+		!heap_managedstring_new( rc->heap, IOBJECT( sym )->name, &t ) )
 		reduce_throw( rc );
 	(void) heap_list_next( list );
 
@@ -672,13 +583,49 @@ dir_scope( Symbol *sym, Reduce *rc, PElement *list )
 	PElement t;
 
 	if( !heap_list_add( rc->heap, list, &t ) ||
-		!heap_string_new( rc->heap, IOBJECT( sym )->name, &t ) )
+		!heap_managedstring_new( rc->heap, IOBJECT( sym )->name, &t ) )
 		reduce_throw( rc );
 	(void) heap_list_next( list );
 
 	return( NULL );
 }
 
+static void *
+dir_gtype( GType type, void *a, void *b )
+{
+	Reduce *rc = (Reduce *) a;
+	PElement *list = (PElement *) b;
+	PElement t;
+
+	if( !heap_list_add( rc->heap, list, &t ) ||
+		!heap_real_new( rc->heap, type, &t ) )
+		return( rc );
+	(void) heap_list_next( list );
+
+	return( NULL );
+}
+
+static void
+dir_gobject( Reduce *rc, 
+	GParamSpec **properties, guint n_properties, PElement *out )
+{
+	int i;
+	PElement list;
+
+	list = *out;
+	heap_list_init( &list );
+
+	for( i = 0; i < n_properties; i++ ) {
+		PElement t;
+
+		if( !heap_list_add( rc->heap, &list, &t ) ||
+			!heap_managedstring_new( rc->heap, 
+				properties[i]->name, &t ) )
+			reduce_throw( rc );
+		(void) heap_list_next( &list );
+	}
+}
+
 /* Do "dir". 
  */
 static void
@@ -709,6 +656,41 @@ apply_dir_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 				(icontainer_map_fn) dir_scope, rc, &list );
 		}
 	}
+	else if( PEISREAL( &rhs ) ) {
+		/* Assume this is a gtype and try to get the children of that
+		 * type.
+		 */
+		GType type = PEGETREAL( &rhs );
+		PElement list;
+
+		list = *out;
+		heap_list_init( &list );
+
+		if( !g_type_name( type ) ) {
+			error_top( _( "No such type" ) );
+			error_sub( _( "GType %u not found." ), 
+				(unsigned int) type );
+			reduce_throw( rc );
+		}
+
+		if( vips_type_map( type, dir_gtype, rc, &list ) ) 
+			reduce_throw( rc );
+	}
+	else if( PEISMANAGEDGOBJECT( &rhs ) ) {
+		guint n_properties;
+		ManagedgobjectClass *class = 
+			MANAGEDGOBJECT_GET_CLASS( PEGETMANAGEDGOBJECT( &rhs ) ); 
+		GParamSpec **properties;
+
+		properties = g_object_class_list_properties( 
+			G_OBJECT_CLASS( class ), &n_properties );
+		dir_gobject( rc, properties, n_properties, out );
+		g_free( properties);
+	}
+	else 
+		/* Just [], ie. no names possible.
+		 */
+		heap_list_init( out );
 }
 
 /* Args for "expand". 
@@ -730,7 +712,7 @@ apply_expand_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	(void) reduce_get_string( rc, &rhs, txt, FILENAME_MAX );
 	expand_variables( txt, txt2 );
 
-	if( !heap_string_new( rc->heap, txt2, out ) )
+	if( !heap_managedstring_new( rc->heap, txt2, out ) )
 		reduce_throw( rc );
 }
 
@@ -776,10 +758,37 @@ apply_gtype2name_call( Reduce *rc, const char *name,
 	PEPOINTRIGHT( arg[0], &rhs );
 	gtype = PEGETREAL( &rhs );
 
-	if( !heap_string_new( rc->heap, g_type_name( gtype ), out ) )
+	if( !heap_managedstring_new( rc->heap, g_type_name( gtype ), out ) )
 		reduce_throw( rc );
 }
 
+/* Args for "vips_object_new". 
+ */
+static BuiltinTypeSpot *vo_new_args[] = { 
+	&string_spot,
+	&list_spot,
+	&list_spot 
+};
+
+/* Do a vips_object_new call.
+ */
+static void
+apply_vo_new_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char buf[256];
+	PElement required;
+	PElement optional;
+
+	PEPOINTRIGHT( arg[2], &rhs );
+	reduce_get_string( rc, &rhs, buf, 256 );
+	PEPOINTRIGHT( arg[1], &required );
+	PEPOINTRIGHT( arg[0], &optional );
+
+	vo_object_new( rc, buf, &required, &optional, out );
+}
+
 /* All ip's builtin functions. 
  */
 static BuiltinInfo builtin_table[] = {
@@ -796,12 +805,8 @@ static BuiltinInfo builtin_table[] = {
 
 	/* vips8 wrapper.
 	 */
-	{ "vips8_image_new", FALSE,  IM_NUMBER( image_new_args ), 
-		&image_new_args[0], apply_image_new_call },
-	{ "vips8_call", FALSE,  IM_NUMBER( call_args ), 
-		&call_args[0], apply_call_call },
-	{ "vips8_header_get", FALSE,  IM_NUMBER( header_get_args ), 
-		&header_get_args[0], apply_header_get_call },
+	{ "vips_object_new", FALSE, IM_NUMBER( vo_new_args ), 
+		&vo_new_args[0], apply_vo_new_call },
 
 	/* Predicates.
 	 */
@@ -903,7 +908,7 @@ builtin_init( void )
 
 		sym = symbol_new( symbol_root->expr->compile,
 			builtin_table[i].name );
-		assert( sym->type == SYM_ZOMBIE );
+		g_assert( sym->type == SYM_ZOMBIE );
 		sym->type = SYM_BUILTIN;
 		sym->builtin = &builtin_table[i];
 		(void) tool_new_sym( kit, -1, sym );
@@ -924,19 +929,19 @@ builtin_init( void )
 /* Make a usage error.
  */
 void
-builtin_usage( BufInfo *buf, BuiltinInfo *builtin )
+builtin_usage( VipsBuf *buf, BuiltinInfo *builtin )
 {
 	int i;
 
-	buf_appendf( buf, 
+	vips_buf_appendf( buf, 
 		ngettext( "Builtin \"%s\" takes %d argument.", 
 			"Builtin \"%s\" takes %d arguments.", 
 			builtin->nargs ),
 		builtin->name, builtin->nargs );
-        buf_appends( buf, "\n" );
+        vips_buf_appends( buf, "\n" );
 
 	for( i = 0; i < builtin->nargs; i++ )
-		buf_appendf( buf, "    %d - %s\n", 
+		vips_buf_appendf( buf, "    %d - %s\n", 
 		i + 1,
 		builtin->args[i]->name );
 }
@@ -946,21 +951,21 @@ static void
 builtin_trace_args( Heap *heap, const char *name, int n, HeapNode **arg )
 {
 	int i;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[100];
 
-	buf_init_static( &buf, txt, 100 );
+	vips_buf_init_static( &buf, txt, 100 );
 
 	for( i = 0; i < n; i++ ) {
 		PElement t;
 
 		PEPOINTRIGHT( arg[n - i - 1], &t );
-		buf_appends( &buf, "(" );
+		vips_buf_appends( &buf, "(" );
 		graph_pelement( hi, &buf, &t, FALSE );
-		buf_appends( &buf, ") " );
+		vips_buf_appends( &buf, ") " );
 	}
 
-	printf( "builtin: %s %s\n", name, buf_all( &buf ) );
+	printf( "builtin: %s %s\n", name, vips_buf_all( &buf ) );
 }
 #endif /*DEBUG*/
 
@@ -981,16 +986,16 @@ builtin_run( Reduce *rc, Compile *compile,
 
 		PEPOINTRIGHT( arg[builtin->nargs - i - 1], &base );
 		if( !ts->pred( rc, &base ) ) {
-			BufInfo buf;
+			VipsBuf buf;
 			char txt[100];
 
-			buf_init_static( &buf, txt, 100 );
+			vips_buf_init_static( &buf, txt, 100 );
 			itext_value_ev( rc, &buf, &base );
 			error_top( _( "Bad argument." ) );
 			error_sub( _( "Argument %d to builtin \"%s\" should "
 				"be \"%s\", you passed:\n  %s" ),
 				i + 1, name, ts->name,
-				buf_all( &buf ) );
+				vips_buf_all( &buf ) );
 			reduce_throw( rc );
 		}
 	}
