@@ -88,7 +88,7 @@ image_value_set( ImageValue *image, Imageinfo *ii )
 /* Generate a descriptive name for an imagevalue. Used by plot.c etc. as well.
  */
 void
-image_value_caption( ImageValue *value, BufInfo *buf )
+image_value_caption( ImageValue *value, VipsBuf *buf )
 {
 	Imageinfo *ii = value->ii;
 	Classmodel *classmodel = value->classmodel;
@@ -97,11 +97,11 @@ image_value_caption( ImageValue *value, BufInfo *buf )
 	 * the class.
 	 */
 	if( ii && imageinfo_is_from_file( ii ) && classmodel->filename ) 
-		buf_appends( buf, im_skip_dir( classmodel->filename ) );
+		vips_buf_appends( buf, im_skip_dir( classmodel->filename ) );
 	else if( !heapmodel_name( HEAPMODEL( classmodel ), buf ) )
 		/* Only if there's no value, I think.
 		 */
-		buf_appends( buf, CLASS_IMAGE );
+		vips_buf_appends( buf, CLASS_IMAGE );
 }
 
 void *
@@ -131,7 +131,7 @@ classmodel_graphic_save_cb( iWindow *iwnd,
 			IM_SETSTR( classmodel->filename, filename );
 			iobject_changed( IOBJECT( classmodel ) );
 
-			nfn( sys, IWINDOW_TRUE );
+			nfn( sys, IWINDOW_YES );
 		}
 		else
 			nfn( sys, IWINDOW_ERROR );
@@ -147,7 +147,7 @@ classmodel_graphic_save( Classmodel *classmodel, GtkWidget *parent )
 {
 	ClassmodelClass *class = CLASSMODEL_GET_CLASS( classmodel );
 	GtkWidget *filesel;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[100];
 
 	if( !class->graphic_save ) {
@@ -159,10 +159,10 @@ classmodel_graphic_save( Classmodel *classmodel, GtkWidget *parent )
 	}
 
 	filesel = filesel_new();
-	buf_init_static( &buf, txt, 100 );
+	vips_buf_init_static( &buf, txt, 100 );
 	row_qualified_name( HEAPMODEL( classmodel )->row, &buf );
 	iwindow_set_title( IWINDOW( filesel ), _( "Save %s \"%s\"" ), 
-		G_OBJECT_TYPE_NAME( classmodel ), buf_all( &buf ) );
+		G_OBJECT_TYPE_NAME( classmodel ), vips_buf_all( &buf ) );
 	filesel_set_flags( FILESEL( filesel ), TRUE, TRUE );
 	filesel_set_filetype( FILESEL( filesel ), 
 		class->filetype, 
@@ -205,7 +205,7 @@ classmodel_graphic_replace_cb( iWindow *iwnd,
 
 			g_object_unref( G_OBJECT( classmodel ) );
 
-			nfn( sys, IWINDOW_TRUE );
+			nfn( sys, IWINDOW_YES );
 		}
 		else
 			nfn( sys, IWINDOW_ERROR );
@@ -221,7 +221,7 @@ classmodel_graphic_replace( Classmodel *classmodel, GtkWidget *parent )
 {
 	ClassmodelClass *class = CLASSMODEL_GET_CLASS( classmodel );
 	GtkWidget *filesel;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[100];
 
 	if( !class->graphic_replace ) {
@@ -233,11 +233,11 @@ classmodel_graphic_replace( Classmodel *classmodel, GtkWidget *parent )
 		return;
 	}
 
-	buf_init_static( &buf, txt, 100 );
+	vips_buf_init_static( &buf, txt, 100 );
 	row_qualified_name( HEAPMODEL( classmodel )->row, &buf );
 	filesel = filesel_new();
 	iwindow_set_title( IWINDOW( filesel ), _( "Replace %s \"%s\"" ), 
-		G_OBJECT_TYPE_NAME( classmodel ), buf_all( &buf ) );
+		G_OBJECT_TYPE_NAME( classmodel ), vips_buf_all( &buf ) );
 	filesel_set_flags( FILESEL( filesel ), TRUE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), 
 		class->filetype, 
@@ -400,7 +400,7 @@ classmodel_dict_new( Classmodel *classmodel,
 		if( !heap_list_add( heap, &list, &pair ) ||
 			!heap_list_add( heap, &pair, &key ) ||
 			!heap_list_add( heap, &pair, &value ) ||
-			!heap_string_new( heap, 
+			!heap_managedstring_new( heap, 
 				options[i].member_name, &key ) ||
 			!classmodel_class_member_new( classmodel,
 				&options[i], heap, &value ) )
@@ -440,7 +440,7 @@ classmodel_class_member_new( Classmodel *classmodel,
 		break;
 
 	case CLASSMODEL_MEMBER_STRING:
-		if( !heap_string_new( heap, 
+		if( !heap_managedstring_new( heap, 
 			G_STRUCT_MEMBER( char *, classmodel, m->offset ),
 			out ) )
 			return( FALSE );
@@ -488,7 +488,7 @@ classmodel_class_member_new( Classmodel *classmodel,
 	}
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( TRUE );
@@ -585,14 +585,14 @@ classmodel_dispose( GObject *gobject )
  * Things like iImage define an _info() with useful stuff in.
  */
 static void
-classmodel_info( iObject *iobject, BufInfo *buf )
+classmodel_info( iObject *iobject, VipsBuf *buf )
 {
 }
 
 static void
 classmodel_parent_add( iContainer *child )
 {
-	assert( IS_CLASSMODEL( child ) );
+	g_assert( IS_CLASSMODEL( child ) );
 
 	ICONTAINER_CLASS( parent_class )->parent_add( child );
 }
@@ -647,7 +647,7 @@ classmodel_done_member( Classmodel *classmodel,
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( TRUE );
@@ -676,7 +676,7 @@ classmodel_done_cb( iWindow *iwnd, void *client,
 	classmodel_update( classmodel );
 	symbol_recalculate_all();
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 static GtkWidget *
@@ -720,7 +720,7 @@ classmodel_buildedit_member( Classmodel *classmodel,
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( widget );
@@ -861,7 +861,7 @@ classmodel_save_member( Classmodel *classmodel,
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( TRUE );
@@ -1022,7 +1022,7 @@ classmodel_load_member( Classmodel *classmodel,
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( found );
@@ -1156,7 +1156,7 @@ classmodel_get_item( Classmodel *classmodel,
 	}
 
 	case CLASSMODEL_MEMBER_REALVEC_FIXED:
-		assert( m->extent < 4 );
+		g_assert( m->extent < 4 );
 		if( (l = heap_get_realvec( value, vec, m->extent )) < 0 )
 			return( FALSE );
 		if( l != m->extent ) {
@@ -1192,7 +1192,7 @@ classmodel_get_item( Classmodel *classmodel,
 		/* If there are optional fields, we have to have a reset
 		 * method for clearing the ones we don't use.
 		 */
-		assert( class->reset );
+		g_assert( class->reset );
 
 		if( heap_map_dict( value, 
 			(heap_map_dict_fn) classmodel_parse_option, 
@@ -1218,7 +1218,7 @@ classmodel_get_item( Classmodel *classmodel,
 	}
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( TRUE );
