@@ -115,14 +115,10 @@ colourdisplay_drag_begin( GtkWidget *widget, GdkDragContext *context )
 	double colours[4];
 	GdkColor bg;
 
-	window = gtk_window_new( GTK_WINDOW_POPUP );
-	gtk_widget_set_app_paintable( GTK_WIDGET( window ), TRUE );
-	gtk_widget_set_size_request( window, 48, 32 );
-	gtk_widget_realize( window );
+	window = iimageview_drag_window_new( 48, 32 ); 
 	gtk_object_set_data_full( GTK_OBJECT( widget ),
-		"colourdisplay-drag-window", window,
+		"nip2-drag-window", window,
 		(GtkDestroyNotify) gtk_widget_destroy );
-
 #ifdef DEBUG
 	printf( "colourdisplay_drag_begin: generating drag swatch colour\n" );
 #endif /*DEBUG*/
@@ -139,7 +135,7 @@ static void
 colourdisplay_drag_end( GtkWidget *widget, GdkDragContext *context )
 {
 	gtk_object_set_data( GTK_OBJECT( widget ), 
-		"colourdisplay-drag-window", NULL );
+		"nip2-drag-window", NULL );
 }
 
 static void
@@ -150,8 +146,8 @@ colourdisplay_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 	Imageinfo *imageinfo = IMAGEDISPLAY( colourdisplay )->conv->ii;
 	double colours[3];
 	guint16 vals[4];
-	char buf_text[256];
-	BufInfo buf;
+	char vips_buf_text[256];
+	VipsBuf buf;
 
 	switch( info ) {
 	case TARGET_COLOUR:
@@ -173,12 +169,12 @@ colourdisplay_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 		break;
 
 	case TARGET_TEXT:
-		buf_init_static( &buf, buf_text, 256 );
+		vips_buf_init_static( &buf, vips_buf_text, 256 );
 		imageinfo_to_text( imageinfo, &buf );
 
 		gtk_selection_data_set( selection_data,
 			gdk_atom_intern( "text/plain", FALSE ), 8, 
-			(guchar *) buf_all( &buf ), strlen( buf_all( &buf ) ) );
+			(guchar *) vips_buf_all( &buf ), strlen( vips_buf_all( &buf ) ) );
 
 #ifdef DEBUG
 		printf( "colourdisplay_drag_data_get: sending text/plain\n" );
@@ -187,7 +183,7 @@ colourdisplay_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 		break;
 	}
 }
@@ -248,20 +244,20 @@ colourdisplay_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 		break;
 	}
 }
 
 static void
-colourdisplay_generate_tooltip( Colourdisplay *colourdisplay, BufInfo *buf )
+colourdisplay_generate_tooltip( Colourdisplay *colourdisplay, VipsBuf *buf )
 {
 	Imagedisplay *id = IMAGEDISPLAY( colourdisplay );
 
 	if( id->conv && id->conv->ii ) {
 		imageinfo_to_text( id->conv->ii, buf );
-		buf_appends( buf, "\n" );
-		buf_appends( buf, _( "Double-click to edit this color, or "
+		vips_buf_appends( buf, "\n" );
+		vips_buf_appends( buf, _( "Double-click to edit this color, or "
 			"drag-and-drop between colors" ) );
 	}
 }
