@@ -97,7 +97,7 @@ column_select_symbols( Column *col )
 static Subcolumn *
 column_get_subcolumn( Column *col )
 {
-	assert( g_slist_length( ICONTAINER( col )->children ) == 1 );
+	g_assert( g_slist_length( ICONTAINER( col )->children ) == 1 );
 
 	return( SUBCOLUMN( ICONTAINER( col )->children->data ) );
 }
@@ -135,7 +135,7 @@ column_parent_add( iContainer *child )
 {
 	Column *col = COLUMN( child );
 
-	assert( IS_WORKSPACE( child->parent ) );
+	g_assert( IS_WORKSPACE( child->parent ) );
 
 	ICONTAINER_CLASS( parent_class )->parent_add( child );
 
@@ -170,14 +170,20 @@ column_save( Model *model, xmlNode *xnode )
 	 */
 	if( !set_prop( xthis, "x", "%d", x ) ||
 		!set_prop( xthis, "y", "%d", y ) ||
-		!set_prop( xthis, "open", bool_to_char( col->open ) ) ||
-		!set_prop( xthis, "selected", bool_to_char( col->selected ) ) ||
-		!set_prop( xthis, "sform", bool_to_char( FALSE ) ) ||
+		!set_sprop( xthis, "open", bool_to_char( col->open ) ) ||
+		!set_sprop( xthis, "selected",
+			bool_to_char( col->selected ) ) ||
+		!set_sprop( xthis, "sform", bool_to_char( FALSE ) ) ||
 		!set_prop( xthis, "next", "%d", col->next ) || 
-		!set_sprop( xthis, "name", IOBJECT( col )->name ) || 
-		!set_sprop( xthis, "caption", IOBJECT( col )->caption ) ) 
+		!set_sprop( xthis, "name", IOBJECT( col )->name ) )
 		return( NULL );
 
+	/* Caption can be NULL for untitled columns.
+	 */
+	if( IOBJECT( col )->caption )
+		if( !set_sprop( xthis, "caption", IOBJECT( col )->caption ) ) 
+			return( NULL );
+
 	return( xthis );
 }
 
@@ -206,7 +212,7 @@ column_load( Model *model,
 
 	char buf[256];
 
-	assert( IS_WORKSPACE( parent ) );
+	g_assert( IS_WORKSPACE( parent ) );
 
 	if( !get_iprop( xnode, "x", &x ) ||
 		!get_iprop( xnode, "y", &y ) ||
@@ -325,7 +331,7 @@ column_new( Workspace *ws, const char *name )
 	}
 
 	col = COLUMN( g_object_new( TYPE_COLUMN, NULL ) );
-	iobject_set( IOBJECT( col ), name, "untitled" );
+	iobject_set( IOBJECT( col ), name, NULL );
 	icontainer_child_add( ICONTAINER( ws ), ICONTAINER( col ), -1 );
 
         subcolumn_new( NULL, col );
@@ -366,14 +372,14 @@ column_get_bottom( Column *col )
  * there.
  */
 gboolean
-column_add_n_names( Column *col, const char *name, BufInfo *buf, int nparam )
+column_add_n_names( Column *col, const char *name, VipsBuf *buf, int nparam )
 {
 	Subcolumn *scol = col->scol;
 	GSList *children = ICONTAINER( scol )->children;
 	int len = g_slist_length( children );
 	GSList *i;
 
-	assert( nparam >= 0 );
+	g_assert( nparam >= 0 );
 
 	if( nparam > 0 && nparam > len ) {
 		error_top( _( "Too few items." ) );
@@ -385,8 +391,8 @@ column_add_n_names( Column *col, const char *name, BufInfo *buf, int nparam )
 	for( i = g_slist_nth( children, len - nparam ); i; i = i->next ) {
 		Row *row = ROW( i->data );
 
-		buf_appends( buf, " " );
-		buf_appends( buf, IOBJECT( row->sym )->name );
+		vips_buf_appends( buf, " " );
+		vips_buf_appends( buf, IOBJECT( row->sym )->name );
 	}
 
 	return( TRUE );
