@@ -107,7 +107,7 @@ columnview_save_as_sub( iWindow *iwnd,
 	if( (filename = filesel_get_filename( filesel )) ) {
 		if( workspace_selected_save( ws, filename ) ) {
 			workspace_deselect_all( ws );
-			nfn( sys, IWINDOW_TRUE );
+			nfn( sys, IWINDOW_YES );
 		}
 		else
 			nfn( sys, IWINDOW_ERROR );
@@ -213,7 +213,7 @@ columnview_to_menu_done_cb( iWindow *iwnd, void *client,
 
 	IM_SETSTR( columnview_to_menu_last_toolkit, toolkit_text );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Make a column into a menu item. 
@@ -349,7 +349,7 @@ columnview_left_press( Columnview *cview, GdkEvent *ev )
                 break;
 
         default:
-		assert( FALSE );
+		g_assert( FALSE );
         }
 }
 
@@ -468,7 +468,7 @@ columnview_left_motion( Columnview *cview, GdkEvent *ev )
                 break;
 
         default:
-                assert( FALSE );
+		g_assert( FALSE );
         }
 }
 
@@ -506,7 +506,7 @@ columnview_left_release( Columnview *cview, GdkEvent *ev )
                 break;
 
         default:
-                assert( FALSE );
+                g_assert( FALSE );
         }
 }
 
@@ -604,7 +604,11 @@ columnview_caption_enter_cb( GtkWidget *wid, Columnview *cview )
 	Workspace *ws = col->ws;
 
         cview->state = COL_WAIT;
-	iobject_set( IOBJECT( col ), NULL, text );
+	iobject_changed( IOBJECT( col ) );
+
+	if( strcmp( text, "" ) != 0 ) 
+		iobject_set( IOBJECT( col ), NULL, text );
+
 	filemodel_set_modified( FILEMODEL( ws ), TRUE );
 
 	/* The ws view needs to update the jumpto menus.
@@ -726,9 +730,8 @@ columnview_refresh( vObject *vobject )
 	widget_visible( cview->title, editable );
 	if( editable ) 
 		gtk_frame_set_label( GTK_FRAME( cview->frame ), NULL );
-	else {
-		GtkWidget *label = gtk_frame_get_label_widget( 
-			GTK_FRAME( cview->frame ) );
+	else if( IOBJECT( col )->caption ) {
+		GtkWidget *label;
 		char buf[256];
 		char buf2[256];
 
@@ -743,7 +746,15 @@ columnview_refresh( vObject *vobject )
 	/* Update names.
 	 */
         set_glabel( cview->lab, "%s - ", IOBJECT( col )->name );
-        set_glabel( cview->head, "%s", IOBJECT( col )->caption );
+        if( IOBJECT( col )->caption )
+		set_glabel( cview->head, "%s", IOBJECT( col )->caption );
+	else {
+		char buf[256];
+
+		im_snprintf( buf, 256, "<i>%s</i>", 
+			_( "doubleclick to set title" ) );
+		gtk_label_set_markup( GTK_LABEL( cview->head ), buf );
+	}
 
 	/* Set open/closed.
 	 */
@@ -770,9 +781,13 @@ columnview_refresh( vObject *vobject )
                 gtk_widget_show( cview->capedit );
                 gtk_widget_hide( cview->headfr );
 
-                set_gentry( cview->capedit, "%s", IOBJECT( col )->caption );
-                gtk_editable_select_region( GTK_EDITABLE( cview->capedit ),
-                        0, strlen( IOBJECT( col )->caption ) );
+		if( IOBJECT( col )->caption ) {
+			set_gentry( cview->capedit, "%s", 
+				IOBJECT( col )->caption );
+			gtk_editable_select_region( 
+				GTK_EDITABLE( cview->capedit ),
+				0, strlen( IOBJECT( col )->caption ) );
+		}
                 gtk_widget_grab_focus( cview->capedit );
         }
         else {
