@@ -80,11 +80,11 @@ compile_name_print( Compile *compile )
 }
 
 static void *
-compile_name_sub( Expr *expr, BufInfo *buf )
+compile_name_sub( Expr *expr, VipsBuf *buf )
 {
 	if( expr->row ) {
-		if( !buf_is_empty( buf ) )
-			buf_appends( buf, ", " );
+		if( !vips_buf_is_empty( buf ) )
+			vips_buf_appends( buf, ", " );
 		row_qualified_name( expr->row, buf );
 	}
 
@@ -92,20 +92,20 @@ compile_name_sub( Expr *expr, BufInfo *buf )
 }
 
 void
-compile_name( Compile *compile, BufInfo *buf )
+compile_name( Compile *compile, VipsBuf *buf )
 {
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt[256];
 
-	buf_appends( buf, "\"" );
+	vips_buf_appends( buf, "\"" );
 	symbol_qualified_name( compile->sym, buf );
-	buf_appends( buf, "\"" );
+	vips_buf_appends( buf, "\"" );
 
-	buf_init_static( &buf2, txt, 256 );
+	vips_buf_init_static( &buf2, txt, 256 );
 	slist_map( compile->exprs,
 		(SListMapFn) compile_name_sub, &buf2 );
-	if( !buf_is_empty( &buf2 ) ) 
-		buf_appendf( buf, " (%s)", buf_all( &buf2 ) );
+	if( !vips_buf_is_empty( &buf2 ) ) 
+		vips_buf_appendf( buf, " (%s)", vips_buf_all( &buf2 ) );
 }
 
 static Compile *
@@ -190,7 +190,7 @@ compile_link_break( Compile *compile, Symbol *child )
 
 	/* Must be there.
 	 */
-	assert( g_slist_find( compile->children, child ) &&
+	g_assert( g_slist_find( compile->children, child ) &&
 		g_slist_find( child->parents, compile ) );
 
 	compile->children = g_slist_remove( compile->children, child );
@@ -209,8 +209,8 @@ compile_link_break( Compile *compile, Symbol *child )
 void *
 compile_expr_link_break( Compile *compile, Expr *expr )
 {
-	assert( expr->compile == compile );
-	assert( g_slist_find( compile->exprs, expr ) );
+	g_assert( expr->compile == compile );
+	g_assert( g_slist_find( compile->exprs, expr ) );
 
 	expr->compile = NULL;
 	compile->exprs = g_slist_remove( compile->exprs, expr );
@@ -229,9 +229,9 @@ compile_expr_link_break_rev( Expr *expr, Compile *compile )
 void 
 compile_expr_link_make( Compile *compile, Expr *expr )
 {
-	assert( !expr->compile );
-	assert( !g_slist_find( compile->exprs, expr ) );
-	assert( compile->sym == expr->sym );
+	g_assert( !expr->compile );
+	g_assert( !g_slist_find( compile->exprs, expr ) );
+	g_assert( compile->sym == expr->sym );
 
 	expr->compile = compile;
 	compile->exprs = g_slist_prepend( compile->exprs, expr );
@@ -280,7 +280,7 @@ compile_finalize( GObject *gobject )
 	/* Remove static strings we created.
 	 */
 	slist_map( compile->statics, 
-		(SListMapFn) heap_static_string_unref, NULL );
+		(SListMapFn) managed_destroy_nonheap, NULL );
 	IM_FREEF( g_slist_free, compile->statics );
 
 	/* Junk heap.
@@ -303,48 +303,20 @@ compile_finalize( GObject *gobject )
 	/* If we're being finalized, we must have a ref count of zero, so
 	 * there shouldn't be any exprs looking at us.
 	 */
-	assert( !compile->exprs );
+	g_assert( !compile->exprs );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
-static void
-compile_child_add( iContainer *parent, iContainer *child, int pos )
-{
-	Compile *compile = COMPILE( parent );
-	Symbol *sym = SYMBOL( child );
-
-	if( sym ) 
-		compile->last_sym = sym;
-
-	ICONTAINER_CLASS( parent_class )->child_add( parent, child, pos );
-}
-
-static void 
-compile_child_remove( iContainer *parent, iContainer *child )
-{
-	Compile *compile = COMPILE( parent );
-	Symbol *sym = SYMBOL( child );
-
-	if( sym && compile->last_sym == sym )
-		compile->last_sym = NULL;
-
-	ICONTAINER_CLASS( parent_class )->child_remove( parent, child );
-}
-
 static void
 compile_class_init( CompileClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
-	iContainerClass *icontainer_class = (iContainerClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->finalize = compile_finalize;
 
-	icontainer_class->child_add = compile_child_add;
-	icontainer_class->child_remove = compile_child_remove;
-
 	/* Create signals.
 	 */
 
@@ -827,19 +799,18 @@ compile_graph( Compile *compile, ParseNode *pn, PElement *out )
 		switch( pn->con.type ) {
 		case PARSE_CONST_STR:
 		{
-			HeapStaticString *string;
-
-			if( !(string = heap_static_string_new( 
-				reduce_context->heap,
+			Managedstring *managedstring;
+				
+			if( !(managedstring = managedstring_find( 
+				reduce_context->heap, 
 				pn->con.val.str )) )
 				return( FALSE );
+			MANAGED_REF( managedstring );
 			compile->statics = g_slist_prepend( compile->statics,
-				string );
-
-			PEPUTP( out, ELEMENT_STATIC, string );
-
-			break;
+				managedstring );
+			PEPUTP( out, ELEMENT_MANAGED, managedstring );
 		}
+			break;
 
 		case PARSE_CONST_CHAR:
 			PEPUTP( out, ELEMENT_CHAR, pn->con.val.ch );
@@ -864,14 +835,14 @@ compile_graph( Compile *compile, ParseNode *pn, PElement *out )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		break;
 
 	case NODE_NONE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( TRUE );
@@ -948,7 +919,7 @@ compile_abstract_body( Compile *compile,
 		case TAG_FILE:
 		case TAG_FREE:	
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		break;
@@ -969,6 +940,7 @@ compile_abstract_body( Compile *compile,
 		*used = TRUE;
 		break;
 
+	case ELEMENT_MANAGED:
 	case ELEMENT_CHAR:
 	case ELEMENT_BOOL:
 	case ELEMENT_BINOP:
@@ -979,16 +951,12 @@ compile_abstract_body( Compile *compile,
 	case ELEMENT_COMPILEREF:
 	case ELEMENT_NOVAL:
 	case ELEMENT_TAG:
-	case ELEMENT_STATIC:
 		/* Leave alone.
 		 */
 		break;
 
-	/* Should not exist at compile time.
-	 */
-	case ELEMENT_MANAGED:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( 0 );
@@ -1214,14 +1182,15 @@ compile_share_scan_element( CompileShare *share, PElement *e )
 		hash = INT_TO_HASH( g_str_hash( PEGETTAG( e ) ) );
 		break;
 
-	case ELEMENT_STATIC:
-		hash = INT_TO_HASH( g_str_hash( PEGETSTATIC( e )->text ) );
+	case ELEMENT_MANAGED:
+		if( PEISMANAGEDSTRING( e ) )
+			hash = INT_TO_HASH( g_str_hash( 
+				PEGETMANAGEDSTRING( e )->string ) );
 		break;
 
 	case ELEMENT_NOVAL:
-	case ELEMENT_MANAGED:
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( hash );
@@ -1266,7 +1235,7 @@ compile_share_scan_node( CompileShare *share, HeapNode *hn )
 	case TAG_SHARED:
 	case TAG_FREE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	/* Add to accumulated table.
@@ -1312,7 +1281,7 @@ compile_equal_node( HeapNode *hn1, HeapNode *hn2 )
 
 	/* If compound type, something is wrong! Only built by reduce.
 	 */
-	assert( hn1->type != TAG_CLASS );
+	g_assert( hn1->type != TAG_CLASS );
 
 	/* In two parts, test tags.
 	 */
@@ -1350,14 +1319,14 @@ compile_transform_reference( Compile *compile, HeapNode *hn1, HeapNode *hn2 )
 {
 #ifdef DEBUG
 	Heap *heap = compile->heap;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[100];
 #endif /*DEBUG*/
 
 #ifdef DEBUG
-	buf_init_static( &buf, txt, 80 );
+	vips_buf_init_static( &buf, txt, 80 );
 	graph_node( heap, &buf, hn1, TRUE );
-	printf( "Found common subexpression: %s\n", buf_all( &buf ) );
+	printf( "Found common subexpression: %s\n", vips_buf_all( &buf ) );
 #endif /*DEBUG*/
 
 #ifdef DEBUG_COMMON
@@ -1422,7 +1391,7 @@ compile_share_trim( CompileShare *share, HeapNode *hn )
 	case TAG_FREE:
 	case TAG_FILE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -1452,7 +1421,7 @@ compile_transform_share( HeapNode *hn, Compile *compile )
 	Heap *heap = compile->heap;
 
 #ifdef DEBUG
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[100];
 #endif /*DEBUG*/
 
@@ -1463,9 +1432,9 @@ compile_transform_share( HeapNode *hn, Compile *compile )
 			HeapNode *hn2;
 
 #ifdef DEBUG
-			buf_init_static( &buf, txt, 80 );
+			vips_buf_init_static( &buf, txt, 80 );
 			graph_node( heap, &buf, hn1, TRUE );
-			printf( "Found shared code: %s\n", buf_all( &buf ) );
+			printf( "Found shared code: %s\n", vips_buf_all( &buf ) );
 #endif /*DEBUG*/
 
 			if( NEWNODE( heap, hn2 ) )
@@ -1551,7 +1520,7 @@ compile_heap( Compile *compile )
 {
 	PElement base;
 #ifdef DEBUG
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 #endif /*DEBUG*/
 
@@ -1591,10 +1560,10 @@ compile_heap( Compile *compile )
 	}
 
 #ifdef DEBUG
-	buf_init_static( &buf, txt, 1024 );
+	vips_buf_init_static( &buf, txt, 1024 );
 	graph_pelement( compile->heap, &buf, &base, TRUE );
 	printf( "before var abstraction, compiled \"%s\" to: %s\n", 
-		IOBJECT( compile->sym )->name, buf_all( &buf ) );
+		IOBJECT( compile->sym )->name, vips_buf_all( &buf ) );
 #endif /*DEBUG*/
 
 	/* Abstract real parameters.
@@ -1622,15 +1591,15 @@ compile_heap( Compile *compile )
 
 #ifdef DEBUG_RESULT
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 
 	printf( "compiled \"" );
 	symbol_name_print( compile->sym );
 	printf( "\" to: " );
-	buf_init_static( &buf, txt, 1024 );
+	vips_buf_init_static( &buf, txt, 1024 );
 	graph_pelement( compile->heap, &buf, &base, TRUE );
-	printf( "%s\n", buf_all( &buf ) );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_RESULT*/
 
@@ -2106,12 +2075,12 @@ compile_copy_sym( Symbol *sym, Compile *dest )
 
 	/* Must be a different place.
 	 */
-	assert( symbol_get_parent( sym )->expr->compile != dest );
+	g_assert( symbol_get_parent( sym )->expr->compile != dest );
 
 	/* Must not be an existing sym of that name. Or if there is, it has to
 	 * be a zombie.
 	 */
-	assert( !compile_lookup( dest, name ) ||
+	g_assert( !compile_lookup( dest, name ) ||
 		compile_lookup( dest, name )->type == SYM_ZOMBIE );
 
 	switch( sym->type ) {
@@ -2153,7 +2122,7 @@ compile_copy_sym( Symbol *sym, Compile *dest )
 	case SYM_EXTERNAL:
 	case SYM_BUILTIN:
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	return( NULL );
@@ -2210,7 +2179,7 @@ compile_copy_tree( Compile *fromscope, ParseNode *tree, Compile *toscope )
 
   Example: after parse we have:
 
-	[(x, y) | x <- [1..3]; y <- [x..3]; x + y > 3];
+	[(x, y) :: x <- [1..3]; y <- [x..3]; x + y > 3];
 
 	... $$lcomp1 ...
 	{
@@ -2424,7 +2393,7 @@ compile_lcomp( Compile *compile )
 
 	/* Copy the code for the final result.
 	 */
-	assert( result );
+	g_assert( result );
 
 	n1 = compile_copy_tree( result->expr->compile, 
 		result->expr->compile->tree, scope );
