@@ -45,14 +45,6 @@ static guint conversion_signals[SIG_LAST] = { 0 };
 
 static ModelClass *parent_class = NULL;
 
-/* What we send down the queue.
- */
-typedef struct _ConversionUpdate {
-	Conversion *conv;
-	IMAGE *im;
-	Rect area;
-} ConversionUpdate;
-
 /* All active conversions.
  */
 static GSList *conversion_all = NULL;
@@ -110,6 +102,7 @@ conversion_finalize( GObject *gobject )
 	conversion_all = g_slist_remove( conversion_all, conv );
 
 	IM_FREEF( im_region_free, conv->ireg );
+	IM_FREEF( im_region_free, conv->mreg );
 	IM_FREEF( im_region_free, conv->reg );
 
 	MANAGED_UNREF( conv->repaint_ii );
@@ -146,6 +139,17 @@ conversion_make_visualise( Conversion *conv, IMAGE *in )
 			in = t;
 		}
 
+		if( in->Coding == IM_CODING_RAD ) {
+			IMAGE *t = im_open_local( out, "conv:1", "p" );
+
+			if( !t || im_rad2float( in, t ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t;
+		}
+
                 if( im_open_local_array( out, t, 3, "conv-1", "p" ) ||
                         im_histnorm( in, t[0] ) ||
                         im_histplot( t[0], t[1] ) ) {
@@ -198,6 +202,15 @@ conversion_make_visualise( Conversion *conv, IMAGE *in )
 	return( out );
 }
 
+/* What we send from the notify callback to the main GUI thread.
+ */
+typedef struct _ConversionUpdate {
+	Conversion *conv;
+
+	IMAGE *im;		
+	Rect area;
+} ConversionUpdate;
+
 static gboolean
 conversion_render_idle_cb( gpointer data )
 {
@@ -283,7 +296,6 @@ static IMAGE *
 conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 {
 	IMAGE *out = im_open( "conversion_display:1", "p" );
-	IMAGE *mask = im_open_local( out, "conv:1a", "p" );
 
 	if( !out )
 		return( NULL );
@@ -331,56 +343,24 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 			return( NULL );
 		}
 	}
-	else
+	else {
+		IMAGE *mask = im_open_local( out, "conv:1a", "p" );
+
 		if( im_render_fade( in, out, mask, 
 			conv->tile_size, conv->tile_size, 
 				conversion_get_default_tiles( conv ),
-			20, conv->fade_steps,
+			20, 0, 
 			conv->priority,
 			conversion_render_notify_cb, conv ) ) {
 			im_close( out );
 			return( NULL );
 		}
 
-	if( mask_out )
-		*mask_out = mask;
-
-	return( out );
-}
-
-static int
-conversion_make_bg( Conversion *conv, IMAGE *repaint, IMAGE *out )
-{
-	IMAGE *tile = im_open_local( out, "tile", "t" );
-	IMAGE *t1 = im_open_local( out, "t1", "p" );
-	IMAGE *t2 = im_open_local( out, "t2", "p" );
-	int i;
-
-	/* Make a 2x2 pixel image with the tile pixels in.
-	 */
-	im_initdesc( tile, 2, 2,
-		repaint->Bands, IM_BBITS_BYTE, IM_BANDFMT_UCHAR,
-		IM_CODING_NONE, IM_TYPE_MULTIBAND, 
-		1.0, 1.0, 0, 0 );
-	if( im_setupout( tile ) )
-		return( -1 );
-	for( i = 0; i < tile->Bands; i++ ) {
-		((PEL *)(tile->data))[i] = 100;
-		((PEL *)(tile->data))[i + tile->Bands] = 90;
-		((PEL *)(tile->data))[i + tile->Bands * 2] = 90;
-		((PEL *)(tile->data))[i + tile->Bands * 3] = 100;
+		if( mask_out )
+			*mask_out = mask;
 	}
 
-	/* Blow it up, tile it and trim it.
-	 */
-	if( im_zoom( tile, t1, 16, 16 ) ||
-		im_replicate( t1, t2, 
-			1 + repaint->Xsize / 16, 1 + repaint->Ysize / 16 ) ||
-		im_extract_area( t2, out, 
-			0, 0, repaint->Xsize, repaint->Ysize ) )
-		return( -1 );
-
-	return( 0 );
+	return( out );
 }
 
 /* Track during lintrauc.
@@ -466,7 +446,7 @@ lintrauc_gen( PEL *in, PEL *out, int width, IMAGE *im, LintraInfo *inf )
 		LOOPCMPLX( double ); break;
 
         default:
-                assert( 0 );
+                g_assert( 0 );
         }
 
         return( 0 );
@@ -515,7 +495,6 @@ static IMAGE *
 conversion_make_repaint( Conversion *conv, IMAGE *in )
 {
 	IMAGE *out = im_open( "conversion_apply:1", "p" );
-	IMAGE *bg = im_open_local( out, "bg", "p" );
 
 	/* 7 is sRGB.
 
@@ -534,7 +513,7 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		return( NULL );
 
 	/* Special case: if this is a IM_CODING_LABQ and the display control 
-	 * bar is turned off we can go straight to RGB for speed.
+	 * bar is turned off, we can go straight to RGB for speed.
 	 */
 	if( in->Coding == IM_CODING_LABQ && !(conv && conv->enabled) ) {
 		IMAGE *t = im_open_local( out, "conv:1", "p" );
@@ -606,6 +585,17 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		in = t;
 	}
 
+	if( in->Coding == IM_CODING_RAD ) {
+		IMAGE *t = im_open_local( out, "conv:1", "p" );
+
+		if( !t || im_rad2float( in, t ) ) {
+			im_close( out );
+			return( NULL );
+		}
+
+		in = t;
+	}
+
 	if( in->Coding != IM_CODING_NONE ) {
 		im_close( out );
 		return( NULL );
@@ -713,19 +703,30 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		}
 	}
 
-	/* im_msb() only works for the int formats.
-	 */
 	if( tconv && 
-		(in->Type == IM_TYPE_RGB16 || in->Type == IM_TYPE_GREY16) &&
-		im_isint( in ) ) {
+		(in->Type == IM_TYPE_RGB16 || in->Type == IM_TYPE_GREY16) ) {
 		IMAGE *t[1];
 
-                if( im_open_local_array( out, t, 1, "conv-1", "p" ) ||
-			im_msb( in, t[0] ) ) {
+		if( im_open_local_array( out, t, 1, "conv-1", "p" ) ) {
 			im_close( out );
 			return( NULL );
 		}
 
+		/* im_msb() only works for the int formats.
+		 */
+		if( im_isint( in ) ) {
+			if( im_msb( in, t[0] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+		}
+		else {
+			if( im_lintrauc( 1 / 256.0, in, 0.0, t[0] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+		}
+
 		in = t[0];
 	}
 
@@ -771,20 +772,10 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		in = t1;
 	}
 
-	/* Combine with background checkerboard and mask to make final image.
-	 */
-	if( conv->synchronous ) {
-		if( im_copy( in, out ) ) {
-			im_close( out );
-			return( NULL );
-		}
+	if( im_copy( in, out ) ) {
+		im_close( out );
+		return( NULL );
 	}
-	else
-		if( conversion_make_bg( conv, in, bg ) ||
-			im_blend( conv->mask, in, bg, out ) ) {
-			im_close( out );
-			return( NULL );
-		}
 
 	return( out );
 }
@@ -852,6 +843,7 @@ conversion_rebuild_display( Conversion *conv )
 	IMAGE *new_display_im;
 	Imageinfo *new_display_ii;
 	IMAGE *mask;
+	REGION *new_mreg;
 
 #ifdef DEBUG
 	g_print( "conversion_remake_display: %p\n", conv );
@@ -866,6 +858,8 @@ conversion_rebuild_display( Conversion *conv )
 	 */
 	new_display_ii = NULL;
 	new_display_im = NULL;
+	new_mreg = NULL;
+	mask = NULL;
 
 	/* Make the new stuff first.
 	 */
@@ -880,13 +874,18 @@ conversion_rebuild_display( Conversion *conv )
 		}
 		managed_sub_add( MANAGED( new_display_ii ), 
 			MANAGED( conv->visual_ii ) );
+		if( mask && 
+			!(new_mreg = im_region_create( mask )) ) 
+			return;
 	}
 
+	IM_FREEF( im_region_free, conv->mreg );
 	MANAGED_UNREF( conv->display_ii );
 
 	if( visual_im ) {
 		conv->display_ii = new_display_ii;
 		conv->mask = mask;
+		conv->mreg = new_mreg;
 		MANAGED_REF( conv->display_ii );
 
 		conv->canvas.width = new_display_im->Xsize;
@@ -931,10 +930,8 @@ conversion_rebuild_visual( Conversion *conv )
 		managed_sub_add( MANAGED( new_visual_ii ), 
 			MANAGED( conv->ii ) );
 
-		if( !(new_reg = im_region_create( im )) ) {
-			IM_FREEF( im_region_free, new_reg );
+		if( !(new_reg = im_region_create( im )) ) 
 			return;
-		}
 	}
 
 	/* Junk old stuff.
@@ -1047,17 +1044,13 @@ conversion_init( Conversion *conv )
 	conv->display_mag = 99999999;
 	conv->repaint_ii = NULL;
 	conv->ireg = NULL;
+	conv->mreg = NULL;
 
 	/* Default tile size ... OK for image display, too big for
 	 * thumbnails.
 	 */
 	conv->tile_size = 128;
 
-	/* Default to the preference ... iimageview sets this to less to stop
-	 * the tile fade animation on thumbnails.
-	 */
-	conv->fade_steps = DISPLAY_FADE_STEPS;
-
 	conv->underlay = emptyrect;
 	conv->image = emptyrect;
 	conv->canvas = emptyrect;
@@ -1198,7 +1191,7 @@ conversion_set_image( Conversion *conv, Imageinfo *ii )
 double
 conversion_dmag( int mag )
 {
-	assert( mag != 0 );
+	g_assert( mag != 0 );
 
 	if( mag > 0 )
 		return( mag );
@@ -1212,7 +1205,7 @@ conversion_dmag( int mag )
 int 
 conversion_double( int mag )
 {
-	assert( mag != -1 );
+	g_assert( mag != -1 );
 
 	if( mag == -3 )
 		return( -2 );
@@ -1227,7 +1220,7 @@ conversion_double( int mag )
 int 
 conversion_halve( int mag )
 {
-	assert( mag != -1 );
+	g_assert( mag != -1 );
 
 	if( mag == 1 )
 		return( -2 );
