@@ -78,13 +78,13 @@ conversionview_scale_cb( GtkWidget *wid, Conversionview *cv )
 	Imagemodel *imagemodel = cv->imagemodel;
 	double min, max;
 
-	set_hourglass();
+	progress_begin();
 	if( !conversionview_findmaxmin( imagemodel, &min, &max ) ) {
-		set_pointer();
+		progress_end();
 		box_alert( wid );
 		return;
 	}
-	set_pointer();
+	progress_end();
 
         if( max - min < 1e-20 ) {
                 error_top( _( "Unable to scale image." ) );
@@ -338,7 +338,7 @@ conversionview_changed_cb( Imagemodel *imagemodel, Conversionview *cv )
 static void
 conversionview_link( Conversionview *cv, Imagemodel *imagemodel )
 {
-	assert( !cv->imagemodel );
+	g_assert( !cv->imagemodel );
 
 	cv->imagemodel = imagemodel;
 	g_signal_connect( G_OBJECT( cv->imagemodel ), 
