@@ -29,7 +29,8 @@
 
 #include "ip.h"
 
-/* A lot of this file is just for debugging.
+/* A lot of this file is just for debugging. Uncomment to enable all the
+ * debugging code.
 #define DEBUG
  */
 
@@ -69,7 +70,7 @@ decode_BinOp( BinOp op )
 	case BI_CONS:		return( ":" );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -100,7 +101,7 @@ decode_UnOp( UnOp op )
 	case UN_PLUS:		return( "+" );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -126,7 +127,7 @@ decode_NodeType( NodeType tag )
 	case TAG_REFERENCE:	return( "TAG_REFERENCE" );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -147,7 +148,7 @@ decode_CombinatorType( CombinatorType comb )
 	case COMB_K:		return( "K" );
 	case COMB_GEN:		return( "G" );
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -171,7 +172,7 @@ decode_SymbolType( SymbolType t )
 	case SYM_BUILTIN:	return( "SYM_BUILTIN" );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 		return( NULL );
 	}
 }
@@ -192,7 +193,7 @@ decode_SymbolType_user( SymbolType t )
 	case SYM_BUILTIN:	return( _( "built-in symbol" ) );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 		return( NULL );
 	}
 }
@@ -208,15 +209,16 @@ decode_SymbolType_user( SymbolType t )
 void *
 dump_tiny( Symbol *sym )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char str[100];
 
 	printf( "(%p) ", sym );
-	buf_init_static( &buf, str, 100 );
+	vips_buf_init_static( &buf, str, 100 );
 	symbol_qualified_name( sym, &buf );
 	if( sym->dirty )
 		printf( "*" );
-	printf( "%s %s; ", decode_SymbolType( sym->type ), buf_all( &buf ) );
+	printf( "%s %s; ", 
+		decode_SymbolType( sym->type ), vips_buf_all( &buf ) );
 
 	return( NULL );
 }
@@ -287,7 +289,7 @@ dump_compile( Compile *compile )
 {
 	Symbol *sym = compile->sym;
 #ifdef VERBOSE
-	BufInfo buf;
+	VipsBuf buf;
 	char str[100];
 #endif /*VERBOSE*/
 
@@ -351,10 +353,10 @@ dump_compile( Compile *compile )
 	(void) slist_map( compile->children, (SListMapFn) dump_tiny, NULL );
 	printf( "\n" );
 
-	buf_init_static( &buf, str, 80 );
+	vips_buf_init_static( &buf, str, 80 );
 	graph_element( compile->heap, &buf, &compile->base, FALSE );
 	printf( "%s->compile->base = %s\n", 
-		IOBJECT( sym )->name, buf_all( &buf ) );
+		IOBJECT( sym )->name, vips_buf_all( &buf ) );
 	if( compile->heap )
 		iobject_dump( IOBJECT( compile->heap ) );
 #endif /*VERBOSE*/
@@ -441,7 +443,7 @@ dump_tiny_tool( Tool *tool )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( NULL );
@@ -487,12 +489,12 @@ psym( char *name )
 void 
 pgraph( PElement *graph )
 {	
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[10240];
 
-	buf_init_static( &buf, txt, 10240 );
+	vips_buf_init_static( &buf, txt, 10240 );
 	graph_pelement( reduce_context->heap, &buf, graph, TRUE );
-	printf( "%s\n", buf_all( &buf ) );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 
 /* Print symbol value from name.
@@ -500,15 +502,15 @@ pgraph( PElement *graph )
 void 
 psymv( char *name )
 {	
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 	Symbol *s = sym( name );
 
 	if( s ) {
-		buf_init_static( &buf, txt, 1024 );
+		vips_buf_init_static( &buf, txt, 1024 );
 		graph_pelement( reduce_context->heap, 
 			&buf, &s->expr->root, TRUE );
-		printf( "%s = %s\n", name, buf_all( &buf ) );
+		printf( "%s = %s\n", name, vips_buf_all( &buf ) );
 	}
 }
 
@@ -570,11 +572,6 @@ print_element( int nsp, EType type, void *arg )
 		printf( "tag \"%s\"", (char*)arg ); 
 		break;
 
-	case ELEMENT_STATIC:
-		printf( "static string \"%s\"", 
-			((HeapStaticString*)arg)->text ); 
-		break;
-
 	case ELEMENT_MANAGED:
 		printf( "Managed* %p", arg ); 
 		break;
@@ -584,7 +581,7 @@ print_element( int nsp, EType type, void *arg )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -672,7 +669,7 @@ graph_heap( int nsp, HeapNode *hn )
 
 	case TAG_FREE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -706,7 +703,7 @@ dump_parseconst( ParseConst *pc )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -799,7 +796,7 @@ dump_tree( ParseNode *n )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( NULL );
@@ -819,10 +816,14 @@ dump_link( Link *link )
 {
 	printf( "link->parent = " );
 	symbol_name_print( link->parent );
+	if( link->parent->dirty )
+		printf( " (dirty)" );
 	printf( "\n" );
 
 	printf( "link->child = " );
 	symbol_name_print( link->child );
+	if( link->child->dirty )
+		printf( " (dirty)" );
 	printf( "\n" );
 
 	printf( "link->serial = %d\n", link->serial );
