@@ -137,18 +137,18 @@ filemodel_save_all( Filemodel *filemodel, const char *filename )
 }
 
 static void
-filemodel_info( iObject *iobject, BufInfo *buf )
+filemodel_info( iObject *iobject, VipsBuf *buf )
 {
 	Filemodel *filemodel = FILEMODEL( iobject );
 
 	IOBJECT_CLASS( parent_class )->info( iobject, buf );
 
-	buf_appendf( buf, "filename = \"%s\"\n", NN( filemodel->filename ) );
-	buf_appendf( buf, "modified = \"%s\"\n", 
+	vips_buf_appendf( buf, "filename = \"%s\"\n", NN( filemodel->filename ) );
+	vips_buf_appendf( buf, "modified = \"%s\"\n", 
 		bool_to_char( filemodel->modified ) );
-	buf_appendf( buf, "registered = \"%s\"\n", 
+	vips_buf_appendf( buf, "registered = \"%s\"\n", 
 		bool_to_char( filemodel->registered ) );
-	buf_appendf( buf, "auto_load = \"%s\"\n", 
+	vips_buf_appendf( buf, "auto_load = \"%s\"\n", 
 		bool_to_char( filemodel->auto_load ) );
 }
 
@@ -368,6 +368,10 @@ filemodel_class_init( FilemodelClass *class )
 	class->top_load = filemodel_real_top_load;
 	class->set_modified = filemodel_real_set_modified;
 	class->save_all = filemodel_real_save_all;
+
+	/* NULL isn't an allowed value -- this gets overridden by our
+	 * subclasses.
+	 */
 	class->filetype = NULL;
 	class->filetype_pref = NULL;
 }
@@ -631,7 +635,7 @@ filemodel_inter_save_cb( iWindow *iwnd,
 
 		if( filemodel_save_all( filemodel, filename ) ) {
 			filemodel_set_modified( filemodel, FALSE );
-			nfn( sys, IWINDOW_TRUE );
+			nfn( sys, IWINDOW_YES );
 		}
 		else
 			nfn( sys, IWINDOW_ERROR );
@@ -702,7 +706,7 @@ filemodel_inter_empty_cb( iWindow *iwnd, void *client,
 	(void) model_empty( MODEL( filemodel ) );
 	filemodel_set_modified( filemodel, FALSE );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 static void
@@ -768,7 +772,7 @@ filemodel_inter_close_cb( iWindow *iwnd, void *client,
 
 	IDESTROY( filemodel );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 void
@@ -805,7 +809,7 @@ filemodel_inter_load_cb( iWindow *iwnd,
 
 		if( filemodel_load_all( filemodel, 
 			MODEL( parent ), filename ) ) 
-			nfn( sys, IWINDOW_TRUE );
+			nfn( sys, IWINDOW_YES );
 		else
 			nfn( sys, IWINDOW_ERROR );
 
@@ -884,7 +888,7 @@ filemodel_inter_close_registered_cb( iWindow *iwnd, void *client,
 			iwindow_susp_comp, susp );
 	}
 	else
-		nfn( sys, IWINDOW_TRUE );
+		nfn( sys, IWINDOW_YES );
 }
 
 /* Mark something as having been loaded (or made) during startup. If we loaded
@@ -907,7 +911,7 @@ filemodel_set_auto_load( Filemodel *filemodel )
 		char *p = strrchr( filemodel->filename, G_DIR_SEPARATOR );
 		char buf[FILENAME_MAX];
 
-		assert( p );
+		g_assert( p );
 
 		im_snprintf( buf, FILENAME_MAX, "$SAVEDIR" G_DIR_SEPARATOR_S
 			"start" G_DIR_SEPARATOR_S "%s", p + 1 );
