@@ -72,38 +72,16 @@ static iDialogClass *parent_class = NULL;
  */
 static char *filesel_last_dir = NULL;
 
-static const char *tiff_suffs[] = { ".tif", ".tiff", NULL };
-static const char *jpeg_suffs[] = { ".jpg", ".jpeg", ".jpe", NULL };
-static const char *png_suffs[] = { ".png", NULL };
-static const char *ppm_suffs[] = { ".ppm", ".pgm", ".pbm", NULL };
-static const char *exr_suffs[] = { ".exr", NULL };
-static const char *analyze_suffs[] = { ".img", ".hdr", NULL };
 static const char *icc_suffs[] = { ".icc", ".icm", NULL };
-static const char *vips_suffs[] = { ".v", NULL };
 static const char *workspace_suffs[] = { ".ws", NULL };
 static const char *rec_suffs[] = { ".rec", NULL };
 static const char *mor_suffs[] = { ".mor", NULL };
 static const char *con_suffs[] = { ".con", NULL };
 static const char *mat_suffs[] = { ".mat", NULL };
 static const char *def_suffs[] = { ".def", NULL };
-static const char *csv_suffs[] = { ".csv", NULL };
 static const char *all_suffs[] = { "", NULL };
 
 FileselFileType
-        filesel_tfile_type = 
-		{ N_( "TIFF image files (*.tif, *.tiff)" ), tiff_suffs },
-        filesel_jfile_type = 
-		{ N_( "JPEG image files (*.jpg, *.jpeg, *.jpe)" ), jpeg_suffs },
-        filesel_nfile_type = 
-		{ N_( "PNG image files (*.png)" ), png_suffs },
-        filesel_vfile_type = 
-		{ N_( "VIPS image files (*.v)" ), vips_suffs },
-        filesel_pfile_type = 
-		{ N_( "PPM image files (*.ppm, *.pgm, *.pbm)" ), ppm_suffs },
-        filesel_efile_type = 
-		{ N_( "OpenEXR image files (*.exr)" ), exr_suffs },
-        filesel_afile_type = 
-		{ N_( "Analyze image files (*.img, *.hdr)" ), analyze_suffs },
         filesel_wfile_type = 
 		{ N_( "Workspace files (*.ws)" ), workspace_suffs },
         filesel_rfile_type = 
@@ -118,51 +96,127 @@ FileselFileType
 		{ N_( "Definition files (*.def)" ), def_suffs },
         filesel_ifile_type = 
 		{ N_( "ICC profiles (*.icc, *.icm)" ), icc_suffs },
-        filesel_sfile_type = 
-		{ N_( "CSV files (*.csv)" ), csv_suffs },
         filesel_allfile_type = 
 		{ N_( "All files (*)" ), all_suffs };
 
 FileselFileType
         *filesel_type_definition[] = { 
-		&filesel_dfile_type, &filesel_allfile_type, NULL 
+		&filesel_dfile_type, NULL 
 	},
         *filesel_type_workspace[] = { 
-		&filesel_wfile_type, &filesel_allfile_type, NULL 
-	},
-        *filesel_type_image[] = { 
-		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
-		&filesel_pfile_type, &filesel_nfile_type, 
-		&filesel_afile_type, &filesel_efile_type,
-		&filesel_sfile_type, 
-		&filesel_allfile_type, NULL 
+		&filesel_wfile_type, NULL 
 	},
+
         *filesel_type_matrix[] = { 
 		&filesel_xfile_type, &filesel_cfile_type, &filesel_rfile_type, 
-		&filesel_mfile_type, &filesel_allfile_type, NULL 
+		&filesel_mfile_type, NULL 
 	},
-        *filesel_type_mainw[] = { 
-		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
-		&filesel_pfile_type, &filesel_nfile_type,
-		&filesel_afile_type, &filesel_efile_type,
-		&filesel_sfile_type, 
-		&filesel_xfile_type, &filesel_cfile_type, &filesel_rfile_type, 
-		&filesel_mfile_type, 
-		&filesel_wfile_type, 
-		&filesel_allfile_type, NULL 
-	},
-        *filesel_type_any[] = { 
-		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
-		&filesel_pfile_type, &filesel_nfile_type,
-		&filesel_afile_type, &filesel_efile_type,
-		&filesel_sfile_type, 
-		&filesel_wfile_type, 
-		&filesel_dfile_type, 
-		&filesel_xfile_type, &filesel_cfile_type, &filesel_rfile_type, 
-		&filesel_mfile_type, 
-		&filesel_ifile_type, 
-		&filesel_allfile_type, NULL 
-	};
+
+	/* Set during startup.
+	 */
+        **filesel_type_image = NULL,
+        **filesel_type_mainw = NULL,
+        **filesel_type_any = NULL;
+
+static void *
+build_vips_formats_sub( VipsFormatClass *format, GSList **types )
+{
+	FileselFileType *type = g_new( FileselFileType, 1 );
+	char txt[MAX_STRSIZE];
+	VipsBuf buf;
+	const char **i;
+
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_appendf( &buf, "%s ", VIPS_OBJECT_CLASS( format )->description );
+	/* Used as eg. "VIPS image files (*.v)"
+	 */
+	vips_buf_appends( &buf, _( "image files" ) );
+	vips_buf_appends( &buf, " (" );
+	if( *format->suffs )
+		for( i = format->suffs; *i; i++ ) {
+			vips_buf_appendf( &buf, "*%s", *i );
+			if( i[1] )
+				vips_buf_appends( &buf, "; " );
+		}
+	else
+		/* No suffix means any allowed.
+		 */
+		vips_buf_appendf( &buf, "*" );
+
+	vips_buf_appends( &buf, ")" );
+
+	type->name = g_strdup( vips_buf_all( &buf ) );
+	type->suffixes = format->suffs;
+
+	*types = g_slist_append( *types, type );
+
+	return( NULL );
+}
+
+/* Look at the registered VIPS formats, build a file type list. Call from
+ * filesel class init.
+ */
+static FileselFileType **
+build_image_file_type( void )
+{
+	GSList *types;
+	FileselFileType **type_array;
+
+	types = NULL;
+	vips_format_map( (VSListMap2Fn) build_vips_formats_sub, &types, NULL );
+
+	type_array = (FileselFileType **) slist_to_array( types );
+	g_slist_free( types );
+
+	return( type_array );
+}
+
+/* Combine a NULL-terminated list of FileselFileType arrays into one.
+ */
+static FileselFileType **
+build_file_type_va( FileselFileType **first, ... )
+{
+	va_list args;
+	int len;
+	FileselFileType **i;
+	FileselFileType **array;
+	int j;
+	int k;
+
+	/* Count total number of items.
+	 */
+	len = 0;
+	va_start( args, first );
+	for( i = first; i; i = va_arg( args, FileselFileType ** ) )
+		len += array_len( (void **) i );
+	va_end( args );
+
+	/* Copy and NULL-terminate.
+	 */
+	array = g_new( FileselFileType *, len + 1 );
+	va_start( args, first );
+	j = 0;
+	for( i = first; i; i = va_arg( args, FileselFileType ** ) ) 
+		for( k = 0; i[k]; k++ )
+			array[j++] = i[k];
+	va_end( args );
+	array[j] = NULL;
+
+	return( array );
+}
+
+/* Here from main() during startup. We can't just put this in class_init,
+ * because we want to be sure this happens early on.
+ */
+void
+filesel_startup( void )
+{
+	filesel_type_image = build_image_file_type();
+	filesel_type_mainw = build_file_type_va( filesel_type_image, 
+		filesel_type_matrix, filesel_type_workspace, NULL );
+	filesel_type_any = build_file_type_va( filesel_type_mainw,
+		filesel_type_definition, NULL );
+}
 
 /* Is a file of type ... just look at the suffix.
  */
@@ -350,47 +404,62 @@ filesel_csv_mode( char *out )
 typedef void (*make_mode_fn)( char *buf );
 
 typedef struct {
-	FileselFileType *type;
-	const char *caption_filter;
-	make_mode_fn mode_fn;
+	const char *caption_filter;	/* nip2 column name for the format */
+	const char *name;		/* vips nickname for the format */
+	make_mode_fn mode_fn;		/* Build a mode string */
 } FileselMode;
 
 static FileselMode filesel_mode_table[] = {
-	{ &filesel_jfile_type, "JPEG", filesel_jpeg_mode },
-	{ &filesel_nfile_type, "PNG", filesel_png_mode },
-	{ &filesel_tfile_type, "TIFF", filesel_tiff_mode },
-	{ &filesel_sfile_type, "CSV", filesel_csv_mode },
-	{ &filesel_pfile_type, "PPM", filesel_ppm_mode }
+	{ "JPEG", "jpeg", filesel_jpeg_mode },
+	{ "PNG", "png", filesel_png_mode },
+	{ "TIFF", "tiff", filesel_tiff_mode },
+	{ "CSV", "csv", filesel_csv_mode },
+	{ "PPM", "ppm", filesel_ppm_mode }
 };
 
+static FileselMode *
+filesel_get_mode( const char *filename )
+{
+	int i;
+	VipsFormatClass *format;
+
+	if( (format = vips_format_for_name( filename )) ) {
+		VipsObjectClass *object_class = VIPS_OBJECT_CLASS( format );
+
+		for( i = 0; i < IM_NUMBER( filesel_mode_table ); i++ )
+			if( strcmp( filesel_mode_table[i].name, 
+				object_class->nickname ) == 0 ) 
+				return( &filesel_mode_table[i] );
+	}
+	else
+		im_error_clear();
+
+	return( NULL );
+}
+
 /* Add our image save settings to the end of a filename.
  */
 void
 filesel_add_mode( char *filename )
 {
-	int i;
+	FileselMode *mode;
 
-	for( i = 0; i < IM_NUMBER( filesel_mode_table ); i++ )
-		if( is_file_type( filesel_mode_table[i].type, filename ) ) {
-			char mode[256];
-			int l = strlen( filename );
+	if( (mode = filesel_get_mode( filename )) ) {
+		char ext[256];
+		int l = strlen( filename );
 
-			filesel_mode_table[i].mode_fn( mode );
-			im_snprintf( filename + l, MAX_STRSIZE - l, 
-				":%s", mode );
-
-			return;
-		}
+		mode->mode_fn( ext );
+		im_snprintf( filename + l, MAX_STRSIZE - l, ":%s", ext );
+	}
 }
 
 static const char *
 filesel_get_filter( const char *filename )
 {
-	int i;
+	FileselMode *mode;
 
-	for( i = 0; i < IM_NUMBER( filesel_mode_table ); i++ )
-		if( is_file_type( filesel_mode_table[i].type, filename ) ) 
-			return( filesel_mode_table[i].caption_filter );
+	if( (mode = filesel_get_mode( filename )) ) 
+		return( mode->caption_filter );
 
 	return( NULL );
 }
@@ -426,15 +495,15 @@ filesel_space_update( Filesel *filesel, const char *dirname )
 				dirname );
 		else {
 			char txt[MAX_STRSIZE];
-			BufInfo buf;
+			VipsBuf buf;
 
-			buf_init_static( &buf, txt, MAX_STRSIZE );
+			vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 			to_size( &buf, sz );
-			buf_appendf( &buf, " " );
+			vips_buf_appendf( &buf, " " );
 			/* Expands to (eg.) '6GB free in "/pics/tmp"'
 			 */
-			buf_appendf( &buf, _( "free in \"%s\"" ), dirname );
-			set_glabel( filesel->space, "%s", buf_all( &buf ) );
+			vips_buf_appendf( &buf, _( "free in \"%s\"" ), dirname );
+			set_glabel( filesel->space, "%s", vips_buf_all( &buf ) );
 		}
 	}
 }
@@ -455,46 +524,46 @@ filesel_add_volume( const char *dir, Filesel *filesel )
 }
 
 static void
-filesel_suffix_to_glob( const char *suffix, BufInfo *patt )
+filesel_suffix_to_glob( const char *suffix, VipsBuf *patt )
 {
 	int i;
 	char ch;
 
-	buf_appends( patt, "*" );
+	vips_buf_appends( patt, "*" );
 
 	for( i = 0; (ch = suffix[i]); i++ ) {
 		if( isalpha( ch ) ) {
-			buf_appends( patt, "[" );
-			buf_appendf( patt, "%c", toupper( ch ) );
-			buf_appendf( patt, "%c", tolower( ch ) );
-			buf_appends( patt, "]" );
+			vips_buf_appends( patt, "[" );
+			vips_buf_appendf( patt, "%c", toupper( ch ) );
+			vips_buf_appendf( patt, "%c", tolower( ch ) );
+			vips_buf_appends( patt, "]" );
 		}
 		else
-			buf_appendf( patt, "%c", ch );
+			vips_buf_appendf( patt, "%c", ch );
 	}
 }
 
 /* Make a shell glob from a filetype.
  */
 void
-filesel_make_patt( FileselFileType *type, BufInfo *patt )
+filesel_make_patt( FileselFileType *type, VipsBuf *patt )
 {
 	int i;
 
 	/* Only use {} braces if there's more than one suffix to match.
 	 */
 	if( type->suffixes[1] )
-		buf_appends( patt, "{" );
+		vips_buf_appends( patt, "{" );
 
 	for( i = 0; type->suffixes[i]; i++ ) {
 		if( i > 0 )
-			buf_appends( patt, "," );
+			vips_buf_appends( patt, "," );
 
 		filesel_suffix_to_glob( type->suffixes[i], patt );
 	}
 
 	if( type->suffixes[1] )
-		buf_appends( patt, "}" );
+		vips_buf_appends( patt, "}" );
 }
 
 static char *
@@ -538,35 +607,41 @@ static void
 filesel_info_update( Filesel *filesel, const char *name )
 {
 	char txt[MAX_STRSIZE];
-	BufInfo buf;
+	VipsBuf buf;
 
 	if( filesel->info ) {
-		buf_init_static( &buf, txt, MAX_STRSIZE );
+		vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 		get_image_info( &buf, name );
-		set_glabel( filesel->info, "%s", buf_firstline( &buf ) );
+		set_glabel( filesel->info, "%s", vips_buf_firstline( &buf ) );
 	}
 }
 
 int
 filesel_get_filetype( Filesel *filesel )
 {
-	if( !filesel->chooser )
-		return( filesel->default_type );
-	else {
-		GtkFileFilter *filter;
+	int type;
+	GtkFileFilter *filter;
+
+	type = filesel->default_type;
+
+	if( filesel->chooser &&
+		(filter = gtk_file_chooser_get_filter( 
+			GTK_FILE_CHOOSER( filesel->chooser ) )) )  {
 		int i;
 
-		filter = gtk_file_chooser_get_filter( 
-			GTK_FILE_CHOOSER( filesel->chooser ) );
-		assert( filter );
-
-		for( i = 0; filesel->type[i]; i++ ) 
+		for( i = 0; filesel->filter[i]; i++ ) 
 			if( filter == filesel->filter[i] )
 				break;
-		assert( filesel->type[i] );
+		g_assert( filesel->filter[i] );
 
-		return( i );
+		type = i;
 	}
+
+#ifdef DEBUG
+	printf( "filesel_get_filetype: %d\n", type ); 
+#endif /*DEBUG*/
+
+	return( type );
 }
 
 /* Find the index of the type which matches this filename.
@@ -584,6 +659,19 @@ filesel_find_file_type( FileselFileType **type, const char *filename )
 	return( -1 );
 }
 
+static void
+filesel_set_filter( Filesel *filesel, GtkFileFilter *filter )
+{
+#ifdef DEBUG
+	printf( "filesel_set_filter: %p\n", filter ); 
+#endif /*DEBUG*/
+
+	g_assert( filter );
+
+	gtk_file_chooser_set_filter( GTK_FILE_CHOOSER( filesel->chooser ),
+		filter );
+}
+
 static void
 filesel_set_filetype_from_filename( Filesel *filesel, const char *name )
 {
@@ -605,14 +693,11 @@ filesel_set_filetype_from_filename( Filesel *filesel, const char *name )
 		return;
 
 	if( (i = filesel_find_file_type( filesel->type, name )) >= 0 )
-		gtk_file_chooser_set_filter( 
-			GTK_FILE_CHOOSER( filesel->chooser ),
-			filesel->filter[i] );
+		filesel_set_filter( filesel, filesel->filter[i] );
 	else
 		/* No match, or no suffix. Set the last type (should be "All").
 		 */
-		gtk_file_chooser_set_filter( 
-			GTK_FILE_CHOOSER( filesel->chooser ),
+		filesel_set_filter( filesel, 
 			filesel->filter[filesel->ntypes - 1] );
 }
 
@@ -627,7 +712,6 @@ filesel_set_filename( Filesel *filesel, const char *name )
         expand_variables( name, buf );
         nativeize_path( buf );
 	absoluteize_path( buf );
-	filesel_set_filetype_from_filename( filesel, buf );
 
 #ifdef DEBUG
 	printf( "filesel_set_filename: %s\n", buf ); 
@@ -639,20 +723,18 @@ filesel_set_filename( Filesel *filesel, const char *name )
 	 */
 	gtk_file_chooser_set_filename( 
 		GTK_FILE_CHOOSER( filesel->chooser ), buf );
+
 	if( filesel->save )
 		gtk_file_chooser_set_current_name(
 			GTK_FILE_CHOOSER( filesel->chooser ), 
 			im_skip_dir( buf ) );
 
-	/* The file isn't actually selected until we hit the main loop again.
-	 * If we don't loop here, the next gtk_file_chooser_get_filename()
-	 * will fail.
-	 */
-	while( g_main_context_iteration( NULL, FALSE ) )
-		;
-
 	filesel->start_name = TRUE;
 
+	/* We have to set this after setting the filename.
+	 */
+	filesel_set_filetype_from_filename( filesel, buf );
+
 	return( TRUE );
 }
 
@@ -683,6 +765,20 @@ filesel_get_filename( Filesel *filesel )
 	return( name );
 }
 
+/* How many selected filenames.
+ */
+int
+filesel_nselected( Filesel *filesel )
+{
+	GSList *names = gtk_file_chooser_get_filenames( 
+		GTK_FILE_CHOOSER( filesel->chooser ) );
+	int n = g_slist_length( names );
+
+	IM_FREEF( slist_free_all, names );
+
+	return( n );
+}
+
 /* Get filename multi ... map over the selected filenames.
  */
 void *
@@ -732,7 +828,7 @@ filesel_selection_changed_cb( GtkWidget *widget, gpointer data )
 }
 
 static void
-filesel_file_sctivated_cb( GtkWidget *widget, gpointer data )
+filesel_file_activated_cb( GtkWidget *widget, gpointer data )
 {
 	idialog_done_trigger( IDIALOG( data ), 0 );
 }
@@ -771,13 +867,55 @@ browse_cb( GtkWidget *wid, Filesel *filesel )
 	}
 }
 
+static GtkFileFilter *
+file_filter_from_file_type( FileselFileType *type )
+{
+	GtkFileFilter *filter;
+	int j;
+	
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name( filter, _( type->name ) );
+
+	if( type->suffixes[0] )
+		for( j = 0; type->suffixes[j]; j++ ) {
+			char buf[FILENAME_MAX];
+			VipsBuf patt;
+
+			vips_buf_init_static( &patt, buf, FILENAME_MAX );
+			filesel_suffix_to_glob( type->suffixes[j], &patt );
+			gtk_file_filter_add_pattern( filter, vips_buf_all( &patt ) );
+		}
+	else
+		/* No suffix list means any suffix allowed.
+		 */
+		gtk_file_filter_add_pattern( filter, "*" );
+
+	return( filter );
+}
+
+static void
+filesel_add_filter( Filesel *filesel, FileselFileType *type, int i )
+{
+	filesel->filter[i] = file_filter_from_file_type( type );
+
+#ifdef DEBUG
+	printf( "filesel_add_filter: %p (%d)\n", filesel->filter[i], i ); 
+#endif /*DEBUG*/
+
+	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( filesel->chooser ),
+		filesel->filter[i] );
+
+	if( i == filesel->default_type )
+		filesel_set_filter( filesel, filesel->filter[i] );
+}
+
 static void
 filesel_build( GtkWidget *widget )
 {
 	Filesel *filesel = FILESEL( widget );
 	iDialog *idlg = IDIALOG( widget );
 
-	int i, j;
+	int i;
 	FileselFileType *type;
 	GtkWidget *vb;
 	GtkWidget *but;
@@ -806,32 +944,11 @@ filesel_build( GtkWidget *widget )
         slist_map( PATH_SEARCH,
 		(SListMapFn) filesel_add_volume, filesel );
 
-	/* Add all the supported file types.
+	/* Add all the supported file types. Add "all" to the end.
 	 */
-	for( i = 0; (type = filesel->type[i]); i++ ) {
-		char buf[FILENAME_MAX];
-		BufInfo patt;
-		
-		filesel->filter[i] = gtk_file_filter_new();
-
-		gtk_file_filter_set_name( filesel->filter[i], _( type->name ) );
-
-		for( j = 0; type->suffixes[j]; j++ ) {
-			buf_init_static( &patt, buf, FILENAME_MAX );
-			filesel_suffix_to_glob( type->suffixes[j], &patt );
-			gtk_file_filter_add_pattern( filesel->filter[i], 
-				buf_all( &patt ) );
-		}
-
-		gtk_file_chooser_add_filter( 
-			GTK_FILE_CHOOSER( filesel->chooser ),
-			filesel->filter[i] );
-
-		if( i == filesel->default_type )
-			gtk_file_chooser_set_filter( 
-				GTK_FILE_CHOOSER( filesel->chooser ),
-				filesel->filter[i] );
-	}
+	for( i = 0; (type = filesel->type[i]); i++ ) 
+		filesel_add_filter( filesel, type, i );
+	filesel_add_filter( filesel, &filesel_allfile_type, i );
 
         /* Spot changes.
          */
@@ -843,7 +960,7 @@ filesel_build( GtkWidget *widget )
                 GTK_SIGNAL_FUNC( filesel_selection_changed_cb ), filesel );
         gtk_signal_connect( GTK_OBJECT( filesel->chooser ), 
 		"file-activated",
-                GTK_SIGNAL_FUNC( filesel_file_sctivated_cb ), filesel );
+                GTK_SIGNAL_FUNC( filesel_file_activated_cb ), filesel );
 
 	/* Pack extra widgets.
 	 */
@@ -971,6 +1088,8 @@ filesel_refresh_all( void )
 static void
 filesel_init( Filesel *filesel )
 {
+	int i;
+
 #ifdef DEBUG
 	printf( "filesel_init: %s\n", NN( IWINDOW( filesel )->title ) );
 #endif /*DEBUG*/
@@ -978,6 +1097,8 @@ filesel_init( Filesel *filesel )
 	filesel->chooser = NULL;
 	filesel->space = NULL;
 	filesel->info = NULL;
+	for( i = 0; i < FILESEL_MAX_FILTERS; i++ )
+		filesel->filter[i] = NULL;
 	filesel->incr = FALSE;
 	filesel->imls = FALSE;
 	filesel->save = FALSE;
@@ -1047,9 +1168,7 @@ filesel_trigger2( void *sys, iWindowResult result )
 	iWindowSusp *susp = (iWindowSusp *) sys;
 	Filesel *filesel = FILESEL( susp->client );
 
-	/* Back from the user function ... restore the pointer.
-	 */
-	set_pointer();
+	progress_end();
 
 	/* If this is a save, assume that there is now a new file, 
 	 * and ask all fsb's to update.
@@ -1057,7 +1176,7 @@ filesel_trigger2( void *sys, iWindowResult result )
 	if( filesel->save && result != IWINDOW_ERROR )
 		filesel_refresh_all();
 
-	if( result != IWINDOW_TRUE ) {
+	if( result != IWINDOW_YES ) {
 		/* Failure ... bomb out.
 		 */
 		iwindow_susp_return( susp, result );
@@ -1092,7 +1211,7 @@ filesel_trigger( Filesel *filesel, iWindow *iwnd,
 		prefs_set( filesel->type_pref, 
 			"%d", filesel_get_filetype( filesel ) );
 
-	set_hourglass();
+	progress_begin();
 	filesel->done_cb( IWINDOW( filesel ), 
 		filesel->client, filesel_trigger2, susp );
 }
@@ -1177,6 +1296,9 @@ filesel_done_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 
 	if( !(filename = filesel_get_filename( filesel )) ) 
 		nfn( sys, IWINDOW_ERROR );
+	else if( isdir( "%s", filename ) ) {
+		nfn( sys, IWINDOW_NO );
+	}
 	else {
 		/* File exists and we are saving? Do a yesno before we carry on.
 		 */
@@ -1212,21 +1334,14 @@ void
 filesel_set_filetype( Filesel *filesel, 
 	FileselFileType **type, int default_type )
 {
-	int i;
-
-	/* Reset the widget if it's there.
+	/* Reset the widget, if it's there.
 	 */
 	if( filesel->chooser )
-		gtk_file_chooser_set_filter( 
-			GTK_FILE_CHOOSER( filesel->chooser ),
-			filesel->filter[default_type] );
+		filesel_set_filter( filesel, filesel->filter[default_type] );
 
 	filesel->type = type;
+	filesel->ntypes = array_len( (void **) type );
 	filesel->default_type = default_type;
-
-	for( i = 0; type[i]; i++ )
-		;
-	filesel->ntypes = i;
 }
 
 void 
