@@ -48,20 +48,19 @@ typedef struct _FileselFileType {
 
 /* Basic types.
  */
-extern FileselFileType filesel_tfile_type, filesel_jfile_type, 
-       filesel_nfile_type, filesel_vfile_type, filesel_pfile_type, 
-       filesel_afile_type, filesel_wfile_type, filesel_rfile_type, 
-       filesel_mfile_type, filesel_cfile_type, filesel_xfile_type, 
-       filesel_dfile_type, filesel_ifile_type, filesel_sfile_type; 
+extern FileselFileType 
+	filesel_wfile_type, filesel_rfile_type, 
+	filesel_mfile_type, filesel_cfile_type, filesel_xfile_type, 
+	filesel_dfile_type, filesel_ifile_type;
 
 /* Suffix sets we support.
  */
-extern FileselFileType *filesel_type_workspace[];
-extern FileselFileType *filesel_type_image[];
-extern FileselFileType *filesel_type_matrix[];
 extern FileselFileType *filesel_type_definition[];
-extern FileselFileType *filesel_type_any[];
-extern FileselFileType *filesel_type_mainw[];
+extern FileselFileType *filesel_type_workspace[];
+extern FileselFileType *filesel_type_matrix[];
+extern FileselFileType **filesel_type_image;
+extern FileselFileType **filesel_type_mainw;
+extern FileselFileType **filesel_type_any;
 
 /* Subclass off gtkfilesel2.c to make one of our fileselectors.
  */
@@ -118,6 +117,8 @@ typedef struct _FileselClass {
 	 */
 } FileselClass;
 
+void filesel_startup( void );
+
 gboolean is_file_type( const FileselFileType *type, const char *filename );
 
 typedef void *(*FileselMapFn)( Filesel *, const char *, void *, void * );
@@ -129,6 +130,7 @@ GtkWidget *filesel_new( void );
 
 gboolean filesel_set_filename( Filesel *filesel, const char *name );
 char *filesel_get_filename( Filesel *filesel );
+int filesel_nselected( Filesel *filesel );
 void *filesel_map_filename_multi( Filesel *filesel,
 	FileselMapFn fn, void *a, void *b );
 
@@ -137,6 +139,6 @@ void filesel_set_filetype( Filesel *filesel,
 	FileselFileType **type, int default_type );
 void filesel_set_filetype_pref( Filesel *filesel, const char *type_pref );
 int filesel_get_filetype( Filesel *filesel ); 
-void filesel_make_patt( FileselFileType *type, BufInfo *patt );
+void filesel_make_patt( FileselFileType *type, VipsBuf *patt );
 void filesel_set_flags( Filesel *filesel, gboolean imls, gboolean save );
 void filesel_set_multi( Filesel *filesel, gboolean multi );
