@@ -49,32 +49,28 @@ static GtkEventBoxClass *parent_class = NULL;
 
 static guint formula_signals[LAST_SIGNAL] = { 0 };
 
-static void
-formula_destroy( GtkObject *object )
-{
-	Formula *formula;
-
-#ifdef DEBUG
-	printf( "formula_destroy\n" );
-#endif /*DEBUG*/
-
-	g_return_if_fail( object != NULL );
-	g_return_if_fail( IS_FORMULA( object ) );
-
-	/* My instance destroy stuff.
-	 */
-	formula = FORMULA( object );
-
-	IM_FREE( formula->caption );
-	IM_FREE( formula->value );
-	IM_FREE( formula->expr );
-
-	GTK_OBJECT_CLASS( parent_class )->destroy( object );
-}
-
-/* Fwd ref.
+/* Formula needing a refresh.
  */
-static void formula_refresh( Formula *formula );
+static GSList *formula_refresh_all = NULL;
+
+/* The idle we add if there are any formula needing a refresh.
+ */
+static gint formula_refresh_idle = 0;
+
+/* Unqueue a refresh.
+ */
+static void
+formula_refresh_unqueue( Formula *formula )
+{
+	if( formula->refresh_queued ) {
+		formula_refresh_all = 
+			g_slist_remove( formula_refresh_all, formula );
+		formula->refresh_queued = FALSE;
+		
+		if( !formula_refresh_all )
+			IM_FREEF( g_source_remove, formula_refresh_idle );
+	}
+}
 
 /* Detect cancel in a text field.
  */
@@ -96,10 +92,9 @@ formula_key_press_event_cb( GtkWidget *widget, GdkEventKey *ev,
 			workspace, should stay in show formula).
 
 		 */
-		formula->edit = FALSE;
-		handled = TRUE;
+		formula_set_edit( formula, FALSE );
 
-		formula_refresh( formula );
+		handled = TRUE;
 	}
 
         return( handled );
@@ -167,6 +162,10 @@ formula_add_edit( Formula *formula )
 static void 
 formula_refresh( Formula *formula )
 {
+#ifdef DEBUG
+	printf( "formula_refresh\n" );
+#endif /*DEBUG*/
+
 	/* Set edit mode.
 	 */
 	if( formula->edit ) {
@@ -206,6 +205,67 @@ formula_refresh( Formula *formula )
 		 * mutiline for class members, for example.
 		 */
 		set_glabel1( formula->right_label, "%s", formula->value );
+
+	if( formula->edit && formula->needs_focus ) {
+		gtk_editable_select_region( 
+			GTK_EDITABLE( formula->entry ),
+			0, strlen( formula->expr ) );
+		gtk_widget_grab_focus( formula->entry );
+		formula->needs_focus = FALSE;
+	}
+}
+
+static gboolean
+formula_refresh_idle_cb( void )
+{
+	formula_refresh_idle = 0;
+
+	while( formula_refresh_all ) {
+		Formula *formula = FORMULA( formula_refresh_all->data );
+
+		formula_refresh_unqueue( formula );
+		formula_refresh( formula );
+	}
+
+	return( FALSE );
+}
+
+static void
+formula_refresh_queue( Formula *formula )
+{
+	if( !formula->refresh_queued ) {
+		formula_refresh_all = 
+			g_slist_prepend( formula_refresh_all, formula );
+		formula->refresh_queued = TRUE;
+		
+		if( !formula_refresh_idle )
+			formula_refresh_idle = g_idle_add( 
+				(GSourceFunc) formula_refresh_idle_cb, NULL );
+	}
+}
+
+static void
+formula_destroy( GtkObject *object )
+{
+	Formula *formula;
+
+#ifdef DEBUG
+	printf( "formula_destroy\n" );
+#endif /*DEBUG*/
+
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( IS_FORMULA( object ) );
+
+	/* My instance destroy stuff.
+	 */
+	formula = FORMULA( object );
+
+	formula_refresh_unqueue( formula );
+	IM_FREE( formula->caption );
+	IM_FREE( formula->value );
+	IM_FREE( formula->expr );
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
 /* Change edit mode.
@@ -213,9 +273,32 @@ formula_refresh( Formula *formula )
 void 
 formula_set_edit( Formula *formula, gboolean edit )
 {
+#ifdef DEBUG
+	printf( "formula_set_edit: %d\n", edit );
+#endif /*DEBUG*/
+
 	if( formula->edit != edit ) {
 		formula->edit = edit;
-		formula_refresh( formula );
+		formula_refresh_queue( formula );
+	}
+
+	/* Can't have been edited yet, whichever way we're turning edit.
+	 */
+	formula->changed = FALSE;
+}
+
+/* Grab focus on next refresh.
+ */
+void 
+formula_set_needs_focus( Formula *formula, gboolean needs_focus )
+{
+#ifdef DEBUG
+	printf( "formula_set_needs_focus: %d\n", needs_focus );
+#endif /*DEBUG*/
+
+	if( formula->needs_focus != needs_focus ) {
+		formula->needs_focus = needs_focus;
+		formula_refresh_queue( formula );
 	}
 }
 
@@ -224,10 +307,18 @@ formula_set_edit( Formula *formula, gboolean edit )
 void 
 formula_set_sensitive( Formula *formula, gboolean sensitive )
 {
-	formula->sensitive = sensitive;
+#ifdef DEBUG
+	printf( "formula_set_sensitive: %d\n", sensitive );
+#endif /*DEBUG*/
 
-	if( !formula->sensitive )
-		formula_set_edit( formula, FALSE );
+	if( formula->sensitive != sensitive ) {
+		formula->sensitive = sensitive;
+
+		if( !formula->sensitive )
+			formula_set_edit( formula, FALSE );
+
+		formula_refresh_queue( formula );
+	}
 }
 
 /* Re-read the text. TRUE if we saw a change.
@@ -237,6 +328,10 @@ formula_scan( Formula *formula )
 {
 	gboolean changed;
 
+#ifdef DEBUG
+	printf( "formula_scan\n" );
+#endif /*DEBUG*/
+
 	changed = FALSE;
 
 	/* Should be in edit mode.
@@ -313,7 +408,7 @@ formula_button_press_event( GtkWidget *widget, GdkEventButton *event )
 		if( event->button == 1 && formula->sensitive ) {
 			if( !formula->edit ) {
 				formula_set_edit( formula, TRUE );
-				gtk_widget_grab_focus( formula->entry );
+				formula_set_needs_focus( formula, TRUE );
 			}
 
 			handled = TRUE;
@@ -326,6 +421,10 @@ formula_button_press_event( GtkWidget *widget, GdkEventButton *event )
 static void
 formula_real_changed( Formula *formula )
 {
+#ifdef DEBUG
+	printf( "formula_real_changed\n" );
+#endif /*DEBUG*/
+
 	formula->changed = TRUE;
 }
 
@@ -390,10 +489,11 @@ static void
 formula_init( Formula *formula )
 {
 	/* How annoying! To avoid vertical resizes on edit/view toggles we
-	 * need to add differing amounts of padding to the label. 
-	 
+	 * need to add differing amounts of padding to the label depending on
+	 * the theme.
+
 	  	FIXME ... get this from the style somehow
-	
+
 	 */
 #ifdef OS_WIN32
 	/* with either wimp theme or gtk default.
@@ -411,6 +511,9 @@ formula_init( Formula *formula )
 	formula->edit = FALSE;
 	formula->sensitive = TRUE;
 	formula->changed = FALSE;
+	formula->refresh_queued = FALSE;
+	formula->needs_focus = FALSE;
+
 	formula->entry_frame = NULL;
 
 	formula->hbox = gtk_hbox_new( FALSE, 12 );
@@ -426,7 +529,7 @@ formula_init( Formula *formula )
 
         formula->right_label = gtk_label_new( "" );
         gtk_misc_set_alignment( GTK_MISC( formula->right_label ), 0, 0.5 );
-        gtk_misc_set_padding( GTK_MISC( formula->right_label ), 2, vpadding );
+        gtk_misc_set_padding( GTK_MISC( formula->right_label ), 7, vpadding );
 	gtk_box_pack_start( GTK_BOX( formula->hbox ), 
 		formula->right_label, TRUE, TRUE, 0 );
         gtk_widget_show( formula->right_label );
@@ -469,39 +572,32 @@ formula_set_caption( Formula *formula, const char *caption )
 {
 	if( !caption && formula->caption ) {
 		IM_FREE( formula->caption );
-		formula_refresh( formula );
+		formula_refresh_queue( formula );
 	}
 	else if( caption && (!formula->caption || 
 		strcmp( caption, formula->caption ) != 0) ) {
 		IM_SETSTR( formula->caption, caption );
-		formula_refresh( formula );
+		formula_refresh_queue( formula );
 	}
 }
 
 void
 formula_set_value_expr( Formula *formula, const char *value, const char *expr )
 {
-	gboolean changed;
-
 #ifdef DEBUG
 	printf( "formula_set_value_expr: value=\"%s\", expr=\"%s\"\n",
 		value, expr );
 #endif /*DEBUG*/
 
-	changed = FALSE;
-
 	if( value && (!formula->value || 
 		strcmp( value, formula->value ) != 0) ) {
 		IM_SETSTR( formula->value, value );
-		changed = TRUE;
+		formula_refresh_queue( formula );
 	}
 
 	if( expr && (!formula->expr || 
 		strcmp( expr, formula->expr ) != 0) ) {
 		IM_SETSTR( formula->expr, expr );
-		changed = TRUE;
+		formula_refresh_queue( formula );
 	}
-
-	if( changed )
-		formula_refresh( formula );
 }
