@@ -35,13 +35,13 @@
 
 /* Fwd ref.
  */
-static void lisp_pelement( BufInfo *buf, PElement *base, 
+static void lisp_pelement( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent );
 
 /* Print a sym-value list.
  */
 static void
-lisp_symval( BufInfo *buf, PElement *base, 
+lisp_symval( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent, PElement *stop )
 {
 	gboolean error = FALSE;
@@ -67,10 +67,10 @@ lisp_symval( BufInfo *buf, PElement *base,
 
 			PEPOINTLEFT( hn2, &pe );
 			if( !error && PEISSYMREF( &pe ) ) {
-				buf_appendf( buf, "\n%s", spc( indent ) );
+				vips_buf_appendf( buf, "\n%s", spc( indent ) );
 				symbol_qualified_name( 
 					PEGETSYMREF( &pe ), buf );
-				buf_appendf( buf, " = " );
+				vips_buf_appendf( buf, " = " );
 
 				PEPOINTRIGHT( hn2, &pe );
 				lisp_pelement( buf, &pe, 
@@ -89,7 +89,7 @@ lisp_symval( BufInfo *buf, PElement *base,
 		error = TRUE;
 
 	if( error )
-		buf_appendf( buf, "\n%s<*** malformed symval list>", 
+		vips_buf_appendf( buf, "\n%s<*** malformed symval list>", 
 			spc( indent ) );
 }
 
@@ -97,14 +97,14 @@ lisp_symval( BufInfo *buf, PElement *base,
  * element, so we print a ", " followed by us.
  */
 static void
-lisp_list( BufInfo *buf, PElement *base, 
+lisp_list( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
 	if( PEISNODE( base ) ) {
 		HeapNode *hn = PEGETVAL( base );
 		PElement pe;
 
-		buf_appends( buf, ", " );
+		vips_buf_appends( buf, ", " );
 
 		if( hn->type == TAG_CONS ) {
 			PEPOINTLEFT( hn, &pe );
@@ -124,7 +124,7 @@ lisp_list( BufInfo *buf, PElement *base,
  * element.
  */
 static void
-lisp_string( BufInfo *buf, PElement *base, 
+lisp_string( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
 	gboolean error = FALSE;
@@ -139,18 +139,18 @@ lisp_string( BufInfo *buf, PElement *base,
 		PEPOINTLEFT( hn, &pe );
 		if( !error ) {
 			if( PEISCHAR( &pe ) ) {
-				buf_appendf( buf, "%c", PEGETCHAR( &pe ) );
+				vips_buf_appendf( buf, "%c", PEGETCHAR( &pe ) );
 
 				PEPOINTRIGHT( hn, &pe );
 				lisp_string( buf, &pe, back, fn, indent );
 			}
 			else {
-				buf_appends( buf, "\":[" );
+				vips_buf_appends( buf, "\":[" );
 				lisp_pelement( buf, &pe, back, fn, indent );
 
 				PEPOINTRIGHT( hn, &pe );
 				lisp_list( buf, &pe, back, fn, indent );
-				buf_appends( buf, "]" );
+				vips_buf_appends( buf, "]" );
 
 				error = TRUE;
 			}
@@ -165,7 +165,7 @@ lisp_string( BufInfo *buf, PElement *base,
 /* Print a graph LISP-style.
  */
 static void
-lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
+lisp_node( VipsBuf *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 {
 	int i;
 	PElement p1, p2;
@@ -175,14 +175,14 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 	if( hn->flgs & FLAG_PRINT ) {
 		if( (i = g_slist_index( *back, hn )) == -1 ) {
 			*back = g_slist_prepend( *back, hn );
-			buf_appendf( buf, "<" ); 
-			buf_appendf( buf, _( "circular" ) ); 
-			buf_appendf( buf, " (%p)>", hn );
+			vips_buf_appendf( buf, "<" ); 
+			vips_buf_appendf( buf, _( "circular" ) ); 
+			vips_buf_appendf( buf, " (%p)>", hn );
 		}
 		else {
-			buf_appendf( buf, "<" );
-			buf_appendf( buf, _( "circular to label %d" ), i );
-			buf_appendf( buf, ">" );
+			vips_buf_appendf( buf, "<" );
+			vips_buf_appendf( buf, _( "circular to label %d" ), i );
+			vips_buf_appendf( buf, ">" );
 		}
 
 		return;
@@ -190,9 +190,9 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 	hn->flgs |= FLAG_PRINT;
 
 	if( (i = g_slist_index( *back, hn )) != -1 ) {
-		buf_appendf( buf, "*" );
-		buf_appendf( buf, _( "label %d" ), i );
-		buf_appendf( buf, ": " );
+		vips_buf_appendf( buf, "*" );
+		vips_buf_appendf( buf, _( "label %d" ), i );
+		vips_buf_appendf( buf, ": " );
 	}
 
 	switch( hn->type ) {
@@ -200,16 +200,16 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 		if( fn ) {
 			PEPOINTLEFT( hn, &p1 );
 			PEPOINTRIGHT( hn, &p2 );
-			buf_appends( buf, "(" );
+			vips_buf_appends( buf, "(" );
 			lisp_pelement( buf, &p1, back, fn, indent );
-			buf_appends( buf, " " );
+			vips_buf_appends( buf, " " );
 			lisp_pelement( buf, &p2, back, fn, indent );
-			buf_appends( buf, ")" );
+			vips_buf_appends( buf, ")" );
 		}
 		else {
-			buf_appends( buf, "<" );
-			buf_appends( buf, _( "unevaluated" ) );
-			buf_appends( buf, ">" );
+			vips_buf_appends( buf, "<" );
+			vips_buf_appends( buf, _( "unevaluated" ) );
+			vips_buf_appends( buf, ">" );
 		}
 
 		break;
@@ -217,34 +217,34 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 	case TAG_CONS:
 		PEPOINTLEFT( hn, &p1 );
 		if( PEISCHAR( &p1 ) ) {
-			buf_appendf( buf, "\"%c", PEGETCHAR( &p1 ) );
+			vips_buf_appendf( buf, "\"%c", PEGETCHAR( &p1 ) );
 			PEPOINTRIGHT( hn, &p2 );
 			lisp_string( buf, &p2, back, fn, indent );
-			buf_appends( buf, "\"" );
+			vips_buf_appends( buf, "\"" );
 		}
 		else {
-			buf_appends( buf, "[" );
+			vips_buf_appends( buf, "[" );
 			lisp_pelement( buf, &p1, back, fn, indent );
 			PEPOINTRIGHT( hn, &p2 );
 			lisp_list( buf, &p2, back, fn, indent );
-			buf_appends( buf, "]" );
+			vips_buf_appends( buf, "]" );
 		}
 		break;
 
 	case TAG_DOUBLE:
-		buf_appendf( buf, "%g", hn->body.num );
+		vips_buf_appendf( buf, "%g", hn->body.num );
 		break;
 
 	case TAG_COMPLEX:
-		buf_appendf( buf, "(%g,%g)", 
+		vips_buf_appendf( buf, "(%g,%g)", 
 			GETLEFT( hn )->body.num, GETRIGHT( hn )->body.num );
 		break;
 
 	case TAG_CLASS:
 		if( fn ) {
-			buf_appendf( buf, "\n%s", spc( indent ) );
-			buf_appendf( buf, _( "class (%p)" ), hn );
-			buf_appendf( buf, " " );
+			vips_buf_appendf( buf, "\n%s", spc( indent ) );
+			vips_buf_appendf( buf, _( "class (%p)" ), hn );
+			vips_buf_appendf( buf, " " );
 		}
 
 		PEPOINTLEFT( hn, &p1 );
@@ -253,22 +253,22 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 		if( fn ) {
 			hn = GETRIGHT( hn );
 
-			buf_appendf( buf, "\n%s", spc( indent + TAB ) ); 
-			buf_appendf( buf, _( "members" ) );
-			buf_appendf( buf, " = { " );
+			vips_buf_appendf( buf, "\n%s", spc( indent + TAB ) ); 
+			vips_buf_appendf( buf, _( "members" ) );
+			vips_buf_appendf( buf, " = { " );
 			PEPOINTRIGHT( hn, &p1 );
 			lisp_symval( buf, &p1, 
 				back, fn, indent + TAB * 2, NULL );
-			buf_appendf( buf, "\n%s}", spc( indent + TAB ) ); 
+			vips_buf_appendf( buf, "\n%s}", spc( indent + TAB ) ); 
 
 			PEPOINTLEFT( hn, &p2 );
 			if( *p1.type != *p2.type || *p1.ele != *p2.ele ) { 
-				buf_appendf( buf, "\n%s", spc( indent + TAB ) );
-				buf_appendf( buf, _( "secret" ) );
-				buf_appendf( buf, " = { " );
+				vips_buf_appendf( buf, "\n%s", spc( indent + TAB ) );
+				vips_buf_appendf( buf, _( "secret" ) );
+				vips_buf_appendf( buf, " = { " );
 				lisp_symval( buf, &p2, 
 					back, fn, indent + TAB * 2, &p1 );
-				buf_appendf( buf, 
+				vips_buf_appendf( buf, 
 					"\n%s} ", spc( indent + TAB ) ); 
 			}
 		}
@@ -276,39 +276,39 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 		break;
 
 	case TAG_GEN:
-		buf_appendf( buf, "[%g,%g...", 
+		vips_buf_appendf( buf, "[%g,%g...", 
 			GETLEFT( hn )->body.num, 
 			GETLEFT( GETRIGHT( hn ) )->body.num ); 
 		if( GETRT( GETRIGHT( hn ) ) == ELEMENT_ELIST )
-			buf_appends( buf, "[ ]]" );
+			vips_buf_appends( buf, "[ ]]" );
 		else
-			buf_appendf( buf, "%g]",
+			vips_buf_appendf( buf, "%g]",
 				GETRIGHT( GETRIGHT( hn ) )->body.num ); 
 		break;
 
 	case TAG_SHARED:
 		PEPOINTLEFT( hn, &p1 );
 		i = GPOINTER_TO_INT( GETRIGHT( hn ) );
-		buf_appendf( buf, "SHARE%d[", i );
+		vips_buf_appendf( buf, "SHARE%d[", i );
 		lisp_pelement( buf, &p1, back, fn, indent );
-		buf_appends( buf, "]" );
+		vips_buf_appends( buf, "]" );
 		break;
 
 	case TAG_REFERENCE:
 		i = GPOINTER_TO_INT( GETRIGHT( GETLEFT( hn ) ) );
-		buf_appendf( buf, "REF%d", i );
+		vips_buf_appendf( buf, "REF%d", i );
 		break;
 
 	case TAG_FREE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
 /* Print a pelement LISP-style.
  */
 static void
-lisp_pelement( BufInfo *buf, PElement *base, 
+lisp_pelement( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
 	HeapNode *hn;
@@ -316,99 +316,92 @@ lisp_pelement( BufInfo *buf, PElement *base,
 
 	switch( type ) {
 	case ELEMENT_NOVAL:
-		buf_appends( buf, "<" );
-		buf_appendf( buf, _( "no value (type %d)" ), 
+		vips_buf_appends( buf, "<" );
+		vips_buf_appendf( buf, _( "no value (type %d)" ), 
 			GPOINTER_TO_INT( PEGETVAL( base ) ) );
-		buf_appends( buf, ">" );
+		vips_buf_appends( buf, ">" );
 		break;
 
 	case ELEMENT_NODE:
 		if( !(hn = PEGETVAL( base )) ) {
-			buf_appends( buf, "<" );
-			buf_appends( buf, _( "NULL pointer" ) );
-			buf_appends( buf, ">" );
+			vips_buf_appends( buf, "<" );
+			vips_buf_appends( buf, _( "NULL pointer" ) );
+			vips_buf_appends( buf, ">" );
 		}
 		else
 			lisp_node( buf, hn, back, fn, indent );
 		break;
 
 	case ELEMENT_SYMBOL:
-		buf_appends( buf, "<" ); 
-		buf_appends( buf, _( "symbol" ) ); 
-		buf_appends( buf, " \"" ); 
+		vips_buf_appends( buf, "<" ); 
+		vips_buf_appends( buf, _( "symbol" ) ); 
+		vips_buf_appends( buf, " \"" ); 
 		symbol_qualified_name( PEGETSYMBOL( base ), buf );
-		buf_appends( buf, "\">" ); 
+		vips_buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_CONSTRUCTOR:
-		buf_appends( buf, "<" ); 
-		buf_appends( buf, _( "constructor" ) ); 
-		buf_appends( buf, " \"" ); 
+		vips_buf_appends( buf, "<" ); 
+		vips_buf_appends( buf, _( "constructor" ) ); 
+		vips_buf_appends( buf, " \"" ); 
 		symbol_qualified_name( PEGETCOMPILE( base )->sym, buf );
-		buf_appends( buf, "\">" ); 
+		vips_buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_SYMREF:
-		buf_appends( buf, "<" ); 
-		buf_appends( buf, _( "symref" ) ); 
-		buf_appends( buf, " \"" ); 
+		vips_buf_appends( buf, "<" ); 
+		vips_buf_appends( buf, _( "symref" ) ); 
+		vips_buf_appends( buf, " \"" ); 
 		symbol_qualified_name( PEGETSYMBOL( base ), buf );
-		buf_appends( buf, "\">" ); 
+		vips_buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_COMPILEREF:
-		buf_appends( buf, "<" ); 
-		buf_appends( buf, _( "compileref" ) ); 
-		buf_appends( buf, " \"" ); 
+		vips_buf_appends( buf, "<" ); 
+		vips_buf_appends( buf, _( "compileref" ) ); 
+		vips_buf_appends( buf, " \"" ); 
 		symbol_qualified_name( PEGETCOMPILE( base )->sym, buf );
-		buf_appends( buf, "\">" ); 
+		vips_buf_appends( buf, "\">" ); 
 		break;
 
 	case ELEMENT_CHAR:
-		buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
+		vips_buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
 		break;
 
 	case ELEMENT_BOOL:
-		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
+		vips_buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
 		break;
 
 	case ELEMENT_BINOP:
-		buf_appends( buf, decode_BinOp( PEGETBINOP( base ) ) );
+		vips_buf_appends( buf, decode_BinOp( PEGETBINOP( base ) ) );
 		break;
 
 	case ELEMENT_UNOP:
-		buf_appends( buf, decode_UnOp( PEGETUNOP( base ) ) );
+		vips_buf_appends( buf, decode_UnOp( PEGETUNOP( base ) ) );
 		break;
 
 	case ELEMENT_ELIST:
-		buf_appends( buf, "[ ]" ); 
+		vips_buf_appends( buf, "[ ]" ); 
 		break;
 
 	case ELEMENT_TAG:
-		buf_appendf( buf, "<" );
-		buf_appendf( buf, _( "tag \"%s\"" ), PEGETTAG( base ) );
-		buf_appendf( buf, ">" );
-		break;
-
-	case ELEMENT_STATIC:
-		buf_appendf( buf, "<" );
-		buf_appendf( buf, _( "static string \"%s\"" ), 
-			PEGETSTATIC( base )->text );
-		buf_appendf( buf, ">" );
+		vips_buf_appendf( buf, "<" );
+		vips_buf_appendf( buf, _( "tag \"%s\"" ), PEGETTAG( base ) );
+		vips_buf_appendf( buf, ">" );
 		break;
 
 	case ELEMENT_MANAGED:
-		buf_appendf( buf, "<Managed* %p>", PEGETVAL( base ) );
+		vips_buf_appendf( buf, "<Managed* %p>", PEGETVAL( base ) );
 		break;
 
 	case ELEMENT_COMB:
-		buf_appends( buf, decode_CombinatorType( PEGETCOMB( base ) ) );
+		vips_buf_appends( buf, decode_CombinatorType( PEGETCOMB( base ) ) );
 		break;
 
 	default:
-		buf_appendf( buf, "<" );
-		buf_appendf( buf, _( "unknown element tag %d" ), type );
-		buf_appendf( buf, ">" );
+		vips_buf_appendf( buf, "<" );
+		vips_buf_appendf( buf, _( "unknown element tag %d" ), type );
+		vips_buf_appendf( buf, ">" );
 		break;
 	}
 }
@@ -416,10 +409,10 @@ lisp_pelement( BufInfo *buf, PElement *base,
 /* Print a node to a buffer. If fn is true, trace into functions.
  */
 void
-graph_node( Heap *heap, BufInfo *buf, HeapNode *root, gboolean fn )
+graph_node( Heap *heap, VipsBuf *buf, HeapNode *root, gboolean fn )
 {
 	GSList *back;
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt[4];
 
 	/* May be called before heap is built.
@@ -428,7 +421,7 @@ graph_node( Heap *heap, BufInfo *buf, HeapNode *root, gboolean fn )
 		return;
 
 	back = NULL;
-	buf_init_static( &buf2, txt, 4 );
+	vips_buf_init_static( &buf2, txt, 4 );
 	heap_clear( heap, FLAG_PRINT );
 	lisp_node( &buf2, root, &back, fn, 0 );
 	heap_clear( heap, FLAG_PRINT );
@@ -439,10 +432,10 @@ graph_node( Heap *heap, BufInfo *buf, HeapNode *root, gboolean fn )
 /* As above, but start from a pelement.
  */
 void
-graph_pelement( Heap *heap, BufInfo *buf, PElement *root, gboolean fn )
+graph_pelement( Heap *heap, VipsBuf *buf, PElement *root, gboolean fn )
 {
 	GSList *back;
-	BufInfo buf2;
+	VipsBuf buf2;
 	char txt[4];
 
 	/* May be called before heap is built.
@@ -453,7 +446,7 @@ graph_pelement( Heap *heap, BufInfo *buf, PElement *root, gboolean fn )
 	/* We print twice ... the first time through we build the list of back
 	 * pointers so we can label the graph correctly.
 	 */
-	buf_init_static( &buf2, txt, 4 );
+	vips_buf_init_static( &buf2, txt, 4 );
 	back = NULL;
 
 	heap_clear( heap, FLAG_PRINT );
@@ -468,7 +461,7 @@ graph_pelement( Heap *heap, BufInfo *buf, PElement *root, gboolean fn )
 /* As above, but start from an element.
  */
 void
-graph_element( Heap *heap, BufInfo *buf, Element *root, gboolean fn )
+graph_element( Heap *heap, VipsBuf *buf, Element *root, gboolean fn )
 {
 	PElement base;
 
@@ -479,71 +472,12 @@ graph_element( Heap *heap, BufInfo *buf, Element *root, gboolean fn )
 void
 graph_pointer( PElement *root )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1000];
 
-	buf_init_static( &buf, txt, 1000 );
+	vips_buf_init_static( &buf, txt, 1000 );
 	graph_pelement( reduce_context->heap, &buf, root, TRUE );
-	printf( "%s\n", buf_all( &buf ) );
-}
-
-static void *
-save_image( Imageinfo *ii, char *filename )
-{
-#ifdef DEBUG
-	printf( "set_output: writing image to %s\n", filename );
-#endif /*DEBUG*/
-
-	if( !imageinfo_write( ii, NULL, filename ) ) 
-		return( ii );
-
-	increment_filename( filename );
-
-	return( NULL );
-}
-
-/* Write all images to files. Generate filenames with increment-file,
- * overwriting filename.
- */
-void *
-save_objects( PElement *base, char *filename )
-{
-	gboolean result;
-	Imageinfo *ii;
-
-	if( PEISIMAGE( base ) && (ii = PEGETII( base )) ) {
-		if( save_image( ii, filename ) )
-			return( base );
-	}
-	else if( PEISCLASS( base ) ) {
-		if( !heap_is_instanceof( CLASS_IMAGE, base, &result ) )
-			error( error_get_top() );
-		if( result ) {
-			if( !class_get_member_image( base, MEMBER_VALUE, &ii ) )
-				error( error_get_top() );
-			if( ii && save_image( ii, filename ) )
-				return( base );
-		}
-
-		if( !heap_is_instanceof( CLASS_MATRIX, base, &result ) )
-			error( error_get_top() );
-		if( result ) {
-			DOUBLEMASK *m;
-
-			if( !(m = matrix_ip_to_dmask( base )) ) 
-				error( error_get_top() );
-			if( im_write_dmask_name( m, filename ) ) 
-				error( error_get_top() );
-			increment_filename( filename );
-		}
-	}
-	else if( PEISLIST( base ) ) {
-		if( heap_map_list( base, 
-			(heap_map_list_fn) save_objects, filename, NULL ) )
-			return( base );
-	}
-
-	return( NULL );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 
 /* Fwd ref.
@@ -611,7 +545,7 @@ shell_node( HeapNode *hn )
 
 	case TAG_FREE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -643,10 +577,6 @@ shell_pelement( PElement *base )
 		printf( "%c", (int)PEGETCHAR( base ) );
 		break;
 
-	case ELEMENT_STATIC:
-		printf( "%s", PEGETSTATIC( base )->text );
-		break;
-
 	case ELEMENT_BOOL:
 		printf( "%s", bool_to_char( PEGETBOOL( base ) ) );
 		break;
@@ -661,7 +591,7 @@ shell_pelement( PElement *base )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
