@@ -56,14 +56,26 @@ group_finalize( GObject *gobject )
 }
 
 static gboolean
-group_save_list( PElement *list, GtkWidget *parent, char *filename );
+group_save_list( PElement *list, char *filename );
 
-static gboolean
-group_save_item( PElement *item, GtkWidget *parent, char *filename )
+/* Exported, since main.c uses this to save 'main' to a file.
+ */
+gboolean
+group_save_item( PElement *item, char *filename )
 {
 	gboolean result;
 	Imageinfo *ii;
 
+	if( !heap_is_instanceof( CLASS_GROUP, item, &result ) )
+		return( FALSE );
+	if( result ) {
+		PElement value;
+
+		if( !class_get_member( item, MEMBER_VALUE, NULL, &value ) ||
+			!group_save_list( &value, filename ) )
+			return( FALSE );
+	}
+
 	if( !heap_is_instanceof( CLASS_IMAGE, item, &result ) )
 		return( FALSE );
 	if( result ) {
@@ -71,8 +83,9 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 
 		if( !class_get_member( item, MEMBER_VALUE, NULL, &value ) || 
 			!heap_get_image( &value, &ii ) ||
-			!imageinfo_write( ii, parent, filename ) )
+			!imageinfo_write( ii, filename ) )
 			return( FALSE );
+		increment_filename( filename );
 	}
 
 	if( !heap_is_instanceof( CLASS_MATRIX, item, &result ) )
@@ -82,7 +95,6 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 
 		if( !(dmask = matrix_ip_to_dmask( item )) )
 			return( FALSE );
-
 		if( im_write_dmask_name( dmask, filename ) ) {
 			error_vips_all();
 			IM_FREEF( im_free_dmask, dmask );
@@ -90,16 +102,19 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 			return( FALSE );
 		}
 		IM_FREEF( im_free_dmask, dmask );
+
+		increment_filename( filename );
 	}
 
 	if( PEISIMAGE( item ) ) {
 		if( !heap_get_image( item, &ii ) ||
-			!imageinfo_write( ii, parent, filename ) )
+			!imageinfo_write( ii, filename ) )
 			return( FALSE );
+		increment_filename( filename );
 	}
 
 	if( PEISLIST( item ) ) {
-		if( !group_save_list( item, parent, filename ) )
+		if( !group_save_list( item, filename ) )
 			return( FALSE );
 	}
 
@@ -107,7 +122,7 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 }
 
 static gboolean
-group_save_list( PElement *list, GtkWidget *parent, char *filename )
+group_save_list( PElement *list, char *filename )
 {
 	int i;
 	int length;
@@ -119,12 +134,10 @@ group_save_list( PElement *list, GtkWidget *parent, char *filename )
 		PElement item;
 
 		if( !heap_list_index( list, i, &item ) ||
-			!group_save_item( &item, parent, filename ) )
+			!group_save_item( &item, filename ) )
 			return( FALSE );
-
-		increment_filename( filename );
 	}
-	
+
 	return( TRUE );
 }
 
@@ -136,17 +149,13 @@ group_graphic_save( Classmodel *classmodel,
 	Row *row = HEAPMODEL( group )->row;
 	PElement *root = &row->expr->root;
 	char buf[FILENAME_MAX];
-	PElement value;
 
 	/* We are going to increment the filename ... make sure there's some
 	 * space at the end of the string.
 	 */
 	im_strncpy( buf, filename, FILENAME_MAX - 5 );
 
-	if( !class_get_member( root, MEMBER_VALUE, NULL, &value ) )
-		return( FALSE );
-
-	if( !group_save_list( &value, parent, buf ) )
+	if( !group_save_item( root, buf ) )
 		return( FALSE );
 
 	return( TRUE );
