@@ -36,10 +36,6 @@
 #define DEBUG_HEAP_GC
  */
 
-/* Trace create/destroy of static strings.
-#define DEBUG_STATIC
- */
-
 /* Count GCs and %full, handy for tuning.
 #define DEBUG_GETMEM
  */
@@ -114,7 +110,7 @@ heap_map( HeapNode *hn, heap_map_fn fn, void *a, void *b )
 
 	case TAG_FREE:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 
 		/* Keep gcc happy.
 		 */
@@ -146,7 +142,7 @@ heap_check_free( Heap *heap )
 	/* Check free list.
 	 */
 	for( hn = heap->free; hn; hn = GETLEFT( hn ) ) {
-		assert( hn->type == TAG_FREE );
+		g_assert( hn->type == TAG_FREE );
 
 		hn->flgs |= FLAG_DEBUG;
 	}
@@ -159,7 +155,7 @@ heap_check_free( Heap *heap )
 		for( i = 0; i < hb->sz; i++ ) {
 			HeapNode *hn = &hb->node[i];
 
-			assert( hn->type != TAG_FREE || 
+			g_assert( hn->type != TAG_FREE || 
 				(hn->flgs & FLAG_DEBUG) );
 		}
 	}
@@ -216,6 +212,12 @@ heapblock_free( HeapBlock *hb )
 	IM_FREE( hb );
 }
 
+static void
+heap_set_flush( Heap *heap, gboolean flush )
+{
+	heap->flush = flush;
+}
+
 static void 
 heap_dispose_print( void *key, void *value )
 { 
@@ -233,6 +235,7 @@ heap_dispose( GObject *gobject )
 	 * closes, so we need to loop until done.
 	 */
 	managed_clear( heap );
+	heap_set_flush( heap, TRUE );
 	while( managed_free_unused( heap ) )
 		;
 
@@ -246,36 +249,11 @@ heap_dispose( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
 
-static void
-heap_static_string_free( HeapStaticString *string )
-{
-	assert( string->count == 0 );
-
-	IM_FREE( string->text );
-	heap_unregister_element( string->heap, &string->e );
-	IM_FREE( string );
-}
-
-static gboolean 
-heap_finalize_static_check( const char *text, HeapStaticString *string )
-{
-	assert( string->count == 0 );
-
-	heap_static_string_free( string );
-
-	return( TRUE );
-}
-
 static void
 heap_finalize( GObject *gobject )
 {
 	Heap *heap = HEAP( gobject );
 
-	if( heap->statics )
-		g_hash_table_foreach_remove( heap->statics, 
-			(GHRFunc) heap_finalize_static_check, NULL );
-	IM_FREEF( g_hash_table_destroy, heap->statics );
-
 	if( heap->hb )
 		heapblock_free( heap->hb );
 
@@ -291,30 +269,30 @@ heap_finalize( GObject *gobject )
 }
 
 static void
-heap_info( iObject *iobject, BufInfo *buf )
+heap_info( iObject *iobject, VipsBuf *buf )
 {
 	Heap *heap = HEAP( iobject );
 
-	buf_appendf( buf, "compile = " );
+	vips_buf_appendf( buf, "compile = " );
 	if( heap->compile )
 		if( heap->compile->sym ) {
 			symbol_qualified_name( heap->compile->sym, buf );
-			buf_appendf( buf, "(%p) (sym)\n", heap->compile->sym ); 
+			vips_buf_appendf( buf, "(%p) (sym)\n", heap->compile->sym ); 
 		}
 		else
-			buf_appendf( buf, "(compile, but no sym)\n" );
+			vips_buf_appendf( buf, "(compile, but no sym)\n" );
 	else
-		buf_appendf( buf, "(no compile)\n" );
-	buf_appendf( buf, "mxb (max blocks) = %d\n", heap->mxb );
-	buf_appendf( buf, "rsz (nodes per block) = %d\n", heap->rsz );
-	buf_appendf( buf, "nb (number of blocks) = %d\n", heap->nb );
-	buf_appendf( buf, "emark = %d pointers\n", 
+		vips_buf_appendf( buf, "(no compile)\n" );
+	vips_buf_appendf( buf, "mxb (max blocks) = %d\n", heap->mxb );
+	vips_buf_appendf( buf, "rsz (nodes per block) = %d\n", heap->rsz );
+	vips_buf_appendf( buf, "nb (number of blocks) = %d\n", heap->nb );
+	vips_buf_appendf( buf, "emark = %d pointers\n", 
 		g_hash_table_size( heap->emark ) );
-	buf_appendf( buf, "rmark = %d pointers\n", 
+	vips_buf_appendf( buf, "rmark = %d pointers\n", 
 		g_hash_table_size( heap->rmark ) );
-	buf_appendf( buf, "ncells (cells allocated) = %d\n", heap->ncells );
-	buf_appendf( buf, "nfree (cells free at last GC) = %d\n", heap->nfree );
-	buf_appendf( buf, "mtable (Managed blocks) = %d pointers\n", 
+	vips_buf_appendf( buf, "ncells (cells allocated) = %d\n", heap->ncells );
+	vips_buf_appendf( buf, "nfree (cells free at last GC) = %d\n", heap->nfree );
+	vips_buf_appendf( buf, "mtable (Managed blocks) = %d pointers\n", 
 		g_hash_table_size( heap->mtable ) );
 
 	IOBJECT_CLASS( parent_class )->info( iobject, buf );
@@ -417,10 +395,11 @@ heap_init( Heap *heap )
 	heap->emark = g_hash_table_new( NULL, g_direct_equal );
 	heap->rmark = g_hash_table_new( NULL, g_direct_equal );
 	heap->mtable = g_hash_table_new( NULL, g_direct_equal );
-	heap->statics = g_hash_table_new( g_str_hash, g_str_equal );
 
 	heap->gc_tid = 0;
 
+	heap->flush = FALSE;
+
 	heap_all = g_slist_prepend( heap_all, heap );
 }
 
@@ -578,7 +557,7 @@ heap_mark_tree( Heap *heap, HeapNode *hn )
 				break;
 
 			case TAG_FILE:
-				assert( GETLT( hn ) == ELEMENT_MANAGED );
+				g_assert( GETLT( hn ) == ELEMENT_MANAGED );
 				managed_mark( (Managed *) GETLEFT( hn ) );
 				break;
 
@@ -593,7 +572,7 @@ heap_mark_tree( Heap *heap, HeapNode *hn )
 
 			case TAG_FREE:
 			default:
-				assert( FALSE );
+				g_assert( FALSE );
 			}
 		}
 	}
@@ -642,24 +621,6 @@ mark_reduce( void *key, void *value, Heap *heap )
 	return( NULL );
 }
 
-/* Check for unreffed static strings.
- */
-static gboolean 
-heap_static_check( const char *text, HeapStaticString *string )
-{
-	if( string->count == 0 ) {
-#ifdef DEBUG_STATIC
-		printf( "heap_static_check: destroying \"%s\"\n", text );
-#endif /*DEBUG_STATIC*/
-
-		heap_static_string_free( string );
-
-		return( TRUE );
-	}
-	else
-		return( FALSE );
-}
-
 /* Do a garbage collect.
  */
 gboolean
@@ -685,11 +646,6 @@ heap_gc( Heap *heap )
 	 */
 	managed_clear( heap );
 
-	/* Free all unreffed static strings.
-	 */
-	g_hash_table_foreach_remove( heap->statics, 
-		(GHRFunc) heap_static_check, NULL );
-
 	/* All flags should be clear, so just mark.
 	 */
 	g_hash_table_foreach( heap->emark, (GHFunc) mark_element, heap );
@@ -855,14 +811,14 @@ heap_getmem( Heap *heap )
 	if( !heap->free ) {
 		error_top( _( "Heap full." ) );
 		if( heap->compile ) {
-			BufInfo buf;
+			VipsBuf buf;
 			char txt[100];
 
-			buf_init_static( &buf, txt, 100 );
+			vips_buf_init_static( &buf, txt, 100 );
 			compile_name( heap->compile, &buf );
 			error_sub( _( "The compile heap for %s has filled. "
 				"Make it smaller and less complicated." ),
-				buf_all( &buf ) );
+				vips_buf_all( &buf ) );
 		}
 		else
 			error_sub( _( "The main calculation heap has filled. "
@@ -958,6 +914,21 @@ heap_complex_new( Heap *heap, double rp, double ip, PElement *out )
 	return( TRUE );
 }
 
+/* 'get' a list: move the PE to point at the list.
+ */
+gboolean
+heap_get_list( PElement *list )
+{
+	g_assert( PEISLIST( list ) );
+
+	if( PEISMANAGEDSTRING( list ) ) {
+		if( !managedstring_get( PEGETMANAGEDSTRING( list ), list ) )
+			return( FALSE );
+	}
+
+	return( TRUE );
+}
+
 /* Set list to [].
  */
 void
@@ -1081,7 +1052,7 @@ heap_file_new( Heap *heap, const char *filename, PElement *out )
 	return( TRUE );
 }
 
-/* Make a string.
+/* Make a heap string.
  */
 gboolean
 heap_string_new( Heap *heap, const char *str, PElement *out )
@@ -1090,12 +1061,8 @@ heap_string_new( Heap *heap, const char *str, PElement *out )
 	const int n = strlen( str );
 	int i;
 
-	/* Make first RHS ... the end of the list. 
-	 */
 	heap_list_init( &list ); 
 
-	/* Build a CONS node for each element. 
-	 */
 	for( i = 0; i < n; i++ ) {
 		PElement t;
 
@@ -1108,6 +1075,25 @@ heap_string_new( Heap *heap, const char *str, PElement *out )
 	return( TRUE );
 }
 
+/* Make a managed string.
+ */
+gboolean
+heap_managedstring_new( Heap *heap, const char *str, PElement *out )
+{
+	Managedstring *managedstring;
+
+	if( strcmp( str, "" ) == 0 ) {
+		PEPUTP( out, ELEMENT_ELIST, NULL );
+	}
+	else {
+		if( !(managedstring = managedstring_find( heap, str )) )
+			return( FALSE );
+		PEPUTP( out, ELEMENT_MANAGED, managedstring );
+	}
+
+	return( TRUE );
+}
+
 /* Make a [[char]].
  */
 gboolean
@@ -1127,7 +1113,7 @@ heap_lstring_new( Heap *heap, GSList *labels, PElement *out )
 		PElement t;
 
 		if( !heap_list_add( heap, &list, &t ) ||
-			!heap_string_new( heap, 
+			!heap_managedstring_new( heap, 
 				g_slist_nth_data( labels, i ), &t ) )
 			return( FALSE );
 		(void) heap_list_next( &list );
@@ -1249,7 +1235,7 @@ heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b )
 		return( base );
 
 	if( !PEISLIST( &e ) ) {
-		heap_error_typecheck( &e, "heap_map_list", "list" );
+		heap_error_typecheck( &e, "heap_map_list", "[*]" );
 		return( base );
 	}
 
@@ -1257,6 +1243,9 @@ heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b )
 		PElement head;
 		void *res;
 
+		if( !heap_get_list( &e ) )
+			return( base );
+
 		/* Apply user function to the head.
 		 */
 		PEGETHD( &head, &e );
@@ -1285,7 +1274,12 @@ heap_get_list_next( PElement *list, PElement *data )
 		return( FALSE );
 
 	if( PEISFLIST( list ) ) {
-		HeapNode *hn = PEGETVAL( list );
+		HeapNode *hn;
+
+		if( !heap_get_list( list ) )
+			return( FALSE );
+
+		hn = PEGETVAL( list );
 
 		PEPOINTRIGHT( hn, list );
 		PEPOINTLEFT( hn, data );
@@ -1313,9 +1307,11 @@ heap_map_dict_entry( PElement *head, HeapMapDict *map_dict )
 	if( !reduce_pelement( rc, reduce_spine, head ) )
 		return( head );
 	if( !PEISFLIST( head ) ) {
-		heap_error_typecheck( head, "heap_map_dict", "list" );
+		heap_error_typecheck( head, "heap_map_dict", "[*]" );
 		return( head );
 	}
+	if( !heap_get_list( head ) )
+		return( head );
 	PEGETHD( &p1, head );
 	if( !heap_get_string( &p1, key, 256 ) )
 		return( head );
@@ -1324,9 +1320,11 @@ heap_map_dict_entry( PElement *head, HeapMapDict *map_dict )
 	if( !reduce_pelement( rc, reduce_spine, &p2 ) )
 		return( head );
 	if( !PEISFLIST( &p2 ) ) {
-		heap_error_typecheck( &p2, "heap_map_dict", "list" );
+		heap_error_typecheck( &p2, "heap_map_dict", "[*]" );
 		return( head );
 	}
+	if( !heap_get_list( &p2 ) )
+		return( head );
 	PEGETHD( &p1, &p2 );
 	if( (result = map_dict->fn( key, &p1, map_dict->a, map_dict->b )) )
 		return( result );
@@ -1774,7 +1772,7 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 	case ELEMENT_COMB:
 	case ELEMENT_TAG:
 	case ELEMENT_ELIST:
-	case ELEMENT_STATIC:
+	case ELEMENT_MANAGED:
 		/* Copy value.
 		 */
 		PEPUTP( out, root->type, root->ele );
@@ -1789,68 +1787,13 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 			return( FALSE );
 		break;
 
-	case ELEMENT_MANAGED:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( TRUE );
 }
 
-HeapStaticString *
-heap_static_string_new( Heap *heap, const char *text )
-{
-	HeapStaticString *string;
-
-	if( !(string = g_hash_table_lookup( heap->statics, text )) ) {
-		PElement pe;
-
-		if( !(string = INEW( NULL, HeapStaticString )) )
-			return( NULL );
-
-		string->heap = heap;
-		string->text = NULL;
-		string->count = 0;
-		string->e.type = ELEMENT_NOVAL;
-		string->e.ele = NULL;
-		heap_register_element( heap, &string->e );
-
-		PEPOINTE( &pe, &string->e );
-		if( !(string->text = im_strdup( NULL, text )) ||
-			!heap_string_new( heap, text, &pe ) ) {
-			heap_static_string_free( string );
-			return( NULL );
-		}
-
-#ifdef DEBUG_STATIC
-		printf( "heap_static_string_new: \"%s\"\n", text );
-#endif /*DEBUG_STATIC*/
-
-		g_hash_table_insert( heap->statics, string->text, string );
-	}
-
-	assert( string->count >= 0 );
-	string->count += 1;
-
-	return( string );
-}
-
-void *
-heap_static_string_unref( HeapStaticString *string )
-{
-	assert( g_hash_table_lookup( string->heap->statics, string->text ) ==
-		string );
-	assert( string->count >= 0 );
-
-	string->count -= 1;
-
-	/* Don't destroy on count == 0; GC frees statics for us next time it
-	 * runs.
-	 */
-
-	return( NULL );
-}
-
 /* Try to make a gvalue from a heap object. 
  */
 gboolean
@@ -1883,8 +1826,13 @@ heap_ip_to_gvalue( PElement *in, GValue *out )
 
 			if( !heap_get_string( in, name, 256 ) )
 				return( FALSE );
-			g_value_init( out, G_TYPE_STRING );
-			g_value_set_string( out, name );
+
+			/* We want a refstring, not a G_TYPE_STRING, since
+			 * this GValue will (probably) be used by vips with
+			 * im_header_string() etc.
+			 */
+			g_value_init( out, IM_TYPE_REF_STRING );
+			im_ref_string_set( out, name );
 		}
 		else {
 			error_top( _( "Unimplemented list type." ) );
@@ -1956,7 +1904,8 @@ heap_gvalue_to_ip( GValue *in, PElement *out )
 			return( FALSE );
 	}
 	else if( G_VALUE_HOLDS_STRING( in ) ) {
-		if( !heap_string_new( heap, g_value_get_string( in ), out ) )
+		if( !heap_managedstring_new( heap, 
+			g_value_get_string( in ), out ) )
 			return( FALSE );
 	}
 	else if( G_VALUE_HOLDS_OBJECT( in ) ) {
@@ -1967,6 +1916,19 @@ heap_gvalue_to_ip( GValue *in, PElement *out )
 		managed = MANAGED( managedgobject_new( heap, object ) );
 		PEPUTP( out, ELEMENT_MANAGED, managed );
 	}
+	else if( g_value_type_transformable( G_VALUE_TYPE( in ), 
+		G_TYPE_STRING ) ) {
+		GValue temp = { 0 };
+
+		g_value_init( &temp, G_TYPE_STRING );
+		g_value_transform( in, &temp );
+		if( !heap_managedstring_new( heap, 
+			g_value_get_string( &temp ), out ) ) {
+			return( FALSE );
+			g_value_unset( &temp );
+		}
+		g_value_unset( &temp );
+	}
 	else {
 		error_top( _( "Unimplemented type." ) );
 		error_sub( _( "Unable to convert %s to a nip type." ), 
