@@ -93,10 +93,10 @@ idialog_popdown_notify( void *sys, iWindowResult result )
         iWindowSusp *susp = IWINDOW_SUSP( sys );
 	iDialog *idlg = IDIALOG( susp->client );
 
-	if( result == IWINDOW_TRUE ) 
+	if( result == IWINDOW_YES ) 
 		/* If our caller hasn't been notified yet, post off a FALSE.
 		 */
-		idialog_notify_parent( idlg, IWINDOW_FALSE );
+		idialog_notify_parent( idlg, IWINDOW_NO );
 
 	/* Pass result on to our suspension (ie. back to iwindow).
 	 */
@@ -141,7 +141,7 @@ idialog_done_notify( void *sys, iWindowResult result )
 
 	/* If all ok, popdown and tell our parent.
 	 */
-	if( result == IWINDOW_TRUE ) {
+	if( result == IWINDOW_YES ) {
 		/* Unless we're pinned up, that is.
 		 */
 		if( !(idlg->tog_pin && 
@@ -177,7 +177,7 @@ idialog_done_trigger( iDialog *idlg, int pos )
 
 	/* Trigger user done callback.
 	 */
-	assert( fn );
+	g_assert( fn );
 	idialog_set_button_state( idlg, FALSE );
 	iwindow_notify_send( IWINDOW( idlg ), 
 		fn, idlg->client, idialog_done_notify, idlg );
@@ -198,8 +198,8 @@ idialog_cancel_notify( void *sys, iWindowResult result )
 
 	/* Send cancel message back to parent if our client cancel was OK.
 	 */
-	if( result == IWINDOW_TRUE ) {
-		idialog_notify_parent( idlg, IWINDOW_FALSE );
+	if( result == IWINDOW_YES ) {
+		idialog_notify_parent( idlg, IWINDOW_NO );
 		iwindow_kill( IWINDOW( idlg ) );
 	}
 
@@ -234,7 +234,7 @@ idialog_done_cb( GtkWidget *w, iDialog *idlg )
 {
 	int pos = g_slist_index( idlg->ok_l, w );
 
-	assert( pos != -1 );
+	g_assert( pos != -1 );
 
 	idialog_done_trigger( idlg, pos );
 }
