@@ -51,7 +51,7 @@ iimage_dispose( GObject *gobject )
 
 	slist_map( iimage->classmodels, 
 		(SListMapFn) classmodel_iimage_unlink, iimage );
-	assert( !iimage->classmodels );
+	g_assert( !iimage->classmodels );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -72,7 +72,7 @@ iimage_finalize( GObject *gobject )
 
 	image_value_destroy( &iimage->value );
 	IM_FREEF( g_slist_free, iimage->views );
-	buf_destroy( &iimage->caption_buffer );
+	vips_buf_destroy( &iimage->caption_buffer );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
@@ -84,22 +84,22 @@ iimage_generate_caption( iObject *iobject )
 {
 	iImage *iimage = IIMAGE( iobject );
 	Imageinfo *ii = iimage->value.ii;
-	BufInfo *buf = &iimage->caption_buffer;
+	VipsBuf *buf = &iimage->caption_buffer;
 
-	buf_rewind( buf );
+	vips_buf_rewind( buf );
 
 	image_value_caption( &iimage->value, buf );
 
 	if( ii ) {
-		buf_appends( buf, ", " );
+		vips_buf_appends( buf, ", " );
 		iobject_info( IOBJECT( iimage->value.ii ), buf );
 	}
 
-	return( buf_all( buf ) );
+	return( vips_buf_all( buf ) );
 }
 
 static void
-iimage_info( iObject *iobject, BufInfo *buf )
+iimage_info( iObject *iobject, VipsBuf *buf )
 {
 	iImage *iimage = IIMAGE( iobject );
 	Imageinfo *ii = iimage->value.ii;
@@ -111,8 +111,8 @@ iimage_info( iObject *iobject, BufInfo *buf )
 		if( im_header_get_type( im, ORIGINAL_FILENAME ) != 0 ) {
 			if( !im_header_string( im, 
 				ORIGINAL_FILENAME, &filename ) ) {
-				buf_appends( buf, _( "Original filename" ) );
-				buf_appendf( buf, ": %s\n", filename );
+				vips_buf_appends( buf, _( "Original filename" ) );
+				vips_buf_appendf( buf, ": %s\n", filename );
 			}
 		}
 	}
@@ -150,19 +150,19 @@ iimage_save( Model *model, xmlNode *xnode )
 	if( !set_prop( xthis, "image_left", "%d", iimage->image_left ) ||
 		!set_prop( xthis, "image_top", "%d", iimage->image_top ) ||
 		!set_prop( xthis, "image_mag", "%d", iimage->image_mag ) ||
-		!set_prop( xthis, "show_status",
+		!set_sprop( xthis, "show_status",
 			bool_to_char( iimage->show_status ) ) ||
-		!set_prop( xthis, "show_paintbox",
+		!set_sprop( xthis, "show_paintbox",
 			bool_to_char( iimage->show_paintbox ) ) ||
-		!set_prop( xthis, "show_convert",
+		!set_sprop( xthis, "show_convert",
 			bool_to_char( iimage->show_convert ) ) ||
-		!set_prop( xthis, "show_rulers",
+		!set_sprop( xthis, "show_rulers",
 			bool_to_char( iimage->show_rulers ) ) ||
 		!set_dprop( xthis, "scale", iimage->scale ) ||
 		!set_dprop( xthis, "offset", iimage->offset ) ||
-		!set_prop( xthis, "falsecolour", "%s",
+		!set_sprop( xthis, "falsecolour", 
 			bool_to_char( iimage->falsecolour ) ) ||
-		!set_prop( xthis, "type", "%s", bool_to_char( iimage->type ) ) )
+		!set_sprop( xthis, "type", bool_to_char( iimage->type ) ) )
 		return( NULL );
 
 	return( xthis );
@@ -174,7 +174,7 @@ iimage_load( Model *model,
 {
         iImage *iimage = IIMAGE( model );
 
-	assert( IS_RHS( parent ) );
+	g_assert( IS_RHS( parent ) );
 
 	(void) get_iprop( xnode, "image_left", &iimage->image_left );
 	(void) get_iprop( xnode, "image_top", &iimage->image_top );
@@ -325,7 +325,7 @@ iimage_graphic_save( Classmodel *classmodel,
 	g_timer_reset( timer );
 
 	if( value->ii )
-		if( !imageinfo_write( value->ii, parent, buf ) )
+		if( !imageinfo_write( value->ii, buf ) )
 			return( FALSE );
 
 	mainw_recent_add( &mainw_recent_image, filename );
@@ -349,14 +349,14 @@ iimage_replace( iImage *iimage, const char *filename )
 	Row *row = HEAPMODEL( iimage )->row;
 	iText *itext = ITEXT( HEAPMODEL( iimage )->rhs->itext );
 	char txt[MAX_STRSIZE];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Image_file \"" );
-	buf_appendsc( &buf, TRUE, filename );
-	buf_appends( &buf, "\"" );
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_appends( &buf, "Image_file \"" );
+	vips_buf_appendsc( &buf, TRUE, filename );
+	vips_buf_appends( &buf, "\"" );
 
-	if( itext_set_formula( itext, buf_all( &buf ) ) ) {
+	if( itext_set_formula( itext, vips_buf_all( &buf ) ) ) {
 		itext_set_edited( itext, TRUE );
 		(void) expr_dirty( row->expr, link_serial_new() );
 
@@ -439,7 +439,7 @@ iimage_init( iImage *iimage )
 	iimage->falsecolour = FALSE;
 	iimage->type = TRUE;
 
-	buf_init_dynamic( &iimage->caption_buffer, MAX_LINELENGTH );
+	vips_buf_init_dynamic( &iimage->caption_buffer, MAX_LINELENGTH );
 
 	iobject_set( IOBJECT( iimage ), CLASS_IMAGE, NULL );
 }
