@@ -58,6 +58,22 @@ iimageview_realize( GtkWidget *widget )
 	set_symbol_drag_type( widget );
 }
 
+GtkWidget *
+iimageview_drag_window_new( int width, int height )
+{
+	GtkWidget *window;
+
+	window = gtk_window_new( GTK_WINDOW_POPUP );
+	gtk_widget_set_app_paintable( GTK_WIDGET( window ), TRUE );
+	gtk_widget_set_size_request( window, width, height );
+	gtk_widget_realize( window );
+#ifdef HAVE_SET_OPACITY
+	gdk_window_set_opacity( window->window, 0.5 );
+#endif /*HAVE_SET_OPACITY*/
+
+	return( window );
+}
+
 static void
 iimageview_drag_begin( GtkWidget *widget, GdkDragContext *context )
 {
@@ -70,19 +86,14 @@ iimageview_drag_begin( GtkWidget *widget, GdkDragContext *context )
 	printf( "iimageview_drag_begin: \n" );
 #endif /*DEBUG*/
 
-	window = gtk_window_new( GTK_WINDOW_POPUP );
-	gtk_widget_set_app_paintable( GTK_WIDGET( window ), TRUE );
-	gtk_widget_set_size_request( window, 
+	window = iimageview_drag_window_new( 
 		conv->canvas.width, conv->canvas.height );
-	gtk_widget_realize( window );
 	gtk_object_set_data_full( GTK_OBJECT( widget ),
-		"iimageview-drag-window", window,
+		"nip2-drag-window", window,
 		(GtkDestroyNotify) gtk_widget_destroy );
-
 	id = imagedisplay_new( conv );
 	gtk_container_add( GTK_CONTAINER( window ), GTK_WIDGET( id ) );
 	gtk_widget_show( GTK_WIDGET( id ) );
-
 	gtk_drag_set_icon_widget( context, window, -2, -2 );
 }
 
@@ -94,7 +105,7 @@ iimageview_drag_end( GtkWidget *widget, GdkDragContext *context )
 #endif /*DEBUG*/
 
 	gtk_object_set_data( GTK_OBJECT( widget ), 
-		"iimageview-drag-window", NULL );
+		"nip2-drag-window", NULL );
 }
 
 static void
@@ -109,17 +120,17 @@ iimageview_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 		iImageview *iimageview = IIMAGEVIEW( widget );
 		iImage *iimage = IIMAGE( VOBJECT( iimageview )->iobject );
 		Row *row = HEAPMODEL( iimage )->row;
-		char buf_text[256];
-		BufInfo buf;
+		char vips_buf_text[256];
+		VipsBuf buf;
 
 		/* Drag the fully-qualified row name.
 		 */
-		buf_init_static( &buf, buf_text, 256 );
+		vips_buf_init_static( &buf, vips_buf_text, 256 );
 		row_qualified_name_relative( main_workspacegroup->sym, 
 			row, &buf );
 		gtk_selection_data_set( selection_data,
 			gdk_atom_intern( "text/symbol", FALSE ), 8, 
-			(guchar *) buf_all( &buf ), strlen( buf_all( &buf ) ) );
+			(guchar *) vips_buf_all( &buf ), strlen( vips_buf_all( &buf ) ) );
 	}
 }
 
@@ -152,17 +163,17 @@ iimageview_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 			from_row_path )) && 
 			from_row != row ) {
 			iText *itext = ITEXT( HEAPMODEL( iimage )->rhs->itext );
-			char buf_text[256];
-			BufInfo buf;
+			char vips_buf_text[256];
+			VipsBuf buf;
 
 			/* Qualify relative to us. We don't want to embed
 			 * workspace names unless we have to.
 			 */
-			buf_init_static( &buf, buf_text, 256 );
+			vips_buf_init_static( &buf, vips_buf_text, 256 );
 			row_qualified_name_relative( row->top_row->sym, 
 				from_row, &buf );
 
-			if( itext_set_formula( itext, buf_all( &buf ) ) ) {
+			if( itext_set_formula( itext, vips_buf_all( &buf ) ) ) {
 				itext_set_edited( itext, TRUE );
 				(void) expr_dirty( row->expr, 
 					link_serial_new() );
@@ -321,20 +332,20 @@ iimageview_filedrop( iImageview *iimageview, const char *file )
 
 static void
 iimageview_tooltip_generate( GtkWidget *widget, 
-	BufInfo *buf, iImageview *iimageview )
+	VipsBuf *buf, iImageview *iimageview )
 {
 	iImage *iimage = IIMAGE( VOBJECT( iimageview )->iobject );
 	Imageinfo *ii = iimage->value.ii;
 	IMAGE *im = imageinfo_get( FALSE, ii );
 
-	buf_rewind( buf );
-	buf_appends( buf, buf_all( &iimage->caption_buffer ) );
+	vips_buf_rewind( buf );
+	vips_buf_appends( buf, vips_buf_all( &iimage->caption_buffer ) );
 	if( im ) {
 		double size = (double) im->Ysize * IM_IMAGE_SIZEOF_LINE( im );
 
-		buf_appends( buf, ", " );
+		vips_buf_appends( buf, ", " );
 		to_size( buf, size );
-		buf_appendf( buf, ", %.3gx%.3g p/mm", im->Xres, im->Yres );
+		vips_buf_appendf( buf, ", %.3gx%.3g p/mm", im->Xres, im->Yres );
 	}
 }
 
@@ -356,7 +367,6 @@ iimageview_init( iImageview *iimageview )
 
 	iimageview->conv = conversion_new( NULL );
 	iimageview->conv->tile_size = 16;
-	iimageview->conv->fade_steps = 1;
         iimageview->id = imagedisplay_new( iimageview->conv );
 	imagedisplay_set_shrink_to_fit( iimageview->id, TRUE );
         gtk_box_pack_start( GTK_BOX( vbox ), 
