@@ -31,7 +31,8 @@
 
 jobs:
 
-- reference counting layer ... in Managed base class
+- reference counting layer ... in Managed base class, plus links to heap 
+  garbage collection
 
 - filesystem tracking: we stat open files and signal file_changed if we see a 
   change
@@ -57,10 +58,6 @@ jobs:
 
 - paint stuff: also undo/redo buffers, each with a "*_changed" signal
 
-every Imageinfo is associated with a heap: when we GC a heap, we
-imageinfo_mark() any references to images inside that heap; after GC we
-imageinfo_destroy_heap() any unreferenced imageinfo
-
  */
 
 /* 
@@ -79,20 +76,17 @@ most of the jobs above are pushed down into vips8 now ... except for
 - cache: several open( "fred.v" )s share a single Imageinfo, provided their 
   mtimes are all the same
 
-
-- 
-
  */
 
 
 #include "ip.h"
 
 /*
-#define DEBUG_OPEN
 #define DEBUG
-#define DEBUG_RGB
 #define DEBUG_CHECK
 #define DEBUG_MAKE
+#define DEBUG_OPEN
+#define DEBUG_RGB
  */
 
 static iContainerClass *imageinfogroup_parent_class = NULL;
@@ -135,7 +129,7 @@ imageinfogroup_child_remove( iContainer *parent, iContainer *child )
 
 	hits = (GSList *) g_hash_table_lookup( imageinfogroup->filename_hash,
 		name );
-	assert( hits );
+	g_assert( hits );
 	hits = g_slist_remove( hits, imageinfo );
 
 	/* child is going away (probably), so we don't want to link hits back
@@ -252,6 +246,7 @@ enum {
 	SIG_AREA_PAINTED,	/* Area of image has been painted */
 	SIG_UNDO_CHANGED,	/* Undo/redo state has changed */
 	SIG_FILE_CHANGED,	/* Underlying file seems to have changed */
+	SIG_INVALIDATE,		/* IMAGE* has been invalidated */
 	SIG_LAST
 };
 
@@ -274,7 +269,7 @@ imageinfo_print( Imageinfo *imageinfo )
 void *
 imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty )
 {
-	assert( IS_IMAGEINFO( imageinfo ) );
+	g_assert( IS_IMAGEINFO( imageinfo ) );
 
 #ifdef DEBUG
 	printf( "imageinfo_area_changed: left = %d, top = %d, "
@@ -291,7 +286,7 @@ imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty )
 void *
 imageinfo_area_painted( Imageinfo *imageinfo, Rect *dirty )
 {
-	assert( IS_IMAGEINFO( imageinfo ) );
+	g_assert( IS_IMAGEINFO( imageinfo ) );
 
 #ifdef DEBUG
 	printf( "imageinfo_area_painted: left = %d, top = %d, "
@@ -308,7 +303,7 @@ imageinfo_area_painted( Imageinfo *imageinfo, Rect *dirty )
 static void *
 imageinfo_undo_changed( Imageinfo *imageinfo )
 {
-	assert( IS_IMAGEINFO( imageinfo ) );
+	g_assert( IS_IMAGEINFO( imageinfo ) );
 
 	g_signal_emit( G_OBJECT( imageinfo ), 
 		imageinfo_signals[SIG_UNDO_CHANGED], 0 );
@@ -319,7 +314,7 @@ imageinfo_undo_changed( Imageinfo *imageinfo )
 static void *
 imageinfo_file_changed( Imageinfo *imageinfo )
 {
-	assert( IS_IMAGEINFO( imageinfo ) );
+	g_assert( IS_IMAGEINFO( imageinfo ) );
 
 #ifdef DEBUG_CHECK
 	printf( "imageinfo_file_changed:" );
@@ -332,6 +327,22 @@ imageinfo_file_changed( Imageinfo *imageinfo )
 	return( NULL );
 }
 
+static void *
+imageinfo_invalidate( Imageinfo *imageinfo )
+{
+	g_assert( IS_IMAGEINFO( imageinfo ) );
+
+#ifdef DEBUG_CHECK
+	printf( "imageinfo_invalidate:" );
+	imageinfo_print( imageinfo );
+#endif /*DEBUG_CHECK*/
+
+	g_signal_emit( G_OBJECT( imageinfo ), 
+		imageinfo_signals[SIG_INVALIDATE], 0 );
+
+	return( NULL );
+}
+
 void
 imageinfo_expr_add( Imageinfo *imageinfo, Expr *expr )
 {
@@ -341,8 +352,8 @@ imageinfo_expr_add( Imageinfo *imageinfo, Expr *expr )
 	printf( "has imageinfo \"%s\" as value\n", imageinfo->im->filename );
 #endif /*DEBUG*/
 
-	assert( !g_slist_find( imageinfo->exprs, expr ) );
-	assert( !expr->imageinfo );
+	g_assert( !g_slist_find( imageinfo->exprs, expr ) );
+	g_assert( !expr->imageinfo );
 
 	expr->imageinfo = imageinfo;
 	imageinfo->exprs = g_slist_prepend( imageinfo->exprs, expr );
@@ -358,9 +369,9 @@ imageinfo_expr_remove( Expr *expr, Imageinfo *imageinfo )
 		imageinfo->im->filename );
 #endif /*DEBUG*/
 
-	assert( expr->imageinfo );
-	assert( g_slist_find( imageinfo->exprs, expr ) );
-	assert( expr->imageinfo == imageinfo );
+	g_assert( expr->imageinfo );
+	g_assert( g_slist_find( imageinfo->exprs, expr ) );
+	g_assert( expr->imageinfo == imageinfo );
 
 	expr->imageinfo = NULL;
 	imageinfo->exprs = g_slist_remove( imageinfo->exprs, expr );
@@ -419,26 +430,9 @@ imageinfo_undo_free( Imageinfo *imageinfo )
 	IM_FREEF( imageinfo_undobuffer_free, imageinfo->cundo );
 }
 
-static void
-imageinfo_set_eval_parent( Imageinfo *imageinfo, GtkWidget *eval_parent )
-{
-	if( imageinfo->eval_parent ) {
-		g_object_remove_weak_pointer( 
-			G_OBJECT( imageinfo->eval_parent ), 
-			(gpointer *) &imageinfo->eval_parent );
-		imageinfo->eval_parent = NULL;
-	}
-	imageinfo->eval_parent = eval_parent;
-	if( imageinfo->eval_parent ) 
-		g_object_add_weak_pointer( G_OBJECT( eval_parent ), 
-			(gpointer *) &imageinfo->eval_parent );
-}
-
 static void
 imageinfo_dispose_eval( Imageinfo *imageinfo )
 {
-	DESTROY_GTK( imageinfo->eval_progress );
-	imageinfo_set_eval_parent( imageinfo, NULL );
 	imageinfo->monitored = FALSE;
 
 	/* Make sure any callbacks from the IMAGE stop working.
@@ -461,7 +455,7 @@ imageinfo_dispose( GObject *gobject )
 
 	slist_map( imageinfo->exprs, 
 		(SListMapFn) imageinfo_expr_remove, imageinfo );
-	assert( !imageinfo->exprs );
+	g_assert( !imageinfo->exprs );
 
 	imageinfo_dispose_eval( imageinfo );
 
@@ -516,11 +510,11 @@ imageinfo_finalize( GObject *gobject )
 /* Make an info string about an imageinfo.
  */
 static void
-imageinfo_info( iObject *iobject, BufInfo *buf )
+imageinfo_info( iObject *iobject, VipsBuf *buf )
 {
 	Imageinfo *imageinfo = IMAGEINFO( iobject );
 
-	buf_appendi( buf, imageinfo_get( FALSE, imageinfo ) );
+	vips_buf_appendi( buf, imageinfo_get( FALSE, imageinfo ) );
 
 	/* Don't chain up to parent->info(), we don't want all the other
 	 * stuff, this is going to be used for a caption.
@@ -551,11 +545,17 @@ imageinfo_real_file_changed( Imageinfo *imageinfo )
 {
 }
 
+static void
+imageinfo_real_invalidate( Imageinfo *imageinfo )
+{
+}
+
 static void
 imageinfo_class_init( ImageinfoClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	iObjectClass *iobject_class = IOBJECT_CLASS( class );
+	ManagedClass *managed_class = MANAGED_CLASS( class );
 
 	parent_class = g_type_class_peek_parent( class );
 
@@ -564,10 +564,15 @@ imageinfo_class_init( ImageinfoClass *class )
 
 	iobject_class->info = imageinfo_info;
 
+	/* Timeout on unreffed images.
+	 */
+	managed_class->keepalive = 60.0;
+
 	class->area_changed = imageinfo_real_area_changed;
 	class->area_painted = imageinfo_real_area_painted;
 	class->undo_changed = imageinfo_real_undo_changed;
 	class->file_changed = imageinfo_real_file_changed;
+	class->invalidate = imageinfo_real_invalidate;
 
 	/* Create signals.
 	 */
@@ -601,6 +606,13 @@ imageinfo_class_init( ImageinfoClass *class )
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0 );
+	imageinfo_signals[SIG_INVALIDATE] = g_signal_new( "invalidate",
+		G_OBJECT_CLASS_TYPE( gobject_class ),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET( ImageinfoClass, invalidate ),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0 );
 }
 
 static void
@@ -626,7 +638,6 @@ imageinfo_init( Imageinfo *imageinfo )
 	imageinfo->cundo = NULL;
 
 	imageinfo->monitored = FALSE;
-	imageinfo->eval_progress = NULL;
 
 	imageinfo->check_mtime = 0;
 	imageinfo->check_tid = 0;
@@ -658,78 +669,33 @@ imageinfo_get_type( void )
 }
 
 static int
-imageinfo_progress_start( ImageinfoIMAGE *proxy )
+imageinfo_proxy_eval( Imageinfoproxy *proxy )
 {
 	Imageinfo *imageinfo = proxy->imageinfo;
 
-	if( imageinfo )
-		/* Note the start time.
-		 */
-		imageinfo->eval_last = 
-			g_timer_elapsed( imageinfo->im->time->start, NULL );
-
-	return( 0 );
-}
-
-static int
-imageinfo_progress_eval( ImageinfoIMAGE *proxy )
-{
-	const double update_threshold = 0.2;
-	Imageinfo *imageinfo = proxy->imageinfo;
-
-	if( imageinfo ) {
-		double latest = 
-			g_timer_elapsed( imageinfo->im->time->start, NULL );
-		double elapsed = latest - imageinfo->eval_last;
-
-		if( imageinfo->im->time->eta > 1 &&
-			!imageinfo->eval_progress ) {
-			imageinfo->eval_progress = progress_new( imageinfo );
-			iwindow_set_parent( IWINDOW( imageinfo->eval_progress ),
-				imageinfo->eval_parent );
-			idialog_set_iobject( 
-				IDIALOG( imageinfo->eval_progress ), 
-				IOBJECT( imageinfo ) );
-			iwindow_build( IWINDOW( imageinfo->eval_progress ) );
-			gtk_widget_show( 
-				GTK_WIDGET( imageinfo->eval_progress ) );
-		}
-
-		if( elapsed > update_threshold ) {
-			if( imageinfo->eval_progress ) 
-				progress_update( imageinfo->eval_progress,
-					imageinfo->im->time->percent,
-					imageinfo->im->time->eta );
-
-			while( g_main_context_iteration( NULL, FALSE ) )
-				;
-
-			if( imageinfo->eval_progress &&
-				imageinfo->eval_progress->cancelled )
-				imageinfo->im->kill = 1;
-
-			imageinfo->eval_last = latest;
-		}
-	}
-
-	return( 0 );
-}
-
-static int
-imageinfo_progress_stop( ImageinfoIMAGE *proxy )
-{
-	Imageinfo *imageinfo = proxy->imageinfo;
-
-	/* Pop down the dialog.
-	 */
 	if( imageinfo ) 
-		DESTROY_GTK( imageinfo->eval_progress );
+		if( progress_update_percent( imageinfo->im->time->percent,
+			imageinfo->im->time->eta ) ) {
+			im_error( "nip2", _( "User cancelled operation" ) );
+			return( -1 );
+		}
 
 	return( 0 );
 }
 
 static int
-imageinfo_progress_close( ImageinfoIMAGE *proxy )
+imageinfo_proxy_invalidate( Imageinfoproxy *proxy )
+{
+	Imageinfo *imageinfo = proxy->imageinfo;
+
+	if( imageinfo ) 
+		imageinfo_invalidate( imageinfo );
+
+	return( 0 );
+}
+
+static int
+imageinfo_proxy_preclose( Imageinfoproxy *proxy )
 {
 	Imageinfo *imageinfo = proxy->imageinfo;
 
@@ -741,20 +707,16 @@ imageinfo_progress_close( ImageinfoIMAGE *proxy )
 	return( 0 );
 }
 
-/* Add a progress box for an image evaluation.
+/* Add a proxy to track IMAGE events.
  */
 static void
-imageinfo_progress_add( Imageinfo *imageinfo, GtkWidget *eval_parent )
+imageinfo_proxy_add( Imageinfo *imageinfo )
 {
 	/* Only if we're running interactively.
 	 */
 	if( !main_window_top )
 		return;
 
-	/* Always update eval_parent. 
-	 */
-	imageinfo_set_eval_parent( imageinfo, eval_parent );
-
 	/* Already being monitored?
 	 */
 	if( imageinfo->monitored ) 
@@ -764,25 +726,25 @@ imageinfo_progress_add( Imageinfo *imageinfo, GtkWidget *eval_parent )
         /* Need a proxy on IMAGE.
          */ 
 	g_assert( !imageinfo->proxy );
-	if( !(imageinfo->proxy = IM_NEW( imageinfo->im, ImageinfoIMAGE )) )
+	if( !(imageinfo->proxy = IM_NEW( imageinfo->im, Imageinfoproxy )) )
+	if( !(imageinfo->proxy = IM_NEW( NULL, Imageinfoproxy )) )
 		return;
 	imageinfo->proxy->im = imageinfo->im;
 	imageinfo->proxy->imageinfo = imageinfo;
 
-	(void) im_add_evalstart_callback( imageinfo->im, 
-		(im_callback_fn) imageinfo_progress_start, 
-		imageinfo->proxy, NULL );
 	(void) im_add_eval_callback( imageinfo->im, 
-		(im_callback_fn) imageinfo_progress_eval, 
-		imageinfo->proxy, NULL );
-	(void) im_add_evalend_callback( imageinfo->im, 
-		(im_callback_fn) imageinfo_progress_stop, 
+		(im_callback_fn) imageinfo_proxy_eval, 
 		imageinfo->proxy, NULL );
 
-	/* On close, remove everything.
+	(void) im_add_invalidate_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_proxy_invalidate, 
+		imageinfo->proxy, NULL );
+
+	/* Has to be preclose, because we want to be sure we disconnect before 
+	 * the proxy is freed on a close callback.
 	 */
-	(void) im_add_close_callback( imageinfo->im, 
-		(im_callback_fn) imageinfo_progress_close, 
+	(void) im_add_preclose_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_proxy_preclose, 
 		imageinfo->proxy, NULL );
 }
 
@@ -798,7 +760,7 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 	char buf[FILENAME_MAX];
 
 #ifdef DEBUG_OPEN
-	printf( "imageinfo_new: \"%s\"\n", im->filename );
+	printf( "imageinfo_new: %p \"%s\"\n", imageinfo, im->filename );
 #endif /*DEBUG_OPEN*/
 
 	managed_link_heap( MANAGED( imageinfo ), heap );
@@ -820,32 +782,20 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 
 	icontainer_child_add( ICONTAINER( imageinfogroup ),
 		ICONTAINER( imageinfo ), -1 );
-	
-	/* Sometimes overridden later to set a real parent.
-	 */
-	imageinfo_progress_add( imageinfo, NULL );
+	imageinfo_proxy_add( imageinfo );
 
 	return( imageinfo );
 }
 
-/* An image is a transformed LUT (eg. after passing a LUT from above through
- * VIPS) ... wrap it around the underlying image.
+/* An image is a result of a LUT operation on an earlier imageinfo.
  */
-Imageinfo *
-imageinfo_new_modlut( Imageinfogroup *imageinfogroup, 
-	Heap *heap, Imageinfo *imageinfo, IMAGE *im )
+void
+imageinfo_set_underlying( Imageinfo *top_imageinfo, Imageinfo *imageinfo )
 {
-	Imageinfo *top_imageinfo;
+	g_assert( !top_imageinfo->underlying );
 
-	if( !(top_imageinfo = imageinfo_new( imageinfogroup, heap, im, NULL )) )
-		return( NULL );
-
-	/* Link together.
-	 */
 	top_imageinfo->underlying = imageinfo;
 	MANAGED_REF( top_imageinfo->underlying );
-
-	return( top_imageinfo );
 }
 
 /* Make a temp image. Deleted on close. No refs: closed on next GC. If you
@@ -856,8 +806,8 @@ imageinfo_new_temp( Imageinfogroup *imageinfogroup,
 	Heap *heap, const char *name, const char *mode )
 {
 	IMAGE *im;
-	Imageinfo *imageinfo;
 	char tname[FILENAME_MAX];
+	Imageinfo *imageinfo;
 
 	if( !temp_name( tname, "v" ) || !(im = im_open( tname, mode )) )
 		return( NULL );
@@ -886,170 +836,12 @@ static Imageinfo *
 imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 {
 	Imageinfo *imageinfo;
+	VipsFormatClass *format;
 
-	/*
+	if( !(format = vips_format_for_file( filename )) ) 
+		return( NULL );
 
-		FIXME ... very repetitive, replace with a proper file loader
-		plug in thing
-
-	 */
-
-	if( im_istiff( filename ) && im_istifftiled( filename ) ) {
-		/* Tiled TIFF - open and read partially.
-		 */
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "p" )) ||
-			im_tiff2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_tiff2vips %s %s",
-				filename, imageinfo->im->filename ) ) 
-			return( NULL );
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened tiled TIFF \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-
-		MANAGED_REF( imageinfo );
-	}
-
-	else if( im_istiff( filename ) && !im_istifftiled( filename ) ) {
-		/* Plain TIFF ... convert to VIPS in a file.
-		 */
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "w" )) )
-			return( NULL );
-		MANAGED_REF( imageinfo );
-		if( im_tiff2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_tiff2vips %s %s",
-				filename, imageinfo->im->filename ) ) {
-			MANAGED_UNREF( imageinfo );
-			return( NULL );
-		}
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened plain TIFF \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_isjpeg( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "w" )) )
-			return( NULL );
-		MANAGED_REF( imageinfo );
-		if( im_jpeg2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_jpeg2vips %s %s",
-				filename, imageinfo->im->filename ) ) {
-			MANAGED_UNREF( imageinfo );
-			return( NULL );
-		}
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened JPEG \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_ispng( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "w" )) )
-			return( NULL );
-		MANAGED_REF( imageinfo );
-		if( im_png2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_png2vips %s %s",
-				filename, imageinfo->im->filename ) ) {
-			MANAGED_UNREF( imageinfo );
-			return( NULL );
-		}
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened PNG \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_isppm( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "p" )) ||
-			im_ppm2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_ppm2vips %s %s",
-				filename, imageinfo->im->filename ) ) 
-			return( NULL );
-		MANAGED_REF( imageinfo );
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened PPM \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_isexr( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "p" )) ||
-			im_exr2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_exr2vips %s %s",
-				filename, imageinfo->im->filename ) ) 
-			return( NULL );
-		MANAGED_REF( imageinfo );
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened OpenEXR \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_isanalyze( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "p" )) ||
-			im_analyze2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_analyze2vips %s %s",
-				filename, imageinfo->im->filename ) ) 
-			return( NULL );
-		MANAGED_REF( imageinfo );
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened Analyze \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	/* CSV ... test off the filename.
-	 */
-	else if( is_file_type( &filesel_sfile_type, filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "p" )) ||
-			im_csv2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_csv2vips %s %s",
-				filename, imageinfo->im->filename ) ) 
-			return( NULL );
-		MANAGED_REF( imageinfo );
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened CSV \"%s\"\n", filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_ismagick( filename ) ) {
-		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "w" )) )
-			return( NULL );
-		MANAGED_REF( imageinfo );
-		if( im_magick2vips( filename, imageinfo->im ) ||
-			im_histlin( imageinfo->im, "im_magick2vips %s %s",
-				filename, imageinfo->im->filename ) ) {
-			MANAGED_UNREF( imageinfo );
-			return( NULL );
-		}
-
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened via libMagick \"%s\"\n", 
-			filename );
-#endif /*DEBUG_OPEN*/
-	}
-
-	else if( im_isvips( filename ) ) {
+	if( strcmp( VIPS_OBJECT_CLASS( format )->nickname, "vips" ) == 0 ) {
 		IMAGE *im;
 
 		if( !(im = im_open( filename, "r" )) ) 
@@ -1067,11 +859,26 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			filename );
 #endif /*DEBUG_OPEN*/
 	}
-
 	else {
-		im_error( "open", _( "\"%s\" is not in a supported "
-			"image file format" ), filename );
-		return( NULL );
+		VipsFormatFlags flags = 
+			vips_format_get_flags( format, filename );
+		const char *mode = flags & VIPS_FORMAT_PARTIAL ? "p" : "w";
+
+		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
+			open->heap, open->filename, mode )) )
+			return( NULL );
+		MANAGED_REF( imageinfo );
+		if( format->load( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_copy %s %s",
+				filename, imageinfo->im->filename ) ) {
+			MANAGED_UNREF( imageinfo );
+			return( NULL );
+		}
+
+#ifdef DEBUG_OPEN
+		printf( "imageinfo_open_image_input: "
+			"opened %s \"%s\"\n", format->name, filename );
+#endif /*DEBUG_OPEN*/
 	}
 
 	/* Get ready for input.
@@ -1082,7 +889,7 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 	/* The rewind will have removed everything from the IMAGE. Reattach
 	 * progress.
 	 */
-	imageinfo_progress_add( imageinfo, NULL );
+	imageinfo_proxy_add( imageinfo );
 
 	/* Attach the original filename ... pick this up again later as a
 	 * save default.
@@ -1093,27 +900,6 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 	return( imageinfo );
 }
 
-/* Open for write ... returns a non-heap pointer, destroy if it goes in the
- * heap.
- */
-static Imageinfo *
-imageinfo_open_image_output( const char *filename,
-	Imageinfogroup *imageinfogroup, Heap *heap, const char *name )
-{
-	Imageinfo *imageinfo;
-	IMAGE *im;
-
-	if( !(im = im_open( filename, "w" )) )
-		return( NULL );
-	if( !(imageinfo = imageinfo_new( imageinfogroup, heap, im, name )) ) {
-		im_close( im );
-		return( NULL );
-	}
-	MANAGED_REF( imageinfo );
-
-	return( imageinfo );
-}
-
 /* Was this ii loaded from a file (ie. ->name contains a filename the user
  * might recognise).
  */
@@ -1203,39 +989,6 @@ imageinfo_new_input( Imageinfogroup *imageinfogroup, GtkWidget *parent,
 	return( imageinfo );
 }
 
-/* Open a filename for output.
- */
-static Imageinfo *
-imageinfo_new_output( Imageinfogroup *imageinfogroup, 
-	Heap *heap, const char *name )
-{
-	Imageinfo *imageinfo;
-	char filename[FILENAME_MAX];
-	char filemode[FILENAME_MAX];
-
-	im_filename_split( name, filename, filemode );
-	if( (imageinfo = imageinfogroup_lookup( imageinfogroup, filename )) ) {
-                error_top( _( "Unable to write to file." ) );
-		error_sub( _( "File \"%s\" is already open for read." ), 
-			filename );
-		error_vips();
-                return( NULL );
-	}
-
-        if( !(imageinfo = (Imageinfo *) callv_string_filename( 
-		(callv_string_fn) imageinfo_open_image_output, 
-		name, imageinfogroup, heap, (char *) filename )) ) {
-                error_top( _( "Unable to write to file." ) );
-		error_sub( _( "Unable to open \"%s\" for write." ), filename );
-		error_vips();
-                return( NULL );
-        }
-
-	imageinfo->from_file = TRUE;
-
-	return( imageinfo );
-}
-
 /* Add an identity lut, if this is a LUTtable image.
  */
 static IMAGE *
@@ -1340,58 +1093,43 @@ imageinfo_same_underlying( Imageinfo *imageinfo[], int n )
 	}
 }
 
-static int
-imageinfo_write_im( Imageinfo *out, IMAGE *im )
-{
-	int result;
-
-	/* im_close() can fail if we write a TIFF on evalend. So close
-	 * ourselves, carefully. Zap out->im so it's not closed again when we
-	 * destroy out.
-	 */
-	if( im_copy( im, out->im ) )
-		return( -1 );
-
-	result = im_close( out->im );
-	out->im = NULL;
-
-	return( result );
-}
-
 /* Write to a filename.
  */
 gboolean
-imageinfo_write( Imageinfo *imageinfo, GtkWidget *parent, const char *filename )
+imageinfo_write( Imageinfo *imageinfo, const char *name )
 {
-	Heap *heap = MANAGED( imageinfo )->heap;
 	Imageinfogroup *imageinfogroup = 
 		IMAGEINFOGROUP( ICONTAINER( imageinfo )->parent );
-	IMAGE *in = imageinfo_get( FALSE, imageinfo );
-	Imageinfo *out;
+	IMAGE *im = imageinfo_get( FALSE, imageinfo );
+	char filename[FILENAME_MAX];
+	char filemode[FILENAME_MAX];
 
-	if( !(out = imageinfo_new_output( imageinfogroup, heap, filename )) ) 
-		return( FALSE );
-	managed_sub_add( MANAGED( out ), MANAGED( imageinfo ) );
-	imageinfo_progress_add( out, parent );
+	im_filename_split( name, filename, filemode );
 
-	if( imageinfo_write_im( out, in ) ) {
-		MANAGED_UNREF( out );
+	if( (imageinfo = imageinfogroup_lookup( imageinfogroup, filename )) ) {
+                error_top( _( "Unable to write to file." ) );
+		error_sub( _( "File \"%s\" is already open for read." ), 
+			filename );
+
+                return( FALSE );
+	}
+
+	if( vips_format_write( im, name ) ) {
 		error_top( _( "Unable to write to file." ) );
 		error_sub( _( "Error writing image to file \"%s\"." ), 
 			filename );
 		error_vips();
+
 		return( FALSE );
 	}
 
-	MANAGED_UNREF( out );
-
 	return( TRUE );
 }
 
 /* Change an imageinfo to be a file, rather than a memory object. 
  */
 static gboolean
-imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
+imageinfo_file( Imageinfo *imageinfo )
 {
 	IMAGE *im;
 	char filename[FILENAME_MAX];
@@ -1407,7 +1145,7 @@ imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
 	 */
 	if( !temp_name( filename, "v" ) )
 		return( FALSE );
-	if( !imageinfo_write( imageinfo, parent, filename ) )
+	if( !imageinfo_write( imageinfo, filename ) )
 		return( FALSE );
 	if( !(im = im_open( filename, "r" )) ) {
 		error_vips_all();
@@ -1430,9 +1168,9 @@ imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
 }
 
 static gboolean
-imageinfo_make_paintable( Imageinfo *imageinfo, GtkWidget *parent )
+imageinfo_make_paintable( Imageinfo *imageinfo )
 {
-	if( !imageinfo_file( imageinfo, parent ) ) 
+	if( !imageinfo_file( imageinfo ) ) 
 		return( FALSE );
 	if( im_rwcheck( imageinfo->im ) ) {
 		error_top( _( "Unable to paint on image." ) );
@@ -1454,12 +1192,12 @@ imageinfo_check_paintable_cb( iWindow *iwnd, void *client,
 {
 	Imageinfo *imageinfo = IMAGEINFO( client );
 
-	if( !imageinfo_make_paintable( imageinfo, GTK_WIDGET( iwnd ) ) ) {
+	if( !imageinfo_make_paintable( imageinfo ) ) {
 		nfn( sys, IWINDOW_ERROR );
 		return;
 	}
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Check painting is OK. nfn() called on "ok!". Returns FALSE if it's
@@ -1493,13 +1231,13 @@ imageinfo_check_paintable( Imageinfo *imageinfo, GtkWidget *parent,
 		return( FALSE );
 	}
 	else if( !im_isfile( im ) && !imageinfo->ok_to_paint ) {
-		if( !imageinfo_make_paintable( imageinfo, parent ) ) {
+		if( !imageinfo_make_paintable( imageinfo ) ) {
 			nfn( sys, IWINDOW_ERROR );
 			return( FALSE );
 		}
 	}
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 
 	return( TRUE );
 }
@@ -2142,6 +1880,7 @@ imageinfo_paint_dropper( Imageinfo *imageinfo, Imageinfo *ink, int x, int y )
 		error_vips_all();
 		return( FALSE );
 	}
+	im_invalidate( ink_im );
 
 	dirty.left = 0;
 	dirty.top = 0;
@@ -2241,18 +1980,18 @@ imageinfo_paint_mask( Imageinfo *imageinfo,
 /* Print a pixel. Output has to be parseable by imageinfo_from_text().
  */
 void 
-imageinfo_to_text( Imageinfo *imageinfo, BufInfo *buf )
+imageinfo_to_text( Imageinfo *imageinfo, VipsBuf *buf )
 {
 	IMAGE *im = imageinfo_get( FALSE, imageinfo );
 	PEL *p = (PEL *) im->data;
 	int i;
 
-#define PRINT_INT( T, I ) buf_appendf( buf, "%d", ((T *)p)[I] );
-#define PRINT_FLOAT( T, I ) buf_appendg( buf, ((T *)p)[I] );
+#define PRINT_INT( T, I ) vips_buf_appendf( buf, "%d", ((T *)p)[I] );
+#define PRINT_FLOAT( T, I ) vips_buf_appendg( buf, ((T *)p)[I] );
 
 	for( i = 0; i < im->Bands; i++ ) {
 		if( i )
-			buf_appends( buf, ", " );
+			vips_buf_appends( buf, ", " );
 
 		switch( im->BandFmt ) {
 		case IM_BANDFMT_UCHAR:
@@ -2284,11 +2023,11 @@ imageinfo_to_text( Imageinfo *imageinfo, BufInfo *buf )
 			break;
 			
 		case IM_BANDFMT_COMPLEX:
-			buf_appends( buf, "(" );
+			vips_buf_appends( buf, "(" );
 			PRINT_FLOAT( float, (i << 1) );
-			buf_appends( buf, ", " );
+			vips_buf_appends( buf, ", " );
 			PRINT_FLOAT( float, (i << 1) + 1 );
-			buf_appends( buf, ")" );
+			vips_buf_appends( buf, ")" );
 			break;
 			
 		case IM_BANDFMT_DOUBLE:
@@ -2296,15 +2035,15 @@ imageinfo_to_text( Imageinfo *imageinfo, BufInfo *buf )
 			break;
 			
 		case IM_BANDFMT_DPCOMPLEX:
-			buf_appends( buf, "(" );
+			vips_buf_appends( buf, "(" );
 			PRINT_FLOAT( double, i << 1 );
-			buf_appends( buf, ", " );
+			vips_buf_appends( buf, ", " );
 			PRINT_FLOAT( double, (i << 1) + 1 );
-			buf_appends( buf, ")" );
+			vips_buf_appends( buf, ")" );
 			break;
 
 		default:
-			buf_appends( buf, "???" );
+			vips_buf_appends( buf, "???" );
 			break;
 		}
 	}
@@ -2419,13 +2158,13 @@ imageinfo_from_text( Imageinfo *imageinfo, const char *text )
 
 #ifdef DEBUG_RGB
 {
-	char buf_text[256];
-	BufInfo buf;
+	char vips_buf_text[256];
+	VipsBuf buf;
 
 	printf( "imageinfo_from_text: out: " );
-	buf_init_static( &buf, buf_text, 256 );
+	vips_buf_init_static( &buf, vips_buf_text, 256 );
 	imageinfo_to_text( imageinfo, &buf );
-	printf( "%s\n", buf_all( &buf ) );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_RGB*/
 
@@ -2450,13 +2189,13 @@ imageinfo_to_rgb( Imageinfo *imageinfo, double *rgb )
 
 #ifdef DEBUG_RGB
 {
-	char buf_text[256];
-	BufInfo buf;
+	char vips_buf_text[256];
+	VipsBuf buf;
 
 	printf( "imageinfo_to_rgb: in: " );
-	buf_init_static( &buf, buf_text, 256 );
+	vips_buf_init_static( &buf, vips_buf_text, 256 );
 	imageinfo_to_text( imageinfo, &buf );
-	printf( "%s\n", buf_all( &buf ) );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_RGB*/
 
@@ -2542,64 +2281,71 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 	for( i = 0; i < 3; i++ )
 		((PEL *) in->im->data)[i] = IM_RINT( rgb[i] * 255.0 );
 
-	/* To imageinfo->type. Make sure we get a float ... except for LABQ.
+	/* To imageinfo->type. Make sure we get a float ... except for LABQ
+	 * and RAD.
 	 */
-	switch( im->Type ) {
-	case IM_TYPE_XYZ:
-		if( im_disp2XYZ( in->im, out->im, display ) )
+	if( im->Coding == IM_CODING_LABQ ) {
+		if( im_disp2Lab( in->im, t1, display ) ||
+			im_Lab2LabQ( t1, out->im ) )
 			return;
-		break;
-
-	case IM_TYPE_YXY:
+	}
+	else if( im->Coding == IM_CODING_RAD ) {
 		if( im_disp2XYZ( in->im, t1, display ) ||
-			im_XYZ2Yxy( t1, out->im ) )
+			im_float2rad( t1, out->im ) )
 			return;
-		break;
-
-	case IM_TYPE_LAB:
-		if( imageinfo->im->Coding == IM_CODING_LABQ ) {
-			if( im_disp2Lab( in->im, t1, display ) ||
-				im_Lab2LabQ( t1, out->im ) )
+	}
+	else if( im->Coding == IM_CODING_NONE ) {
+		switch( im->Type ) {
+		case IM_TYPE_XYZ:
+			if( im_disp2XYZ( in->im, out->im, display ) )
 				return;
-		}
-		else {
+			break;
+
+		case IM_TYPE_YXY:
+			if( im_disp2XYZ( in->im, t1, display ) ||
+				im_XYZ2Yxy( t1, out->im ) )
+				return;
+			break;
+
+		case IM_TYPE_LAB:
 			if( im_disp2Lab( in->im, out->im, display ) )
 				return;
+			break;
+
+		case IM_TYPE_LCH:
+			if( im_disp2Lab( in->im, t1, display ) ||
+				im_Lab2LCh( t1, out->im ) )
+				return;
+			break;
+
+		case IM_TYPE_UCS:
+			if( im_disp2Lab( in->im, t1, display ) ||
+				im_Lab2LCh( t1, t2 ) ||
+				im_LCh2UCS( t2, out->im ) )
+				return;
+			break;
+
+		case IM_TYPE_RGB16:
+		case IM_TYPE_GREY16:
+			if( im_lintra( 1.0 / 256.0, in->im, 0.0, out->im ) )
+				return;
+			break;
+
+		case IM_TYPE_RGB:
+		case IM_TYPE_sRGB:
+		default:
+			if( im_clip2f( in->im, out->im ) )
+				return;
+			break;
 		}
-		break;
-
-	case IM_TYPE_LCH:
-		if( im_disp2Lab( in->im, t1, display ) ||
-			im_Lab2LCh( t1, out->im ) )
-			return;
-		break;
-
-	case IM_TYPE_UCS:
-		if( im_disp2Lab( in->im, t1, display ) ||
-			im_Lab2LCh( t1, t2 ) ||
-			im_LCh2UCS( t2, out->im ) )
-			return;
-		break;
-
-	case IM_TYPE_RGB16:
-	case IM_TYPE_GREY16:
-		if( im_lintra( 1.0 / 256.0, in->im, 0.0, out->im ) )
-			return;
-		break;
-
-	case IM_TYPE_RGB:
-	case IM_TYPE_sRGB:
-	default:
-		if( im_clip2f( in->im, out->im ) )
-			return;
-		break;
 	}
 
 #define SET( TYPE, i ) ((TYPE *) im->data)[i] = ((float *) out->im->data)[i];
 
 	/* Now ... overwrite imageinfo.
 	 */
-	if( im->Coding == IM_CODING_LABQ ) {
+	if( im->Coding == IM_CODING_LABQ ||
+		im->Coding == IM_CODING_RAD ) {
 		for( i = 0; i < im->Bands; i++ ) 
 			((PEL *) im->data)[i] = ((PEL *) out->im->data)[i];
 	}
@@ -2649,20 +2395,20 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 				break;
 
 			default:
-				assert( FALSE );
+				g_assert( FALSE );
 			}
 	}
 	im_invalidate( im );
 
 #ifdef DEBUG_RGB
 {
-	char buf_text[256];
-	BufInfo buf;
+	char vips_buf_text[256];
+	VipsBuf buf;
 
 	printf( "imageinfo_from_rgb: out: " );
-	buf_init_static( &buf, buf_text, 256 );
+	vips_buf_init_static( &buf, vips_buf_text, 256 );
 	imageinfo_to_text( imageinfo, &buf );
-	printf( "%s\n", buf_all( &buf ) );
+	printf( "%s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_RGB*/
 
@@ -2699,7 +2445,7 @@ imageinfo_colour_done_cb( iWindow *iwnd, void *client,
 	 */
 	imageinfo_from_rgb( imageinfo, rgb );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Build the insides of colour edit.
