@@ -106,7 +106,7 @@ imagepresent_destroy( GtkObject *object )
 
 	/* Child views should all have removed themselves.
 	 */
-	assert( ip->regionviews == NULL );
+	g_assert( ip->regionviews == NULL );
 }
 
 static void
@@ -1027,7 +1027,7 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 			/* Make a new region.
 			 */
 			char txt[MAX_STRSIZE];
-			BufInfo buf;
+			VipsBuf buf;
 			Symbol *sym;
 
 			/* In compatibility mode, we need to un-offset, since
@@ -1041,7 +1041,7 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 				ip->floating.top -= im->Yoffset;
 			}
 
-			buf_init_static( &buf, txt, MAX_STRSIZE );
+			vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 
 			switch( ip->regionview->type ) {
 			case REGIONVIEW_MARK:
@@ -1049,53 +1049,53 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 				 * "Point".
 				 */
 				if( row->ws->compat_78 )
-					buf_appends( &buf, "Point " );
+					vips_buf_appends( &buf, "Point " );
 				else
-					buf_appendf( &buf, "%s ", CLASS_MARK );
+					vips_buf_appendf( &buf, "%s ", CLASS_MARK );
 
 				row_qualified_name( row, &buf );
-				buf_appendd( &buf, ip->floating.left );
-				buf_appendd( &buf, ip->floating.top );
+				vips_buf_appendd( &buf, ip->floating.left );
+				vips_buf_appendd( &buf, ip->floating.top );
 				break;
 
 			case REGIONVIEW_REGION:
-				buf_appendf( &buf, "%s ", CLASS_REGION );
+				vips_buf_appendf( &buf, "%s ", CLASS_REGION );
 				row_qualified_name( row, &buf );
-				buf_appendd( &buf, ip->floating.left );
-				buf_appendd( &buf, ip->floating.top );
-				buf_appendd( &buf, ip->floating.width );
-				buf_appendd( &buf, ip->floating.height );
+				vips_buf_appendd( &buf, ip->floating.left );
+				vips_buf_appendd( &buf, ip->floating.top );
+				vips_buf_appendd( &buf, ip->floating.width );
+				vips_buf_appendd( &buf, ip->floating.height );
 				break;
 
 			case REGIONVIEW_ARROW:
-				buf_appendf( &buf, "%s ", CLASS_ARROW );
+				vips_buf_appendf( &buf, "%s ", CLASS_ARROW );
 				row_qualified_name( row, &buf );
-				buf_appendd( &buf, ip->floating.left );
-				buf_appendd( &buf, ip->floating.top );
-				buf_appendd( &buf, ip->floating.width );
-				buf_appendd( &buf, ip->floating.height );
+				vips_buf_appendd( &buf, ip->floating.left );
+				vips_buf_appendd( &buf, ip->floating.top );
+				vips_buf_appendd( &buf, ip->floating.width );
+				vips_buf_appendd( &buf, ip->floating.height );
 				break;
 
 			case REGIONVIEW_HGUIDE:
-				buf_appendf( &buf, "%s ", CLASS_HGUIDE );
+				vips_buf_appendf( &buf, "%s ", CLASS_HGUIDE );
 				row_qualified_name( row, &buf );
-				buf_appendd( &buf, ip->floating.top );
+				vips_buf_appendd( &buf, ip->floating.top );
 				break;
 
 			case REGIONVIEW_VGUIDE:
-				buf_appendf( &buf, "%s ", CLASS_VGUIDE );
+				vips_buf_appendf( &buf, "%s ", CLASS_VGUIDE );
 				row_qualified_name( row, &buf );
-				buf_appendd( &buf, ip->floating.left );
+				vips_buf_appendd( &buf, ip->floating.left );
 				break;
 
 			default:
-				assert( FALSE );
+				g_assert( FALSE );
 			}
 
 			DESTROY_GTK( ip->regionview );
 
 			if( !(sym = workspace_add_def( row->ws, 
-				buf_all( &buf ) )) ) 
+				vips_buf_all( &buf ) )) ) 
 				box_alert( GTK_WIDGET( ip ) );
 
 			workspace_deselect_all( row->ws );
@@ -1525,7 +1525,7 @@ imagepresent_init( Imagepresent *ip )
 		GTK_SIGNAL_FUNC( imagepresent_scroll_event_cb ), ip );
 
 	port = gtk_bin_get_child( GTK_BIN( ip->swin ) );
-	assert( GTK_IS_VIEWPORT( port ) );
+	g_assert( GTK_IS_VIEWPORT( port ) );
 	gtk_signal_connect( GTK_OBJECT( ip->hadj ), "changed",
 		GTK_SIGNAL_FUNC( imagepresent_hadj_changed_cb ), ip );
 	gtk_signal_connect( GTK_OBJECT( ip->hadj ), "value_changed",
