@@ -64,7 +64,6 @@
 #define ngettext(S, P, N) ((N) == 1 ? (S) : (P))
 #endif /* ENABLE_NLS */
 
-#include <assert.h>
 #include <time.h>
 #include <ctype.h>
 #include <errno.h>
@@ -131,21 +130,6 @@ extern int statfs();
 
 #include <vips/vips.h>
 
-/* Experimental vips8 links.
- */
-#ifdef HAVE_VIPS8
-#include <vips8/vips.h>
-#else /*!HAVE_VIPS8*/
-/* Some typedefs so we can build code containing vips8 names without falling
- * over.
- */
-typedef void VipsImage;
-typedef void VipsObject;
-#define VIPS_OBJECT(O) (O)
-#define VIPS_IS_OBJECT(O) (O)
-#define VIPS_NN(O) (O)
-#endif /*HAVE_VIPS8*/
-
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
@@ -178,8 +162,9 @@ typedef void VipsObject;
 #define MAX_TRACE (1024)	/* Biggest thing we print in trace */
 #define MAX_SSTACK (40)		/* Scope stack for parser */
 #define VIPS_HOMEPAGE "http://www.vips.ecs.soton.ac.uk"
+#define IP_NAME PACKAGE "-" VERSION
 #define NIP_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
-	"doc" G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "html"
+        "doc" G_DIR_SEPARATOR_S IP_NAME G_DIR_SEPARATOR_S "html"
 #define VIPS_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
 	"doc" G_DIR_SEPARATOR_S "vips" G_DIR_SEPARATOR_S "html"
 #define IP_NAME PACKAGE "-" VERSION
@@ -187,7 +172,7 @@ typedef void VipsObject;
 				/* XML namespace ... note, not nip2! */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
 #define MAX_RECENT (10)		/* Number of recent items in file menu */
-#define NIP_COPYRIGHT "%s: (c) 2008 Imperial College, London"
+#define NIP_COPYRIGHT "%s: &#0169;2009 Imperial College, London"
 
 /* Our stock_ids.
  */
@@ -263,6 +248,7 @@ typedef struct _Managed Managed;
 typedef struct _Managedfile Managedfile;
 typedef struct _Managedgvalue Managedgvalue;
 typedef struct _Managedgobject Managedgobject;
+typedef struct _Managedstring Managedstring;
 typedef struct _Imageinfo Imageinfo;
 typedef struct _Imagepresent Imagepresent;
 typedef struct _Imagemodel Imagemodel;
@@ -352,11 +338,13 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "imageview.h"
 #include "tslider.h"
 #include "pane.h"
+#include "progress.h"
 
 /* Basic ip includes (order important).
  */
 #include "tree.h"
 #include "heap.h"
+#include "managedstring.h"
 #include "class.h"
 #include "link.h"
 #include "expr.h"
@@ -374,6 +362,7 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "action.h"
 #include "reduce.h"
 #include "vobject.h"
+#include "vipsobject.h"
 #include "view.h"
 #include "graphicview.h"
 #include "spin.h"
@@ -393,6 +382,7 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "toolview.h"
 #include "watch.h"
 #include "value.h"
+#include "panechild.h"
 
 /* Per module includes, any order
  */
@@ -458,7 +448,6 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "toggle.h"
 #include "toggleview.h"
 #include "vips_call.h"
-#include "call8.h"
 #include "parser.h"
 
 #ifdef WITH_DMALLOC
