@@ -41,7 +41,7 @@
     	iwindow_kill()
 
 		'cancellable' kill ... user popdown can return IWINDOW_ERROR
-		or IWINDOW_FALSE to prevent popdown
+		or IWINDOW_NO to prevent popdown
 
 	gtk_widget_destroy()
 
@@ -307,81 +307,49 @@ iwindow_cursor_update( iWindow *iwnd )
 /* Set a global cursor for a window.
  */
 static void *
-iwindow_cursor_set( iWindow *iwnd, iWindowShape shape )
+iwindow_cursor_set( iWindow *iwnd, iWindowShape *shape )
 {
-	iwnd->shape = shape;
-	iwindow_cursor_update( iwnd );
+	if( iwnd->shape != *shape ) {
+		iwnd->shape = *shape;
+		iwindow_cursor_update( iwnd );
+	}
 
 	return( NULL );
 }
 
-static int iwindow_hglass_count = 0;
-static GTimer *iwindow_hglass_timer = NULL;
-static GTimer *iwindow_hglass_update_timer = NULL;
+static gboolean hourglass_showing = FALSE;
 
-/* Delay before we start showing the spinning hglass.
- */
-static const double iwindow_hglass_delay = 0.2;
-
-/* Delay between cursor updates.
- */
-static const double iwindow_hglass_update = 0.5;
-
-void
-animate_hourglass( void )
+static void
+hourglass_begin( void )
 {
-	static iWindowShape shape = IWINDOW_SHAPE_HGLASS1;
+	hourglass_showing = TRUE;
+}
 
-	if( iwindow_hglass_count &&
-		g_timer_elapsed( iwindow_hglass_timer, NULL ) > 
-			iwindow_hglass_delay ) {
+static void
+hourglass_update( void )
+{
+	if( hourglass_showing ) {
+		static iWindowShape shape = IWINDOW_SHAPE_HGLASS1;
 
-		if( g_timer_elapsed( iwindow_hglass_update_timer, NULL ) > 
-			iwindow_hglass_update ) {
-			iwindow_map_all( 
-				(iWindowMapFn) iwindow_cursor_set, 
-				(void *) shape );
-			g_timer_start( iwindow_hglass_update_timer );
+		iwindow_map_all( (iWindowMapFn) iwindow_cursor_set, &shape );
 
-			shape += 1;
-			if( shape > IWINDOW_SHAPE_HGLASS8 )
-				shape = IWINDOW_SHAPE_HGLASS1;
-		}
+		shape += 1;
+		if( shape > IWINDOW_SHAPE_HGLASS8 )
+			shape = IWINDOW_SHAPE_HGLASS1;
 	}
 }
 
-void
-set_hourglass( void )
+static void
+hourglass_end( void )
 {
-	assert( iwindow_hglass_count >= 0 );
+	if( hourglass_showing ) {
+		iWindowShape shape = IWINDOW_SHAPE_NONE;
 
-	iwindow_hglass_count += 1;
-
-	if( iwindow_hglass_count == 1 ) {
-		if( !iwindow_hglass_timer )
-			iwindow_hglass_timer = g_timer_new();
-		if( !iwindow_hglass_update_timer )
-			iwindow_hglass_update_timer = g_timer_new();
-
-		g_timer_start( iwindow_hglass_timer );
-		g_timer_start( iwindow_hglass_update_timer );
-
-		animate_hourglass();
+		iwindow_map_all( (iWindowMapFn) iwindow_cursor_set, &shape );
+		hourglass_showing = FALSE;
 	}
 }
 
-void
-set_pointer( void )
-{
-	assert( iwindow_hglass_count > 0 );
-
-	iwindow_hglass_count -= 1;
-
-	if( !iwindow_hglass_count ) 
-		iwindow_map_all( (iWindowMapFn) iwindow_cursor_set, 
-			(void *) IWINDOW_SHAPE_NONE );
-}
-
 iWindowCursorContext *
 iwindow_cursor_context_new( iWindow *iwnd, int priority, const char *name )
 {
@@ -463,7 +431,7 @@ iwindow_susp_comp( void *sys, iWindowResult result )
 {
 	iWindowSusp *susp = IWINDOW_SUSP( sys );
 
-	if( result == IWINDOW_TRUE ) {
+	if( result == IWINDOW_YES ) {
 		susp->fn( susp->iwnd, susp->client, susp->nfn, susp->sys );
 		im_free( susp );
 	}
@@ -476,13 +444,13 @@ iwindow_susp_comp( void *sys, iWindowResult result )
 void 
 iwindow_true_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys ) 
 { 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 void 
 iwindow_false_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys ) 
 { 
-	nfn( sys, IWINDOW_FALSE );
+	nfn( sys, IWINDOW_NO );
 }
 
 /* Null notify callback.
@@ -498,7 +466,7 @@ iwindow_final_death( iWindow *iwnd )
 	printf( "iwindow_final_death: %s\n", iwnd->title );
 #endif /*DEBUG*/
 
-	assert( iwnd->pending == 0 && iwnd->destroy );
+	g_assert( iwnd->pending == 0 && iwnd->destroy );
 
 	/* Clean up.
 	 */
@@ -516,7 +484,7 @@ iwindow_notify_return( iWindow *iwnd )
 		iwnd->title, iwnd->pending );
 #endif /*DEBUG*/
 
-	assert( iwnd->pending > 0 );
+	g_assert( iwnd->pending > 0 );
 
 	iwnd->pending--;
 	if( iwnd->destroy && iwnd->pending == 0 ) {
@@ -543,7 +511,7 @@ iwindow_notify_send( iWindow *iwnd,
 	if( fn )
 		fn( iwnd, client, back, sys );
 	else
-		back( sys, IWINDOW_TRUE );
+		back( sys, IWINDOW_YES );
 }
 
 static void
@@ -594,10 +562,10 @@ iwindow_popdown_notify( iWindow *iwnd, iWindowResult result )
 
 	if( result == IWINDOW_ERROR )
 		box_alert( GTK_WIDGET( iwnd ) );
-	else if( result == IWINDOW_TRUE )
+	else if( result == IWINDOW_YES )
 		iwindow_kill( iwnd );
 
-	if( result != IWINDOW_TRUE ) {
+	if( result != IWINDOW_YES ) {
 #ifdef DEBUG
 		printf( "iwindow_popdown_notify: %s: kill cancelled!\n", 
 			iwnd->title );
@@ -770,6 +738,12 @@ iwindow_class_init( iWindowClass *class )
 	/* Static class data init.
 	 */
 	iwindow_make_cursors();
+
+	/* Link to busy signals.
+	 */
+	g_signal_connect( progress_get(), "begin", hourglass_begin, NULL );
+	g_signal_connect( progress_get(), "update", hourglass_update, NULL );
+	g_signal_connect( progress_get(), "end", hourglass_end, NULL );
 }
 
 static void
@@ -896,7 +870,7 @@ iwindow_set_work_window( iWindow *iwnd, GdkWindow *work_window )
 void 
 iwindow_set_parent( iWindow *iwnd, GtkWidget *parent )
 {
-	assert( !iwnd->parent );
+	g_assert( !iwnd->parent );
 
 	iwnd->parent = parent;
 
