@@ -33,14 +33,24 @@
 #define DEBUG
  */
 
+/* Show all paint actions with flashing stuff.
+#define DEBUG_UPDATES
+ */
+
 /* Stop startup creation of externs for all VIPS functions etc.
 #define DEBUG_NOAUTO
  */
 
-/* stop on any gtk error/warning/whatever
+/* Stop on any gtk error/warning/whatever. Usually set by configure for dev
+ * builds. 
 #define DEBUG_FATAL
  */
 
+/* But some themes can trigger warnings, argh, so sometimes we need to
+ * undef it. VipsObject sets can also trigger warnings.
+ */
+#undef DEBUG_FATAL
+
 /* Time startup.
 #define DEBUG_TIME
  */
@@ -93,6 +103,8 @@ gboolean main_option_i18n = FALSE;
 static gboolean main_option_print_main = FALSE;
 static gboolean main_option_version = FALSE;
 static gboolean main_option_verbose = FALSE;
+static gboolean main_option_test = FALSE;
+static char *main_option_prefix = NULL;
 
 static GOptionEntry main_option[] = {
 	{ "expression", 'e', 0, G_OPTION_ARG_STRING, &main_option_expression, 
@@ -113,7 +125,7 @@ static GOptionEntry main_option[] = {
 		&main_option_no_load_menus, 
 		N_( "don't load menu definitions" ), NULL },
 	{ "no-load-args", 'a', 0, G_OPTION_ARG_NONE, &main_option_no_load_args, 
-		N_( "don't try to load extra cmd-line args as files" ), NULL },
+		N_( "don't try to load command-line arguments" ), NULL },
 	{ "stdin-ws", 'w', 0, G_OPTION_ARG_NONE, &main_option_stdin_ws, 
 		N_( "load stdin as a workspace" ), NULL },
 	{ "stdin-def", 'd', 0, G_OPTION_ARG_NONE, &main_option_stdin_def, 
@@ -127,20 +139,24 @@ static GOptionEntry main_option[] = {
 	{ "time-save", 't', 0, G_OPTION_ARG_NONE, &main_option_time_save, 
 		N_( "time image save operations" ), 
 		NULL },
+	{ "prefix", 'x', 0, G_OPTION_ARG_FILENAME, &main_option_prefix, 
+		N_( "start as if installed to PREFIX" ), "PREFIX" },
 	{ "i18n", 'i', 0, G_OPTION_ARG_NONE, &main_option_i18n, 
 		N_( "output strings for internationalisation" ), 
 		NULL },
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &main_option_version, 
 		N_( "print version number" ), 
 		NULL },
+	{ "test", 'T', 0, G_OPTION_ARG_NONE, &main_option_test, 
+		N_( "test for errors and quit" ), NULL },
 	{ NULL }
 };
 
 /* Accumulate startup errors here.
  */
 static char main_start_error_txt[MAX_STRSIZE];
-static BufInfo main_start_error = 
-	BUF_STATIC( main_start_error_txt, MAX_STRSIZE );
+static VipsBuf main_start_error = 
+	VIPS_BUF_STATIC( main_start_error_txt, MAX_STRSIZE );
 
 static void
 main_log_add( const char *fmt, ... )
@@ -148,20 +164,20 @@ main_log_add( const char *fmt, ... )
 	va_list ap;
 
         va_start( ap, fmt );
-	buf_vappendf( &main_start_error, fmt, ap );
+	vips_buf_vappendf( &main_start_error, fmt, ap );
         va_end( ap );
 }
 
 static const char *
 main_log_get( void )
 {
-	return( buf_all( &main_start_error ) );
+	return( vips_buf_all( &main_start_error ) );
 }
 
 static gboolean
 main_log_is_empty( void )
 {
-	return( buf_is_empty( &main_start_error ) );
+	return( vips_buf_is_empty( &main_start_error ) );
 }
 
 /* NULL log handler. Used to suppress output on win32 without DEBUG_FATAL.
@@ -191,13 +207,13 @@ main_error_exit( const char *fmt, ... )
 		error_get_top(), error_get_sub() );
 
 	if( main_option_verbose ) {
-		BufInfo buf;
+		VipsBuf buf;
 		char txt[MAX_STRSIZE];
 
-		buf_init_static( &buf, txt, MAX_STRSIZE );
+		vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 		slist_map( expr_error_all,
 			(SListMapFn) expr_error_print, &buf );
-		fprintf( stderr, "%s\n", buf_all( &buf ) );
+		fprintf( stderr, "%s\n", vips_buf_all( &buf ) );
 	}
 
 	exit( 1 );
@@ -220,8 +236,8 @@ main_print_main( Symbol *sym )
 		char filename[FILENAME_MAX];
 
 		im_strncpy( filename, main_option_output, FILENAME_MAX );
-		if( save_objects( root, filename ) )
-			main_error_exit( _( "error calculating \"%s\"" ), 
+		if( !group_save_item( root, filename ) )
+			main_error_exit( _( "error saving \"%s\"" ), 
 				symbol_name( sym ) );
 	}
 
@@ -357,7 +373,7 @@ main_quit( void )
 static void
 main_quit_test_cb( void *sys, iWindowResult result )
 {
-	if( result == IWINDOW_TRUE )
+	if( result == IWINDOW_YES )
 		/* No return from this.
 		 */
 		main_quit();
@@ -386,20 +402,6 @@ main_splash_enabled( void )
 		get_savedir(), NO_SPLASH ) );
 }
 
-void
-main_splash_update( const char *fmt, ... )
-{
-	if( main_splash ) {
-		va_list ap;
-
-		va_start( ap, fmt );
-		splash_updatev( main_splash, fmt, ap );
-		va_end( ap );
-	}
-
-	animate_hourglass();
-}
-
 static void
 main_watchgroup_changed_cb( void )
 {
@@ -418,8 +420,6 @@ main_load( Workspace *ws, const char *filename )
 {
 	Workspace *new_ws;
 
-	main_splash_update( _( "Loading \"%s\"" ), im_skip_dir( filename ) );
-
 	if( (new_ws = 
 		workspace_new_from_file( main_workspacegroup, filename )) ) {
 		Mainw *new_mainw;
@@ -477,8 +477,7 @@ main_load_def( const char *filename )
 	Toolkit *kit;
 
 	if( !main_option_no_load_menus || im_skip_dir( filename )[0] == '_' ) {
-		main_splash_update( _( "Loading toolkit \"%s\"" ), 
-			im_skip_dir( filename ) );
+		progress_update_loading( 0, im_skip_dir( filename ) );
 
 		if( !(kit = toolkit_new_from_file( main_toolkitgroup, 
 			filename )) )
@@ -499,8 +498,7 @@ main_load_ws( const char *filename )
 	printf( "main_load_ws: %s\n", filename );
 #endif/*DEBUG*/
 
-	main_splash_update( _( "Loading workspace \"%s\"" ), 
-		im_skip_dir( filename ) );
+	progress_update_loading( 0, im_skip_dir( filename ) );
 
 	if( !(ws = workspace_new_from_file( main_workspacegroup, filename )) ) 
 		box_alert( NULL );
@@ -530,7 +528,7 @@ main_link_package( im_package *pack)
 
 			sym = symbol_new( symbol_root->expr->compile,
 				pack->table[i]->name );
-			assert( sym->type == SYM_ZOMBIE );
+			g_assert( sym->type == SYM_ZOMBIE );
 			sym->type = SYM_EXTERNAL;
 			sym->function = pack->table[i];
 			sym->fn_nargs = vips_n_args( pack->table[i] );
@@ -572,8 +570,6 @@ main_load_startup( void )
 	printf( "plug-ins init\n" );
 #endif/*DEBUG*/
 
-	main_splash_update( "%s", _( "Loading plugins" ) );
-
 	/* Load any plug-ins on PATH_START. 
 	 */
 	(void) path_map_exact( PATH_START, "*.plg", 
@@ -604,7 +600,7 @@ main_load_startup( void )
 static void *
 main_junk_auto_load( Filemodel *filemodel )
 {
-	assert( IS_FILEMODEL( filemodel ) );
+	g_assert( IS_FILEMODEL( filemodel ) );
 
 	if( filemodel->auto_load )
 		IDESTROY( filemodel );
@@ -617,7 +613,7 @@ main_junk_auto_load( Filemodel *filemodel )
 void
 main_reload( void )
 {
-	set_hourglass();
+	progress_begin();
 
 	/* Remove.
 	 */
@@ -635,7 +631,7 @@ main_reload( void )
 	 */
 	watch_relink_all();
 
-	set_pointer();
+	progress_end();
 }
 
 /* Use a file to paint a named stock item.
@@ -796,16 +792,22 @@ main_x_init( int *argc, char ***argv )
         gtk_widget_realize( main_window_top );
 	main_window_gdk = main_window_top->window;
 
-#ifdef DEBUG
+#ifdef DEBUG_UPDATES
 	printf( "*** debug updates is on\n" );
 	gdk_window_set_debug_updates( TRUE );
-#endif /*DEBUG*/
+#endif /*DEBUG_UPDATES*/
 
 	main_register_icons();
 
 	if( main_splash_enabled() ) {
 		main_splash = splash_new();
 		gtk_widget_show( GTK_WIDGET( main_splash ) );
+
+		/* Wait for the splashscreen to pop up. We want this to appear
+		 * as quickly as possible.
+		 */
+		while( g_main_context_iteration( NULL, FALSE ) )
+			;
 	}
 
 	/* Next window we make is end of startup.
@@ -845,7 +847,7 @@ main_toobig_done( iWindow *iwnd,
 	if( main_imageinfogroup )
 		iobject_changed( IOBJECT( main_imageinfogroup ) );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Test for a bunch of stuff in the TMP area. Need to do this before
@@ -858,14 +860,14 @@ main_check_temp( double total )
 {
 	if( total > 10 * 1024 * 1024 ) {
 		char txt[256];
-		BufInfo buf;
+		VipsBuf buf;
 		char tmp[FILENAME_MAX];
 
 		expand_variables( PATH_TMP, tmp );
 		nativeize_path( tmp );
 		absoluteize_path( tmp );
 
-		buf_init_static( &buf, txt, 256 );
+		vips_buf_init_static( &buf, txt, 256 );
 		to_size( &buf, total );
 
 		box_yesno( NULL,
@@ -877,7 +879,7 @@ main_check_temp( double total )
 			"Would you like to empty the temp area? "
 			"This will delete any workspace backups and "
 			"cannot be undone." ),
-			tmp, buf_all( &buf ) );
+			tmp, vips_buf_all( &buf ) );
 	}
 }
 
@@ -964,7 +966,7 @@ main( int argc, char *argv[] )
 #endif /*HAVE_FFTW*/
 	Toolkit *kit;
 	char txt[MAX_STRSIZE];
-	BufInfo buf;
+	VipsBuf buf;
 
 #ifdef DEBUG_LEAK
 #ifdef PROFILE
@@ -1072,15 +1074,6 @@ main( int argc, char *argv[] )
 	bindtextdomain( GETTEXT_PACKAGE, name );
 	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
 
-#ifdef G_THREADS_ENABLED
-	/* If VIPS has been configured without threads, we will need to start
-	 * the thread system ourselves so that we can make the async queue in
-	 * conversion.c
-	 */
-        if( !g_thread_supported() )
-                g_thread_init( NULL );
-#endif /*G_THREADS_ENABLED*/
-
 	context = g_option_context_new( _( "- image processing spreadsheet" ) );
 	g_option_context_add_main_entries( context, 
 		main_option, GETTEXT_PACKAGE );
@@ -1099,12 +1092,25 @@ main( int argc, char *argv[] )
 
 	g_option_context_free( context );
 
+	/* Override the install guess from vips. This won't pick up msg
+	 * cats sadly :( since we have to init i18n before arg parsing. Handy
+	 * for testing without installing.
+	 */
+	if( main_option_prefix ) {
+		char tmp[FILENAME_MAX];
+
+		im_strncpy( tmp, main_option_prefix, FILENAME_MAX );
+		nativeize_path( tmp );
+		absoluteize_path( tmp );
+		prefix = im_strdupn( tmp );
+		setenvf( "VIPSHOME", "%s", prefix );
+	}
+
 	if( main_option_version ) {
 		printf( "%s-%s", PACKAGE, VERSION );
 		printf( "\n" );
 
-		printf( _( "linked to vips-%s" ), 
-			im_version_string() );
+		printf( _( "linked to vips-%s" ), im_version_string() );
 		printf( "\n" );
 
 		exit( 0 );
@@ -1147,9 +1153,12 @@ main( int argc, char *argv[] )
 	main_stdin = file_open_read_stdin();
 
 #ifdef HAVE_GETRLIMIT
-	/* Make sure we have lots of file descriptors.
+	/* Make sure we have lots of file descriptors. Some platforms have cur
+	 * as 256 and max at 1024 to keep stdio happy.
 	 */
 	if( getrlimit( RLIMIT_NOFILE, &rlp ) == 0 ) {
+		rlim_t old_limit = rlp.rlim_cur;
+
 		rlp.rlim_cur = rlp.rlim_max;
 		if( setrlimit( RLIMIT_NOFILE, &rlp ) == 0 ) {
 #ifdef DEBUG
@@ -1157,11 +1166,13 @@ main( int argc, char *argv[] )
 				(int) rlp.rlim_max );
 #endif /*DEBUG*/
 		}
-		else {
-			g_warning( _( "unable to change max file "
-				"descriptors\n"
+		else if( (int) rlp.rlim_max != -1 ) {
+			/* -1 means can't-be-set, at least on os x, so don't
+			 * warn.
+			 */
+			g_warning( _( "unable to change max file descriptors\n"
 				"max file descriptors still set to %d" ),
-				(int) rlp.rlim_cur );
+				(int) old_limit );
 		}
 	}
 	else {
@@ -1169,6 +1180,10 @@ main( int argc, char *argv[] )
 	}
 #endif /*HAVE_GETRLIMIT*/
 
+	/* Make our file types.
+	 */
+	filesel_startup();
+
 	/* Set default values for paths.
 	 */
 	path_init();
@@ -1217,38 +1232,22 @@ main( int argc, char *argv[] )
 	g_object_ref( G_OBJECT( main_imageinfogroup ) );
 	iobject_sink( IOBJECT( main_imageinfogroup ) );
 
-#ifdef DEBUG
-	printf( "arg processing\n" );
-#endif/*DEBUG*/
-
-	/* Might make this from stdin/whatever if we have a special
-	 * command-line flag.
-	 */
-	ws = NULL;
-
-	/* Interpret compound options first: these imply other options.
+	/* First pass at command-line options. Just look at the flags that
+	 * imply other flags, don't do any processing yet.
 	 */
 	if( main_option_script ) {
-		if( !toolkit_new_from_file( main_toolkitgroup,
-			main_option_script ) )
-			main_log_add( "%s\n", error_get_sub() );
-
 		main_option_batch = TRUE;
 		main_option_no_load_menus = TRUE;
 		main_option_no_load_args = TRUE;
 		main_option_print_main = TRUE;
 	}
 
+	if( main_option_test ) {
+		main_option_batch = TRUE;
+		main_option_verbose = TRUE;
+	}
+
 	if( main_option_expression ) {
-		kit = toolkit_new( main_toolkitgroup, "_expression" );
-
-		buf_init_static( &buf, txt, MAX_STRSIZE );
-		buf_appendf( &buf, "main = %s;", main_option_expression );
-		attach_input_string( buf_all( &buf ) );
-		(void) parse_onedef( kit, -1 );
-
-		filemodel_set_modified( FILEMODEL( kit ), FALSE );
-
 		main_option_batch = TRUE;
 		main_option_no_load_menus = TRUE;
 		main_option_no_load_args = TRUE;
@@ -1268,18 +1267,58 @@ main( int argc, char *argv[] )
 		main_option_no_load_menus = FALSE;
 	}
 
+#ifdef DEBUG
+	if( main_option_batch ) 
+		printf( "non-interactive mode\n" );
+#endif /*DEBUG*/
+
+	/* Start the X connection. We need this before _load_all(), so that
+	 * we can pop up error dialogs.
+	 */
+	if( !main_option_batch )
+		main_x_init( &argc, &argv );
+
+	/* Load start-up stuff. Builtins, plugins, externals etc. We need to
+	 * do this before we load any user code so we can prevent redefinition
+	 * of builtins.
+	 */
+	main_load_startup();
+
+#ifdef DEBUG
+	printf( "arg processing\n" );
+#endif/*DEBUG*/
+
+	/* Might make this from stdin/whatever if we have a special
+	 * command-line flag.
+	 */
+	ws = NULL;
+
+	/* Second command-line pass. This time we do any actions.
+	 */
+	if( main_option_script ) {
+		if( !toolkit_new_from_file( main_toolkitgroup,
+			main_option_script ) )
+			main_log_add( "%s\n", error_get_sub() );
+	}
+
+	if( main_option_expression ) {
+		kit = toolkit_new( main_toolkitgroup, "_expression" );
+
+		vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+		vips_buf_appendf( &buf, "main = %s;", main_option_expression );
+		attach_input_string( vips_buf_all( &buf ) );
+		(void) parse_onedef( kit, -1 );
+
+		filemodel_set_modified( FILEMODEL( kit ), FALSE );
+	}
+
 	if( main_option_stdin_def ) {
-		/* Load stdin as a set of defs. Good for HEREIS 
-		 * shell scripts.
-		 */
 		if( !(kit = toolkit_new_from_openfile( 
 			main_toolkitgroup, main_stdin )) )
 			main_log_add( "%s\n", error_get_sub() );
 	}
 
 	if( main_option_stdin_ws ) {
-		/* Load stdin as a workspace.
-		 */
 		if( !(ws = workspace_new_from_openfile( 
 			main_workspacegroup, main_stdin )) ) 
 			main_log_add( "%s\n", error_get_sub() );
@@ -1296,33 +1335,17 @@ main( int argc, char *argv[] )
 		ws = workspace_new_blank( main_workspacegroup, name );
 	}
 
-#ifdef DEBUG
-	if( main_option_batch ) 
-		printf( "non-interactive mode\n" );
-#endif /*DEBUG*/
-
-	/* Start the X connection. We need this before _load_all(), so that
-	 * we can pop up error dialogs.
-	 */
-	if( !main_option_batch )
-		main_x_init( &argc, &argv );
-
 	/* Reset IM_CONCURRENCY if a watch changes. Need to do this after
 	 * parsing options so we skip in batch mode.
 	 */
 	g_signal_connect( main_watchgroup, "watch_changed", 
 		G_CALLBACK( main_watchgroup_changed_cb ), NULL );
 
-	/* Load start-up stuff.
-	 */
-	main_load_startup();
-
 	/* Recalc to build all classes. We have to do this in batch
 	 * mode since we can find dirties through dynamic lookups. Even though
 	 * you might think we could just follow recomps.
 	 */
-	main_splash_update( "%s", _( "First recalculation ..." ) );
-	symbol_recalculate_all();
+	symbol_recalculate_all_force( TRUE );
 
 	/* Measure amount of stuff in temp area ... need this for checking
 	 * temps later. We pop a dialog if there are too many, so only useful
@@ -1334,21 +1357,21 @@ main( int argc, char *argv[] )
 	/* Make nip's argc/argv[].
 	 */
 	kit = toolkit_new( main_toolkitgroup, "_args" );
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appendf( &buf, "argc = %d;", argc );
-	attach_input_string( buf_all( &buf ) );
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_appendf( &buf, "argc = %d;", argc );
+	attach_input_string( vips_buf_all( &buf ) );
 	(void) parse_onedef( kit, -1 );
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appendf( &buf, "argv = [" );
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_appendf( &buf, "argv = [" );
 	for( i = 0; i < argc; i++ ) {
 		if( i > 0 )
-			buf_appendf( &buf, ", " );
-		buf_appendf( &buf, "\"%s\"", argv[i] );
+			vips_buf_appendf( &buf, ", " );
+		vips_buf_appendf( &buf, "\"%s\"", argv[i] );
 	}
-	buf_appendf( &buf, "];" );
+	vips_buf_appendf( &buf, "];" );
 
-	attach_input_string( buf_all( &buf ) );
+	attach_input_string( vips_buf_all( &buf ) );
 	if( !parse_onedef( kit, -1 ) ) 
 		main_log_add( "%s\n", error_get_sub() );
 
@@ -1382,8 +1405,7 @@ main( int argc, char *argv[] )
 
 	/* Recalc to load any args.
 	 */
-	main_splash_update( "%s", _( "Final recalculation ..." ) );
-	symbol_recalculate_all();
+	symbol_recalculate_all_force( TRUE );
 
 	/* Make sure our start ws doesn't have modified set. We may have
 	 * loaded some images or whatever into it.
@@ -1404,7 +1426,6 @@ main( int argc, char *argv[] )
 		if( !workspace_is_empty( ws ) || mainw_number() == 0 ) {
 			Mainw *mainw;
 			
-			main_splash_update( _( "Building main window" ) );
 			mainw = mainw_new( ws );
 			gtk_widget_show( GTK_WIDGET( mainw ) );
 		}
@@ -1461,6 +1482,11 @@ _( "A new directory has been created in your home directory to hold startup, "
 		gtk_main();
 	}
 
+	if( main_option_test && expr_error_all )
+		main_error_exit( "--test: errors found" );
+
+	/* No return from this.
+	 */
 	main_quit();
 
 	return( 0 );
