@@ -91,7 +91,7 @@ mainw_recent_freeze( void )
 void
 mainw_recent_thaw( void )
 {
-	assert( mainw_recent_freeze_count > 0 );
+	g_assert( mainw_recent_freeze_count > 0 );
 
 	mainw_recent_freeze_count -= 1;
 }
@@ -150,6 +150,9 @@ mainw_destroy( GtkObject *object )
 	FREESID( mainw->imageinfo_changed_sid, main_imageinfogroup );
 	FREESID( mainw->heap_changed_sid, reduce_context->heap );
 	FREESID( mainw->watch_changed_sid, main_watchgroup );
+	FREESID( mainw->begin_sid, progress_get() );
+	FREESID( mainw->update_sid, progress_get() );
+	FREESID( mainw->end_sid, progress_get() );
 	DESTROY_GTK( mainw->popup );
 	UNREF( mainw->kitgview );
 
@@ -180,7 +183,7 @@ mainw_map( GtkWidget *widget )
 {
 	Mainw *mainw = MAINW( widget );
 
-	assert( !mainw->compat_timeout );
+	g_assert( !mainw->compat_timeout );
 
 	if( mainw->ws->compat_major )
 		mainw->compat_timeout = g_timeout_add( 500, 
@@ -215,6 +218,32 @@ mainw_class_init( MainwClass *class )
 	widget_class->configure_event = mainw_configure_event;
 }
 
+static void
+mainw_progress_begin( Progress *progress, Mainw *mainw )
+{
+	mainw->cancel = FALSE;
+        gtk_widget_show( mainw->progress_box );
+}
+
+static void
+mainw_progress_update( Progress *progress, gboolean *cancel, Mainw *mainw )
+{
+	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( mainw->progress ), 
+		vips_buf_all( &progress->feedback ) );
+	gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( mainw->progress ), 
+		IM_CLIP( 0.0, (double) progress->percent / 100.0, 1.0 ) );
+
+	if( mainw->cancel )
+		*cancel = TRUE;
+} 
+
+static void
+mainw_progress_end( Progress *progress, Mainw *mainw )
+{
+        gtk_widget_hide( mainw->progress_box );
+	mainw->cancel = FALSE;
+}
+
 static void
 mainw_init( Mainw *mainw )
 {
@@ -225,6 +254,14 @@ mainw_init( Mainw *mainw )
 	mainw->heap_changed_sid = 0;
 	mainw->watch_changed_sid = 0;
 
+	mainw->begin_sid = g_signal_connect( progress_get(), "begin", 
+		G_CALLBACK( mainw_progress_begin ), mainw );
+	mainw->update_sid = g_signal_connect( progress_get(), "update", 
+		G_CALLBACK( mainw_progress_update ), mainw );
+	mainw->end_sid = g_signal_connect( progress_get(), "end", 
+		G_CALLBACK( mainw_progress_end ), mainw );
+	mainw->cancel = FALSE;
+
 	mainw->free_type = FALSE;
 
 	mainw->toolbar_visible = MAINW_TOOLBAR;
@@ -243,6 +280,8 @@ mainw_init( Mainw *mainw )
 	mainw->statusbar = NULL;
 	mainw->space_free = NULL;
 	mainw->space_free_eb = NULL;
+	mainw->progress_box = NULL;
+	mainw->progress = NULL;
 
 	mainw->lpane = NULL;
 	mainw->rpane = NULL;
@@ -275,6 +314,12 @@ mainw_get_type( void )
 	return( type );
 }
 
+static void
+mainw_cancel_cb( GtkWidget *wid, Mainw *mainw )
+{
+	mainw->cancel = TRUE;
+}
+
 static void
 mainw_workspace_destroy_cb( Workspace *ws, Mainw *mainw )
 {
@@ -286,24 +331,24 @@ mainw_workspace_destroy_cb( Workspace *ws, Mainw *mainw )
 }
 
 static void
-mainw_find_disc( BufInfo *buf )
+mainw_find_disc( VipsBuf *buf )
 {
 	double sz = find_space( PATH_TMP );
 
 	if( sz < 0 )
-		buf_appendf( buf, _( "No temp area" ) );
+		vips_buf_appendf( buf, _( "No temp area" ) );
 	else {
 		char txt[MAX_STRSIZE];
-		BufInfo buf2;
+		VipsBuf buf2;
 
-		buf_init_static( &buf2, txt, MAX_STRSIZE );
+		vips_buf_init_static( &buf2, txt, MAX_STRSIZE );
 		to_size( &buf2, sz );
-		buf_appendf( buf, _( "%s free" ), buf_all( &buf2 ) );
+		vips_buf_appendf( buf, _( "%s free" ), vips_buf_all( &buf2 ) );
 	}
 }
 
 static void
-mainw_find_heap( BufInfo *buf, Heap *heap )
+mainw_find_heap( VipsBuf *buf, Heap *heap )
 {
 	/* How much we can still expand the heap by ... this 
 	 * can be -ve if we've closed a workspace, or changed 
@@ -311,7 +356,7 @@ mainw_find_heap( BufInfo *buf, Heap *heap )
 	 */
 	int togo = IM_MAX( 0, (heap->mxb - heap->nb) * heap->rsz );
 
-	buf_appendf( buf, _( "%d cells free" ), heap->nfree + togo );
+	vips_buf_appendf( buf, _( "%d cells free" ), heap->nfree + togo );
 }
 
 /* Update the space remaining indicator. 
@@ -321,15 +366,15 @@ mainw_free_update( Mainw *mainw )
 {
 	Heap *heap = reduce_context->heap;
 	char txt[80];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 80 );
+	vips_buf_init_static( &buf, txt, 80 );
 
 	if( workspace_selected_any( mainw->ws ) ) {
 		/* Display select message instead.
 		 */
-		buf_appends( &buf, _( "Selected:" ) );
-		buf_appends( &buf, " " );
+		vips_buf_appends( &buf, _( "Selected:" ) );
+		vips_buf_appends( &buf, " " );
 		workspace_selected_names( mainw->ws, &buf, ", " );
 	}
 	else {
@@ -344,47 +389,51 @@ mainw_free_update( Mainw *mainw )
 			mainw_find_disc( &buf );
 	}
 
-	set_glabel( mainw->space_free, "%s", buf_all( &buf ) );
+	set_glabel( mainw->space_free, "%s", vips_buf_all( &buf ) );
 }
 
 static void
 mainw_title_update( Mainw *mainw )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[512];
 
-	buf_init_static( &buf, txt, 512 );
-	buf_appendf( &buf, "%s", NN( IOBJECT( mainw->ws->sym )->name ) );
+	vips_buf_init_static( &buf, txt, 512 );
+	if( FILEMODEL( mainw->ws )->modified ) 
+		vips_buf_appendf( &buf, "*" ); 
+	vips_buf_appendf( &buf, "%s", NN( IOBJECT( mainw->ws->sym )->name ) );
 	if( mainw->ws->compat_major ) {
-		buf_appends( &buf, " - " );
-		buf_appends( &buf, _( "compatibility mode" ) );
-		buf_appendf( &buf, " %d.%d", 
+		vips_buf_appends( &buf, " - " );
+		vips_buf_appends( &buf, _( "compatibility mode" ) );
+		vips_buf_appendf( &buf, " %d.%d", 
 			mainw->ws->compat_major, 
 			mainw->ws->compat_minor ); 
 	}
 	if( FILEMODEL( mainw->ws )->filename )
-		buf_appendf( &buf, " - %s", FILEMODEL( mainw->ws )->filename );
-	if( FILEMODEL( mainw->ws )->modified ) {
-		buf_appendf( &buf, " [" ); 
-		buf_appendf( &buf, _( "modified" ) ); 
-		buf_appendf( &buf, "]" ); 
+		vips_buf_appendf( &buf, " - %s", FILEMODEL( mainw->ws )->filename );
+	else {
+		vips_buf_appends( &buf, " - " );
+		vips_buf_appends( &buf, _( "unsaved workspace" ) );
 	}
 
-	iwindow_set_title( IWINDOW( mainw ), buf_all( &buf ) );
+	iwindow_set_title( IWINDOW( mainw ), "%s", vips_buf_all( &buf ) );
 }
 
 static void 
 mainw_status_update( Mainw *mainw )
 {
-	Workspace *ws = mainw->ws;
+	if( mainw->ws->status ) {
+		gtk_label_set_text( GTK_LABEL( mainw->statusbar ), 
+			mainw->ws->status ); 
+	}
+	else {
+		char txt[256];
 
-	gtk_statusbar_pop( GTK_STATUSBAR( mainw->statusbar ), 0 );
-	if( symbol_busy() )
-		gtk_statusbar_push( GTK_STATUSBAR( mainw->statusbar ), 0, 
-			_( "Calculating ..." ) );
-	else if( ws->status )
-		gtk_statusbar_push( GTK_STATUSBAR( mainw->statusbar ), 0, 
-			ws->status );
+		im_snprintf( txt, 256, 
+			_( "%s: &#0169;2009 Imperial College, London" ), 
+			PACKAGE );
+		gtk_label_set_markup( GTK_LABEL( mainw->statusbar ), txt );
+	}
 }
 
 static gint
@@ -412,12 +461,12 @@ mainw_jump_build( Column *column, GtkWidget *menu )
 {
 	GtkWidget *item;
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 256 );
-	buf_appendf( &buf, "%s - %s", 
+	vips_buf_init_static( &buf, txt, 256 );
+	vips_buf_appendf( &buf, "%s - %s", 
 		IOBJECT( column )->name, IOBJECT( column )->caption );
-	item = gtk_menu_item_new_with_label( buf_all( &buf ) );
+	item = gtk_menu_item_new_with_label( vips_buf_all( &buf ) );
 	g_signal_connect( item, "activate",
 		G_CALLBACK( mainw_jump_column_cb ), column );
 	gtk_menu_append( GTK_MENU( menu ), item );
@@ -540,7 +589,7 @@ mainw_space_free_event( GtkWidget *widget, GdkEvent *ev, Mainw *mainw )
 }
 
 static void
-mainw_space_free_tooltip_generate( GtkWidget *widget, BufInfo *buf, 
+mainw_space_free_tooltip_generate( GtkWidget *widget, VipsBuf *buf, 
 	Mainw *mainw )
 {
 	Heap *heap = reduce_context->heap;
@@ -548,20 +597,19 @@ mainw_space_free_tooltip_generate( GtkWidget *widget, BufInfo *buf,
 
 	mainw_find_disc( buf );
 	/* Expands to (eg.) "14GB free in /pics/tmp" */
-        buf_appendf( buf, _( " in \"%s\"" ), PATH_TMP );
-        buf_appends( buf, ", " );
+        vips_buf_appendf( buf, _( " in \"%s\"" ), PATH_TMP );
+        vips_buf_appends( buf, ", " );
 
-        buf_appendf( buf, 
+        vips_buf_appendf( buf, 
 		_( "%d cells in heap, %d cells free, %d cells maximum" ),
                 heap->ncells, heap->nfree, heap->max_fn( heap ) );
-        buf_appends( buf, ", " );
+        vips_buf_appends( buf, ", " );
 
-        buf_appendf( buf, _( "%d objects in workspace" ),
+        vips_buf_appendf( buf, _( "%d objects in workspace" ),
 		g_slist_length( ICONTAINER( sym->expr->compile )->children ) );
-        buf_appends( buf, ", " );
+        vips_buf_appends( buf, ", " );
 
-        buf_appendf( buf, _( "%d vips calls cached" ),
-		vips_history_size );
+        vips_buf_appendf( buf, _( "%d vips calls cached" ), vips_history_size );
 }
 
 static void
@@ -614,15 +662,15 @@ mainw_clone( Mainw *mainw )
 
 	/* Clone selected symbols.
 	 */
-	set_hourglass();
+	progress_begin();
 	if( !workspace_clone_selected( ws ) ) { 
 		box_alert( GTK_WIDGET( mainw ) );
-		set_pointer();
+		progress_end();
 		return;
 	}
 	symbol_recalculate_all();
 	workspace_deselect_all( ws );
-	set_pointer();
+	progress_end();
 
 	model_scrollto( MODEL( ws->current ), MODEL_SCROLL_TOP );
 }
@@ -638,10 +686,10 @@ mainw_duplicate_action_cb( GtkAction *action, Mainw *mainw )
 static void
 mainw_ungroup_action_cb( GtkAction *action, Mainw *mainw )
 {
-	set_hourglass();
+	progress_begin();
 	if( !workspace_selected_ungroup( mainw->ws ) )
 		box_alert( GTK_WIDGET( mainw ) );
-	set_pointer();
+	progress_end();
 }
 
 /* Group the selected object(s).
@@ -650,7 +698,7 @@ static void
 mainw_group_action_cb( GtkAction *action, Mainw *mainw )
 {
 	Workspace *ws = mainw->ws;
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[MAX_STRSIZE];
 
 	if( !workspace_selected_any( ws ) ) {
@@ -665,11 +713,11 @@ mainw_group_action_cb( GtkAction *action, Mainw *mainw )
 		row_select( row );
 	}
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Group [" );
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	vips_buf_appends( &buf, "Group [" );
 	workspace_selected_names( ws, &buf, "," );
-	buf_appends( &buf, "]" );
-	if( !workspace_add_def( ws, buf_all( &buf ) ) ) {
+	vips_buf_appends( &buf, "]" );
+	if( !workspace_add_def( ws, vips_buf_all( &buf ) ) ) {
 		box_alert( GTK_WIDGET( mainw ) );
 		return;
 	}
@@ -709,7 +757,7 @@ mainw_find_again_action_cb( GtkAction *action, Mainw *mainw )
 static Row *
 mainw_test_error( Row *row, Mainw *mainw, int *found )
 {
-	assert( row->err );
+	g_assert( row->err );
 
 	/* Found next?
 	 */
@@ -731,7 +779,7 @@ mainw_test_error( Row *row, Mainw *mainw, int *found )
 static void
 mainw_next_error_action_cb( GtkAction *action, Mainw *mainw )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char str[MAX_LINELENGTH];
 
 	int found;
@@ -759,15 +807,15 @@ mainw_next_error_action_cb( GtkAction *action, Mainw *mainw )
 
 	/* *must* have one now.
 	 */
-	assert( mainw->row_last_error && mainw->row_last_error->err );
+	g_assert( mainw->row_last_error && mainw->row_last_error->err );
 
 	model_scrollto( MODEL( mainw->row_last_error ), MODEL_SCROLL_TOP );
 
-	buf_init_static( &buf, str, MAX_LINELENGTH );
+	vips_buf_init_static( &buf, str, MAX_LINELENGTH );
 	row_qualified_name( mainw->row_last_error->expr->row, &buf );
-	buf_appends( &buf, ": " );
-	buf_appends( &buf, mainw->row_last_error->expr->error_top );
-	workspace_set_status( mainw->ws, "%s", buf_firstline( &buf ) );
+	vips_buf_appends( &buf, ": " );
+	vips_buf_appends( &buf, mainw->row_last_error->expr->error_top );
+	workspace_set_status( mainw->ws, "%s", vips_buf_firstline( &buf ) );
 }
 
 static void
@@ -785,7 +833,7 @@ mainw_selected_recalc_dia( iWindow *iwnd, void *client,
 	Mainw *mainw = MAINW( client );
 
 	if( workspace_selected_recalc( mainw->ws ) )
-		nfn( sys, IWINDOW_TRUE );
+		nfn( sys, IWINDOW_YES );
 	else
 		nfn( sys, IWINDOW_ERROR );
 }
@@ -799,10 +847,10 @@ mainw_force_calc_action_cb( GtkAction *action, Mainw *mainw )
 	Workspace *ws = mainw->ws;
 
         if( workspace_selected_any( ws ) ) {
-		BufInfo buf;
+		VipsBuf buf;
 		char str[30];
 
-		buf_init_static( &buf, str, 30 );
+		vips_buf_init_static( &buf, str, 30 );
 		workspace_selected_names( ws, &buf, ", " );
 
 		box_yesno( GTK_WIDGET( mainw ), 
@@ -811,7 +859,7 @@ mainw_force_calc_action_cb( GtkAction *action, Mainw *mainw )
 			_( "Recalculate" ), 
 			_( "Completely recalculate?" ),
 			( "Are you sure you want to "
-			"completely recalculate %s?" ), buf_all( &buf ) );
+			"completely recalculate %s?" ), vips_buf_all( &buf ) );
 	}
 	else 
 		/* Recalculate.
@@ -835,7 +883,7 @@ mainw_workspace_save_as_action_cb( GtkAction *action, Mainw *mainw )
 	filemodel_inter_saveas( IWINDOW( mainw ), FILEMODEL( mainw->ws ) );
 }
 
-static Workspace *
+Workspace *
 mainw_open_file_into_workspace( Mainw *mainw, const char *filename )
 {
 	Workspacegroup *wsg = WORKSPACEGROUP( ICONTAINER( mainw->ws )->parent );
@@ -868,61 +916,49 @@ mainw_open_file_into_workspace( Mainw *mainw, const char *filename )
 		iwindow_kill( IWINDOW( mainw ) );
 	}
 
+	/* Can't rely on filename still being around ... read filename
+	 * from new ws instead.
+	 */
+	mainw_recent_add( &mainw_recent_workspace, 
+		FILEMODEL( new_ws )->filename );
+
+	/* Process some events to make sure we rethink the layout and
+	 * are able to get the append-at-RHS offsets.
+	 */
+	while( g_main_context_iteration( NULL, FALSE ) )
+		;
+
+	symbol_recalculate_all();
+
 	return( new_ws );
 }
 
-/* Try to open a file ... workspace, image or matrix.
+/* Track these during a load.
  */
-Filemodel *
-mainw_open_file( Mainw *mainw, const char *filename )
-{
-	Workspace *ws;
-	Symbol *sym;
-	char *utf8;
+typedef struct {
+	Mainw *mainw;
+	VipsBuf *buf;
+	int nitems;
+} MainwLoad;
 
-	/* Do this first since into_ws needs to be able to spot blank
-	 * workspaces.
-	 */
-	if( (ws = mainw_open_file_into_workspace( mainw, filename )) ) {
-		/* Can't rely on filename still being around ... read filename
-		 * from new ws instead.
-		 */
-		mainw_recent_add( &mainw_recent_workspace, 
-			FILEMODEL( ws )->filename );
-
-		/* Process some events to make sure we rethink the layout and
-		 * are able to get the append-at-RHS offsets.
-		 */
-		while( g_main_context_iteration( NULL, FALSE ) )
-			;
-
-		symbol_recalculate_all();
-
-		return( FILEMODEL( ws ) );
-	}
-	error_clear();
-
-	if( (sym = workspace_load_file( mainw->ws, filename )) )
-		return( FILEMODEL( sym ) );
-
-	utf8 = f2utf8( filename );
-	error_top( _( "Load failed." ) );
-	error_sub( _( "Unable to load from file \"%s\". "
-		"Error loading as image, workspace or matrix." ), utf8 );
-	g_free( utf8 );
-
-	return( NULL );
-}
-
-/* Try to open a file ... Image or matrix.
+/* Try to open a file. Workspace files we load immediately, other ones we
+ * add the load to a buffer.
  */
 static void *
-mainw_open_fn( Filesel *filesel, const char *filename, void *a, void *b )
+mainw_open_fn( Filesel *filesel, const char *filename, MainwLoad *load )
 {
-	Mainw *mainw = MAINW( a );
-
-	if( !mainw_open_file( mainw, filename ) )
-		return( filesel );
+	if( is_file_type( &filesel_wfile_type, filename ) ) {
+		if( !mainw_open_file_into_workspace( load->mainw, filename ) )
+			return( filesel );
+	}
+	else {
+		if( load->nitems ) 
+			vips_buf_appends( load->buf, ", " );
+		if( !workspace_load_file_buf( load->buf, filename ) )
+			return( filesel );
+		mainw_recent_add( &mainw_recent_image, filename );
+		load->nitems += 1;
+	}
 
 	return( NULL );
 }
@@ -935,14 +971,40 @@ mainw_open_done_cb( iWindow *iwnd, void *client,
 {
 	Mainw *mainw = MAINW( client );
 	Filesel *filesel = FILESEL( iwnd );
+	int nselected = filesel_nselected( filesel );
+	char txt[MAX_STRSIZE];
+	VipsBuf buf;
+	MainwLoad load;
 
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	load.mainw = mainw;
+	load.buf = &buf;
+	load.nitems = 0;
+
+	if( nselected > 1 )
+		vips_buf_appends( &buf, "Group [" );
 	if( filesel_map_filename_multi( filesel,
-		mainw_open_fn, mainw, NULL ) ) {
+		(FileselMapFn) mainw_open_fn, &load, NULL ) ) {
+		nfn( sys, IWINDOW_ERROR );
+		return;
+	}
+	if( nselected > 1 )
+		vips_buf_appends( &buf, "]" );
+
+	/* We may have loaded a workspace and caused this workspace to
+	 * vanish, argh. mainw may no longer be valid, but check ->ws anyway.
+	 * We'll probably see a NULL if this ws has been junked.
+	 */
+	if( load.nitems && 
+		mainw->ws &&
+		!workspace_add_def( mainw->ws, vips_buf_all( &buf ) ) ) {
+		error_top( _( "Load failed." ) );
+		error_sub( _( "Unable to execute:\n   %s" ), vips_buf_all( &buf ) );
 		nfn( sys, IWINDOW_ERROR );
 		return;
 	}
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Show an open file dialog ... any type, but default to one of the image
@@ -1015,18 +1077,33 @@ mainw_open_examples_action_cb( GtkAction *action, Mainw *mainw )
 	mainw_open_examples( mainw );
 }
 
+static gboolean
+mainw_recent_open( Mainw *mainw, const char *filename )
+{
+	if( is_file_type( &filesel_wfile_type, filename ) ) {
+		if( !mainw_open_file_into_workspace( mainw, filename ) )
+			return( FALSE );
+	}
+	else {
+		if( !workspace_load_file( mainw->ws, filename ) )
+			return( FALSE );
+	}
+
+	return( TRUE );
+}
+
 static void
 mainw_recent_open_cb( GtkWidget *widget, const char *filename )
 {
 	Mainw *mainw = MAINW( iwindow_get_root( widget ) );
 
-	set_hourglass();
-	if( !mainw_open_file( mainw, filename ) ) {
+	progress_begin();
+	if( !mainw_recent_open( mainw, filename ) ) {
 		box_alert( widget );
-		set_pointer();
+		progress_end();
 		return;
 	}
-	set_pointer();
+	progress_end();
 
 	symbol_recalculate_all();
 }
@@ -1040,16 +1117,16 @@ mainw_recent_build( GtkWidget *menu, GSList *recent )
 		const char *filename = (const char *) p->data;
 		GtkWidget *item;
 		char txt[80];
-		BufInfo buf;
+		VipsBuf buf;
 		char *utf8;
 
-		buf_init_static( &buf, txt, 80 );
-		buf_appendf( &buf, "    %s", im_skip_dir( filename ) );
-		utf8 = f2utf8( buf_all( &buf ) );
+		vips_buf_init_static( &buf, txt, 80 );
+		vips_buf_appendf( &buf, "    %s", im_skip_dir( filename ) );
+		utf8 = f2utf8( vips_buf_all( &buf ) );
 		item = gtk_menu_item_new_with_label( utf8 );
 		g_free( utf8 );
 		utf8 = f2utf8( filename );
-		set_tooltip( item, utf8 );
+		set_tooltip( item, "%s", utf8 );
 		g_free( utf8 );
 		gtk_menu_append( GTK_MENU( menu ), item );
 		gtk_widget_show( item );
@@ -1166,7 +1243,7 @@ mainw_workspace_merge_done_cb( iWindow *iwnd,
 
 	symbol_recalculate_all();
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Merge ws file into current ws.
@@ -1195,14 +1272,14 @@ mainw_workspace_duplicate_action_cb( GtkAction *action, Mainw *mainw )
 	Workspace *new_ws;
 	Mainw *new_mainw;
 
-        set_hourglass();
+        progress_begin();
 	if( !(new_ws = workspace_clone( mainw->ws )) ) {
-		set_pointer();
+		progress_end();
 		box_alert( GTK_WIDGET( mainw ) );
 		return;
 	}
 	symbol_recalculate_all();
-	set_pointer();
+	progress_end();
 
 	new_mainw = mainw_new( new_ws );
 	gtk_widget_show( GTK_WIDGET( new_mainw ) );
@@ -1280,10 +1357,12 @@ mainw_column_new_cb( iWindow *iwnd, void *client,
 		return;
 	}
 
-	iobject_set( IOBJECT( col ), NULL, caption_text );
+	if( strcmp( caption_text, "" ) != 0 )
+		iobject_set( IOBJECT( col ), NULL, caption_text );
+
 	workspace_column_select( mainw->ws, col );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* Make a new column with a specified name.
@@ -1427,7 +1506,7 @@ mainw_revert_ok_cb( iWindow *iwnd, void *client,
 		symbol_recalculate_all();
 	}
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 void 
@@ -1576,7 +1655,7 @@ static GtkActionEntry mainw_actions[] = {
 		G_CALLBACK( mainw_workspace_save_as_action_cb ) },
 
 	{ "Recover", 
-		NULL, N_( "Re_cover After Crash" ), NULL,
+		NULL, N_( "Search for Workspace _Backups" ), NULL,
 		N_( "Load last automatically backed-up workspace" ), 
 		G_CALLBACK( mainw_recover_action_cb ) },
 
@@ -1828,7 +1907,9 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	GtkWidget *ebox;
 	GtkAccelGroup *accel_group;
 	GError *error;
+	GtkWidget *cancel;
 	GtkWidget *item;
+	Panechild *panechild;
 
 #ifdef DEBUG
 	printf( "mainw_init: %p\n", mainw );
@@ -1899,7 +1980,7 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	 */
         mainw->statusbar_main = gtk_hbox_new( FALSE, 2 );
         gtk_box_pack_end( GTK_BOX( vbox ), 
-		mainw->statusbar_main, FALSE, FALSE, 0 );
+		mainw->statusbar_main, FALSE, FALSE, 2 );
         widget_visible( mainw->statusbar_main, MAINW_STATUSBAR );
 
 	/* Make space free label.
@@ -1930,15 +2011,35 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 
 	/* Make message label.
 	 */
-	mainw->statusbar = gtk_statusbar_new();
+	mainw->statusbar = gtk_label_new( "" );
+	gtk_label_set_ellipsize( GTK_LABEL( mainw->statusbar ), 
+		PANGO_ELLIPSIZE_MIDDLE );
+	/* 6 is enough to stop the statusbar changing height when the progress
+	 * indicator changes visibility.
+	 */
+	gtk_misc_set_padding( GTK_MISC( mainw->statusbar ), 2, 6 );
+	gtk_misc_set_alignment( GTK_MISC( mainw->statusbar ), 0.0, 0.5 );
         gtk_box_pack_start( GTK_BOX( mainw->statusbar_main ), 
 		mainw->statusbar, TRUE, TRUE, 0 );
-	/* Yuk!
-	 */
-	gtk_misc_set_padding( 
-		GTK_MISC( GTK_STATUSBAR( mainw->statusbar )->label ), 2, 2 );
         gtk_widget_show( mainw->statusbar );
 
+        mainw->progress_box = gtk_hbox_new( FALSE, 2 );
+
+	mainw->progress = gtk_progress_bar_new();
+        gtk_box_pack_end( GTK_BOX( mainw->progress_box ), mainw->progress, 
+		FALSE, TRUE, 0 );
+        gtk_widget_show( mainw->progress );
+
+        cancel = gtk_button_new_with_label( "Cancel" );
+        g_signal_connect( cancel, "clicked",
+                G_CALLBACK( mainw_cancel_cb ), mainw );
+        gtk_box_pack_end( GTK_BOX( mainw->progress_box ), cancel, 
+		FALSE, TRUE, 0 );
+        gtk_widget_show( cancel );
+
+        gtk_box_pack_end( GTK_BOX( mainw->statusbar_main ), 
+		mainw->progress_box, FALSE, TRUE, 0 );
+
 	/* Make toolkit/workspace displays.
 	 */
 	mainw->kitgview = TOOLKITGROUPVIEW( 
@@ -1989,11 +2090,18 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 		POPUP_FUNC( mainw_next_error_action_cb2 ) ); 
         popup_attach( mainw->wsview->fixed, mainw->popup, mainw );
 
+	/* Toolkit Browser pane.
+	 */
+	panechild = panechild_new( mainw->rpane, _( "Toolkit Browser" ) );
+	gtk_paned_pack2( GTK_PANED( mainw->rpane ), 
+		GTK_WIDGET( panechild ), TRUE, TRUE );
+	gtk_widget_show( GTK_WIDGET( panechild ) );
+
 	/* Have to put toolkitbrowser in an ebox so the search entry gets
 	 * clipped to the pane size.
 	 */
 	ebox = gtk_event_box_new();
-	gtk_paned_pack2( GTK_PANED( mainw->rpane ), ebox, TRUE, TRUE );
+	gtk_container_add( GTK_CONTAINER( panechild ), GTK_WIDGET( ebox ) );
 	gtk_widget_show( ebox );
 
 	mainw->toolkitbrowser = toolkitbrowser_new();
@@ -2006,10 +2114,15 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 
 	/* Workspace-local defs pane.
 	 */
+	panechild = panechild_new( mainw->lpane, _( "Workspace Definitions" ) );
+	gtk_paned_pack1( GTK_PANED( mainw->lpane ), 
+		GTK_WIDGET( panechild ), TRUE, TRUE );
+	gtk_widget_show( GTK_WIDGET( panechild ) );
+
 	mainw->workspacedefs = workspacedefs_new( mainw );
 	vobject_link( VOBJECT( mainw->workspacedefs ), IOBJECT( mainw->ws ) );
-	gtk_paned_pack1( GTK_PANED( mainw->lpane ), 
-		GTK_WIDGET( mainw->workspacedefs ), TRUE, TRUE );
+	gtk_container_add( GTK_CONTAINER( panechild ), 
+		GTK_WIDGET( mainw->workspacedefs ) );
 	gtk_widget_show( GTK_WIDGET( mainw->workspacedefs ) );
 
 	/* Set start state.
@@ -2037,7 +2150,7 @@ mainw_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 static void
 mainw_link( Mainw *mainw, Workspace *ws )
 {
-	assert( !mainw->ws );
+	g_assert( !mainw->ws );
 
 	mainw->ws = ws;
 	ws->iwnd = IWINDOW( mainw );
