@@ -331,7 +331,7 @@ matrixview_slider_change_cb( Tslider *tslider, Matrixview *matrixview )
 	int y = pos / matrixview->width;
 	int i = x + y * matrix->value.width;
 
-    	assert( pos >= 0 );
+    	g_assert( pos >= 0 );
 
     	/* Install value.
     	 */
@@ -486,7 +486,7 @@ matrixview_text_traverse( GtkSheet *sheet,
 
     	char *new_text = gtk_sheet_cell_get_text( sheet, *new_row, *new_col );
 
-    	BufInfo buf;
+    	VipsBuf buf;
     	char str[MAX_LINELENGTH];
 
 	/* Make a note of what's in this cell before any editing ... "changed"
@@ -496,14 +496,14 @@ matrixview_text_traverse( GtkSheet *sheet,
 	matrixview->cell_row = *new_row;
 	matrixview->cell_col = *new_col;
 
-    	buf_init_static( &buf, str, MAX_LINELENGTH );
+    	vips_buf_init_static( &buf, str, MAX_LINELENGTH );
     	row_qualified_name( row, &buf );
 	/* Expands to (eg) "A2: cell (1,2): 45" ... status line display during
 	 * matrix traverse.
 	 */
-    	buf_appendf( &buf, _( ": cell (%d, %d): %s" ), 
+    	vips_buf_appendf( &buf, _( ": cell (%d, %d): %s" ), 
 		*new_col, *new_row, new_text );
-	workspace_set_status( row->ws, "%s", buf_all( &buf ) );
+	workspace_set_status( row->ws, "%s", vips_buf_all( &buf ) );
 
     	return( TRUE );
 }
@@ -997,7 +997,7 @@ matrixview_refresh( vObject *vobject )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		if( hclip ) {
@@ -1052,7 +1052,7 @@ matrixview_refresh( vObject *vobject )
     		break;
 
     	default:
-    		assert( FALSE );
+    		g_assert( FALSE );
     	}
 
     	/* If we've built a new display, need to show after _refresh.
