@@ -117,7 +117,7 @@ model_loadstate_error( ModelLoadState *state, const char *fmt, ... )
 	va_list ap;
 
 	va_start( ap, fmt );
-	(void) buf_vappendf( &state->error_log, fmt, ap );
+	(void) vips_buf_vappendf( &state->error_log, fmt, ap );
 	va_end( ap );
 }
 
@@ -126,7 +126,7 @@ model_loadstate_error_get( ModelLoadState *state )
 {
 	char *utf8;
 
-	utf8 = f2utf8( buf_all( &state->error_log ) );
+	utf8 = f2utf8( vips_buf_all( &state->error_log ) );
 	error_top( _( "Load failed." ) );
 	error_sub( _( "Unable to load from file \"%s\". Error log is:\n%s" ),
 		state->filename, utf8 );
@@ -149,7 +149,7 @@ model_loadstate_new( const char *filename )
 		model_loadstate_destroy( state );
 		return( NULL );
 	}
-	buf_init_static( &state->error_log, 
+	vips_buf_init_static( &state->error_log, 
 		state->error_log_buffer, MAX_STRSIZE );
 
 	xmlSetGenericErrorFunc( state, 
@@ -179,7 +179,7 @@ model_loadstate_new_openfile( iOpenFile *of )
 		model_loadstate_destroy( state );
 		return( NULL );
 	}
-	buf_init_static( &state->error_log, 
+	vips_buf_init_static( &state->error_log, 
 		state->error_log_buffer, MAX_STRSIZE );
 
 	xmlSetGenericErrorFunc( state, 
@@ -247,7 +247,7 @@ model_loadstate_rewrite( ModelLoadState *state, char *old_rhs, char *new_rhs )
 
 	/* Take copy of lexed and rewritten stuff.
 	 */
-	im_strncpy( new_rhs, buf_all( &lex_text ), MAX_STRSIZE );
+	im_strncpy( new_rhs, vips_buf_all( &lex_text ), MAX_STRSIZE );
 }
 
 View *
@@ -278,7 +278,7 @@ model_register_loadable( ModelClass *model_class )
 void
 model_scrollto( Model *model, ModelScrollPosition position )
 {
-	assert( IS_MODEL( model ) );
+	g_assert( IS_MODEL( model ) );
 
 	g_signal_emit( G_OBJECT( model ), model_signals[SIG_SCROLLTO], 0, 
 		position );
@@ -287,7 +287,7 @@ model_scrollto( Model *model, ModelScrollPosition position )
 void
 model_layout( Model *model )
 {
-	assert( IS_MODEL( model ) );
+	g_assert( IS_MODEL( model ) );
 
 	g_signal_emit( G_OBJECT( model ), model_signals[SIG_LAYOUT], 0 );
 }
@@ -295,7 +295,7 @@ model_layout( Model *model )
 void *
 model_reset( Model *model )
 {
-	assert( IS_MODEL( model ) );
+	g_assert( IS_MODEL( model ) );
 
 	g_signal_emit( G_OBJECT( model ), model_signals[SIG_RESET], 0 );
 
@@ -633,7 +633,7 @@ model_check_destroy_sub( iWindow *iwnd, void *client,
 	IDESTROY( mcd->model );
 	symbol_recalculate_all();
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 /* The model we are watching has been killed, maybe by us.
@@ -641,9 +641,9 @@ model_check_destroy_sub( iWindow *iwnd, void *client,
 static void
 model_check_destroy_destroy_cb( Model *model, ModelCheckDestroy *mcd )
 {
-	assert( IS_MODEL( model ) );
-	assert( IS_MODEL( mcd->model ) );
-	assert( !mcd->idlg || IS_IDIALOG( mcd->idlg ) );
+	g_assert( IS_MODEL( model ) );
+	g_assert( IS_MODEL( mcd->model ) );
+	g_assert( !mcd->idlg || IS_IDIALOG( mcd->idlg ) );
 
 	mcd->model = NULL;
 	mcd->destroy_sid = 0;
@@ -670,7 +670,7 @@ model_check_destroy_finished( void *client, iWindowResult result )
 void
 model_check_destroy( GtkWidget *parent, Model *model )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char str[30];
 	const char *name;
 
@@ -680,9 +680,9 @@ model_check_destroy( GtkWidget *parent, Model *model )
 	mcd->model = model;
 
 	if( IS_SYMBOL( model ) ) {
-		buf_init_static( &buf, str, 30 );
+		vips_buf_init_static( &buf, str, 30 );
 		symbol_qualified_name( SYMBOL( model ), &buf );
-		name = buf_all( &buf );
+		name = vips_buf_all( &buf );
 	}
 	else
 		name = IOBJECT( model )->name;
