@@ -68,7 +68,7 @@ plot_finalize( GObject *gobject )
 	 */
 	image_value_destroy( &plot->value );
 	plot_free_columns( plot );
-	buf_destroy( &plot->caption_buffer );
+	vips_buf_destroy( &plot->caption_buffer );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
@@ -104,15 +104,15 @@ static const char *
 plot_generate_caption( iObject *iobject )
 {
 	Plot *plot = PLOT( iobject );
-	BufInfo *buf = &plot->caption_buffer;
+	VipsBuf *buf = &plot->caption_buffer;
 
-	buf_rewind( buf );
+	vips_buf_rewind( buf );
 	image_value_caption( &plot->value, buf );
-	buf_appendf( buf, ", %d series, %d points", plot->columns, plot->rows );
-	buf_appendf( buf, ", xrange [%g, %g]", plot->xmin, plot->xmax );
-	buf_appendf( buf, ", yrange [%g, %g]", plot->ymin, plot->ymax );
+	vips_buf_appendf( buf, ", %d series, %d points", plot->columns, plot->rows );
+	vips_buf_appendf( buf, ", xrange [%g, %g]", plot->xmin, plot->xmax );
+	vips_buf_appendf( buf, ", yrange [%g, %g]", plot->ymin, plot->ymax );
 
-	return( buf_all( buf ) );
+	return( vips_buf_all( buf ) );
 }
 
 /* Unpack all data formats to XYXYXY.
@@ -155,7 +155,7 @@ plot_unpack( Plot *plot, DOUBLEMASK *mask )
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	if( plot->columns != columns || plot->rows != rows ) {
@@ -218,7 +218,7 @@ plot_unpack( Plot *plot, DOUBLEMASK *mask )
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	xmin = plot->xcolumn[0][0];
@@ -279,7 +279,7 @@ plot_save( Model *model, xmlNode *xnode )
 	if( !set_prop( xthis, "plot_left", "%d", plot->left ) ||
 		!set_prop( xthis, "plot_top", "%d", plot->top ) ||
 		!set_prop( xthis, "plot_mag", "%d", plot->mag ) ||
-		!set_prop( xthis, "show_status", 
+		!set_sprop( xthis, "show_status", 
 			bool_to_char( plot->show_status ) ) )
 		return( NULL );
 
@@ -292,7 +292,7 @@ plot_load( Model *model,
 {
         Plot *plot = PLOT( model );
 
-	assert( IS_RHS( parent ) );
+	g_assert( IS_RHS( parent ) );
 
 	(void) get_iprop( xnode, "plot_left", &plot->left );
 	(void) get_iprop( xnode, "plot_top", &plot->top );
@@ -424,7 +424,7 @@ plot_graphic_save( Classmodel *classmodel,
 	filesel_add_mode( buf );
 
 	if( value->ii )
-		if( !imageinfo_write( value->ii, parent, buf ) )
+		if( !imageinfo_write( value->ii, buf ) )
 			return( FALSE );
 
 	mainw_recent_add( &mainw_recent_image, filename );
@@ -489,7 +489,7 @@ plot_init( Plot *plot )
 	plot->left = 0;
 	plot->top = 0;
 
-	buf_init_dynamic( &plot->caption_buffer, MAX_LINELENGTH );
+	vips_buf_init_dynamic( &plot->caption_buffer, MAX_LINELENGTH );
 
 	iobject_set( IOBJECT( plot ), CLASS_PLOT, NULL );
 
