@@ -67,6 +67,10 @@ static int nargs[] = {3, 3, 3, 1, 2, 3};
  */
 int reduce_total_recomputations = 0;
 
+/* The current expr being reduced. Used for computation feedback messages.
+ */
+static Expr *reduce_current_expr = NULL;
+
 /* Eval error here. Longjmp back a ways.
  */
 void
@@ -105,15 +109,15 @@ void
 reduce_error_typecheck( Reduce *rc, 
 	PElement *e, const char *name, const char *type )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 
 	error_top( _( "Typecheck error." ) );
-	buf_init_static( &buf, txt, 1024 );
-	buf_appendf( &buf, _( "%s expected %s, instead saw:" ), name, type );
-	buf_appends( &buf, "\n  " );
+	vips_buf_init_static( &buf, txt, 1024 );
+	vips_buf_appendf( &buf, _( "%s expected %s, instead saw:" ), name, type );
+	vips_buf_appends( &buf, "\n  " );
 	itext_value_ev( rc, &buf, e );
-	error_sub( "%s", buf_all( &buf ) );
+	error_sub( "%s", vips_buf_all( &buf ) );
 
 	reduce_throw( rc );
 }
@@ -126,6 +130,15 @@ reduce_error_toobig( Reduce *rc, const char *name )
 	reduce_throw( rc );
 }
 
+/* 'get' a list: convert a MANAGEDSTRING into a list, if necessary.
+ */
+void
+reduce_get_list( Reduce *rc, PElement *list )
+{
+	if( !heap_get_list( list ) )
+		reduce_throw( rc );
+}
+
 /* Map over a heap list. Reduce the list spine as we go, don't reduce the
  * heads. 
  */
@@ -144,6 +157,8 @@ reduce_map_list( Reduce *rc,
 		PElement head;
 		void *res;
 
+		reduce_get_list( rc, &e );
+
 		/* Apply user function to the head.
 		 */
 		PEGETHD( &head, &e );
@@ -159,6 +174,61 @@ reduce_map_list( Reduce *rc,
 	return( NULL );
 }
 
+typedef struct _ReduceMapDict {
+	reduce_map_dict_fn fn;
+	void *a;
+	void *b;
+} ReduceMapDict;
+
+static void *
+reduce_map_dict_entry( Reduce *rc, PElement *head, ReduceMapDict *map_dict )
+{
+	char key[256];
+	PElement p1, p2;
+	void *result;
+
+	reduce_spine( rc, head );
+	if( !PEISFLIST( head ) ) 
+		reduce_error_typecheck( rc, head, "reduce_map_dict", "[*]" );
+
+	reduce_get_list( rc, head );
+	PEGETHD( &p1, head );
+	reduce_get_string( rc, &p1, key, 256 );
+	PEGETTL( &p2, head );
+
+	reduce_spine( rc, &p2 );
+	if( !PEISFLIST( &p2 ) ) 
+		reduce_error_typecheck( rc, &p2, "reduce_map_dict", "[*]" );
+
+	reduce_get_list( rc, &p2 );
+	PEGETHD( &p1, &p2 );
+	if( (result = map_dict->fn( rc, key, &p1, map_dict->a, map_dict->b )) )
+		return( result );
+
+	PEGETTL( &p1, &p2 );
+	reduce_spine( rc, &p1 );
+	if( !PEISELIST( &p1 ) ) 
+		reduce_error_typecheck( rc, &p1, "reduce_map_dict", "[]" );
+
+	return( NULL );
+}
+
+/* Map over a list of ["key", value] pairs.
+ */
+void *
+reduce_map_dict( Reduce *rc, PElement *base, 
+	reduce_map_dict_fn fn, void *a, void *b )
+{
+	ReduceMapDict map_dict;
+
+	map_dict.fn = fn;
+	map_dict.a = a;
+	map_dict.b = b;
+
+	return( reduce_map_list( rc, base, 
+		(reduce_map_list_fn) reduce_map_dict_entry, &map_dict, NULL ) );
+}
+
 static void *
 reduce_clone_list_sub( Reduce *rc, PElement *data, PElement *out )
 {
@@ -222,13 +292,15 @@ reduce_get_string( Reduce *rc, PElement *base, char *buf, int n )
 {
 	int sz = n - 1;
 
-	if( PEISSTATIC( base ) ) {
+	reduce_spine( rc, base );
+
+	if( PEISMANAGEDSTRING( base ) ) {
 		/* A static string ... rather than expanding to a list and
 		 * parsing, we can copy directly.
 		 */
-		HeapStaticString *string = PEGETSTATIC( base );
+		Managedstring *managedstring = PEGETMANAGEDSTRING( base );
 
-		im_strncpy( buf, string->text, n );
+		im_strncpy( buf, managedstring->string, n );
 		sz -= strlen( buf );
 	}
 	else {
@@ -595,19 +667,26 @@ reduce_test_char( Reduce *rc, PElement *base, int *sz )
 static gboolean
 reduce_n_is_string( Reduce *rc, PElement *base, int sz )
 {
-	void *result;
-
 	reduce_spine( rc, base );
+
 	if( !PEISLIST( base ) ) 
 		return( FALSE );
 
-	result = reduce_map_list( rc, base, 
-		(reduce_map_list_fn) reduce_test_char, &sz, NULL );
+	/* We know managedstrings are strings without needing to expand them.
+	 */
+	if( PEISMANAGEDSTRING( base ) )
+		return( TRUE );
+	else {
+		void *result;
 
-	if( result == (void *) -1 )
-		return( FALSE );
+		result = reduce_map_list( rc, base, 
+			(reduce_map_list_fn) reduce_test_char, &sz, NULL );
 
-	return( TRUE );
+		if( result == (void *) -1 )
+			return( FALSE );
+
+		return( TRUE );
+	}
 }
 
 /* Test for object is string. Just test the first few elements, so we
@@ -694,15 +773,15 @@ reduce_is_class( Reduce *rc, PElement *klass )
 gboolean
 reduce_is_instanceof_exact( Reduce *rc, const char *name, PElement *instance )
 {
-        BufInfo buf;
+        VipsBuf buf;
         char txt[256];
 
 	if( !reduce_is_class( rc, instance ) )
 		return( FALSE );
 
-        buf_init_static( &buf, txt, 256 );
+        vips_buf_init_static( &buf, txt, 256 );
         symbol_qualified_name( PEGETCLASSCOMPILE( instance )->sym, &buf );
-        if( strcmp( name, buf_all( &buf ) ) == 0 )
+        if( strcmp( name, vips_buf_all( &buf ) ) == 0 )
                 return( TRUE );
 
 	return( FALSE );
@@ -726,40 +805,6 @@ reduce_is_instanceof( Reduce *rc, const char *name, PElement *instance )
 	return( FALSE );
 }
 
-/* Find the length of a list.
- */
-int
-reduce_list_length( Reduce *rc, PElement *base )
-{
-	PElement p;
-	int i;
-
-	/* Reduce to first element.
-	 */
-	p = *base;
-	reduce_spine( rc, &p );
-
-	/* Does it look like the start of a list? 
-	 */
-	if( !PEISLIST( &p ) ) 
-		reduce_error_typecheck( rc, &p, _( "List length" ), "list" );
-
-	/* Loop down list.
-	 */
-	for( i = 0; PEISFLIST( &p ); i++ ) {
-		HeapNode *hn;
-
-		hn = PEGETVAL( &p );
-		PEPOINTRIGHT( hn, &p );
-
-		reduce_spine( rc, &p );
-	}
-
-	assert( PEISELIST( &p ) );
-
-	return( i );
-}
-
 /* Find the length of a list, with a bailout for the largest size we test.
  * Handy for avoiding finding the length of "[1..]".
  */
@@ -779,25 +824,42 @@ reduce_list_length_max( Reduce *rc, PElement *base, int max_length )
 	if( !PEISLIST( &p ) ) 
 		reduce_error_typecheck( rc, &p, _( "List length" ), "list" );
 
-	/* Loop down list.
-	 */
-	for( i = 0; PEISFLIST( &p ); i++ ) {
-		HeapNode *hn;
+	if( PEISMANAGEDSTRING( &p ) ) {
+		Managedstring *managedstring = PEGETMANAGEDSTRING( &p );
 
-		if( i > max_length ) 
-			reduce_error_toobig( rc, "list" );
+		i = strlen( managedstring->string );
+	}
+	else {
+		/* Loop down list.
+		 */
+		for( i = 0; PEISFLIST( &p ); i++ ) {
+			HeapNode *hn;
 
-		hn = PEGETVAL( &p );
-		PEPOINTRIGHT( hn, &p );
+			if( max_length != -1 && i > max_length ) 
+				reduce_error_toobig( rc, "list" );
 
-		reduce_spine( rc, &p );
+			reduce_get_list( rc, &p );
+
+			hn = PEGETVAL( &p );
+			PEPOINTRIGHT( hn, &p );
+
+			reduce_spine( rc, &p );
+		}
+
+		g_assert( PEISELIST( &p ) );
 	}
 
-	assert( PEISELIST( &p ) );
-
 	return( i );
 }
 
+/* Find the length of a list.
+ */
+int
+reduce_list_length( Reduce *rc, PElement *base )
+{
+	return( reduce_list_length_max( rc, base, -1 ) );
+}
+
 /* Point "out" at the nth element of a list. Index from 0.
  */
 void
@@ -827,7 +889,9 @@ reduce_list_index( Reduce *rc, PElement *base, int n, PElement *out )
 			reduce_throw( rc );
 		}
 
-		assert( PEISFLIST( &p ) );
+		g_assert( PEISFLIST( &p ) );
+
+		reduce_get_list( rc, &p );
 
 		hn = PEGETVAL( &p );
 		PEPOINTRIGHT( hn, &p );
@@ -839,10 +903,10 @@ reduce_list_index( Reduce *rc, PElement *base, int n, PElement *out )
 	}
 
 	if( trace_flags & TRACE_OPERATOR ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
 		trace_pelement( base );
-		buf_appendf( buf, " \"?\" %d ->\n", n );
+		vips_buf_appendf( buf, " \"?\" %d ->\n", n );
 	}
 
 	PEPOINTLEFT( hn, out );
@@ -858,15 +922,15 @@ reduce_list_index( Reduce *rc, PElement *base, int n, PElement *out )
 static void
 argserror( Reduce *rc,  PElement *a )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[MAX_ERROR_FRAG];
 
-	buf_init_static( &buf, txt, MAX_ERROR_FRAG );
+	vips_buf_init_static( &buf, txt, MAX_ERROR_FRAG );
 	itext_value_ev( rc, &buf, a );
 
 	error_top( _( "No arguments allowed." ) );
 	error_sub( _( "Object \"%s\" should have no arguments." ),
-		buf_all( &buf ) );
+		vips_buf_all( &buf ) );
 	reduce_throw( rc );
 }
 
@@ -889,7 +953,7 @@ is_WHNF( PElement *out )
 		PEISCOMPLEX( out ) || PEISNUM( out ) || PEISCHAR( out ) ||
 		PEISBOOL( out ) || PEISTAG( out ) || PEISIMAGE( out ) ||
 		PEISLIST( out ) || PEISCLASS( out ) || PEISSYMREF( out ) ||
-		PEISCOMPILEREF( out ) || PEISNOVAL( out ) || PEISSTATIC( out ) )
+		PEISCOMPILEREF( out ) || PEISNOVAL( out ) )
 		return( TRUE );
 
 	/* Must be a function or generator ... loop down the spine, counting 
@@ -980,26 +1044,22 @@ reduce_spine( Reduce *rc, PElement *out )
 
 reduce_start:
 	reduce_total_recomputations += 1;
-	/*
-
-		FIXME ... where could we show the progress box?
-
-	if( (reduce_total_recomputations % 1000) == 0 ) {
-
-
-		if( mainw_countdown_animate( 99 ) )
+	if( (reduce_total_recomputations % 100000) == 0 ) {
+		if( progress_update_expr( reduce_current_expr ) ) {
+			error_top( _( "Cancelled." ) );
+			error_sub( _( "Evaluation cancelled." ) );
 			reduce_throw( rc );
+		}
 	}
-	 */
 
 	switch( PEGETTYPE( &np ) ) {
 	case ELEMENT_CHAR:
 	case ELEMENT_BOOL:
 	case ELEMENT_ELIST:
 	case ELEMENT_TAG:
-	case ELEMENT_MANAGED:
 	case ELEMENT_SYMREF:
 	case ELEMENT_COMPILEREF:
+	case ELEMENT_MANAGED:
 		/* Base type .. no more reduction needed.
 		 */
 
@@ -1020,7 +1080,7 @@ reduce_start:
 		/* Class constructor.
 		 */
 		compile = PEGETCOMPILE( &np );
-		assert( is_class( compile ) );
+		g_assert( is_class( compile ) );
 
 		/* Class args ... real params, secret params.
 		 */
@@ -1057,22 +1117,11 @@ reduce_start:
 		goto reduce_start;
 	}
 
-	case ELEMENT_STATIC:
-	{
-		HeapStaticString *string = PEGETSTATIC( &np );
-
-		/* Link to compiled string.
-		 */
-		PEPUTE( &np, &string->e );
-
-		goto reduce_start;
-	}
-
 	case ELEMENT_SYMBOL:
 	{
 		Symbol *sym = PEGETSYMBOL( &np );
 
-		assert( sym );
+		g_assert( sym );
 
 		switch( sym->type ) {
 		case SYM_VALUE:
@@ -1084,7 +1133,7 @@ reduce_start:
 			 */
 			if( sym->dirty ) {
 				error_top( _( "No value." ) );
-				error_sub( _( "Symbol \"%s\" has no value" ), 
+				error_sub( _( "Symbol \"%s\" has no value." ), 
 					symbol_name( sym ) );
 				reduce_throw( rc );
 			}
@@ -1127,7 +1176,7 @@ reduce_start:
 		case SYM_PARAM:
 			/* All params should be taken out by var abstract.
 			 */
-			assert( FALSE );
+			g_assert( FALSE );
 			break;
 
 		case SYM_EXTERNAL:
@@ -1248,7 +1297,7 @@ reduce_start:
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		break;
@@ -1312,14 +1361,14 @@ reduce_start:
 				d3 = GETRIGHT( GETRIGHT( hn ) )->body.num;
 
 			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_push();
+				VipsBuf *buf = trace_push();
 
 				if( limit )
-					buf_appendf( buf, 
+					vips_buf_appendf( buf, 
 						"generator %g %g %g ->\n",
 						d1, d2, d3 );
 				else
-					buf_appendf( buf, 
+					vips_buf_appendf( buf, 
 						"generator %g %g ->\n",
 						d1, d2 );
 			}
@@ -1417,10 +1466,10 @@ reduce_start:
 		}
 
 		case TAG_FREE:
-			assert( FALSE );
+			g_assert( FALSE );
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		break;
@@ -1572,9 +1621,9 @@ reduce_start:
 				d3 = PEGETREAL( &rhs3 ); 
 
 			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_push();
+				VipsBuf *buf = trace_push();
 
-				buf_appends( buf, "generator constructor " );
+				vips_buf_appends( buf, "generator constructor " );
 				trace_args( arg, 3 );
 			}
 
@@ -1625,13 +1674,14 @@ reduce_start:
 				ELEMENT_NODE, hn2 );
 
 			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_current();
+				VipsBuf *buf = trace_current();
 
-				buf_appends( buf, "    " ); 
+				vips_buf_appends( buf, "    " ); 
 				trace_node( arg[0] );
-				buf_appends( buf, "\n" ); 
+				vips_buf_appends( buf, "\n" ); 
 
-				trace_text( TRACE_OPERATOR, buf_all( buf ) ); 
+				trace_text( TRACE_OPERATOR, 
+					"%s", vips_buf_all( buf ) ); 
 
 				trace_pop();
 			}
@@ -1650,7 +1700,7 @@ reduce_start:
 		}
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		/* Find output element.
@@ -1709,13 +1759,14 @@ reduce_start:
 				PEGETTYPE( &rhs1 ), PEGETVAL( &rhs1 ) );
 
 			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_current();
+				VipsBuf *buf = trace_current();
 
-				buf_appends( buf, "    " ); 
+				vips_buf_appends( buf, "    " ); 
 				trace_node( arg[0] );
-				buf_appends( buf, "\n" ); 
+				vips_buf_appends( buf, "\n" ); 
 
-				trace_text( TRACE_OPERATOR, buf_all( buf ) );
+				trace_text( TRACE_OPERATOR, 
+					"%s", vips_buf_all( buf ) );
 
 				trace_pop();
 			}
@@ -1790,7 +1841,7 @@ reduce_start:
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	/* Unwind stack, restore frame pointer.
@@ -1801,14 +1852,14 @@ reduce_start:
 	/* Should now be in WHNF ... test!
 	 */
 	if( !is_WHNF( out ) ) {
-		BufInfo buf;
+		VipsBuf buf;
 		char txt[1000];
 
-		buf_init_static( &buf, txt, 1000 );
+		vips_buf_init_static( &buf, txt, 1000 );
 		graph_pelement( heap, &buf, out, TRUE );
 		printf( "*** internal error:\n" );
 		printf( "result of reduce_spine not in WHNF: " );
-		printf( "%s\n", buf_all( &buf ) );
+		printf( "%s\n", vips_buf_all( &buf ) );
 		reduce_throw( rc );
 	}
 #endif /*WHNF_DEBUG*/
@@ -1825,9 +1876,10 @@ reduce_spine_strict( Reduce *rc, PElement *np )
 	 */
 	reduce_spine( rc, np );
 
-	/* If it's a non-empty list, may need to reduce inside.
+	/* If it's a non-empty list, may need to reduce inside. Not managed
+	 * strings though, we can leave them unevaluated.
 	 */
-	if( PEISFLIST( np ) ) {
+	if( PEISFLIST( np ) && !PEISMANAGEDSTRING( np ) ) {
 		/* Recurse for head and tail.
 		 */
 		HeapNode *hn = PEGETVAL( np );
@@ -1950,28 +2002,29 @@ reduce_regenerate( Expr *expr, PElement *out )
 
 #ifdef DEBUG_REGEN
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 
-	buf_init_static( &buf, txt, 1024 );
+	vips_buf_init_static( &buf, txt, 1024 );
 	graph_pelement( heap, &buf, out, TRUE );
 	printf( "reduce_regenerate: reducing " );
 	expr_name_print( expr );
-	printf( "graph: %s\n", buf_all( &buf ) );
+	printf( "graph: %s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_REGEN*/
 
-	/* Do initial reduction.
-	 */
+	reduce_current_expr = expr;
 	if( !reduce_pelement( rc, reduce_spine, out ) ) {
+		reduce_current_expr = NULL;
 		expr_error_set( expr );
 		(void) heap_gc( heap );
 		return( FALSE );
 	}
+	reduce_current_expr = NULL;
 
 #ifdef DEBUG_REGEN
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 
 	/* Force immediate GC to pick up any stray pointers.
@@ -1981,11 +2034,11 @@ reduce_regenerate( Expr *expr, PElement *out )
 		return( FALSE );
 	}
 
-	buf_init_static( &buf, txt, 1024 );
+	vips_buf_init_static( &buf, txt, 1024 );
 	graph_pelement( heap, &buf, out, TRUE );
 	printf( "reduce_regenerate: reduced " );
 	expr_name_print( expr );
-	printf( " to: %s\n", buf_all( &buf ) );
+	printf( " to: %s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_REGEN*/
 
@@ -2022,14 +2075,14 @@ reduce_regenerate_member( Expr *expr, PElement *ths, PElement *out )
 
 #ifdef DEBUG_REGEN_MEMBER
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1024];
 
-	buf_init_static( &buf, txt, 1024 );
+	vips_buf_init_static( &buf, txt, 1024 );
 	graph_pelement( heap, &buf, out, TRUE );
 	printf( "reduce_regenerate_member: " );
 	expr_name_print( expr );
-	printf( " new code: %s\n", buf_all( &buf ) );
+	printf( " new code: %s\n", vips_buf_all( &buf ) );
 }
 #endif /*DEBUG_REGEN_MEMBER*/
 
