@@ -455,7 +455,7 @@ regionview_queue_draw( Regionview *regionview )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	if( regionview->classmodel )
@@ -523,13 +523,13 @@ regionview_paint( Regionview *regionview )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	if( regionview->classmodel ) 
-		regionview_paint_label( regionview,
-			draw, &regionview->label, regionview->ascent,
-			buf_all( &regionview->caption ) );
+		regionview_paint_label( regionview, draw, 
+			&regionview->label, regionview->ascent,
+			vips_buf_all( &regionview->caption ) );
 }
 
 /* Stop tracking.
@@ -538,7 +538,7 @@ static void
 regionview_detach( Regionview *regionview )
 {
 	if( regionview->grabbed ) {
-		assert( regionview->ip->grabbed == regionview );
+		g_assert( regionview->ip->grabbed == regionview );
 
 #ifdef DEBUG_GRAB
 		printf( "regionview_detach: %p\n", regionview );
@@ -583,7 +583,7 @@ regionview_destroy( GtkObject *object )
 	FREESID( regionview->model_changed_sid, regionview->classmodel ); 
 	IM_FREEF( g_source_remove, regionview->dash_crawl );
 	IM_FREEF( iwindow_cursor_context_destroy, regionview->cntxt );
-	buf_destroy( &regionview->caption );
+	vips_buf_destroy( &regionview->caption );
 
 	if( regionview->ip ) {
 		if( regionview->ip->regionview == regionview )
@@ -611,7 +611,7 @@ static void
 regionview_label_geo( Regionview *regionview )
 {
 	int n = regionview_label_border;
-	const char *str = buf_all( &regionview->caption );
+	const char *str = vips_buf_all( &regionview->caption );
 	int width, height;
 	PangoLayout *layout;
 
@@ -631,14 +631,14 @@ regionview_refresh_label( Regionview *regionview )
 	if( regionview->classmodel ) {
 		Row *row = HEAPMODEL( regionview->classmodel )->row;
 
-		buf_rewind( &regionview->caption );
+		vips_buf_rewind( &regionview->caption );
 		row_qualified_name_relative( row->ws->sym, row, 
 			&regionview->caption );
 		regionview_label_geo( regionview );
 	}
 }
 
-/* Move label to try to keep it within the image, and away from the
+/* Move label to try to keep it within the window, and away from the
  * selected pixels.
  */
 static void
@@ -646,6 +646,7 @@ regionview_position_label( Regionview *regionview )
 {
 	Imagepresent *ip = regionview->ip;
 	Conversion *conv = ip->id->conv;
+	Rect *visible = &conv->visible;
 	Rect *label = &regionview->label;
 	const int b = regionview_border_width + 2;
 
@@ -662,27 +663,27 @@ regionview_position_label( Regionview *regionview )
 	case REGIONVIEW_REGION:
 	case REGIONVIEW_AREA:
 	case REGIONVIEW_BOX:
-		if( dr.top > label->height + b ) {
+		if( dr.top > visible->top + label->height + b ) {
 			/* Space above region for label.
 			 */
 			label->left = dr.left - b;
 			label->top = dr.top - label->height - b;
 		}
-		else if( dr.left > label->width + b ) {
+		else if( dr.left > visible->left + label->width + b ) {
 			/* Space to left of region for label
 			 */
 			label->left = dr.left - label->width - b;
 			label->top = dr.top - b;
 		}
-		else if( IM_RECT_RIGHT( &dr ) < conv->canvas.width - 
-			label->width - b ) {
+		else if( IM_RECT_RIGHT( &dr ) < 
+			IM_RECT_RIGHT( visible ) - label->width - b ) {
 			/* Space at right.
 			 */
 			label->left = IM_RECT_RIGHT( &dr ) + b;
 			label->top = dr.top - b;
 		}
-		else if( IM_RECT_BOTTOM( &dr ) < conv->canvas.height - 
-			label->height - b ) {
+		else if( IM_RECT_BOTTOM( &dr ) < 
+			IM_RECT_BOTTOM( visible ) - label->height - b ) {
 			/* Space at bottom.
 			 */
 			label->left = dr.left - b;
@@ -703,8 +704,8 @@ regionview_position_label( Regionview *regionview )
 	case REGIONVIEW_LINE:
 		/* Space above? 
 		 */
-		if( dr.top > label->height + b/2 + 2 ) {
-			if( dr.left > conv->canvas.width - 
+		if( dr.top > visible->top + label->height + b/2 + 2 ) {
+			if( dr.left > IM_RECT_RIGHT( visible ) - 
 				label->width - b/2 - 2 ) {
 				/* Above left.
 				 */
@@ -718,7 +719,7 @@ regionview_position_label( Regionview *regionview )
 				label->top = dr.top - b/2 - 2 - label->height;
 			}
 		}
-		else if( dr.left > conv->canvas.width - 
+		else if( dr.left > IM_RECT_RIGHT( visible ) -
 			label->width - b/2 - 2 ) {
 			/* Below left.
 			 */
@@ -734,7 +735,7 @@ regionview_position_label( Regionview *regionview )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -943,7 +944,7 @@ regionview_remove_yes( iWindow *iwnd, void *client,
 
 	IDESTROY( row->sym );
 
-	nfn( sys, IWINDOW_TRUE );
+	nfn( sys, IWINDOW_YES );
 }
 
 static void
@@ -965,7 +966,7 @@ regionview_remove_cb( GtkWidget *menu,
 		GTK_STOCK_DELETE, 
 		_( "Delete Region?" ),
 		_( "Are you sure you want to delete Region \"%s\"?" ), 
-		buf_all( &regionview->caption ) );
+		vips_buf_all( &regionview->caption ) );
 }
 
 static void
@@ -1041,7 +1042,7 @@ regionview_init( Regionview *regionview )
 	regionview->first = TRUE;
 	regionview->label_geo = TRUE;
 
-	buf_init_dynamic( &regionview->caption, REGIONVIEW_LABEL_MAX );
+	vips_buf_init_dynamic( &regionview->caption, REGIONVIEW_LABEL_MAX );
 
 	gtk_widget_set_name( GTK_WIDGET( regionview ), "regionview_widget" );
 }
@@ -1177,7 +1178,7 @@ regionview_rect_touches_region( Regionview *regionview, Rect *expose )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( FALSE );
@@ -1193,7 +1194,7 @@ regionview_expose( Regionview *regionview, Rect *expose )
 		expose->left, expose->top, expose->width, expose->height );
 #endif /*DEBUG_PAINT*/
 
-	assert( expose->width >= 0 && expose->height >= 0 );
+	g_assert( expose->width >= 0 && expose->height >= 0 );
 
 	/* If we've not finished init, don't paint.
 	 */
@@ -1346,7 +1347,7 @@ regionview_find_resize( Regionview *regionview, int x, int y )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( REGIONVIEW_RESIZE_NONE );
@@ -1375,8 +1376,8 @@ regionview_attach( Regionview *regionview, int x, int y )
 	Conversion *conv = ip->id->conv;
 	int dx, dy;
 
-	assert( !regionview->grabbed );
-	assert( !regionview->ip->grabbed );
+	g_assert( !regionview->grabbed );
+	g_assert( !regionview->ip->grabbed );
 
 #ifdef DEBUG_GRAB
 	printf( "regionview_attach: %p\n", regionview );
@@ -1438,7 +1439,7 @@ regionview_left_press( Regionview *regionview, GdkEvent *ev, int x, int y )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( handled );
@@ -1537,7 +1538,7 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		if( abs( our_area->width ) < th && 
@@ -1610,7 +1611,7 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 				break;
 
 			default:
-				assert( FALSE );
+				g_assert( FALSE );
 			}
 			break;
 
@@ -1635,7 +1636,7 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 				break;
 
 			default:
-				assert( FALSE );
+				g_assert( FALSE );
 			}
 			break;
 
@@ -1655,7 +1656,7 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 	}
 }
@@ -1737,7 +1738,7 @@ regionview_motion_grab( Regionview *regionview, int x, int y )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 
 		snap = *our_area;
@@ -1829,7 +1830,7 @@ regionview_motion( Regionview *regionview, GdkEvent *ev, int x, int y )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( FALSE );
