@@ -101,11 +101,11 @@ row_get_parent( Row *row )
  * a local of context.
  */
 void
-row_qualified_name_relative( Symbol *context, Row *row, BufInfo *buf )
+row_qualified_name_relative( Symbol *context, Row *row, VipsBuf *buf )
 {
 	if( !row_get_parent( row ) ) {
 		if( !row->sym )
-			buf_appends( buf, "(null)" );
+			vips_buf_appends( buf, "(null)" );
 		else
 			symbol_qualified_name_relative( context, 
 				row->sym, buf );
@@ -115,8 +115,8 @@ row_qualified_name_relative( Symbol *context, Row *row, BufInfo *buf )
 		 */
 		row_qualified_name_relative( context, 
 			row_get_parent( row ), buf );
-		buf_appends( buf, "." );
-		buf_appends( buf, row_name( row ) );
+		vips_buf_appends( buf, "." );
+		vips_buf_appends( buf, row_name( row ) );
 	}
 }
 
@@ -124,7 +124,7 @@ row_qualified_name_relative( Symbol *context, Row *row, BufInfo *buf )
  * hierarchy. eg. "A1.fred.x".
  */
 void
-row_qualified_name( Row *row, BufInfo *buf )
+row_qualified_name( Row *row, VipsBuf *buf )
 {
 	row_qualified_name_relative( row->ws->sym, row, buf );
 }
@@ -135,12 +135,12 @@ void *
 row_name_print( Row *row )
 {
 	if( row ) {
-		BufInfo buf;
+		VipsBuf buf;
 		char txt[100];
 
-		buf_init_static( &buf, txt, 100 );
+		vips_buf_init_static( &buf, txt, 100 );
 		row_qualified_name( row, &buf );
-		printf( "%s ", buf_all( &buf ) );
+		printf( "%s ", vips_buf_all( &buf ) );
 	}
 	else
 		printf( "(null)" );
@@ -178,10 +178,10 @@ row_error_set( Row *row )
 		 * recomp on this tree.
 		 */
 		if( row != row->top_row ) {
-			BufInfo buf;
+			VipsBuf buf;
 			char txt[100];
 
-			buf_init_static( &buf, txt, 100 );
+			vips_buf_init_static( &buf, txt, 100 );
 			row_qualified_name( row, &buf );
 
 			error_top( _( "Error in row." ) );
@@ -189,7 +189,7 @@ row_error_set( Row *row )
 			 * secondary error.
 			 */
 			error_sub( _( "Error in row %s: %s\n%s" ),	
-				buf_all( &buf ),
+				vips_buf_all( &buf ),
 				row->expr->error_top,
 				row->expr->error_sub );
 
@@ -249,7 +249,7 @@ row_dirty_clear( Row *row )
 	Row *top_row = row->top_row;
 
 	if( row->dirty )
-		assert( g_slist_find( top_row->recomp, row ) );
+		g_assert( g_slist_find( top_row->recomp, row ) );
 }
 #endif /*DEBUG_DIRTY*/
 
@@ -281,9 +281,9 @@ row_dirty_set_single( Row *row, gboolean clear_error )
 	Row *top_row = row->top_row;
 
 	if( row->dirty )
-		assert( g_slist_find( top_row->recomp, row ) );
+		g_assert( g_slist_find( top_row->recomp, row ) );
 	if( !row->dirty )
-		assert( !g_slist_find( top_row->recomp, row ) );
+		g_assert( !g_slist_find( top_row->recomp, row ) );
 }
 #endif /*DEBUG_DIRTY*/
 
@@ -317,7 +317,7 @@ row_dirty_set_sub( Model *model, gboolean clear_error )
 		Row *row = ROW( model );
 		Rhs *rhs = row->child_rhs;
 
-		assert( !rhs || IS_RHS( rhs ) );
+		g_assert( !rhs || IS_RHS( rhs ) );
 
 		if( rhs && rhs->itext && ITEXT( rhs->itext )->edited )
 			row_dirty_set_single( row, clear_error );
@@ -350,7 +350,7 @@ row_link_break( Row *parent, Row *child )
 {
 	/* Must be there.
 	 */
-	assert( g_slist_find( parent->children, child ) &&
+	g_assert( g_slist_find( parent->children, child ) &&
 		g_slist_find( child->parents, parent ) );
 
 	parent->children = g_slist_remove( parent->children, child );
@@ -400,17 +400,17 @@ row_dispose( GObject *gobject )
 	 */
 	slist_map( row->parents, (SListMapFn) row_link_break, row );
 	slist_map( row->children, (SListMapFn) row_link_break_rev, row );
-	assert( !row->parents && !row->children );
+	g_assert( !row->parents && !row->children );
 	(void) slist_map( row->recomp, (SListMapFn) row_dirty_clear, NULL );
 	if( row->top_row )
 		row->top_row->recomp_save = 
 			g_slist_remove( row->top_row->recomp_save, row );
 	IM_FREEF( g_slist_free, row->recomp_save );
 
-	assert( !row->recomp );
+	g_assert( !row->recomp );
 
 	if( row->expr ) {
-		assert( row->expr->row == row );
+		g_assert( row->expr->row == row );
 
 		/* If we're a local row, we will have a private expr 
 		 * allocated for us. Junk it.
@@ -437,51 +437,51 @@ row_dispose( GObject *gobject )
 }
 
 static void *
-row_add_parent_name( Link *link, BufInfo *buf )
+row_add_parent_name( Link *link, VipsBuf *buf )
 {
 	Row *row;
 
 	if( link->parent->expr && (row = link->parent->expr->row) ) {
 		row_qualified_name_relative( link->child, row, buf );
-		buf_appends( buf, " " );
+		vips_buf_appends( buf, " " );
 	}
 
 	return( NULL );
 }
 
 static void *
-row_add_child_name( Link *link, BufInfo *buf )
+row_add_child_name( Link *link, VipsBuf *buf )
 {
 	Row *row;
 
 	if( link->child->expr && (row = link->child->expr->row) ) {
 		row_qualified_name_relative( link->parent, row, buf );
-		buf_appends( buf, " " );
+		vips_buf_appends( buf, " " );
 	}
 
 	return( NULL );
 }
 
 static void *
-row_add_dirty_child_name( Link *link, BufInfo *buf )
+row_add_dirty_child_name( Link *link, VipsBuf *buf )
 {
 	if( link->child->dirty ) {
 		symbol_qualified_name( link->child, buf );
-		buf_appends( buf, " " );
+		vips_buf_appends( buf, " " );
 	}
 
 	return( NULL );
 }
 
 static void
-row_info( iObject *iobject, BufInfo *buf )
+row_info( iObject *iobject, VipsBuf *buf )
 {
 	Row *row = ROW( iobject );
 
-	buf_appends( buf, _( "Name" ) );
-	buf_appends( buf, ": " );
+	vips_buf_appends( buf, _( "Name" ) );
+	vips_buf_appends( buf, ": " );
 	row_qualified_name( row, buf );
-	buf_appends( buf, "\n" );
+	vips_buf_appends( buf, "\n" );
 
 	if( row->expr ) 
 		iobject_info( IOBJECT( row->expr ), buf );
@@ -492,25 +492,25 @@ row_info( iObject *iobject, BufInfo *buf )
 	if( row->top_row->sym ) {
 		if( row->top_row->sym->topchildren ) {
 			row_qualified_name( row, buf );
-			buf_appends( buf, " " );
+			vips_buf_appends( buf, " " );
 			/* Expands to eg. "B1 refers to: B2, B3".
 			 */
-			buf_appends( buf, _( "refers to" ) );
-			buf_appends( buf, ": " );
+			vips_buf_appends( buf, _( "refers to" ) );
+			vips_buf_appends( buf, ": " );
 			slist_map( row->top_row->sym->topchildren, 
 				(SListMapFn) row_add_child_name, buf );
-			buf_appends( buf, "\n" );
+			vips_buf_appends( buf, "\n" );
 		}
 		if( row->top_row->sym->topparents ) {
 			row_qualified_name( row, buf );
-			buf_appends( buf, " " );
+			vips_buf_appends( buf, " " );
 			/* Expands to eg. "B1 is referred to by: B2, B3".
 			 */
-			buf_appends( buf, _( "is referred to by" ) );
-			buf_appends( buf, ": " );
+			vips_buf_appends( buf, _( "is referred to by" ) );
+			vips_buf_appends( buf, ": " );
 			slist_map( row->top_row->sym->topparents, 
 				(SListMapFn) row_add_parent_name, buf );
-			buf_appends( buf, "\n" );
+			vips_buf_appends( buf, "\n" );
 		}
 	}
 	if( row == row->top_row && row->sym->dirty ) {
@@ -518,12 +518,12 @@ row_info( iObject *iobject, BufInfo *buf )
 
 		if( sym->ndirtychildren ) {
 			row_qualified_name( row, buf );
-			buf_appends( buf, " " );
-			buf_appends( buf, _( "is blocked on" ) );
-			buf_appends( buf, ": " );
+			vips_buf_appends( buf, " " );
+			vips_buf_appends( buf, _( "is blocked on" ) );
+			vips_buf_appends( buf, ": " );
 			slist_map( sym->topchildren,
 				(SListMapFn) row_add_dirty_child_name, buf );
-			buf_appends( buf, "\n" );
+			vips_buf_appends( buf, "\n" );
 		}
 	}
 }
@@ -531,7 +531,7 @@ row_info( iObject *iobject, BufInfo *buf )
 static Rhs *
 row_get_rhs( Row *row )
 {
-	assert( g_slist_length( ICONTAINER( row )->children ) == 1 );
+	g_assert( g_slist_length( ICONTAINER( row )->children ) == 1 );
 
 	return( RHS( ICONTAINER( row )->children->data ) );
 }
@@ -595,7 +595,7 @@ row_parent_add( iContainer *child )
 {
 	Row *row = ROW( child );
 
-	assert( IS_SUBCOLUMN( child->parent ) );
+	g_assert( IS_SUBCOLUMN( child->parent ) );
 
 	ICONTAINER_CLASS( parent_class )->parent_add( child );
 
@@ -658,7 +658,7 @@ row_load( Model *model,
 
 	char name[256];
 
-	assert( IS_SUBCOLUMN( parent ) );
+	g_assert( IS_SUBCOLUMN( parent ) );
 
 	if( !get_sprop( xnode, "name", name, 256 ) ) 
 		return( FALSE );
@@ -758,7 +758,7 @@ row_save( Model *model, xmlNode *xnode )
 	/* Top-level only.
 	 */
 	if( row->top_row == row ) 
-		if( !set_prop( xthis, "popup", bool_to_char( row->popup ) ) )
+		if( !set_sprop( xthis, "popup", bool_to_char( row->popup ) ) )
 			return( NULL );
 	if( !set_sprop( xthis, "name", IOBJECT( row )->name ) )
 		return( NULL );
@@ -1046,9 +1046,9 @@ row_get_type( void )
 void
 row_link_symbol( Row *row, Symbol *sym, PElement *root )
 {
-	assert( !row->sym );
-	assert( !row->expr );
-	assert( !sym->expr || !sym->expr->row );
+	g_assert( !row->sym );
+	g_assert( !row->expr );
+	g_assert( !sym->expr || !sym->expr->row );
 
 	row->sym = sym;
 
@@ -1058,7 +1058,7 @@ row_link_symbol( Row *row, Symbol *sym, PElement *root )
 	 */
 	if( is_top( sym ) ) {
 		row->expr = sym->expr;
-		assert( !row->expr->row );
+		g_assert( !row->expr->row );
 		row->expr->row = row;
 	}
 	else {
@@ -1406,7 +1406,7 @@ row_recomp_sort_func( Row *a, Row *b )
 			row_name_print( a );
 			printf( "and " );
 			row_name_print( b );
-			printf( "independant\n" );
+			printf( "independent\n" );
 		}
 #endif /*DEBUG_SORT_VERBOSE*/
 
@@ -1423,7 +1423,6 @@ row_recomp_sort_func( Row *a, Row *b )
 	return( order );
 }
 
-
 /* Insert-sort an slist.
  */
 static GSList *
@@ -1473,10 +1472,10 @@ row_recomp_sort( Row *row )
 	g_timer_reset( sort_timer );
 #endif /*DEBUG_TIME_SORT*/
 
-	assert( row == row->top_row );
+	g_assert( row == row->top_row );
 
 	/* Nope, can't use g_slist_sort(). We have a partial order and
-	 * g_slist_sort() uses an algorith that assumes a full order. Do a
+	 * g_slist_sort() uses an algorithm that assumes a full order. Do a
 	 * simple insert-sort, it'll do enough comparisons that we won't miss
 	 * things.
 
@@ -1554,22 +1553,6 @@ row_regenerate( Row *row )
 	return( TRUE );
 }
 
-/* Does this row have a dirty enclosing row?
- */
-static gboolean 
-row_is_enclosing_dirty( Row *row )
-{
-	Row *enclosing;
-
-	if( row == row->top_row )
-		return( FALSE );
-
-	if( (enclosing = row_get_parent( row ))->dirty )
-		return( TRUE );
-
-	return( row_is_enclosing_dirty( enclosing ) );
-}
-
 static gboolean
 row_recomp_row( Row *row )
 {
@@ -1622,7 +1605,7 @@ row_recomp_row( Row *row )
 			return( FALSE );
 	}
 
-	animate_hourglass();
+	progress_update_tick();
 
 	return( TRUE );
 }
@@ -1812,8 +1795,8 @@ row_deselect( Row *row )
 	if( !row->selected )
 		return( NULL );
 
-	assert( ws && IS_WORKSPACE( ws ) );
-	assert( g_slist_find( ws->selected, row ) );
+	g_assert( ws && IS_WORKSPACE( ws ) );
+	g_assert( g_slist_find( ws->selected, row ) );
 
 	ws->selected = g_slist_remove( ws->selected, row );
 	row->selected = FALSE;
@@ -1900,7 +1883,7 @@ row_select_extend( Row *row )
 		int step = pos > pos_last ? 1 : -1;
 		int i;
 
-		assert( pos != -1 && pos_last != -1 );
+		g_assert( pos != -1 && pos_last != -1 );
 
 		for( i = pos_last; i != pos + step; i += step )
 			row_select2( ROW( g_slist_nth_data( rows, i ) ) );
@@ -2022,33 +2005,33 @@ row_set_status( Row *row )
 	Expr *expr = row->expr;
 
 	char str[MAX_LINELENGTH];
-	BufInfo buf;
+	VipsBuf buf;
 
 	/* No symbol? eg. on load error.
 	 */
 	if( !expr )
 		return;
 
-	buf_init_static( &buf, str, MAX_LINELENGTH );
+	vips_buf_init_static( &buf, str, MAX_LINELENGTH );
 
 	row_qualified_name( row, &buf );
 
 	if( expr->err ) {
-		buf_appends( &buf, ": " );
-		buf_appends( &buf, expr->error_top );
+		vips_buf_appends( &buf, ": " );
+		vips_buf_appends( &buf, expr->error_top );
 	}
 	else if( row->child_rhs->itext ) {
 		iText *itext = ITEXT( row->child_rhs->itext );
 
-		buf_appends( &buf, " = " );
+		vips_buf_appends( &buf, " = " );
 
 		if( row->ws->mode != WORKSPACE_MODE_FORMULA )
-			buf_appends( &buf, NN( itext->formula ) );
+			vips_buf_appends( &buf, NN( itext->formula ) );
 		else 
-			buf_appends( &buf, buf_all( &itext->value ) );
+			vips_buf_appends( &buf, vips_buf_all( &itext->value ) );
 	}
 
-	workspace_set_status( row->ws, "%s", buf_firstline( &buf ) );
+	workspace_set_status( row->ws, "%s", vips_buf_firstline( &buf ) );
 }
 
 /* Sub fn of below ... search inside a row hierarcy. Context is (eg.) row
