@@ -41,15 +41,22 @@
 #define DEBUG_TIME
  */
 
+/* Show symbols as we recalc
+#define DEBUG_RECALC
+ */
+
 /* If DEBUG is on, make sure other debugs are on too.
  */
 #ifdef DEBUG
-#ifndef DEBUG_MAKE
-#define DEBUG_MAKE
-#endif
-#ifndef DEBUG_TIME
-#define DEBUG_TIME
-#endif
+# ifndef DEBUG_MAKE
+#  define DEBUG_MAKE
+# endif
+# ifndef DEBUG_TIME
+#  define DEBUG_TIME
+# endif
+# ifndef DEBUG_RECALC
+#  define DEBUG_RECALC
+# endif
 #endif
 
 /* Our signals. 
@@ -149,32 +156,32 @@ symbol_get_scope( Symbol *sym )
  * static scopes.
  */
 void
-symbol_qualified_name( Symbol *sym, BufInfo *buf )
+symbol_qualified_name( Symbol *sym, VipsBuf *buf )
 {
 	Symbol *parent = symbol_get_parent( sym );
 
 	if( parent && !is_scope( parent ) ) {
 		symbol_qualified_name( parent, buf );
-		buf_appends( buf, "." );
+		vips_buf_appends( buf, "." );
 	}
 
-	buf_appends( buf, NN( IOBJECT( sym )->name ) );
+	vips_buf_appends( buf, NN( IOBJECT( sym )->name ) );
 }
 
 /* Make a symbol name relative to a scope context ... ie. from the point of
  * view of a local of context, what name will find sym.
  */
 void
-symbol_qualified_name_relative( Symbol *context, Symbol *sym, BufInfo *buf )
+symbol_qualified_name_relative( Symbol *context, Symbol *sym, VipsBuf *buf )
 {
 	Symbol *parent = symbol_get_parent( sym );
 
 	if( parent && !is_ancestor( context, parent ) ) {
 		symbol_qualified_name_relative( context, parent, buf );
-		buf_appends( buf, "." );
+		vips_buf_appends( buf, "." );
 	}
 
-	buf_appends( buf, NN( IOBJECT( sym )->name ) );
+	vips_buf_appends( buf, NN( IOBJECT( sym )->name ) );
 }
 
 /* Handy for error messages ... but nowt else. Return string overwritten on
@@ -183,13 +190,13 @@ symbol_qualified_name_relative( Symbol *context, Symbol *sym, BufInfo *buf )
 const char *
 symbol_name( Symbol *sym )
 {
-	static BufInfo buf;
+	static VipsBuf buf;
 	static char txt[200];
 
-	buf_init_static( &buf, txt, 200 );
+	vips_buf_init_static( &buf, txt, 200 );
 	symbol_qualified_name( sym, &buf );
 
-	return( buf_all( &buf ) );
+	return( vips_buf_all( &buf ) );
 }
 
 /* Convenience ... print a qual name to stdout.
@@ -208,15 +215,15 @@ symbol_name_scope( Symbol *sym )
 {
 	Symbol *scope = symbol_get_scope( sym );
 
-	static BufInfo buf;
+	static VipsBuf buf;
 	static char txt[200];
 
-	buf_init_static( &buf, txt, 200 );
-	buf_appends( &buf, NN( IOBJECT( scope )->name ) );
-	buf_appends( &buf, "." );
+	vips_buf_init_static( &buf, txt, 200 );
+	vips_buf_appends( &buf, NN( IOBJECT( scope )->name ) );
+	vips_buf_appends( &buf, "." );
 	symbol_qualified_name_relative( scope, sym, &buf );
 
-	return( buf_all( &buf ) );
+	return( vips_buf_all( &buf ) );
 }
 
 /* Convenience ... print a qual name to stdout.
@@ -238,7 +245,7 @@ symbol_new_value( Symbol *sym )
 void *
 symbol_patch_add( void **pnt, Symbol *sym )
 {
-	assert( sym->type == SYM_ZOMBIE );
+	g_assert( sym->type == SYM_ZOMBIE );
 
 	sym->patch = g_slist_prepend( sym->patch, pnt );
 
@@ -457,7 +464,7 @@ symbol_made( Symbol *sym )
 }
 
 static void *
-symbol_not_defined_sub( Link *link, BufInfo *buf )
+symbol_not_defined_sub( Link *link, VipsBuf *buf )
 {
 	symbol_qualified_name( link->parent, buf );
 
@@ -474,22 +481,22 @@ void
 symbol_not_defined( Symbol *sym )
 {
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
 	error_top( _( "Not found." ) );
-	buf_init_static( &buf, txt, 256 );
-	buf_appendf( &buf, _( "Symbol %s is not defined." ), 
+	vips_buf_init_static( &buf, txt, 256 );
+	vips_buf_appendf( &buf, _( "Symbol %s is not defined." ), 
 		symbol_name( sym ) );
-	buf_appends( &buf, "\n" );
+	vips_buf_appends( &buf, "\n" );
 	if( sym->topparents ) {
-		buf_appendf( &buf, _( "%s is referred to by" ),
+		vips_buf_appendf( &buf, _( "%s is referred to by" ),
 			symbol_name( sym ) );
-		buf_appends( &buf, ": " );
+		vips_buf_appends( &buf, ": " );
 		slist_map2( sym->topparents,
 			(SListMap2Fn) symbol_not_defined_sub, &buf, NULL );
-		buf_appends( &buf, "\n" );
+		vips_buf_appends( &buf, "\n" );
 	}
-	error_sub( buf_all( &buf ) );
+	error_sub( "%s", vips_buf_all( &buf ) );
 }
 
 /* Compile refers to sym, which is going ... mark compile as containing an 
@@ -508,11 +515,13 @@ static void
 symbol_dispose( GObject *gobject )
 {
 	Symbol *sym;
+	Compile *compile;
 
 	g_return_if_fail( gobject != NULL );
 	g_return_if_fail( IS_SYMBOL( gobject ) );
 
 	sym = SYMBOL( gobject );
+	compile = COMPILE( ICONTAINER( sym )->parent );
 
 #ifdef DEBUG_MAKE
 	printf( "symbol_dispose: " );
@@ -520,6 +529,11 @@ symbol_dispose( GObject *gobject )
 	printf( " (%p)\n", sym );
 #endif /*DEBUG_MAKE*/
 
+	/* Make sure we're not leaving last_sym dangling.
+	 */
+	if( compile && compile->last_sym == sym )
+		compile->last_sym = NULL;
+
 	/* Note the impending destruction of this symbol. It's a state
 	 * change, take off the leaf set if it's there.
 	 */
@@ -560,10 +574,10 @@ symbol_dispose( GObject *gobject )
 	 */
 	sym->type = SYM_ZOMBIE; 
 
-	assert( !sym->tool );
-	assert( !sym->parents );
-	assert( !sym->topparents );
-	assert( !sym->topchildren );
+	g_assert( !sym->tool );
+	g_assert( !sym->parents );
+	g_assert( !sym->topparents );
+	g_assert( !sym->topchildren );
 
 	IM_FREEF( g_slist_free, sym->patch );
 	IM_FREEF( g_slist_free, sym->parents );
@@ -687,19 +701,19 @@ symbol_new( Compile *compile, const char *name )
 void
 symbol_error_redefine( Symbol *sym )
 {
-	static BufInfo buf;
+	static VipsBuf buf;
 	static char txt[200];
 
-	buf_init_static( &buf, txt, 200 );
-	buf_appendf( &buf, _( "Redefinition of \"%s\"." ), 
+	vips_buf_init_static( &buf, txt, 200 );
+	vips_buf_appendf( &buf, _( "Redefinition of \"%s\"." ), 
 		IOBJECT( sym )->name );
 	if( sym->tool && sym->tool->lineno != -1 ) {
-		buf_appendf( &buf, "\n" );
-		buf_appendf( &buf, _( "Previously defined at line %d." ),
+		vips_buf_appendf( &buf, "\n" );
+		vips_buf_appendf( &buf, _( "Previously defined at line %d." ),
 			sym->tool->lineno );
 	}
 
-	yyerror( "%s", buf_all( &buf ) );
+	yyerror( "%s", vips_buf_all( &buf ) );
 }
 
 /* Name in defining occurence. If this is a top-level definition, clean the
@@ -796,7 +810,7 @@ symbol_link_break( Symbol *child, Compile *compile )
 gboolean
 symbol_user_init( Symbol *sym )
 {
-	assert( sym->type == SYM_ZOMBIE );
+	g_assert( sym->type == SYM_ZOMBIE );
 
 	sym->type = SYM_VALUE;
 	reduce_register( sym );
@@ -817,7 +831,7 @@ symbol_parameter_init( Symbol *sym )
 {
 	Compile *parent = COMPILE( ICONTAINER( sym )->parent );
 
-	assert( sym->type == SYM_ZOMBIE );
+	g_assert( sym->type == SYM_ZOMBIE );
 
 	sym->type = SYM_PARAM;
 	parent->param = g_slist_append( parent->param, sym );
@@ -832,7 +846,7 @@ symbol_parameter_init( Symbol *sym )
 gboolean
 symbol_parameter_builtin_init( Symbol *sym )
 {
-	assert( sym->type == SYM_ZOMBIE );
+	g_assert( sym->type == SYM_ZOMBIE );
 
 	sym->type = SYM_PARAM;
 	symbol_made( sym );
@@ -879,7 +893,7 @@ symbol_set_leaf( Symbol *sym, gboolean leaf )
 				g_slist_prepend( symbol_leaf_set, sym );
 		}
 		else {
-			assert( symbol_leaf_set );
+			g_assert( symbol_leaf_set );
 
 			symbol_leaf_set = 
 				g_slist_remove( symbol_leaf_set, sym );
@@ -901,7 +915,7 @@ symbol_set_leaf( Symbol *sym, gboolean leaf )
 void
 symbol_state_change( Symbol *sym )
 {
-	assert( sym->ndirtychildren >= 0 );
+	g_assert( sym->ndirtychildren >= 0 );
 
 	/* Used to do more ... now we just set leaf.
 	 */
@@ -923,35 +937,22 @@ symbol_recalculate_sub( Symbol *sym )
 	g_timer_reset( timer );
 #endif /*DEBUG_TIME*/
 
-	if( is_value( sym ) ) {
-		if( sym->expr->row ) {
-			/* This is the root of a display ... use that recomp
-			 * mechanism.
-			 */
-			row_recomp( sym->expr->row );
-			if( sym->expr->row->err )
-				result = FALSE;
-		}
-		else if( sym->expr->compile->nparam > 0 ) {
-			/* This is a function ... just mark it clean and wait
-			 * until link time before we try to copy the code into
-			 * the heap.
-			 */
-			symbol_dirty_clear( sym );
-			expr_new_value( sym->expr );
-		}
-		else {
-			if( reduce_regenerate( sym->expr, &sym->expr->root ) ) {
-				symbol_dirty_clear( sym );
-				expr_new_value( sym->expr );
-			}
-			else
-				result = FALSE;
+	g_assert( is_value( sym ) );
 
-		}
+	if( sym->expr->row ) {
+		/* This is the root of a display ... use that recomp
+		 * mechanism. 
+		 */
+		row_recomp( sym->expr->row );
+		if( sym->expr->row->err )
+			result = FALSE;
+	}
+	else if( sym->expr->compile->nparam == 0 ) {
+		/* No params: this ought to have a value.
+		 */
+		if( !reduce_regenerate( sym->expr, &sym->expr->root ) ) 
+			result = FALSE;
 	}
-	else
-		symbol_dirty_clear( sym );
 
 #ifdef DEBUG_TIME
 	printf( "symbol_recalculate_sub: " );
@@ -973,13 +974,20 @@ static gboolean symbol_running = FALSE;
 void *
 symbol_recalculate_check( Symbol *sym )
 {
-#ifdef DEBUG
-	printf( "symbol_recalculate_check: %s\n", symbol_name( sym ) );
-#endif /*DEBUG*/
+#ifdef DEBUG_RECALC
+	printf( "symbol_recalculate_check: %s\n", symbol_name_scope( sym ) );
+	g_assert( symbol_is_leafable( sym ) );
+	g_assert( symbol_ndirty( sym ) == 0 );
+#endif /*DEBUG_RECALC*/
 
 	error_clear();
 	if( sym->expr->err ) {
 		expr_error_get( sym->expr );
+
+#ifdef DEBUG_RECALC
+		printf( "\t(error: previous error)\n" );
+#endif /*DEBUG_RECALC*/
+
 		return( sym );
 	}
 	if( !sym->dirty ) 
@@ -993,13 +1001,39 @@ symbol_recalculate_check( Symbol *sym )
 
 	reduce_context->heap->filled = FALSE;
 	symbol_running = TRUE;
+	progress_begin();
 	if( !symbol_recalculate_sub( sym ) || 
 		reduce_context->heap->filled ) {
 		expr_error_set( sym->expr );
 		symbol_running = FALSE;
+		progress_end();
+
+#ifdef DEBUG_RECALC
+		printf( "\t(error: %s %s)\n", 
+			sym->expr->error_top, sym->expr->error_sub );
+#endif /*DEBUG_RECALC*/
+
 		return( sym );
 	}
 	symbol_running = FALSE;
+	progress_end();
+
+	/* Have we discovered any dirty children? If not, we've cleaned this
+	 * sym.
+	 */
+	if( !sym->ndirtychildren ) {
+		symbol_dirty_clear( sym );
+		expr_new_value( sym->expr );
+
+#ifdef DEBUG_RECALC
+		printf( "\t(success)\n" ); 
+#endif /*DEBUG_RECALC*/
+	}
+#ifdef DEBUG_RECALC
+	else {
+		printf( "\t(found dirty children)\n" );
+	}
+#endif /*DEBUG_RECALC*/
 
 	return( NULL );
 }
@@ -1026,10 +1060,10 @@ symbol_recalculate_leaf( void )
 		/* Should be dirty with no dirty children. Unless it's a
 		 * function, in which case dirty kids are OK.
 		 */
-		assert( sym->dirty );
-		assert( !sym->expr->err );
-		assert( is_top( sym ) );
-		assert( symbol_ndirty( sym ) == 0 || is_value( sym ) );
+		g_assert( sym->dirty );
+		g_assert( !sym->expr->err );
+		g_assert( is_top( sym ) );
+		g_assert( symbol_ndirty( sym ) == 0 || is_value( sym ) );
 
 		/* Found a symbol!
 		 */
@@ -1059,17 +1093,16 @@ symbol_recalculate_idle_cb( void )
 	run_again = TRUE;
 
 	if( !symbol_running ) {
-		set_hourglass();
+		progress_begin();
 
 		if( !symbol_recalculate_leaf() ) {
 			/* Nothing more to do: shut down idle recomp.
 			 */
 			symbol_idle_id = 0;
-
 			run_again = FALSE;
 		}
 
-		set_pointer();
+		progress_end();
 	}
 
 	return( run_again );
@@ -1095,12 +1128,12 @@ symbol_recalculate_all_force( gboolean now )
 		 */
 		;
 	else if( main_option_batch || now ) {
-		set_hourglass();
+		progress_begin();
 
 		while( symbol_recalculate_leaf() )
 			;
 
-		set_pointer();
+		progress_end();
 	}
 	else if( !symbol_idle_id ) 
 		symbol_idle_id = g_idle_add( 
