@@ -43,18 +43,72 @@ static FilemodelClass *parent_class = NULL;
 #define MAX_NAME (256)
 
 void
-tool_error( Tool *tool, BufInfo *buf )
+tool_error( Tool *tool, VipsBuf *buf )
 {
 	if( tool->lineno != -1 ) {
-		buf_appends( buf, " (" );
+		vips_buf_appends( buf, " (" );
 		if( FILEMODEL( tool->kit )->filename )
-			buf_appends( buf, FILEMODEL( tool->kit )->filename );
+			vips_buf_appends( buf, 
+				FILEMODEL( tool->kit )->filename );
 		else
-			buf_appends( buf, IOBJECT( tool->kit )->name );
-		buf_appendf( buf, ":%d)", tool->lineno );
+			vips_buf_appends( buf, IOBJECT( tool->kit )->name );
+		vips_buf_appendf( buf, ":%d)", tool->lineno );
 	}
 }
 
+static void *
+tool_linkreport_sym_sym( Symbol *child, 
+	Symbol *parent, VipsBuf *buf, gboolean *found )
+{
+	/* Don't report generated syms eg. from lcomps or pattern
+	 * matches.
+	 */
+	if( child->type == SYM_ZOMBIE && 
+		!child->generated && !parent->generated &&
+		!compile_resolve_top( child ) ) {
+		Tool *tool = symbol_get_tool( parent );
+
+		symbol_qualified_name( parent, buf );
+		tool_error( tool, buf );
+
+		vips_buf_appendf( buf, " " );
+		/* used as in "fred refers to undefined symbol jim"
+		 */
+		vips_buf_appendf( buf, _( "refers to undefined symbol" ) );
+		vips_buf_appendf( buf, " " );
+		symbol_qualified_name( child, buf );
+		vips_buf_appendf( buf, "\n" );
+
+		*found = TRUE;
+	}
+
+	return( NULL );
+}
+
+static void *
+tool_linkreport_sym( Symbol *sym, VipsBuf *buf, gboolean *found )
+{
+	if( sym->expr )
+		return( slist_map3( sym->expr->compile->children,
+			(SListMap3Fn) tool_linkreport_sym_sym, 
+			sym, buf, found ) );
+
+	return( NULL );
+}
+
+void *
+tool_linkreport_tool( Tool *tool, VipsBuf *buf, gboolean *found )
+{
+	Symbol *sym;
+
+	if( tool->type != TOOL_SYM )
+		return( NULL );
+	sym = tool->sym;
+
+	return( symbol_map_all( tool->sym, 
+		(symbol_map_fn) tool_linkreport_sym, buf, found ) );
+}
+
 static void *toolitem_free( Toolitem *toolitem );
 
 /* Remove a tool. Also strip the sym, if any.
@@ -135,7 +189,7 @@ tool_save_text( Model *model, iOpenFile *of )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( TRUE );
@@ -150,25 +204,25 @@ tool_type_to_char( Tooltype type )
 	case TOOL_SEP:	return( "separator" );
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
 static void
-tool_info( iObject *iobject, BufInfo *buf )
+tool_info( iObject *iobject, VipsBuf *buf )
 {
 	Tool *tool = TOOL( iobject );
 
 	IOBJECT_CLASS( parent_class )->info( iobject, buf );
 
-	buf_appendf( buf, "type = \"%s\"\n", tool_type_to_char( tool->type ) );
+	vips_buf_appendf( buf, "type = \"%s\"\n", tool_type_to_char( tool->type ) );
 	if( tool->type == TOOL_SYM )
-		buf_appendf( buf, "symbol = \"%s\"\n", 
+		vips_buf_appendf( buf, "symbol = \"%s\"\n", 
 			IOBJECT( tool->sym )->name );
 	if( tool->lineno != -1 )
-		buf_appendf( buf, "lineno = %d\n", tool->lineno );
+		vips_buf_appendf( buf, "lineno = %d\n", tool->lineno );
 	if( tool->kit )
-		buf_appendf( buf, "toolkit = \"%s\"\n", 
+		vips_buf_appendf( buf, "toolkit = \"%s\"\n", 
 			IOBJECT( tool->kit )->name );
 }
 
@@ -267,7 +321,7 @@ toolitem_free( Toolitem *toolitem )
 
 	slist_map( toolitem->children, (SListMapFn) toolitem_free, NULL );
 
-	assert( !toolitem->children );
+	g_assert( !toolitem->children );
 
 	if( parent ) {
 		parent->children = g_slist_remove( parent->children, toolitem );
@@ -448,13 +502,13 @@ toolitem_set_action( Toolitem *toolitem, PElement *root )
 {
 	gboolean result;
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 256 );
+	vips_buf_init_static( &buf, txt, 256 );
 
 	if( toolitem->parent )
-		buf_appendf( &buf, "%s.", toolitem->parent->action );
-	buf_appendf( &buf, "%s", IOBJECT( toolitem->compile->sym )->name );
+		vips_buf_appendf( &buf, "%s.", toolitem->parent->action );
+	vips_buf_appendf( &buf, "%s", IOBJECT( toolitem->compile->sym )->name );
 
 	/* If this is a Menuaction, we need the action member.
 	 */
@@ -471,9 +525,9 @@ toolitem_set_action( Toolitem *toolitem, PElement *root )
 	/* If there's an action member, use that. 
 	 */
 	if( toolitem->is_action )
-		buf_appends( &buf, "." MEMBER_ACTION ); 
+		vips_buf_appends( &buf, "." MEMBER_ACTION ); 
 
-	IM_SETSTR( toolitem->action, buf_all( &buf ) );
+	IM_SETSTR( toolitem->action, vips_buf_all( &buf ) );
 
 	/* No action member found and this is an item (ie. not a pullright)?
 	 * Default to the sym itself.
@@ -486,38 +540,38 @@ static void
 toolitem_set_path( Toolitem *toolitem )
 {
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 256 );
+	vips_buf_init_static( &buf, txt, 256 );
 	if( toolitem->parent )
-		buf_appendf( &buf, "%s", toolitem->parent->path );
+		vips_buf_appendf( &buf, "%s", toolitem->parent->path );
 	else 
-		buf_appendf( &buf, "<mainw>/Toolkits/%s", 
+		vips_buf_appendf( &buf, "<mainw>/Toolkits/%s", 
 			IOBJECT( toolitem->tool->kit )->name );
-	buf_appendf( &buf, "/%s", toolitem->name );
-	IM_SETSTR( toolitem->path, buf_all( &buf ) );
+	vips_buf_appendf( &buf, "/%s", toolitem->name );
+	IM_SETSTR( toolitem->path, vips_buf_all( &buf ) );
 }
 
 static void
 toolitem_set_user_path( Toolitem *toolitem )
 {
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 256 );
+	vips_buf_init_static( &buf, txt, 256 );
 	if( toolitem->parent )
-		buf_appends( &buf, toolitem->parent->user_path );
+		vips_buf_appends( &buf, toolitem->parent->user_path );
 	else
-		buf_appends( &buf, IOBJECT( toolitem->tool->kit )->name );
-	buf_appendf( &buf, " / %s", toolitem->name );
-	IM_SETSTR( toolitem->user_path, buf_all( &buf ) );
+		vips_buf_appends( &buf, IOBJECT( toolitem->tool->kit )->name );
+	vips_buf_appendf( &buf, " / %s", toolitem->name );
+	IM_SETSTR( toolitem->user_path, vips_buf_all( &buf ) );
 }
 
 static void *
-toolitem_set_help_sub( Symbol *param, BufInfo *buf )
+toolitem_set_help_sub( Symbol *param, VipsBuf *buf )
 {
-	buf_appends( buf, " " );
-	buf_appends( buf, IOBJECT( param )->name );
+	vips_buf_appends( buf, " " );
+	vips_buf_appends( buf, IOBJECT( param )->name );
 
 	return( NULL );
 }
@@ -526,10 +580,10 @@ static void
 toolitem_set_help( Toolitem *toolitem )
 {
 	char txt[256];
-	BufInfo buf;
+	VipsBuf buf;
 
-	buf_init_static( &buf, txt, 256 );
-	buf_appends( &buf, toolitem->name );
+	vips_buf_init_static( &buf, txt, 256 );
+	vips_buf_appends( &buf, toolitem->name );
 
 	/* Get the params from the action member if we can.
 	 */
@@ -539,10 +593,10 @@ toolitem_set_help( Toolitem *toolitem )
 		slist_map( toolitem->action_sym->expr->compile->param,
 			(SListMapFn) toolitem_set_help_sub, &buf );
 
-	buf_appends( &buf, ": " );
+	vips_buf_appends( &buf, ": " );
 	if( toolitem->tooltip )
-		buf_appends( &buf, toolitem->tooltip );
-	IM_SETSTR( toolitem->help, buf_firstline( &buf ) );
+		vips_buf_appends( &buf, toolitem->tooltip );
+	IM_SETSTR( toolitem->help, vips_buf_firstline( &buf ) );
 }
 
 static Toolitem *
@@ -708,7 +762,7 @@ tool_toolitem_rebuild( Tool *tool )
 		break;
 
 	default:
-		assert( 0 );
+		g_assert( 0 );
 	}
 
 	iobject_changed( IOBJECT( tool ) );
@@ -740,7 +794,7 @@ tool_new_sym( Toolkit *kit, int pos, Symbol *sym )
 {
 	Tool *tool;
 
-	assert( kit && sym );
+	g_assert( kit && sym );
 
 	/* Is there a tool we can reuse? Don't update pos .. assume we want to
 	 * keep the old one.
@@ -784,7 +838,7 @@ tool_new_sep( Toolkit *kit, int pos )
 {
 	Tool *tool;
 
-	assert( kit );
+	g_assert( kit );
 
 	tool = TOOL( g_object_new( TYPE_TOOL, NULL ) );
 	tool->type = TOOL_SEP;
@@ -813,7 +867,7 @@ tool_new_dia( Toolkit *kit, int pos,
 {
 	Tool *tool;
 
-	assert( kit && name && filename );
+	g_assert( kit && name && filename );
 
 	if( (tool = tool_find( kit, name )) ) {
 		if( tool->type != TOOL_DIA ) {
