@@ -31,9 +31,9 @@
 
 /*
 #define DEBUG_TIME
-#define DEBUG
 #define DEBUG_HISTORY_SANITY
 #define DEBUG_HISTORY_MISS
+#define DEBUG
 #define DEBUG_HISTORY
  */
 
@@ -75,7 +75,7 @@ typedef struct _VipsInfo {
 	int ninii;			
 	Imageinfo *inii[MAX_VIPS_ARGS];	
 	unsigned int inii_destroy_sid[MAX_VIPS_ARGS];
-	unsigned int inii_paint_sid[MAX_VIPS_ARGS];
+	unsigned int inii_invalidate_sid[MAX_VIPS_ARGS];
 
 	/* Output images. 
 	 */
@@ -108,7 +108,8 @@ static im_arg_type vips_supported[] = {
 	IM_TYPE_IMASK,
 	IM_TYPE_IMAGEVEC,
 	IM_TYPE_INTVEC,
-	IM_TYPE_GVALUE
+	IM_TYPE_GVALUE,
+	IM_TYPE_INTERPOLATE
 };
 
 typedef enum _VipsArgumentType {
@@ -123,7 +124,8 @@ typedef enum _VipsArgumentType {
 	VIPS_IMASK,
 	VIPS_IMAGEVEC,
 	VIPS_INTVEC,
-	VIPS_GVALUE
+	VIPS_GVALUE,
+	VIPS_INTERPOLATE
 } VipsArgumentType;
 
 /* The previous function calls we are caching, plus an LRU queue for flushing.
@@ -148,6 +150,8 @@ vips_check_all_destroyed( void )
 
 		printf( "** %d VipsInfo leaked!\n", 
 			g_slist_length( vips_info_all ) );
+		printf( "(ignore operations which do not take "
+			"image arguments)\n" );
 
 		for( p = vips_info_all; p; p = p->next ) {
 			VipsInfo *vi = (VipsInfo *) p->data;
@@ -171,7 +175,7 @@ vips_error( VipsInfo *vi )
 /* Look up a VIPS type. 
  */
 static VipsArgumentType
-vips_type_find( im_arg_type type )
+vips_lookup_type( im_arg_type type )
 {
 	int i;
 
@@ -216,7 +220,7 @@ vips_hash( VipsInfo *vi )
 
         for( i = 0; i < vi->fn->argc; i++ ) {
                 im_type_desc *vips = vi->fn->argv[i].desc;
-		VipsArgumentType vt = vips_type_find( vips->type );
+		VipsArgumentType vt = vips_lookup_type( vips->type );
 
                 if( !(vips->flags & IM_TYPE_OUTPUT) ) {
 			switch( vt ) {
@@ -237,6 +241,8 @@ vips_hash( VipsInfo *vi )
 				HASH_S( (char *) vi->vargv[i] );
 				break;
 
+			case VIPS_GVALUE:
+			case VIPS_INTERPOLATE:
 			case VIPS_IMAGE:
 				HASH_P( vi->vargv[i] );
 				break;
@@ -321,10 +327,6 @@ vips_hash( VipsInfo *vi )
 				break;
 			}
 
-			case VIPS_GVALUE:
-				HASH_P( vi->vargv[i] );
-				break;
-
 			default:
 			case VIPS_NONE:
 				break;
@@ -354,7 +356,7 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 
         for( i = 0; i < fn->argc; i++ ) {
                 im_type_desc *vips = fn->argv[i].desc;
-		VipsArgumentType vt = vips_type_find( vips->type );
+		VipsArgumentType vt = vips_lookup_type( vips->type );
 
                 if( !(vips->flags & IM_TYPE_OUTPUT) ) {
 			switch( vt ) {
@@ -498,6 +500,11 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 					return( FALSE );
 				break;
 
+			case VIPS_INTERPOLATE:
+				if( vi1->vargv[i] != vi2->vargv[i] )
+					return( FALSE );
+				break;
+
 			default:
 			case VIPS_NONE:
 				break;
@@ -561,8 +568,36 @@ vips_history_sanity( void )
 		if( !g_hash_table_lookup( vips_history_table, vi ) )
 			g_assert( !vi->in_cache );
 	}
+
+	/* Every image argument on every in-cache call should be sane.
+	 */
+	for( p = vips_call_all; p; p = p->next ) {
+		VipsCall *call = (VipsCall *) p->data;
+		int i;
+
+		if( !call->in_cache )
+			continue;
+
+		for( i = 0; i < call->fn->argc; i++ ) {
+			im_type_desc *ty = call->fn->argv[i].desc;
+
+			if( strcmp( ty->type, IM_TYPE_IMAGE ) == 0 ) {
+				IMAGE *im = (IMAGE *) call->vargv[i];
+
+				g_assert (!im_image_sanity (im));
+			}
+			else if( strcmp( ty->type, IM_TYPE_IMAGEVEC ) == 0 ) {
+				im_imagevec_object *iv = 
+					(im_imagevec_object *) call->vargv[i];
+				int j;
+
+				for( j = 0; j < iv->n; j++) 
+					g_assert( !im_image_sanity( iv->vec[j] ) );
+			}
+		}
+	}
 }
-#endif /*DEBUG_HISTORY_SANITY*/
+#endif /*DEBUG_HISTORY_SANITY */
 
 /* Is a function call in our history? Return the old one. 
  */
@@ -667,17 +702,14 @@ vips_destroy( VipsInfo *vi )
 	vips_info_all = g_slist_remove( vips_info_all, vi );
 #endif /*DEBUG_LEAK*/
 
-	/* Break "destroy" links to iis.
+	/* Remove signals.
 	 */
-	for( i = 0; i < vi->ninii; i++ ) 
-		FREESID( vi->inii_destroy_sid[i], vi->inii[i] ); 
 	for( i = 0; i < vi->noutii; i++ ) 
 		FREESID( vi->outii_destroy_sid[i], vi->outii[i] ); 
-
-	/* Break paint links.
-	 */
-	for( i = 0; i < vi->ninii; i++ ) 
-		FREESID( vi->inii_paint_sid[i], vi->inii[i] ); 
+	for( i = 0; i < vi->ninii; i++ ) {
+		FREESID( vi->inii_destroy_sid[i], vi->inii[i] ); 
+		FREESID( vi->inii_invalidate_sid[i], vi->inii[i] ); 
+	}
 
 	/* Free any VIPS args we built and haven't used.
 	 */
@@ -686,11 +718,11 @@ vips_destroy( VipsInfo *vi )
 		im_object *obj = vi->vargv[i];
 		VipsArgumentType vt;
 
-		/* Make sure we don't damage any error magessage we might
+		/* Make sure we don't damage any error message we might
 		 * have.
 		 */
 		error_block();
-		vt = vips_type_find( ty->type );
+		vt = vips_lookup_type( ty->type );
 		error_unblock();
 
 		switch( vt ) {
@@ -700,6 +732,7 @@ vips_destroy( VipsInfo *vi )
 		case VIPS_COMPLEX: 
 		case VIPS_IMAGEVEC: 	/* Input only, so no freeing reqd */
 		case VIPS_GVALUE:
+		case VIPS_INTERPOLATE:
 			/* Do nothing.
 			 */
 			break;
@@ -781,11 +814,11 @@ vips_history_destroy_cb( Imageinfo *ii, VipsInfo *vi )
 }
 
 static void
-vips_history_painted_cb( Imageinfo *ii, Rect *dirty, VipsInfo *vi )
+vips_history_invalidate_cb( Imageinfo *ii, VipsInfo *vi )
 {
 #ifdef DEBUG_HISTORY
-	printf( "vips_history_painted_cb: "
-		"on paint of ii, uncaching \"%s\"\n", vi->name );
+	printf( "vips_history_invalidate_cb: "
+		"on invalidate of ii, uncaching \"%s\"\n", vi->name );
 #endif /*DEBUG_HISTORY*/
 
 	vips_destroy( vi );
@@ -807,21 +840,22 @@ vips_history_add( VipsInfo *vi )
 
 	/* If any of our ii are destroyed, we must go too.
 	 */
-	for( i = 0; i < vi->ninii; i++ )
-		vi->inii_destroy_sid[i] = g_signal_connect( vi->inii[i], 
-			"destroy", 
-			G_CALLBACK( vips_history_destroy_cb ), vi );
 	for( i = 0; i < vi->noutii; i++ )
 		vi->outii_destroy_sid[i] = g_signal_connect( vi->outii[i], 
 			"destroy", 
 			G_CALLBACK( vips_history_destroy_cb ), vi );
 
-	/* If any of our input ii are painted on, we must also invalidate.
+	/* If any of our input ii are destroyed or painted on, we must also 
+	 * uncache.
 	 */
-	for( i = 0; i < vi->ninii; i++ )
-		vi->inii_paint_sid[i] = g_signal_connect( vi->inii[i], 
-			"area_painted", 
-			G_CALLBACK( vips_history_painted_cb ), vi );
+	for( i = 0; i < vi->ninii; i++ ) {
+		vi->inii_destroy_sid[i] = g_signal_connect( vi->inii[i], 
+			"destroy", 
+			G_CALLBACK( vips_history_destroy_cb ), vi );
+		vi->inii_invalidate_sid[i] = g_signal_connect( vi->inii[i], 
+			"invalidate", 
+			G_CALLBACK( vips_history_invalidate_cb ), vi );
+	}
 
 	/* History too big? Flush!
 	 */
@@ -832,65 +866,69 @@ vips_history_add( VipsInfo *vi )
 /* VIPS types -> a buffer. For tracing calls.
  */
 static void
-vips_tochar( VipsInfo *vi, int i, BufInfo *buf )
+vips_tochar( VipsInfo *vi, int i, VipsBuf *buf )
 {
 	im_object obj = vi->vargv[i];
 	im_type_desc *vips = vi->fn->argv[i].desc;
-	VipsArgumentType vt = vips_type_find( vips->type );
+	VipsArgumentType vt = vips_lookup_type( vips->type );
 
 	switch( vt ) {
 	case VIPS_DOUBLE:
-		buf_appendf( buf, "%g", *((double*)obj) );
+		vips_buf_appendf( buf, "%g", *((double*)obj) );
 		break;
 
 	case VIPS_INT:
-		buf_appendf( buf, "%d", *((int*)obj) );
+		vips_buf_appendf( buf, "%d", *((int*)obj) );
 		break;
 
 	case VIPS_COMPLEX:
-		buf_appendf( buf, "(%g, %g)", 
+		vips_buf_appendf( buf, "(%g, %g)", 
 			((double*)obj)[0], ((double*)obj)[1] );
 		break;
 
 	case VIPS_STRING:
-		buf_appendf( buf, "\"%s\"", (char*) obj );
+		vips_buf_appendf( buf, "\"%s\"", (char*) obj );
 		break;
 
 	case VIPS_IMAGE:
-		buf_appendi( buf, (IMAGE *) obj );
+		vips_buf_appendi( buf, (IMAGE *) obj );
 		break;
 
 	case VIPS_DMASK:
-		buf_appendf( buf, "dmask" );
+		vips_buf_appendf( buf, "dmask" );
 		break;
 
 	case VIPS_IMASK:
-		buf_appendf( buf, "imask" );
+		vips_buf_appendf( buf, "imask" );
 		break;
 
 	case VIPS_DOUBLEVEC:
-		buf_appendf( buf, "doublevec" );
+		vips_buf_appendf( buf, "doublevec" );
 		break;
 
 	case VIPS_INTVEC:
-		buf_appendf( buf, "intvec" );
+		vips_buf_appendf( buf, "intvec" );
 		break;
 
 	case VIPS_IMAGEVEC:
-		buf_appendf( buf, "imagevec" );
+		vips_buf_appendf( buf, "imagevec" );
 		break;
 
 	case VIPS_GVALUE:
 	{
 		GValue *value = (GValue *) obj;
 
-		buf_appends( buf, "(gvalue" );
-		buf_appendgv( buf, value );
-		buf_appendf( buf, ")" );
+		vips_buf_appends( buf, "(gvalue" );
+		vips_buf_appendgv( buf, value );
+		vips_buf_appendf( buf, ")" );
 
 		break;
 	}
 
+	case VIPS_INTERPOLATE:
+		vips_object_to_string( VIPS_OBJECT( obj ), buf );
+		break;
+
 	default:
 		g_assert( FALSE );
 	}
@@ -899,41 +937,41 @@ vips_tochar( VipsInfo *vi, int i, BufInfo *buf )
 /* Get the args from the heap.
  */
 static void
-vips_args_heap( VipsInfo *vi, HeapNode **arg, BufInfo *buf )
+vips_args_heap( VipsInfo *vi, HeapNode **arg, VipsBuf *buf )
 {
 	int i;
 
-	buf_appendf( buf, _( "You passed:" ) );
-	buf_appendf( buf, "\n" );
+	vips_buf_appendf( buf, _( "You passed:" ) );
+	vips_buf_appendf( buf, "\n" );
 	for( i = 0; i < vi->nargs; i++ ) {
 		im_arg_desc *varg = &vi->fn->argv[vi->inpos[i]];
 		PElement rhs;
 
 		PEPOINTRIGHT( arg[vi->nargs - i - 1], &rhs );
-		buf_appendf( buf, "  %s - ", varg->name );
+		vips_buf_appendf( buf, "  %s - ", varg->name );
 		itext_value_ev( vi->rc, buf, &rhs );
-		buf_appendf( buf, "\n" );
+		vips_buf_appendf( buf, "\n" );
 	}
 }
 
 /* Get the args from the VIPS call buffer.
  */
 static void
-vips_args_vips( VipsInfo *vi, BufInfo *buf )
+vips_args_vips( VipsInfo *vi, VipsBuf *buf )
 {
 	int i;
 
-	buf_appendf( buf, _( "You passed:" ) );
-	buf_appendf( buf, "\n" );
+	vips_buf_appendf( buf, _( "You passed:" ) );
+	vips_buf_appendf( buf, "\n" );
         for( i = 0; i < vi->fn->argc; i++ ) {
                 im_type_desc *type = vi->fn->argv[i].desc;
                 char *name = vi->fn->argv[i].name;
 
                 if( !(type->flags & IM_TYPE_OUTPUT)  &&
                         strcmp( type->type, IM_TYPE_DISPLAY ) != 0 ) {
-                        buf_appendf( buf, "   %s - ", name );
+                        vips_buf_appendf( buf, "   %s - ", name );
                         vips_tochar( vi, i, buf );
-                        buf_appendf( buf, "\n" );
+                        vips_buf_appendf( buf, "\n" );
                 }
         }
 }
@@ -941,7 +979,7 @@ vips_args_vips( VipsInfo *vi, BufInfo *buf )
 /* Make a usage error for a VIPS function.
  */
 void
-vips_usage( BufInfo *buf, im_function *fn )
+vips_usage( VipsBuf *buf, im_function *fn )
 {
 	im_package *pack = im_package_of_function( fn->name );
 	char input[MAX_STRSIZE];
@@ -976,53 +1014,53 @@ vips_usage( BufInfo *buf, im_function *fn )
 		}
 	}
 
-	buf_appendf( buf, _( "Usage:" ) );
-        buf_appends( buf, "\n" );
-        buf_appendf( buf, _( "VIPS operator \"%s\"" ), fn->name );
-        buf_appends( buf, "\n" );
-        buf_appendf( buf, _( "%s, from package \"%s\"" ), 
+	vips_buf_appendf( buf, _( "Usage:" ) );
+        vips_buf_appends( buf, "\n" );
+        vips_buf_appendf( buf, _( "VIPS operator \"%s\"" ), fn->name );
+        vips_buf_appends( buf, "\n" );
+        vips_buf_appendf( buf, _( "%s, from package \"%s\"" ), 
 		fn->desc, pack->name );
-        buf_appends( buf, "\n" );
+        vips_buf_appends( buf, "\n" );
 
-	buf_appendf( buf, 
+	vips_buf_appendf( buf, 
 		ngettext( "\"%s\" takes %d argument:",
 			"\"%s\" takes %d arguments:",
 			nin ),
 		fn->name, nin );
-        buf_appendf( buf, "\n%s", input );
+        vips_buf_appendf( buf, "\n%s", input );
 
-	buf_appendf( buf, 
+	vips_buf_appendf( buf, 
 		ngettext( "And produces %d result:",
 			"And produces %d results:",
 			nout ),
 		nout );
-	buf_appendf( buf, "\n%s", output );
+	vips_buf_appendf( buf, "\n%s", output );
 
         /* Print any flags this function has.
          */
-        buf_appendf( buf, _( "Flags:" ) );
-        buf_appends( buf, "\n" );
-	buf_appendf( buf, "   (" );
+        vips_buf_appendf( buf, _( "Flags:" ) );
+        vips_buf_appends( buf, "\n" );
+	vips_buf_appendf( buf, "   (" );
         if( fn->flags & IM_FN_PIO )
-                buf_appendf( buf, _( "PIO function" ) );
+                vips_buf_appendf( buf, _( "PIO function" ) );
         else
-                buf_appendf( buf, _( "WIO function" ) );
-	buf_appendf( buf, ") (" );
+                vips_buf_appendf( buf, _( "WIO function" ) );
+	vips_buf_appendf( buf, ") (" );
         if( fn->flags & IM_FN_TRANSFORM ) 
-                buf_appendf( buf, _( "coordinate transformer" ) );
+                vips_buf_appendf( buf, _( "coordinate transformer" ) );
         else
-                buf_appendf( buf, _( "no coordinate transformation" ) );
-	buf_appendf( buf, ") (" );
+                vips_buf_appendf( buf, _( "no coordinate transformation" ) );
+	vips_buf_appendf( buf, ") (" );
         if( fn->flags & IM_FN_PTOP )
-                buf_appendf( buf, _( "point-to-point operation" ) );
+                vips_buf_appendf( buf, _( "point-to-point operation" ) );
         else
-                buf_appendf( buf, _( "area operation" ) );
-	buf_appendf( buf, ") (" );
+                vips_buf_appendf( buf, _( "area operation" ) );
+	vips_buf_appendf( buf, ") (" );
         if( fn->flags & IM_FN_NOCACHE )
-                buf_appendf( buf, _( "uncacheable operation" ) );
+                vips_buf_appendf( buf, _( "uncacheable operation" ) );
         else
-                buf_appendf( buf, _( "operation can be cached" ) );
-        buf_appendf( buf, ")\n" );
+                vips_buf_appendf( buf, _( "operation can be cached" ) );
+        vips_buf_appendf( buf, ")\n" );
 }
 
 /* We know there's a problem exporting a particular arg to VIPS.
@@ -1030,20 +1068,20 @@ vips_usage( BufInfo *buf, im_function *fn )
 static void
 vips_error_arg( VipsInfo *vi, HeapNode **arg, int argi )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1000];
 
 	error_top( _( "Bad argument." ) );
 
-	buf_init_static( &buf, txt, 1000 );
-	buf_appendf( &buf,
+	vips_buf_init_static( &buf, txt, 1000 );
+	vips_buf_appendf( &buf,
 		_( "Argument %d (%s) to \"%s\" is the wrong type." ),
 		argi + 1, vi->fn->argv[argi].name, vi->name );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "\n" );
 	vips_args_heap( vi, arg, &buf );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "\n" );
 	vips_usage( &buf, vi->fn );
-	error_sub( "%s", buf_all( &buf ) );
+	error_sub( "%s", vips_buf_all( &buf ) );
 }
 
 /* There's a problem calling the function. Show args from the vips call
@@ -1052,22 +1090,23 @@ vips_error_arg( VipsInfo *vi, HeapNode **arg, int argi )
 static void
 vips_error_fn_vips( VipsInfo *vi )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char txt[1000];
 
 	error_top( _( "VIPS library error." ) );
 
-	buf_init_static( &buf, txt, 1000 );
-	buf_appendf( &buf, _( "Error calling library function \"%s\" (%s)." ),
+	vips_buf_init_static( &buf, txt, 1000 );
+	vips_buf_appendf( &buf, 
+		_( "Error calling library function \"%s\" (%s)." ),
 		vi->name, vi->fn->desc );
-	buf_appendf( &buf, "\n" );
-	buf_appendf( &buf, _( "VIPS library: %s" ),
-		im_errorstring() );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, _( "VIPS library: %s" ), im_error_buffer() );
+	im_error_clear();
+	vips_buf_appendf( &buf, "\n" );
 	vips_args_vips( vi, &buf );
-	buf_appendf( &buf, "\n" );
+	vips_buf_appendf( &buf, "\n" );
 	vips_usage( &buf, vi->fn );
-	error_sub( "%s", buf_all( &buf ) );
+	error_sub( "%s", vips_buf_all( &buf ) );
 }
 
 static VipsInfo *
@@ -1128,7 +1167,7 @@ vips_new( Reduce *rc, im_function *fn )
 	for( i = 0; i < MAX_VIPS_ARGS; i++ ) {
 		vi->outii_destroy_sid[i] = 0;
 		vi->inii_destroy_sid[i] = 0;
-		vi->inii_paint_sid[i] = 0;
+		vi->inii_invalidate_sid[i] = 0;
 	}
 
 	/* Make the call spine, alloc memory. 
@@ -1156,12 +1195,12 @@ vips_is_callable( im_function *fn )
 		return( FALSE );
 
         /* Check all argument types are supported. As well as the arg types
-         * spotted by vips_type_find, we also allow IM_TYPE_DISPLAY.
+         * spotted by vips_lookup_type, we also allow IM_TYPE_DISPLAY.
          */
         for( i = 0; i < fn->argc; i++ ) {
                 im_arg_desc *arg = &fn->argv[i];
                 im_arg_type vt = arg->desc->type;
-                VipsArgumentType t = vips_type_find( vt );
+                VipsArgumentType t = vips_lookup_type( vt );
 
                 if( t == VIPS_NONE ) {
                         /* Unknown type .. if DISPLAY it's OK.
@@ -1202,7 +1241,7 @@ vips_n_args( im_function *fn )
         for( nin = 0, i = 0; i < fn->argc; i++ ) {
                 im_arg_desc *arg = &fn->argv[i];
                 im_arg_type vt = arg->desc->type;
-                VipsArgumentType t = vips_type_find( vt );
+                VipsArgumentType t = vips_lookup_type( vt );
 
                 if( t == VIPS_NONE ) {
                         /* Unknown type .. if DISPLAY it's OK.
@@ -1274,9 +1313,9 @@ static gboolean
 vips_fromip( Reduce *rc, PElement *arg, 
 	im_type_desc *vips, im_object *obj )
 {
-	VipsArgumentType vt = vips_type_find( vips->type );
+	VipsArgumentType vt = vips_lookup_type( vips->type );
 
-	/* If vips_type_find failed, is it the special DISPLAY type?
+	/* If vips_lookup_type failed, is it the special DISPLAY type?
 	 */
 	if( vt == VIPS_NONE && strcmp( vips->type, IM_TYPE_DISPLAY ) != 0 ) 
 		/* Unknown type, and it's not DISPLAY. Flag an error.
@@ -1428,6 +1467,12 @@ vips_fromip( Reduce *rc, PElement *arg,
 		break;
 	}
 
+	case VIPS_INTERPOLATE:
+		if( !PEISMANAGEDGOBJECT( arg ) )
+			return( FALSE );
+		*obj = PEGETMANAGEDGOBJECT( arg );
+		break;
+
 	default:
 		g_assert( FALSE );
 	}
@@ -1443,7 +1488,7 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 {
 	im_object obj = vi->vargv[i];
 	im_type_desc *vips = vi->fn->argv[i].desc;
-	VipsArgumentType vt = vips_type_find( vips->type );
+	VipsArgumentType vt = vips_lookup_type( vips->type );
 
 #ifdef DEBUG
 	printf( "vips_toip: arg[%d] (%s) = ", i, vips->type );
@@ -1485,7 +1530,7 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 		break;
 
 	case VIPS_STRING:
-		if( !heap_string_new( vi->rc->heap, (char*)obj, arg ) )
+		if( !heap_managedstring_new( vi->rc->heap, (char *) obj, arg ) )
 			reduce_throw( vi->rc );
 		break;
 
@@ -1528,6 +1573,7 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 		break;
 
 	case VIPS_IMAGEVEC:
+	case VIPS_INTERPOLATE:
 	default:
 		g_assert( FALSE );
 	}
@@ -1541,32 +1587,32 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
  * a method on object type.
  */
 static void
-vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
+vips_tobuf( VipsInfo *vi, int i, VipsBuf *buf )
 {
 	im_object obj = vi->vargv[i];
 	im_type_desc *vips = vi->fn->argv[i].desc;
-	VipsArgumentType vt = vips_type_find( vips->type );
+	VipsArgumentType vt = vips_lookup_type( vips->type );
 
 	switch( vt ) {
 	case VIPS_DOUBLE:
-		buf_appendf( buf, "%g", *((double*)obj) );
+		vips_buf_appendf( buf, "%g", *((double*)obj) );
 		break;
 
 	case VIPS_INT:
-		buf_appendf( buf, "%d", *((int*)obj) );
+		vips_buf_appendf( buf, "%d", *((int*)obj) );
 		break;
 
 	case VIPS_COMPLEX:
-		buf_appendf( buf, "(%g, %g)", 
+		vips_buf_appendf( buf, "(%g, %g)", 
 			((double*)obj)[0], ((double*)obj)[1] );
 		break;
 
 	case VIPS_STRING:
-		buf_appendf( buf, "\"%s\"", (char*)obj );
+		vips_buf_appendf( buf, "\"%s\"", (char*)obj );
 		break;
 
 	case VIPS_IMAGE:
-		buf_appendf( buf, "%s", NN( ((IMAGE*)obj)->filename ) );
+		vips_buf_appendf( buf, "%s", NN( ((IMAGE*)obj)->filename ) );
 		break;
 
 	case VIPS_DMASK:
@@ -1574,7 +1620,7 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 	{
 		im_mask_object *mo = obj;
 
-		buf_appendf( buf, "%s", NN( mo->name ) );
+		vips_buf_appendf( buf, "%s", NN( mo->name ) );
 		break;
 	}
 
@@ -1583,10 +1629,10 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 		im_doublevec_object *v = (im_doublevec_object *) obj;
 		int j;
 
-		buf_appendf( buf, "\"" );
+		vips_buf_appendf( buf, "\"" );
 		for( j = 0; j < v->n; j++ )
-			buf_appendf( buf, "%g ", v->vec[j] );
-		buf_appendf( buf, "\"" );
+			vips_buf_appendf( buf, "%g ", v->vec[j] );
+		vips_buf_appendf( buf, "\"" );
 
 		break;
 	}
@@ -1596,10 +1642,10 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 		im_intvec_object *v = (im_intvec_object *) obj;
 		int j;
 
-		buf_appendf( buf, "\"" );
+		vips_buf_appendf( buf, "\"" );
 		for( j = 0; j < v->n; j++ )
-			buf_appendf( buf, "%d ", v->vec[j] );
-		buf_appendf( buf, "\"" );
+			vips_buf_appendf( buf, "%d ", v->vec[j] );
+		vips_buf_appendf( buf, "\"" );
 
 		break;
 	}
@@ -1609,10 +1655,10 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 		im_imagevec_object *v = (im_imagevec_object *) obj;
 		int j;
 
-		buf_appendf( buf, "\"" );
+		vips_buf_appendf( buf, "\"" );
 		for( j = 0; j < v->n; j++ )
-			buf_appendf( buf, "%s ", v->vec[j]->filename );
-		buf_appendf( buf, "\"" );
+			vips_buf_appendf( buf, "%s ", v->vec[j]->filename );
+		vips_buf_appendf( buf, "\"" );
 
 		break;
 	}
@@ -1621,16 +1667,20 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 	{
 		GValue *value = (GValue *) obj;
 
-		buf_appendgv( buf, value );
+		vips_buf_appendgv( buf, value );
 
 		break;
 	}
 
+	case VIPS_INTERPOLATE:
+		vips_object_to_string( VIPS_OBJECT( obj ), buf );
+		break;
+
 	case VIPS_NONE:
 		if( strcmp( vips->type, IM_TYPE_DISPLAY ) == 0 ) 
 			/* Just assume sRGB.
 			 */
-			buf_appendf( buf, "sRGB" );
+			vips_buf_appendf( buf, "sRGB" );
 		break;
 
 	default:
@@ -1721,7 +1771,7 @@ static void
 vips_build_output( VipsInfo *vi, int i )
 {
 	im_type_desc *ty = vi->fn->argv[i].desc;
-	VipsArgumentType vt = vips_type_find( ty->type );
+	VipsArgumentType vt = vips_lookup_type( ty->type );
 	char tname[FILENAME_MAX];
 
 	/* Provide output objects for the function to write to.
@@ -1838,12 +1888,12 @@ vips_build_argv( VipsInfo *vi, char **argv )
 	int i;
 
 	for( i = 0; i < vi->fn->argc; i++ ) {
-		BufInfo buf;
+		VipsBuf buf;
 		char txt[512];
 
-		buf_init_static( &buf, txt, 512 );
+		vips_buf_init_static( &buf, txt, 512 );
 		vips_tobuf( vi, i, &buf );
-		if( !(argv[i] = im_strdup( NULL, buf_all( &buf ) )) )
+		if( !(argv[i] = im_strdup( NULL, vips_buf_all( &buf ) )) )
 			return( FALSE );
 	}
 
@@ -1900,7 +1950,7 @@ vips_update_hist( VipsInfo *vi )
 	for( i = 0; i < vi->nres; i++ ) {
 		int j = vi->outpos[i];
 		im_type_desc *ty = vi->fn->argv[j].desc;
-		VipsArgumentType vt = vips_type_find( ty->type );
+		VipsArgumentType vt = vips_lookup_type( ty->type );
 
 		/* Image output.
 		 */
@@ -1982,24 +2032,17 @@ vips_wrap_output( VipsInfo *vi )
 		int j = vi->outpos[i];
 		IMAGE *im = (IMAGE *) vi->vargv[j];
 		im_type_desc *vips = vi->fn->argv[j].desc;
-		VipsArgumentType vt = vips_type_find( vips->type );
+		VipsArgumentType vt = vips_lookup_type( vips->type );
 		Imageinfo *outii;
 
 		if( vt != VIPS_IMAGE )
 			continue;
 
-		if( vi->use_lut ) {
-			/* We are using a LUT. 
-			 */
-			if( !(outii = imageinfo_new_modlut( main_imageinfogroup,
-				vi->rc->heap, vi->inii[0], im )) )
-				reduce_throw( vi->rc );
-		}
-		else {
-			if( !(outii = imageinfo_new( main_imageinfogroup, 
-				vi->rc->heap, im, NULL )) )
-				reduce_throw( vi->rc );
-		}
+		if( !(outii = imageinfo_new( main_imageinfogroup, 
+			vi->rc->heap, im, NULL )) )
+			reduce_throw( vi->rc );
+		if( vi->use_lut ) 
+			imageinfo_set_underlying( outii, vi->inii[0] );
 
 		/* This output ii depends upon all of the input images.
 		 */
@@ -2032,7 +2075,7 @@ vips_fillva( VipsInfo *vi, va_list ap )
 
 	for( j = 0, i = 0; i < vi->fn->argc; i++ ) {
 		im_type_desc *ty = vi->fn->argv[i].desc;
-		VipsArgumentType vt = vips_type_find( ty->type );
+		VipsArgumentType vt = vips_lookup_type( ty->type );
 
 #ifdef DEBUG
 		printf( "vips_fillva: arg[%d] (%s) = ", i, ty->type );
@@ -2054,7 +2097,7 @@ vips_fillva( VipsInfo *vi, va_list ap )
 			*((double*)vi->vargv[i]) = v;
 
 			if( trace_flags & TRACE_VIPS ) 
-				buf_appendf( trace_current(), "%g ", v );
+				vips_buf_appendf( trace_current(), "%g ", v );
 		}
 		else if( vt == VIPS_INT ) {
 			int v = va_arg( ap, int );
@@ -2062,36 +2105,53 @@ vips_fillva( VipsInfo *vi, va_list ap )
 			*((int*)vi->vargv[i]) = v;
 
 			if( trace_flags & TRACE_VIPS ) 
-				buf_appendf( trace_current(), "%d ", v );
+				vips_buf_appendf( trace_current(), "%d ", v );
 		}
+
 		else if( vt == VIPS_GVALUE ) {
 			GValue *value = va_arg( ap, GValue * );
 
 			vi->vargv[i] = value;
 
 			if( trace_flags & TRACE_VIPS ) {
-				buf_appendgv( trace_current(), value );
-				buf_appends( trace_current(), " " );
+				vips_buf_appendgv( trace_current(), value );
+				vips_buf_appends( trace_current(), " " );
 			}
 		}
+
+		else if( vt == VIPS_INTERPOLATE ) {
+			VipsInterpolate *value = 
+				va_arg( ap, VipsInterpolate * );
+
+			vi->vargv[i] = value;
+
+			if( trace_flags & TRACE_VIPS ) {
+				vips_object_to_string( VIPS_OBJECT( value ), 
+					trace_current() );
+				vips_buf_appends( trace_current(), " " );
+			}
+		}
+
 		else if( vt == VIPS_IMAGE ) {
 			Imageinfo *ii = va_arg( ap, Imageinfo * );
 
 			vi->vargv[i] = ii;
 
 			if( trace_flags & TRACE_VIPS ) {
-				BufInfo *buf = trace_current();
+				VipsBuf *buf = trace_current();
 
 				if( ii && ii->im ) {
-					buf_appends( buf, "<" );
-					buf_appendf( buf, _( "image \"%s\"" ),
+					vips_buf_appends( buf, "<" );
+					vips_buf_appendf( buf, 
+						_( "image \"%s\"" ),
 						ii->im->filename ); 
-					buf_appends( buf, "> " );
+					vips_buf_appends( buf, "> " );
 				}
 				else {
-					buf_appends( buf, "<" );
-					buf_appends( buf, _( "no image" ) );
-					buf_appends( buf, "> " );
+					vips_buf_appends( buf, "<" );
+					vips_buf_appends( buf, 
+						_( "no image" ) );
+					vips_buf_appends( buf, "> " );
 				}
 			}
 	    	}
@@ -2102,14 +2162,14 @@ vips_fillva( VipsInfo *vi, va_list ap )
 			if( vips_make_doublevec( vi->vargv[i], n, vec ) )
 				reduce_throw( vi->rc );
 			if( trace_flags & TRACE_VIPS ) {
-				BufInfo *buf = trace_current();
+				VipsBuf *buf = trace_current();
 				int i;
 
-				buf_appendf( buf, "<" );
-				buf_appendf( buf, _( "doublevec" ) );
+				vips_buf_appendf( buf, "<" );
+				vips_buf_appendf( buf, _( "doublevec" ) );
 				for( i = 0; i < n; i++ )
-					buf_appendf( buf, " %g", vec[i] );
-				buf_appends( buf, "> " );
+					vips_buf_appendf( buf, " %g", vec[i] );
+				vips_buf_appends( buf, "> " );
 			}
 		}
 		/* 
@@ -2125,18 +2185,18 @@ vips_fillva( VipsInfo *vi, va_list ap )
 				n, (IMAGE **) vec ) )
 				reduce_throw( vi->rc );
 			if( trace_flags & TRACE_VIPS ) {
-				BufInfo *buf = trace_current();
+				VipsBuf *buf = trace_current();
 				int i;
 
-				buf_appendf( buf, "<" );
-				buf_appendf( buf, _( "imagevec" ) );
+				vips_buf_appendf( buf, "<" );
+				vips_buf_appendf( buf, _( "imagevec" ) );
 				for( i = 0; i < n; i++ ) {
-					buf_appendf( buf, " <" );
-					buf_appendf( buf, _( "image \"%s\"" ),
+					vips_buf_appendf( buf, " <" );
+					vips_buf_appendf( buf, _( "image \"%s\"" ),
 						vec[i]->im->filename );
-					buf_appendf( buf, ">" );
+					vips_buf_appendf( buf, ">" );
 				}
-				buf_appends( buf, "> " );
+				vips_buf_appends( buf, "> " );
 			}
 		}
 
@@ -2165,7 +2225,7 @@ vips_dispatch( VipsInfo *vi, PElement *out )
 		vi = old_vi;
 
 		if( trace_flags & TRACE_VIPS ) 
-			buf_appendf( trace_current(), "(from cache) " );
+			vips_buf_appendf( trace_current(), "(from cache) " );
 
 #ifdef DEBUG_HISTORY
 		printf( "vips_dispatch: found %s in history\n", vi->name );
@@ -2188,6 +2248,9 @@ vips_dispatch( VipsInfo *vi, PElement *out )
 		printf( "vips_dispatch: calling %s\n", vi->name );
 #endif /*DEBUG_HISTORY_MISS*/
 
+		/* Be careful. Eval callbacks from this may do anything,
+		 * including call vips_dispatch().
+		 */
 		result = vi->fn->disp( vi->vargv );
 
 #ifdef DEBUG_TIME
@@ -2267,9 +2330,9 @@ vips_spine( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 		reduce_throw( rc );
 
 	if( trace_flags & TRACE_VIPS ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "\"%s\" ", name );
+		vips_buf_appendf( buf, "\"%s\" ", name );
 		trace_args( arg, vi->nargs );
 	}
 
@@ -2310,9 +2373,9 @@ vips_run( Reduce *rc, Compile *compile,
 		reduce_throw( rc );
 
 	if( trace_flags & TRACE_VIPS ) {
-		BufInfo *buf = trace_push();
+		VipsBuf *buf = trace_push();
 
-		buf_appendf( buf, "\"%s\" ", name );
+		vips_buf_appendf( buf, "\"%s\" ", name );
 		trace_args( arg, vi->nargs );
 	}
 
@@ -2335,7 +2398,7 @@ vipsva_sub( VipsInfo *vi, PElement *out, va_list ap )
 	REDUCE_CATCH_START( FALSE );
 
 	if( trace_flags & TRACE_VIPS ) 
-		buf_appendf( trace_current(), "\"%s\" ", vi->name );
+		vips_buf_appendf( trace_current(), "\"%s\" ", vi->name );
 
 	/* Fill argv ... input images go in as Imageinfo here, output as
 	 * IMAGE.
@@ -2343,7 +2406,7 @@ vipsva_sub( VipsInfo *vi, PElement *out, va_list ap )
 	vips_fillva( vi, ap );
 
 	if( trace_flags & TRACE_VIPS ) 
-		buf_appends( trace_current(), " ->\n" ); 
+		vips_buf_appends( trace_current(), " ->\n" ); 
 
 	vips_dispatch( vi, out );
 
