@@ -186,7 +186,7 @@ workspace_deselect_all( Workspace *ws )
 /* Track this while we build a names list.
  */
 typedef struct {
-	BufInfo *buf;
+	VipsBuf *buf;
 	const char *separator;
 	gboolean first;
 } NamesInfo;
@@ -197,7 +197,7 @@ static void *
 workspace_selected_names_sub( Row *row, NamesInfo *names )
 {
 	if( !names->first )
-		buf_appends( names->buf, names->separator );
+		vips_buf_appends( names->buf, names->separator );
 
 	/* Hack: if this is a matrix with selected cells, use an extract to
 	 * get those cells out. We should really have a row method for this I
@@ -208,9 +208,9 @@ workspace_selected_names_sub( Row *row, NamesInfo *names )
 		MATRIX( row->child_rhs->graphic )->selected ) {
 		Matrix *matrix = MATRIX( row->child_rhs->graphic );
 
-		buf_appends( names->buf, "(" );
+		vips_buf_appends( names->buf, "(" );
 		row_qualified_name( row, names->buf );
-		buf_appendf( names->buf, ".extract %d %d %d %d)",
+		vips_buf_appendf( names->buf, ".extract %d %d %d %d)",
 			matrix->range.left, 
 			matrix->range.top, 
 			matrix->range.width, 
@@ -227,7 +227,7 @@ workspace_selected_names_sub( Row *row, NamesInfo *names )
 /* Add a list of selected symbol names to a string. 
  */
 void
-workspace_selected_names( Workspace *ws, BufInfo *buf, const char *separator )
+workspace_selected_names( Workspace *ws, VipsBuf *buf, const char *separator )
 {
 	NamesInfo names;
 
@@ -240,7 +240,7 @@ workspace_selected_names( Workspace *ws, BufInfo *buf, const char *separator )
 }
 
 void
-workspace_column_names( Column *col, BufInfo *buf, const char *separator )
+workspace_column_names( Column *col, VipsBuf *buf, const char *separator )
 {
 	NamesInfo names;
 
@@ -379,7 +379,7 @@ workspace_column_pick( Workspace *ws )
 void
 workspace_column_select( Workspace *ws, Column *col )
 {
-	assert( !col || ICONTAINER_IS_CHILD( ws, col ) ); 
+	g_assert( !col || ICONTAINER_IS_CHILD( ws, col ) ); 
 
 	if( col && col == ws->current )
 		return;
@@ -486,34 +486,43 @@ workspace_add_def( Workspace *ws, const char *str )
 	return( sym );
 }
 
+gboolean
+workspace_load_file_buf( VipsBuf *buf, const char *filename )
+{
+	if( !existsf( "%s", filename ) ) {
+		error_top( _( "File does not exist." ) );
+		error_sub( _( "File \"%s\" cannot be read." ), filename );
+
+		return( FALSE );
+	}
+
+	if( vips_format_for_file( filename ) ) 
+		vips_buf_appends( buf, "Image_file" );
+	else
+		vips_buf_appends( buf, "Matrix_file" );
+
+	vips_buf_appends( buf, " \"" );
+	vips_buf_appendsc( buf, TRUE, filename );
+	vips_buf_appends( buf, "\"" );
+
+	return( TRUE );
+}
+
 Symbol *
 workspace_load_file( Workspace *ws, const char *filename )
 {
 	char txt[MAX_STRSIZE];
-	BufInfo buf;
+	VipsBuf buf;
 	Symbol *sym;
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Image_file \"" );
-	buf_appendsc( &buf, TRUE, filename );
-	buf_appends( &buf, "\"" );
-	if( (sym = workspace_add_def( ws, buf_all( &buf ) )) ) {
-		mainw_recent_add( &mainw_recent_image, filename );
-		return( sym );
-	}
-	error_clear();
+	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
+	if( !workspace_load_file_buf( &buf, filename ) )
+		return( NULL );
+	if( !(sym = workspace_add_def( ws, vips_buf_all( &buf ) )) ) 
+		return( NULL );
+	mainw_recent_add( &mainw_recent_image, filename );
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Matrix_file \"" );
-	buf_appendsc( &buf, TRUE, filename );
-	buf_appends( &buf, "\"" );
-	if( (sym = workspace_add_def( ws, buf_all( &buf ) )) ) {
-		mainw_recent_add( &mainw_recent_matrix, filename );
-		return( sym );
-	}
-	error_clear();
-
-	return( NULL );
+	return( sym );
 }
 
 /* Bounding box of columns to be saved. Though we only really set top/left.
@@ -581,14 +590,14 @@ workspace_clone_selected( Workspace *ws )
 
 	/* Try to load the clone file back again.
 	 */
-        set_hourglass();
+        progress_begin();
 	if( !workspace_merge_column_file( ws, filename ) ) {
-		set_pointer();
+		progress_end();
 		unlinkf( "%s", filename );
 
 		return( FALSE );
 	}
-	set_pointer();
+	progress_end();
 	unlinkf( "%s", filename );
 
 	return( TRUE );
@@ -716,23 +725,23 @@ workspace_auto_recover_load( iWindow *iwnd,
 {
 	char *filename = (char *) client;
 	Mainw *mainw = MAINW( iwindow_get_root_noparent( GTK_WIDGET( iwnd ) ) );
-	Filemodel *filemodel;
+	Workspace *ws;
 
 	/* Load ws file.
 	 */
-        set_hourglass();
-	filemodel = mainw_open_file( mainw, filename );
-	set_pointer();
+        progress_begin();
+	ws = mainw_open_file_into_workspace( mainw, filename );
+	progress_end();
 
-	if( filemodel ) {
+	if( ws ) {
 		/* The filename will be something like
 		 * "~/.nip2-7.9.6/tmp/untitled-nip2-0-3904875.ws", very
 		 * unhelpful.
 		 */
-		IM_FREE( filemodel->filename );
-		iobject_changed( IOBJECT( filemodel ) );
+		IM_FREE( FILEMODEL( ws )->filename );
+		iobject_changed( IOBJECT( ws ) );
 
-		nfn( sys, IWINDOW_TRUE );
+		nfn( sys, IWINDOW_YES );
 	}
 	else
 		nfn( sys, IWINDOW_ERROR );
@@ -935,7 +944,7 @@ workspace_load( Model *model,
 	char buf[256];
 	char *txt;
 
-	assert( IS_WORKSPACEGROUP( parent ) );
+	g_assert( IS_WORKSPACEGROUP( parent ) );
 
 	/* "view" is optional, for backwards compatibility.
 	 */
@@ -991,18 +1000,18 @@ workspace_save( Model *model, xmlNode *xnode )
 	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
 		return( NULL );
 
-	if( !set_prop( xthis, "view", workspacemode_to_char( ws->mode ) ) ||
+	if( !set_sprop( xthis, "view", workspacemode_to_char( ws->mode ) ) ||
 		!set_dprop( xthis, "scale", ws->scale ) ||
 		!set_dprop( xthis, "offset", ws->offset ) ||
 		!set_prop( xthis, "window_width", "%d", ws->window_width ) ||
 		!set_prop( xthis, "window_height", "%d", ws->window_height ) ||
 		!set_prop( xthis, "lpane_position", "%d", 
 			ws->lpane_position ) ||
-		!set_prop( xthis, "lpane_open", 
+		!set_sprop( xthis, "lpane_open", 
 			bool_to_char( ws->lpane_open ) ) ||
 		!set_prop( xthis, "rpane_position", "%d", 
 			ws->rpane_position ) ||
-		!set_prop( xthis, "rpane_open", 
+		!set_sprop( xthis, "rpane_open", 
 			bool_to_char( ws->rpane_open ) ) ||
 		!set_sprop( xthis, "local_defs", ws->local_defs ) ||
 		!set_sprop( xthis, "name", IOBJECT( ws )->name ) ||
@@ -1037,7 +1046,7 @@ workspace_rename_row_node( ModelLoadState *state, Column *col, xmlNode *xnode )
 		char *new_name;
 
 		new_name = column_name_new( col );
-		(void) set_prop( xnode, "name", new_name );
+		(void) set_sprop( xnode, "name", new_name );
 		(void) model_loadstate_rename_new( state, name, new_name );
 		IM_FREE( new_name );
 	}
@@ -1070,7 +1079,7 @@ workspace_rename_column_node( Workspace *ws,
 			name, new_name );
 #endif /*DEBUG*/
 
-		(void) set_prop( xnode, "name", new_name );
+		(void) set_sprop( xnode, "name", new_name );
 		IM_FREE( new_name );
 
 		/* And allocate new names for all rows in the subcolumn.
@@ -1241,7 +1250,7 @@ workspace_top_load( Filemodel *filemodel,
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( FILEMODEL_CLASS( parent_class )->top_load( filemodel, 
@@ -1319,8 +1328,6 @@ workspace_class_init( WorkspaceClass *class )
 static void
 workspace_init( Workspace *ws )
 {
-	char txt[256];
-
 	ws->sym = NULL;
 
 	/* We default to using the main toolkitgroup for our definitions.
@@ -1360,8 +1367,6 @@ workspace_init( Workspace *ws )
 	ws->auto_save_timeout = 0;
 
 	ws->status = NULL;
-	im_snprintf( txt, 256, _( NIP_COPYRIGHT ), PACKAGE );
-	IM_SETSTR( ws->status, txt );
 
 	ws->scale = 1.0;
 	ws->offset = 0.0;
@@ -1568,12 +1573,12 @@ workspace_add_action( Workspace *ws,
 {
 	Column *col = workspace_column_pick( ws );
 	char str[1024];
-	BufInfo buf;
+	VipsBuf buf;
 
 	/* Are there any selected symbols?
 	 */
-	buf_init_static( &buf, str, 1024 );
-	buf_appends( &buf, action );
+	vips_buf_init_static( &buf, str, 1024 );
+	vips_buf_appends( &buf, action );
 	if( nparam > 0 && workspace_selected_any( ws ) ) {
 		if( nparam != workspace_selected_num( ws ) ) {
 			error_top( _( "Wrong number of arguments." ) );
@@ -1584,15 +1589,15 @@ workspace_add_action( Workspace *ws,
 			return( FALSE );
 		}
 
-		buf_appends( &buf, " " );
+		vips_buf_appends( &buf, " " );
 		workspace_selected_names( ws, &buf, " " );
-		if( buf_is_full( &buf ) ) {
+		if( vips_buf_is_full( &buf ) ) {
 			error_top( _( "Overflow error." ) );
 			error_sub( _( "Too many names selected." ) );
 			return( FALSE );
 		}
 
-		if( !workspace_add_def( ws, buf_all( &buf ) ) ) 
+		if( !workspace_add_def( ws, vips_buf_all( &buf ) ) ) 
 			return( FALSE );
 		workspace_deselect_all( ws );
 	}
@@ -1601,7 +1606,7 @@ workspace_add_action( Workspace *ws,
 		 * arguments. 
 		 */
 		if( !column_add_n_names( col, name, &buf, nparam ) || 
-			!workspace_add_def( ws, buf_all( &buf ) ) ) 
+			!workspace_add_def( ws, vips_buf_all( &buf ) ) ) 
 			return( FALSE );
 	}
 
@@ -1757,7 +1762,7 @@ workspace_selected_remove_yesno_cb( iWindow *iwnd, void *client,
 	Workspace *ws = WORKSPACE( client );
 
 	if( workspace_selected_remove( ws ) )
-		nfn( sys, IWINDOW_TRUE );
+		nfn( sys, IWINDOW_YES );
 	else
 		nfn( sys, IWINDOW_ERROR );
 }
@@ -1767,13 +1772,13 @@ workspace_selected_remove_yesno_cb( iWindow *iwnd, void *client,
 void
 workspace_selected_remove_yesno( Workspace *ws, GtkWidget *parent )
 {
-	BufInfo buf;
+	VipsBuf buf;
 	char str[30];
 
         if( !workspace_selected_any( ws ) ) 
 		return;
 
-	buf_init_static( &buf, str, 30 );
+	vips_buf_init_static( &buf, str, 30 );
 	workspace_selected_names( ws, &buf, ", " );
 
 	box_yesno( parent, 
@@ -1781,7 +1786,7 @@ workspace_selected_remove_yesno( Workspace *ws, GtkWidget *parent )
 		iwindow_notify_null, NULL,
 		GTK_STOCK_DELETE, 
 		_( "Delete selected objects?" ),
-		_( "Are you sure you want to delete %s?" ), buf_all( &buf ) );
+		_( "Are you sure you want to delete %s?" ), vips_buf_all( &buf ) );
 }
 
 /* Sub fn of below ... add a new index expression.
@@ -1789,13 +1794,13 @@ workspace_selected_remove_yesno( Workspace *ws, GtkWidget *parent )
 static gboolean
 workspace_ungroup_add_index( Row *row, const char *fmt, int i )
 {
-	static BufInfo buf;
+	static VipsBuf buf;
 	static char txt[200];
 
-	buf_init_static( &buf, txt, 200 );
+	vips_buf_init_static( &buf, txt, 200 );
 	row_qualified_name( row, &buf );
-	buf_appendf( &buf, fmt, i );
-	if( !workspace_add_def( row->ws, buf_all( &buf ) ) )
+	vips_buf_appendf( &buf, fmt, i );
+	if( !workspace_add_def( row->ws, vips_buf_all( &buf ) ) )
 		return( FALSE );
 
 	return( TRUE );
@@ -1835,14 +1840,14 @@ workspace_ungroup_row( Row *row )
 					return( row );
 		}
 		else {
-			BufInfo buf;
+			VipsBuf buf;
 			char txt[100];
 
-			buf_init_static( &buf, txt, 100 );
+			vips_buf_init_static( &buf, txt, 100 );
 			row_qualified_name( row, &buf );
 			error_top( _( "Unable to ungroup." ) );
 			error_sub( _( "Row \"%s\" is not a Group or a list." ), 
-				buf_all( &buf ) );  
+				vips_buf_all( &buf ) );  
 
 			return( row );
 		}
